{
  Name    :   ClientMain component.

  Purpose :   Client that uses the services of the AccountTypes component.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.

  Author  :   Paul M. Fairhurst, July 1998.
}
unit ClientMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DelphiBankServer_TLB;

type
  TFrmDelphiBankClient = class(TForm)
    Label1: TLabel;
    EditResultStr: TEdit;
    CbAccountType: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    EditAccountName: TEdit;
    EditInterestRate: TEdit;
    EditDefMinBalance: TEdit;
    Label6: TLabel;
    EditResultCode: TEdit;
    procedure CbAccountTypeChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    AnAccountTypes : IAccountTypes;
  public
    { Public declarations }
  end;

var
  FrmDelphiBankClient: TFrmDelphiBankClient;

implementation

{$R *.DFM}

procedure TFrmDelphiBankClient.CbAccountTypeChange(Sender: TObject);
const
  cUnknownStr : String = '?';
var
  AccountName   : WideString;
  InterestRate  : Single;
  DefMinBalance : Single;
  ResultStr     : WideString;
  ResultCode    : Integer;
begin
  { Query the AccountTypes component about this account. }
  if not Assigned(AnAccountTypes) then
    AnAccountTypes := CoAccountTypes.Create; { Class function from DelphiBankServer_TLB. }

  { If we got here then the call succeeded. }
  try
    ResultCode := AnAccountTypes.GetAccountType(CbAccountType.ItemIndex+1, AccountName,
                                                InterestRate, DefMinBalance, ResultStr);
    if ResultCode = 0 then begin
      EditAccountName.Text   := AccountName;
      EditResultStr.Text     := ResultStr;
      EditInterestRate.Text  := Format('%.2f', [InterestRate])+'%';
      EditDefMinBalance.Text := Format('%.2f', [DefMinBalance]);
      EditResultCode.Text    := IntToStr(ResultCode);
    end else
      raise Exception.Create(ResultStr); { Failure. }
  except
    on E: Exception do begin
      EditAccountName.Text   := cUnknownStr;
      EditInterestRate.Text  := cUnknownStr;
      EditDefMinBalance.Text := cUnknownStr;
      EditResultCode.Text    := cUnknownStr;
      EditResultStr.Text     := 'Exception - ' + E.Message;
    end;
  end;
end;

procedure TFrmDelphiBankClient.FormDestroy(Sender: TObject);
begin
  { Release the AccountTypes component.
    Normally this would be done when work has finished on the component
     and not here. However, I placed it here so you could see the
     component being used in MTS and released when the client closes.
  }
  AnAccountTypes := nil;
end;

procedure TFrmDelphiBankClient.FormShow(Sender: TObject);
begin
  CbAccountType.ItemIndex := 0;
end;

end.
