unit EmpForm;

{
  This program demonstrates using TClientDataSet in a distributed data
  application.  The data comes from a separate OLE Automation server,
  so before running this project you must compile and run EmpServ.dpr.
}

{
This application was lifted largely from the Demos included with Delphi 3 C/S; this
will allow you to refer to the original on your own copy, and if you like, follow
by making the changes I've put in here to assist in understanding what is going on.
}

interface

uses
  SysUtils, Forms, Dialogs, ExtCtrls, StdCtrls, Mask, Classes,
    DBClient, Db, Controls, DBCtrls, ComCtrls;

type
  EInvalidSalary = Class(Exception);
  ELoginRefused = class(Exception);
  TEmployeeForm = class(TForm)
    Label2: TLabel;
    btnApplyUpdates: TButton;
    btnUndo: TButton;
    btnRefresh: TButton;
    DBText1: TDBText;
    FirstName: TDBEdit;
    LastName: TDBEdit;
    PhoneExt: TDBEdit;
    HireDate: TDBEdit;
    Salary: TDBEdit;
    EmpData: TDataSource;
    navEmployees: TDBNavigator;
    Employees: TClientDataSet;
    rsvrDataAccess: TRemoteServer;
    StatusBar1: TStatusBar;
    EmployeesEmpNo: TIntegerField;
    EmployeesLastName: TStringField;
    EmployeesFirstName: TStringField;
    EmployeesPhoneExt: TStringField;
    EmployeesHireDate: TDateTimeField;
    EmployeesSalary: TFloatField;
    cboAppServer: TComboBox;
    procedure btnRefreshClick(Sender: TObject);
    procedure btnApplyUpdatesClick(Sender: TObject);
    procedure btnUndoClick(Sender: TObject);
    procedure EmployeesReconcileError(DataSet: TClientDataSet;
     E: EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
    procedure EmpDataDataChange(Sender: TObject; Field: TField);
    procedure EmployeesSalaryValidate(Sender: TField);
    procedure FormCreate(Sender: TObject);
    procedure cboAppServerChange(Sender: TObject);
    procedure DBLogin;
  end;

var
  EmployeeForm: TEmployeeForm;

implementation

uses RecError, fmLogin;

{$R *.DFM}

procedure TEmployeeForm.btnRefreshClick(Sender: TObject);
begin

{ Get data from the server.  The number of records returned is dependent on
  the PacketRecords property of TClientDataSet.  If PacketRecords is set
  to -1 then all records are returned.  Otherwise, as the user scrolls
  through the data, additional records are returned in separate data packets. }

  Employees.Close;
  Employees.Open;

end;

procedure TEmployeeForm.btnApplyUpdatesClick(Sender: TObject);
begin

{ Apply any edits.  The parameter indicates the number of errors which
  are allowed before the updating is aborted.  A value of -1 indicates that
  all successful updates be applied regardless of the number of errors. }

  Employees.ApplyUpdates(-1);
  btnRefreshClick(nil);
end;

procedure TEmployeeForm.btnUndoClick(Sender: TObject);
begin

{ Here we demonstrate a new feature in TClientDataSet, the ability to undo
  changes in reverse order.  The parameter indicates if the cursor should
  be repositioned to then record association with the change. }


  Employees.UndoLastChange(True);

end;

procedure TEmployeeForm.EmployeesReconcileError(DataSet: TClientDataSet;
  E: EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
begin

{ This is the event handler which is called when there are errors during the
  update process.  To demonstrate, you can create an error by running two
  copies of this application and modifying the same record in each one.
  Here we use the standard reconcile error dialog from the object repository. }

  Action := HandleReconcileError(DataSet, UpdateKind, E);
end;

procedure TEmployeeForm.EmpDataDataChange(Sender: TObject; Field: TField);
begin

{ This code is used to update the status bar to show the number of records
  that have been retrieved an our relative position with the dataset. }

  with Employees do
  begin
    if Active then
    begin
      StatusBar1.Panels[1].Text := Format(' %d of %d', [RecNo, RecordCount]);
    end; {If Active}
  end; {With Employees}
end;


procedure TEmployeeForm.EmployeesSalaryValidate(Sender: TField);
begin
  {Field validation through the DA Server, not at it.}
  If Not rsvrDataAccess.AppServer.IsValidSalary(EmployeesSalary.Value) then
  Begin
    Raise
      EInvalidSalary.Create('Only the CFO may grant a salary greater than $100,000.');
  end;
end;

procedure TEmployeeForm.FormCreate(Sender: TObject);
begin
  cboAppServer.ItemIndex := 0;
end;

procedure TEmployeeForm.cboAppServerChange(Sender: TObject);
begin
  {I provide here the option for the User to change
  which server he or she draws from.}
  With rsvrDataAccess do
  begin
    If ((cboAppServer.ItemIndex = 1) and (ServerName = 'ServPX.EmpServer'))
    or ((cboAppServer.ItemIndex = 0) and (ServerName = 'prDAserverIB.rdmDataAccessIB')) then
    try
      Employees.DisableControls;
      Employees.Close;
      Connected := False;
      DBLogin;
    finally
      Employees.Open;
      Employees.EnableControls;
    end; {try...finally}
  end; {With rsvrDataAccess}
end;

procedure TEmployeeForm.DBLogin;
var
  ErrorString : String;
  frmLogin : TfrmLogin;
begin
  With rsvrDataAccess Do
  begin
    Case cboAppServer.ItemIndex of
      0 : begin
            ServerName := 'ServPX.EmpServer';
            //ServerGUID := '{53BC6562-5B3E-11D0-9FFC-00A0248E4B9A}';
            Connected := True;
          end; {Case 0}
      1 : begin
            ServerName := 'prDAserverIB.rdmDataAccessIB';
            //ServerGUID := '{53BC6562-5B3E-11D0-9FFC-00A0248E4B9A}';
            {If running against a server, collect the login data from the user -
            could also launch stuff that interrogates win32 for name and so forth,
            if desired.}
            frmLogin := TfrmLogin.Create(nil);
            try
              If Not frmLogin.ShowModal = mrOK then
              begin
                cboAppServer.ItemIndex := 0;
                cboAppServerChange(nil);
              end else
              begin
                try
                  Connected := True;
                  If Not rsvrDataAccess.AppServer.DBLogin(frmLogin.UName, frmLogin.Pword) then
                  begin
                    {Login was refused - return to Paradox - note that this will pass through
                    this procedure recursively, once.}
                    ErrorString := 'That username and password combination was refused.';
                    ErrorString := ErrorString + '  This system will attach to the local database.';
                    Raise ELoginRefused.Create(ErrorString);
                  end; {If not logged in}
                except
                  On ELoginRefused do
                  begin
                    cboAppServer.ItemIndex := 0;
                    cboAppServerChange(nil);
                    Raise;
                  end; {On ELoginRefused}
                end; {try...except}
              end; {If Not frmLogin.ShowModal}
            finally
              frmLogin.Free;
            end; {try...finally}
          end; {Case 1}
    end; {Case ItemIndex}
  end; {with rsvrDataAccess}
end;

end.



