unit fRAEditorTest;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  RAEditor, IParser, StdCtrls, ExtCtrls, ComCtrls, RARegAuto, ImgList;

type
  TEditor = class(TForm)
    GutterImages: TImageList;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    RAEditor: TRAEditor;
    TabSheet2: TTabSheet;
    RAEditor1: TRAEditor;
    TabSheet3: TTabSheet;
    RAEditor2: TRAEditor;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    RAEditor3: TRAEditor;
    RAEditor4: TRAEditor;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ilCompletions: TImageList;
    RegAuto1: TRegAuto;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RAEditorGetLineAttr(Sender: TObject; var Line: String;
      index: Integer; var Attrs: TLineAttrs);
    procedure RAEditorPaintGutter(Sender: TObject; Canvas: TCanvas);
    procedure PageControl1Change(Sender: TObject);
    procedure PageControl1Enter(Sender: TObject);
    procedure RAEditorCompletionDrawItem(Control: TWinControl;
      Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure RegAuto1AfterLoad(Sender: TObject);
    procedure RegAuto1AfterSave(Sender: TObject);
  private
    Parser : TIParser;
  end;

var
  Editor: TEditor;

implementation

uses RAUtils, RAConst;

{$R *.DFM}

procedure TEditor.RAEditorGetLineAttr(Sender: TObject; var Line: String;
  index: Integer; var Attrs: TLineAttrs);

  procedure SetColor(FC, BC : TColor; Style : TFontStyles);
  var
    i : integer;
  begin
    for i := Parser.PosBeg[0]+1 to Parser.PosEnd[0] do begin
      Attrs[i].FC := FC;
      Attrs[i].BC := BC;
      Attrs[i].Style := Style;
    end;
  end;
var
  Token : string;
  i : integer;
const
  Symbols = [',', ':', ';', '.', '[', ']', '(', ')', '=', '+',
             '-', '/', '<', '>', '%', '*', '~', '''', '\', '^'];
const
  DelphiKeyWords : string =
    '  constructor  destructor  record  procedure  with  of  '+
    'repeat  until  try  finally  except  for  to  downto  case  '+
    'type  interface  implementation  initialization  finalization  '+
    'default  private  public  protected  published   automated  property  '+
    'program  read  write  override  object  nil  raise  '+
    'on  set  xor  shr  shl  begin  end  args  if  then  else  '+
    'endif  goto  while  do  var  true  false  or  and  not  mod  unit  '+
    'function  uses  external  const  class  inherited  '
  ;

  BuilderKeyWords : string =
    ' __asm  _asm  asm  auto  __automated  break  bool  case  catch  __cdecl  '+
    '_cdecl  cdecl  char  class  __classid  __closure  const  const_cast  '+
    'continue  __declspec  default  delete  __dispid  do  double  dynamic_cast  '+
    'else  enum  __except  explicit  _export  __export  extern  false  __fastcall  '+
    '_fastcall  __finally  float  for  friend  goto  if  __import  _import  inline  '+
    'int  __int8  __int16  __int32  __int64  long  mutable  namespace  new  operator  '+
    '__pascal  _pascal  pascal  private  protected  __property  public  __published  '+
    'register  reinterpret_cast  return  __rtti  short  signed  sizeof  static  static_cast  '+
    '__stdcall  _stdcall  struct  switch  template  this  __thread  throw  true  __try  '+
    'try  typedef  typename  typeid  union  using  unsigned  virtual  void  volatile  '+
    'wchar_t  while  '
  ;

  SQLKeyWords : string =
    '  active  as  add  asc  after  ascending  all  at  alter  auto  '+
    'and  autoddl  any  avg  based  between  basename  blob  '+
    'base_name  blobedit  before  buffer  begin  by  cache   compiletime  '+
    'cast   computed  char   close  character   conditional  character_length   connect  '+
    'char_length   constraint  check   containing  check_point_len   continue  check_point_length   count  '+
    'collate   create  collation   cstring  column   current  commit   cursor  '+
    'committed  database   descending  date   describe  db_key   descriptor  debug   disconnect  '+
    'dec   display  decimal  distinct  declare  do  default   domain  '+
    'delete   double  desc  drop  echo  exception  edit  execute  '+
    'else   exists  end   exit  entry_point   extern  escape   external  '+
    'event   extract  fetch  foreign  file   found  filter   from  '+
    'float   full  for   function  gdscode  grant  generator  group  '+
    'gen_id  commit_group_wait  global  group_commit_wait_time  goto  '+
    'having  help  if   input_type  immediate   insert  in  int  '+
    'inactive   integer  index  into  indicator   is  init   isolation  '+
    'inner  isql  input  join  key  '+
    'lc_messages   like  lc_type   logfile  left  log_buffer_size  length  log_buf_size  '+
    'lev   long  level  manual   merge  max   message  '+
    'maximum   min  maximum_segment  minimum  max_segment   module_name  names  not  '+
    'national   null  natural   numeric  nchar  num_log_bufs  no  num_log_buffers  '+
    'noauto  octet_length  or  of   order  on   outer  only  output  '+
    'open  output_type  option  overflow  page  post_event  pagelength   precision  '+
    'pages   prepare  page_size  procedure  parameter   protected  password   primary  '+
    'plan   privileges  position   public  quit  '+
    'raw_partitions   retain  rdb  db_key   return  read   returning_values  real   returns  '+
    'record_version  revoke  references   right  release   rollback  reserv  runtime  '+
    'reserving  schema   sql  segment   sqlcode  select   sqlerror  set   sqlwarning  '+
    'shadow   stability  shared   starting  shell   starts  show   statement  '+
    'singular   static  size   statistics  smallint   sub_type  snapshot   sum  '+
    'some  suspend  sort  table   translate  terminator   translation  then   trigger  to   trim  '+
    'transaction  uncommitted  upper  union   user  unique  using  update  '+
    'value  varying  values  version  varchar  view  variable  '+
    'wait  while  when  with  whenever  work  where  write  '+

    'term  new  old '
   ;

  function IsDelphiKeyWord(St : string) : boolean;
  begin
    Result := Pos(' '+ANSILowerCase(St)+' ', DelphiKeyWords) <> 0;
  end;
  function IsBuilderKeyWord(St : string) : boolean;
  begin
    Result := Pos(' '+St+' ', BuilderKeyWords) <> 0;
  end;
  function IsSQLKeyWord(St : string) : boolean;
  begin
    Result := Pos(' '+ANSILowerCase(St)+' ', SQLKeyWords) <> 0;
  end;

  function IsStringConstant(St : string) : boolean;
  var
    LS : integer;
  begin
    LS := Length(St);
    if (LS >= 2) and (((St[1] = '''') and (St[LS] = '''')) or ((St[1] = '"') and (St[LS] = '"'))) then
      Result := true else
      Result := false
  end;

  function IsRealConstant(St : string) : boolean;
  var
    i, j : integer;
    Point : boolean;
  begin
    Result := false;
    if (St = '.') or (St = '') then exit;
    if St[1] = '-' then
      if Length(St) = 1 then exit
      else j := 2
    else j := 1;
    Point := false;
    for i := j to Length(St) do
      if St[i] = '.' then
         if Point then exit else Point := true
      else if (St[i] < '0') or (St[i] > '9') then exit;
    Result := true;
  end;

  function IsIntConstant(St : string) : boolean;
  var
    i, j : integer;
    Sym : TSetOfChar;
  begin
    Result := false;
    if (Length(St) = 0) or ((Length(St) = 1) and (St[1]='$')) then exit;
    Sym := StConstSymbols10;
    if (St[1] = '-') or (St[1] = '$') then begin
      if Length(St) = 1 then exit
      else j := 2;
      if St[1] = '$' then Sym := StConstSymbols;
    end else j := 1;
    for i := j to Length(St) do
      if not CharInSet(St[i], Sym) then exit;
    Result := true;
  end;

var
  F : boolean;
  L : integer;
  FC : TColor;
  FS : TFontStyles;
  Ed : TRAEditor;
begin
  Ed := Sender as TRAEditor;
  Parser.pcProgram := PChar(Line);
  Parser.pcPos := Parser.pcProgram;
  L := Ed.Tag;
  if (L = 1) and (Length(Line) > 0) and (Line[1] = '#') then
  begin
    FC := clGreen;
    FS := Ed.Font.Style;
  end else
  begin
    FC := clOlive; { }
    FS := [fsItalic];
  end;
  Attrs[1].FC := FC;
  Attrs[1].Style := FS;
  Attrs[1].BC := Ed.Color;
  for i := 1 to 100 do
     Move(Attrs[1], Attrs[i], sizeof(Attrs[1]));
  if (L = 1) and (Length(Line) > 0) and (Line[1] = '#') then exit;
  try
    Token := Parser.Token;
    while Token <> '' do begin
      F := true;
      case L of
        0 : if IsDelphiKeyWord(Token) then SetColor(clBlack, Ed.Color, [fsBold]) else F := false;
        1 : if IsBuilderKeyWord(Token) then SetColor(clBlack, Ed.Color, [fsBold]) else F := false;
        2 : if IsSQLKeyWord(Token) then SetColor(clBlack, Ed.Color, [fsBold]) else F := false;
        else F := false;
      end;
      if F then {Ok}
      else if (Length(Token) = 1) and (Token[1] in Symbols) then
        SetColor(clBlue, Ed.Color, [])
      else if IsIntConstant(Token) or IsRealConstant(Token) then
        SetColor(clNavy, Ed.Color, [])
      else if IsStringConstant(Token) then
        SetColor(clPurple, Ed.Color, [])
      else
        SetColor(clBlack, Ed.Color, []);
      Token := Parser.Token;
    end;
  except

  end;
end;

procedure TEditor.FormCreate(Sender: TObject);
begin
  Application.Title := Caption;
  Parser := TIParser.Create;
end;

procedure TEditor.FormDestroy(Sender: TObject);
begin
  Parser.Free;
end;

procedure TEditor.RAEditorPaintGutter(Sender: TObject; Canvas: TCanvas);
  procedure Draw(Y, ImageIndex : integer);
  var
    Ro : integer;
    R : TRect;
  begin
    if Y <> -1 then
      with Sender as TRAEditor do begin
        Ro := Y - TopRow;
        R := CalcCellRect(0, Ro);
        GutterImages.Draw(Canvas,
          GutterWidth -GutterRightMargin -GutterImages.Width{R.Left},
          R.Top + (CellRect.Height - GutterImages.Height) div 2 +1,
          ImageIndex);
      end;
  end;
var
  i  : integer;
begin
  for i := 0 to 9 do
    if (Sender as TRAEditor).Bookmarks[i].Valid then
      Draw((Sender as TRAEditor).Bookmarks[i].Y, i);
end;

procedure TEditor.PageControl1Change(Sender: TObject);
begin
  Parser.Style := TIParserStyle(not(PageControl1.ActivePage.PageIndex=0));
  RAEditor.Refresh;
  RAEditor1.Refresh;
  RAEditor2.Refresh;
  RAEditor3.Refresh;
end;

procedure TEditor.PageControl1Enter(Sender: TObject);
begin
  (PageControl1.ActivePage.Controls[0] as TWinControl).SetFocus;
end;

procedure TEditor.RAEditorCompletionDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
	Offset, W : Integer;
  S : string;
  ImageIndex : integer;
begin
  Offset := 3;
  with Control as TListBox, (Control.Owner as TRAEditor).Completion do
  begin
    Canvas.FillRect(Rect);
    case Mode of
      cmIdentifers :
        begin
          ImageIndex := StrToInt(Trim(SubStr(Items[Index], 2, Separator))) - 1;
          ilCompletions.Draw(Canvas, Rect.Left + 2, Rect.Top, ImageIndex);
          Canvas.TextOut(Rect.Left + 3*Offset + ilCompletions.Width, Rect.Top +2, SubStr(Items[Index], 0, Separator));
          S := Trim(SubStr(Items[Index], 1, Separator));
          W := Canvas.TextWidth(S);
          Canvas.TextOut(Rect.Right - 2*Offset - W, Rect.Top +2, S);
        end;
      cmTemplates :
        begin
          Canvas.TextOut(Rect.Left + Offset, Rect.Top +2, SubStr(Items[Index], 1, Separator));
          Canvas.Font.Style := [fsBold];
          S := SubStr(Items[Index], 0, Separator);
          W := Canvas.TextWidth(S);
          Canvas.TextOut(Rect.Right - 2*Offset - W, Rect.Top +2, S);
        end;
    end;
  end;  
end;

procedure TEditor.RegAuto1AfterLoad(Sender: TObject);
begin
  PageControl1.ActivePage := PageControl1.Pages[RegAuto1.ReadInteger(Name, 'PageIndex', PageControl1.ActivePage.PageIndex)];
  PageControl1Change(nil);
end;


procedure TEditor.RegAuto1AfterSave(Sender: TObject);
begin
  RegAuto1.WriteInteger(Name, 'PageIndex', PageControl1.ActivePage.PageIndex);
end;

end.
