{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAIFile;

interface

uses Windows, SysUtils, Classes,
  RAIIntf, RAIClas, RAIProg, RAISystem;

type

  TIFileUtil = class(TIUnit)
  public
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
    constructor Create;
    destructor Destroy; override;
  end;


implementation

type

  TIFileStream = class(TFileStream, IIObject)
  private
    FRefCount : integer;
  protected
    function SetGetValue(var SGI : TSetGetInfo) : boolean; {override;}
  public
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function _AddRef: integer; virtual; stdcall;
    function _Release: integer; virtual; stdcall;
  end;

{*************************** TIFileUtil *************************}
constructor TIFileUtil.Create;
  procedure AddConst(Name : string; Value : integer);
  begin
    Ids.AddObject(Name, TObject(Value));
  end;

begin
  inherited Create;
  Name := 'FileUtil';
{ File open modes }
  AddConst('fmCreate'        , $FFFF);
  AddConst('fmOpenRead'      , $0000);
  AddConst('fmOpenWrite'     , $0001);
  AddConst('fmOpenReadWrite' , $0002);
  AddConst('fmShareCompat'   , $0000);
  AddConst('fmShareExclusive', $0010);
  AddConst('fmShareDenyWrite', $0020);
  AddConst('fmShareDenyRead' , $0030);
  AddConst('fmShareDenyNone' , $0040);
{ TStream seek origins }
  AddConst('soFromBeginning' , 0);
  AddConst('soFromCurrent'   , 1);
  AddConst('soFromEnd'       , 2);
{ File attribute constants }
  AddConst('faReadOnly'  , $00000001);
  AddConst('faHidden'    , $00000002);
  AddConst('faSysFile'   , $00000004);
  AddConst('faVolumeID'  , $00000008);
  AddConst('faDirectory' , $00000010);
  AddConst('faArchive'   , $00000020);
  AddConst('faAnyFile'   , $0000003F);
  Ids.Sort;
end;

destructor TIFileUtil.Destroy;
begin
  inherited Destroy;
end;

function TIFileUtil.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  Ind : integer;
begin
  with SGI do
  if Get then begin
    Result := true;
    if Ids.Find(SGI.Name, Ind) then Res.vInteger := integer(Ids.Objects[Ind])
    else if ProcIs('TFileStream', 0, [ivEmpty]) then Res.vClass := Name
    else if ClassIs('TFileStream') and ProcIs('Create', 3, [ivClass, ivString, ivInteger]) then begin
      Res.vObject := TIFileStream.Create(Args[1].vString, Args[2].vInteger);
    end else Result := false;
  end else Result := false;
end;

{*************************** TIFileStream *************************}
 {************* IIObject interface *************}
function TIFileStream.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

function TIFileStream._AddRef: Longint;
begin
  Result := IObjectAddRef(Self, FRefCount);
end;

function TIFileStream._Release: Longint;
begin
  Result := IObjectRelease(Self, FRefCount);
end;
 {############# IIObject interface #############}

function TIFileStream.SetGetValue(var SGI : TSetGetInfo) : boolean;
  function ProcIs(Name : TIdentifer; Count : integer; Params : array of TIValueTyp) : boolean;
  begin
    try
      Result := Cmp(SGI.Name, Name) and IIArgsCheckParams(SGI.Args, Count, Params);
    except
      on E : EInterError do begin E.VarName := Name; raise; end
      else raise;
    end;
  end;
var
  S : string;
  I : integer;  
begin
  with SGI do
  if Get then begin
    Result := true;
   { Don't use Free method, use assignment "Variable := nil",
     this will automatically _Release object
     
    if ProcIs('Free', 0, [ivEmpty]) then
      begin
        Res.Empty;
        FileClose(Handle);
        _Release
      end
   }
   { This will be uncomment in future verions of RAInter
     else if ProcIs('Read', 2, [ivPointer, ivInteger]) then
      case Args[0].vPointer.Typ of
        ivString :
          begin
            SetLength(S, Args[1].vInteger);
            Res.vInteger := Read(S[1], Args[1].vInteger);
            Args[1].vPointer.vString := iString(S);
          end;
        else InterErrorS(ieTypeMistmach);
      end }
    if ProcIs('ReadString', 1, [ivInteger]) then
      begin
        SetLength(S, Args[0].vInteger);
        ReadBuffer(S[1], Args[0].vInteger);
        Res.vString := iString(S);
      end
    else if ProcIs('ReadInteger', 1, [ivInteger]) then
      begin
        ReadBuffer(I, Args[0].vInteger);
        Res.vInteger := I;
      end
    else if ProcIs('WriteString', 2, [ivString, ivInteger]) then
      begin
        WriteBuffer(Args[0].vString[0], Args[1].vInteger);
        Res.Empty;
      end
    else if ProcIs('WriteInteger', 2, [ivInteger, ivInteger]) then
      begin
        I := Args[0].vInteger;
        WriteBuffer(I, Args[1].vInteger);
        Res.Empty;
      end
    else Result := false;
  end else Result := false;
end;
{########################### TIFileStream #########################}


end.
