unit fEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs,
  RARegAuto, ComCtrls, StdCtrls, Menus, Forms, RAEditor, ExtCtrls, RAHint,
  RAIProc{Parser}, RAIClas{Lang}, RAIIntf, RAIImpl, RAIExpr;

const
  clDontChange = High(TColor);
  FclExecFC = clDontChange;
  FclExecBC = clSilver;
  FclStackFC = clDontChange;
  FclStackBC = $00DDDDDD;
  FclBreakFC = clWhite;
  FclBreakBC = clRed;

  { }
  imExecute = 0;
  imError   = 1;
  imStack   = 2;
  imBreak   = 3;

type

  TEditor = class;

  TEMemo = class(TRAEditor)
  private
    FExecBegX, FExecBegY, FExecEndX, FExecEndY : integer;
    FStackBegX, FStackBegY, FStackEndX, FStackEndY : integer;
    FErrY : integer;
    XX, YY : integer;
  protected
    procedure ChangeAttr(Line, ColBeg, ColEnd : integer); override;
    procedure GutterPaint(Canvas : TCanvas); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure ShowPoint(PosBeg, PosEnd : integer; var BegX, BegY, EndX, EndY : integer);
    procedure TextModified(Pos : integer; Action : TModifiedAction; Text : string); override;
  public
    constructor Create(AOwner : TComponent); override;
    procedure ShowExecutionPoint(PosBeg, PosEnd : integer);
    procedure ShowCallStackPoint(PosBeg, PosEnd : integer);
    procedure ShowErrorPoint(ErrPos : integer);
    procedure ShowBreakPoint(PosBeg, PosEnd : integer);
    function BPIndex(PBeg, PEnd : integer) : integer;
    procedure pmEditorPopup;
    procedure TogleBreakPoint;
  end;

  TTab = class(TTabSheet)
  private
    FileName : TFileName;
    FEMemo    : TEMemo;
    Form     : TEditor;
  //  FModified : boolean;
    Parser : TIParser;
    constructor CreateF(AOwner : TComponent; AForm : TEditor; AFileName : TFileName; Source : string);
    procedure Save;
    function Check4Save : boolean;
    procedure Close;
    procedure MemoChangeStatus(Sender: TObject);
    function GetModified : boolean;
    procedure SetModified(M : boolean);
    procedure RAEditorGetLineAttr(Sender : TObject; var Line: String; index: Integer; var Attrs: TLineAttrs);

    property Modified : boolean read GetModified write SetModified;
  public
    UnitName : string;
    property EMemo : TEMemo read FEMemo;
  end;

  TGetUnitSource = function(UnitName : iString; var Source, UnitFullName : iString) : boolean of object;

  TEditor = class(TForm)
    RegAuto: TRegAuto;
    StatusBar: TStatusBar;
    pcFiles: TPageControl;
    lblFont: TLabel;
    pmEditor: TPopupMenu;
    miClosePage: TMenuItem;
    miOpenFileAtCursor: TMenuItem;
    OpenDialog: TOpenDialog;
    Images: TImageList;
    RAHint1: TRAHint;
    miTogleBreakPoint: TMenuItem;
    N1: TMenuItem;
    miReadOnly: TMenuItem;
    miAddWatch: TMenuItem;
    miEvaluateModify: TMenuItem;
    LargeGutterImages: TImageList;
    procedure miClosePageClick(Sender: TObject);
    procedure pcFilesChange(Sender: TObject);
    procedure RegAutoAfterLoad(Sender: TObject);
    procedure RegAutoAfterSave(Sender: TObject);
    procedure miOpenFileAtCursorClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure pmEditorPopup(Sender: TObject);
    procedure miTogleBreakPointClick(Sender: TObject);
    procedure miReadOnlyClick(Sender: TObject);
    procedure miEvaluateModifyClick(Sender: TObject);
  private
    { Private declarations }
    GetUnitSource : TGetUnitSource;
    function GetTab(index : integer) : TTab;
    procedure UpdateStatus;
    function TabsCount : integer;
    function iFileOpen(lFileName : TFileName) : boolean;
    property Tabs[index : integer] : TTab read GetTab;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMGetMinMaxInfo(var M : TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    { Public declarations }
    SearchPath : string;
    procedure FileOpen;
    procedure FileSave;
    procedure FileSaveAll;
    procedure FileClose;
    procedure FileNew;
    procedure FileSaveAs;
    function ActiveTab : TTab;
    function OpenUnit(lUnitName : string; ShowOpenDialog : boolean) : boolean;
    procedure DeleteAllErrorPoint;
    procedure ShowBreakPoint(index : integer);
    property OnGetUnitSource : TGetUnitSource read GetUnitSource write GetUnitSource;
  end;

var
  Editor: TEditor;

implementation

uses fMain, fConsole, RAUtils, iMTracer, fBreakpoints;

{$R *.DFM}

const
  ISExt = '.itr';

{************************ TTab ***********************}
constructor TTab.CreateF(AOwner : TComponent; AForm : TEditor; AFileName : TFileName; Source : string);
begin
  inherited Create(AOwner);
  Form := AForm;
  PageControl := AOwner as TPageControl;
  Parser := TIParser.Create;
  FEMemo := TEMemo.Create(self);
  with FEMemo do begin
    Parent := self;
    Color := clWhite;
    GutterWidth := 16;
    Align := alClient;
    Font := Form.lblFont.Font;
    Lines.Text := Source;
    OnChangeStatus := MemoChangeStatus;
    ScrollBars := ssBoth;
    OnGetLineAttr := RAEditorGetLineAttr;
    TabStop := true;
  end;
  Parent := PageControl;
  FileName := AFileName;
  AFileName := ExtractFileName(FileName);
  if ANSICompareText(
    Copy(AFileName, Length(AFileName)-3, Length(AFileName))
    , ISExt) = 0 then
    AFileName := Copy(AFileName, 1, Length(AFileName)-4);
  UnitName := AFileName;
  Caption := AFileName;
end;

procedure TTab.Save;
begin
  FEMemo.Lines.SaveToFile(FileName);
  Modified := false;
  Form.UpdateStatus;
end;

function TTab.Check4Save : boolean;
begin
  //   
  Result := true;
  if Modified then
    case MessageDlg(' '''+FileName+''' . ?', mtConfirmation, mbYesNoCancel, 0) of
      mrOk     : Save;
      mrCancel : begin Result := false; exit end;
    end;
  Form.pcFilesChange(PageControl);
end;

procedure TTab.Close;
var
  F : TEditor;
begin
  PageControl.ActivePage := PageControl.FindNextPage(self, false, true);
  F := Form;
  Parser.Free;
  Free;
  F.pcFilesChange(PageControl);
end;

procedure TTab.MemoChangeStatus(Sender: TObject);
begin
  Form.UpdateStatus;
end;

function TTab.GetModified : boolean;
begin
  Result := FEMemo.Modified;
end;

procedure TTab.SetModified(M : boolean);
begin
  FEMemo.Modified := M;
end;

procedure TTab.RAEditorGetLineAttr(Sender : TObject; var Line: String; index: Integer; var Attrs: TLineAttrs);
{Syntax Highlighting}
  procedure SetColor(FC, BC : TColor; Style : TFontStyles);
  var
    i : integer;
  begin
    for i := Parser.PosBeg[0]+1 to Parser.PosEnd[0] do begin
      Attrs[i].FC := FC;
      Attrs[i].BC := BC;
      Attrs[i].Style := Style;
    end;
  end;

  function LenStr(S : string): integer;
  var
    i : integer;
  begin
    Result := Length(S);
    for i := 2 to Result - 1 do
      if S[i] = '''' then inc(Result);
  end;

var
  S : string;
  Token : string;
  i : integer;
const
  Symbols = [',', ':', ';', '.', '[', ']', '(', ')', '=', '+',
             '-', '/', '<', '>', '%', '*', '~', '''', '\'];
begin
  S := Line;
  Parser.pcProgram := PChar(S);
  Parser.pcPos := Parser.pcProgram;
  Attrs[1].FC := clOlive; { }
  Attrs[1].Style := FEMemo.Font.Style;
  Attrs[1].BC := FEMemo.Color;
  for i := 1 to 100 do
     Move(Attrs[1], Attrs[i], sizeof(Attrs[1]));
  try
    Token := Parser.Token;
    while Token <> '' do begin
      if Lang.ReservedWord(Token) then
        SetColor(clBlack, FEMemo.Color, [fsBold])
      else if (Length(Token) = 1) and (Token[1] in Symbols) then
        SetColor(clNavy, FEMemo.Color, [])
      else if Lang.IsStringConstant(Token) then begin
       // SetColor(clPurple, FEMemo.Color, [])
        for i := Parser.PosEnd[0] - LenStr(Token) + 1 to Parser.PosEnd[0] do begin
          Attrs[i].FC := clPurple;
          Attrs[i].BC := FEMemo.Color;
          Attrs[i].Style := [];
        end;
      end
      else if (Lang.IsIntConstant(Token)) or (Lang.IsRealConstant(Token)) then
        SetColor(clNavy, FEMemo.Color, [])
      else
        SetColor(clBlack, FEMemo.Color, []);
      Token := Parser.Token;
    end;
  except

  end;
end;
{######################## TTab #######################}

{************************ TEditor ***********************}
function TEditor.iFileOpen(lFileName : TFileName) : boolean;
var
  Tab : TTab;
  i : integer;
  S, UnitFullName : string;
  P1, P2 : iString;
begin
  Result := false;
  {   ,    }
  for i := 0 to TabsCount-1 do
    if Tabs[i].FileName = lFileName then begin
      pcFiles.ActivePage := Tabs[i];
      Result := true;
      exit;
    end;
  {  }
  if GetUnitSource(iString(lFileName), P1, P2) then begin
    S := P1;
    UnitFullName := P2;
    Tab := TTab.CreateF(pcFiles, self, UnitFullName, S);
    if Tab.FileName = '' then Tab.Free;
    pcFiles.ActivePage := Tab;
    pcFilesChange(pcFiles);
    Visible := true;
    Result := true;
  end;
end;

function TEditor.OpenUnit(lUnitName : string; ShowOpenDialog : boolean) : boolean;
var
  FN : TFileName;
  i : integer;
begin
  Result := false;
  {   ,    }
  for i := 0 to TabsCount-1 do
    if Tabs[i].UnitName = lUnitName then begin
      pcFiles.ActivePage := Tabs[i];
      Result := true;
      break;
    end;
  if not Result then
  Result := iFileOpen(lUnitName);
  if not Result and ShowOpenDialog then begin
    FN := SearchPath+lUnitName+ISExt;
    OpenDialog.FileName := FN;
    if OpenDialog.Execute then iFileOpen(OpenDialog.FileName);
  end;
end;

procedure TEditor.FileOpen;
begin
  if OpenDialog.Execute then
    iFileOpen(OpenDialog.FileName);
end;

procedure TEditor.FileSave;
begin
  if ActiveTab <> nil then ActiveTab.Save
end;

procedure TEditor.FileSaveAll;
var
  i : integer;
begin
  for i := 0 to pcFiles.PageCount-1 do Tabs[i].Save;
end;

procedure TEditor.FileClose;
begin
  if (ActiveTab <> nil) and ActiveTab.Check4Save then
    ActiveTab.Close;
end;

procedure TEditor.FileNew;
var
  Tab : TTab;
begin
  Tab := TTab.CreateF(pcFiles, self, 'Unit1', 'Unit Unit1; '#10#13'EndUnit;');
  if Tab.FileName = '' then Tab.Free;
  Visible := true;
  OpenUnit('Unit1', false);
  Tab.Modified := true;
  UpdateStatus;
end;

procedure TEditor.FileSaveAs;
begin

end;

function TEditor.TabsCount : integer;
begin
  Result := pcFiles.PageCount;
end;

function TEditor.GetTab(index : integer) : TTab;
begin
  Result := pcFiles.Pages[index] as TTab;
end;

function TEditor.ActiveTab : TTab;
begin
  Result := pcFiles.ActivePage as TTab
end;

procedure TEditor.miClosePageClick(Sender: TObject);
begin
  FileClose;
end;

procedure TEditor.pcFilesChange(Sender: TObject);
begin
  if pcFiles.PageCount = 0 then Close
  else begin
    Caption := ActiveTab.FileName;
   // ActiveTab.EMemo.NeedRepaint := true;
   // ActiveTab.EMemo.Repaint;
    UpdateStatus;
  end;
end;

procedure TEditor.UpdateStatus;
const
  Modi : array[boolean] of string[10] = ('', 'Modify');
  Modes : array[boolean] of string[10] = ('Overwrite', 'Insert');
var
  AllModified : boolean;
  i : integer;
begin
  with StatusBar, ActiveTab.EMemo do begin
    Panels[0].Text := IntToStr(CaretY) + ':' + IntToStr(CaretX);
    Panels[1].Text := Modi[Modified];
    if ReadOnly then Panels[2].Text := 'Read only'
    else Panels[2].Text := Modes[InsertMode];
  end;
  Main.sbSave.Enabled := ActiveTab.Modified;
  AllModified := false;
  for i := 0 to TabsCount-1 do
    if Tabs[i].Modified then begin
      AllModified := true;
      break
    end;
  Main.sbSaveAll.Enabled := AllModified;
  Main.miFileSaveAll.Enabled := Main.sbSaveAll.Enabled;
  Main.miFileSave.Enabled := Main.sbSave.Enabled;
  Main.ButtonsEnabled(true);
end;

procedure TEditor.RegAutoAfterLoad(Sender: TObject);
var
  i : integer;
  FN : TFileName;
begin
  for i := 0 to RegAuto.ReadInteger('Editor\OpenFiles', 'Count', 0)-1
  do begin
    iFileOpen(RegAuto.ReadString('Editor\OpenFiles',
      'File'+IntToStr(i), ''));
  end;
  if ActiveTab <> nil then FN := ActiveTab.FileName else FN := '';
  FN := RegAuto.ReadString('Editor\OpenFiles', 'ActiveFile', FN);
  for i := 0 to TabsCount-1 do
    if Tabs[i].FileName = FN then begin
      pcFiles.ActivePage := Tabs[i]
    end;
end;

procedure TEditor.RegAutoAfterSave(Sender: TObject);
var
  i : integer;
begin
  RegAuto.EraseSection('Editor\OpenFiles');
  RegAuto.WriteInteger('Editor\OpenFiles', 'Count', TabsCount);
  for i := 0 to TabsCount-1 do
    RegAuto.WriteString('Editor\OpenFiles', 'File'+IntToStr(i), Tabs[i].FileName);
  if ActiveTab <> nil then
    RegAuto.WriteString('Editor\OpenFiles', 'ActiveFile', ActiveTab.FileName);
end;

procedure TEditor.miOpenFileAtCursorClick(Sender: TObject);
begin
  with ActiveTab.FEMemo do
    OpenUnit(GetWordOnPos(Lines.Text, SelStart), true);
end;
                      
procedure TEditor.FormCreate(Sender: TObject);
begin
  SearchPath := ExtractFilePath(Application.ExeName);
end;

procedure TEditor.FormClose(Sender: TObject; var Action: TCloseAction);
var
  i : integer;
begin
  RegAuto.Save;
  for i := 0 to pcFiles.PageCount-1 do
    if not Tabs[0].Check4Save then begin
      Action := caNone;
      exit;//break;
    end;
  while pcFiles.PageCount > 0 do Tabs[0].Close;
  Main.ButtonsEnabled(false);
end;

procedure TEditor.pmEditorPopup(Sender: TObject);
begin
  if ActiveTab <> nil then
    ActiveTab.EMemo.pmEditorPopup;
end;

procedure TEditor.miTogleBreakPointClick(Sender: TObject);
begin
  if ActiveTab <> nil then
    ActiveTab.EMemo.TogleBreakPoint;
end;

procedure TEditor.DeleteAllErrorPoint;
var
  i : integer;
begin
  for i := 0 to TabsCount-1 do
    Tabs[i].EMemo.ShowErrorPoint(-1);
end;

procedure TEditor.ShowBreakPoint(index : integer);
begin
  with Main.InterObj.IDebugObject.BreakPoints[index] do 
    if OpenUnit(UnitName, false) then
      ActiveTab.EMemo.ShowBreakPoint(PosBeg, PosEnd);
end;

{****************************** TEMemo ********************************}
constructor TEMemo.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
//  Splitter.MinSize := 15;
  FExecBegY := -1;
  FStackBegY := -1;
  FErrY := -1;
end;

procedure TEMemo.ShowPoint(PosBeg, PosEnd : integer; var BegX, BegY, EndX, EndY : integer);
var
  OldBegY, OldEndY ,iR : integer;
begin
  FErrY := -1;
  OldBegY := BegY; OldEndY := EndY;
  CaretFromPos(PosBeg, BegX, BegY);
  CaretFromPos(PosEnd+1, EndX, EndY);
  if (BegY < 0) and (OldBegY < 0) then exit;
  if (BegY > -1) and (BegY < TopRow) or (BegY > LastVisibleRow)
  then SetLeftTop(LeftCol, BegY - VisibleRowCount div 2);
  {    ...}
  {}
  for iR := OldBegY to OldEndY do PaintLine(iR, -1, -1);
  if BegY < OldBegY then OldBegY := BegY else OldBegY := OldEndY+1;
  if EndY > OldEndY then OldEndY := EndY else OldEndY := OldBegY-1;
  for iR := OldBegY to OldEndY do PaintLine(iR, -1, -1);
  {  Gutter'}
  Gutter.Paint;
end;

procedure TEMemo.ShowExecutionPoint(PosBeg, PosEnd : integer);
begin
  ShowPoint(PosBeg, PosEnd, FExecBegX, FExecBegY, FExecEndX, FExecEndY);
end;

procedure TEMemo.ShowCallStackPoint(PosBeg, PosEnd : integer);
begin
  ShowPoint(PosBeg, PosEnd, FStackBegX, FStackBegY, FStackEndX, FStackEndY);
end;

procedure TEMemo.ShowBreakPoint(PosBeg, PosEnd : integer);
var
  BegX, BegY, v3, v4 : integer;
begin
  ShowPoint(PosBeg, PosEnd, BegX, BegY, v3, v4);
  CaretFromPos(PosBeg, BegX, BegY);
  SetCaret(BegX, BegY);
  SetFocus;
end;

procedure TEMemo.ShowErrorPoint(ErrPos : integer);
var
  X, Y : integer;
begin
  CaretFromPos(ErrPos, X, Y);
  FErrY := Y;
  if (Y > -1) and (Y < TopRow) or (Y > LastVisibleRow)
  then SetLeftTop(LeftCol, Y - VisibleRowCount div 2);
  {    ...}
  if (Y > -1) then SetCaret(X, Y);
  {  Gutter'}
  Gutter.Paint;
end;

procedure TEMemo.ChangeAttr(Line, ColBeg, ColEnd : integer);
  procedure Change(BegX, BegY, EndX, EndY : integer; clFC, clBC : TColor);
    procedure DoChange(const iBeg, iEnd : integer);
    var i : integer;
    begin
      for i := iBeg to iEnd do begin
        if clFC <> clDontChange then LineAttrs[i+1].FC := clFC;
        if clBC <> clDontChange then LineAttrs[i+1].BC := clBC;
      end;
    end;
  begin
    if (Line = BegY) and (Line = EndY) then DoChange(BegX, EndX-1)
    else begin
      if Line = BegY then DoChange(BegX, BegX + VisibleColCount);
      if (Line > BegY) and (Line < EndY) then DoChange(ColBeg, ColEnd);
      if Line = EndY then DoChange(ColBeg, EndX-1);
    end;
  end;
var
  i, BegX, BegY, EndX, EndY : integer;
begin
 {Show BreakPoints}
  with Main.InterObj.IDebugObject do
    for i := 0 to BreakPoints.Count-1 do with BreakPoints[i] do begin
      if Cmp(PChar((Owner as TTab).UnitName), PChar(UnitName))
      then begin
        CaretFromPos(PosBeg, BegX, BegY);
        CaretFromPos(PosEnd, EndX, EndY);
        Change(BegX, BegY, EndX, EndY, FclBreakFC, FclBreakBC);
      end;
    end;
 {ShowExecutionPoint}
  Change(FExecBegX, FExecBegY, FExecEndX, FExecEndY, FclExecFC, FclExecBC);
 {ShowCallStackPoint}
  Change(FStackBegX, FStackBegY, FStackEndX, FStackEndY, FclStackFC, FclStackBC);
end;

procedure TEMemo.GutterPaint(Canvas : TCanvas);
  procedure Draw(Y, ImageIndex : integer);
  var
    Ro : integer;
    R : TRect;
  begin
    if Y <> -1 then begin
      Ro := Y - TopRow;
      R := CalcCellRect(0, Ro);
      (Owner as TTab).Form.Images.Draw(Canvas,
        GutterWidth -GutterRightMargin -(Owner as TTab).Form.Images.Width{R.Left},
        R.Top + (CellRect.Height - (Owner as TTab).Form.Images.Height) div 2 +1,
        ImageIndex);
    end;
  end;
var
  i, v, BegY : integer;
begin
  {   ...}
  with Main.InterObj.IDebugObject do
    for i := 0 to BreakPoints.Count-1 do with BreakPoints[i] do begin
      if Cmp(PChar((Owner as TTab).UnitName), PChar(UnitName))
      then begin
        CaretFromPos(PosBeg, v, BegY);
        if (BegY <> FExecBegY) and (BegY <> FStackBegY) then
          Draw(BegY, imBreak);
      end;
    end;
  if FErrY <> -1 then Draw(FErrY, imError);
  if FStackBegY <> -1 then Draw(FStackBegY, imStack);
  if FExecBegY <> -1 then Draw(FExecBegY, imExecute);
end;

procedure TEMemo.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  P : integer;
  W, W1 : string;
  R : TRect;
begin
  XX := X; YY := Y;
  {$IFDEF DEBUG}
  Tracer.Writeln(IntToStr(X)+', '+IntToStr(Y));
  {$ENDIF}
  P := PosFromMouse(X, Y);
  if P = -1 then exit;
  W := GetWordOnPos(Lines.Text, P);
  if Main.InterObj.IDebugObject.EvaluateModify(W, W1, dfHint) then begin
    R := ClientRect;
    OffsetRect(R, ClientOrigin.X, ClientOrigin.Y);
    (Owner as TTab).Form.RAHint1.ActivateHint(R, W+' = '+W1);
  end;
end;

procedure TEditor.WMSize(var Message: TWMSize);
begin
  inherited;
 { if ActiveTab <> nil then
    if Message.SizeType = SIZE_MAXIMIZED then
      ActiveTab.EMemo.Paint
    else if Message.SizeType = SIZE_MINIMIZED then
      ActiveTab.EMemo.NeedRepaint := true; }
end;

function TEMemo.BPIndex(PBeg, PEnd : integer) : integer;
var
  i : integer;
begin
  Result := -1;
  with Main.InterObj.IDebugObject do
    for i := 0 to BreakPoints.Count-1 do with BreakPoints[i] do begin
      if Cmp(PChar((Owner as TTab).UnitName), PChar(UnitName)) and
       ((PosBeg <= PBeg) and (PEnd <= PosEnd))
      then begin Result := i; exit end;
    end;
end;

procedure TEMemo.pmEditorPopup;
var
  P : integer;
begin
  P := PosFromCaret(CaretX, CaretY);
  (Owner as TTab).Form.miTogleBreakPoint.Enabled := P <> -1;
  if (P = -1) or (BPIndex(P, P) = -1) then
    (Owner as TTab).Form.miTogleBreakPoint.Caption := 'Add &breakpoint'
  else
    (Owner as TTab).Form.miTogleBreakPoint.Caption := 'Remove &breakpoint';
  (Owner as TTab).Form.miReadOnly.Checked := ReadOnly;
end;

procedure TEMemo.TogleBreakPoint;
var
  P, PBeg, PEnd, Ind, iR : integer;
  BegY, EndY : integer;
  BP : TBreakPoint;
begin
  P := PosFromCaret(CaretX, CaretY);
  Ind := BPIndex(P, P);
  if Ind = -1 then begin
    ProgGetBlockByPos(iString(Lines.Text), P, PBeg, PEnd);
    if (PBeg = -1) or (PEnd = -1) then exit;
    BP := TBreakPoint.Create;
    BP.UnitName := (Owner as TTab).UnitName;
    BP.PosBeg := PBeg;
    BP.PosEnd := PEnd;
    Main.InterObj.IDebugObject.BreakPoints.Add(BP);
  end else
    with Main.InterObj.IDebugObject do begin
      PBeg := BreakPoints[Ind].PosBeg;
      PEnd := BreakPoints[Ind].PosEnd;
      BreakPoints.Delete(Ind);
    end;
  CaretFromPos(PBeg, iR, BegY);
  CaretFromPos(PEnd, iR, EndY);
  PaintCaret(false);
  for iR := BegY to EndY do PaintLine(iR, -1, -1);
  PaintCaret(true);
  {  Gutter'}
  Gutter.Paint;
  BreakPoints.UpdateBreakPoints;
end;

procedure TEMemo.TextModified(Pos : integer; Action : TModifiedAction; Text : string);
var
  i : integer;
  Modi : boolean;
begin
  Modi := false;
  {   }
  with Main.InterObj.IDebugObject do
    for i := 0 to BreakPoints.Count-1 do with BreakPoints[i] do
      if Cmp(PChar((Owner as TTab).UnitName), PChar(UnitName)) then
        if Action = maInsert then begin
          if PosBeg >= Pos then begin
            inc(PosBeg, Length(Text));
            inc(PosEnd, Length(Text));
            Modi := true;
          end else
          if (PosBeg < Pos) and (Pos <= PosEnd) then begin
            inc(PosEnd, Length(Text));
            Modi := true;
          end
        end else {Action = maDelete}
          if PosBeg >= Pos then begin
            dec(PosBeg, Length(Text));
            dec(PosEnd, Length(Text));
            Modi := true;
          end else
          if (PosBeg < Pos) and (Pos <= PosEnd) then begin
            dec(PosEnd, Length(Text));
            Modi := true;
          end;
 {!!!!}
  if Modi then begin
    BreakPoints.UpdateBreakPoints;
   // NeedRepaint := true;
    Invalidate;//Repaint;
  end;
end;

procedure TEditor.miReadOnlyClick(Sender: TObject);
begin
  if ActiveTab <> nil then begin
    miReadOnly.Checked := not miReadOnly.Checked;
    ActiveTab.EMemo.ReadOnly := miReadOnly.Checked;
  end;
end;

procedure TEditor.miEvaluateModifyClick(Sender: TObject);
begin
  Console.memMess.SelStart := Length(Console.memMess.Text);
  Console.Visible := true;
  Console.memMess.SetFocus;
end;

procedure TEditor.WMGetMinMaxInfo(var M : TWMGetMinMaxInfo);
begin
  inherited;
  if Visible then begin
    M.MinMaxInfo^.ptMinTrackSize.Y := 200;
    M.MinMaxInfo^.ptMinTrackSize.X := 200;
  end;
end;

end.
