unit fMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, StdCtrls, Registry,
  Buttons, RARegAuto,
  RAIIntf, RAIImpl, InterObj;

type               
  TMain = class(TForm)
    ToolBar: TPanel;
    sbTraceInto: TSpeedButton;
    sbRun: TSpeedButton;
    sbPause: TSpeedButton;
    sbReset: TSpeedButton;
    sbStepOver: TSpeedButton;
    sbSave: TSpeedButton;
    Bevel2: TBevel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    miFileNew: TMenuItem;
    miFileOpen: TMenuItem;
    miFileClose: TMenuItem;
    N6: TMenuItem;
    miFileSave: TMenuItem;
    miFileSaveAs: TMenuItem;
    N4: TMenuItem;
    miFileExit: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    RegAuto: TRegAuto;
    miView: TMenuItem;
    sbOpen: TSpeedButton;
    miFileSaveAll: TMenuItem;
    sbSaveAll: TSpeedButton;
    miOutput: TMenuItem;
    miOptions: TMenuItem;
    miWatches: TMenuItem;
    N1: TMenuItem;
    miCallStack: TMenuItem;
    miBreakpoints: TMenuItem;
    N2: TMenuItem;
    miRunRun: TMenuItem;
    Panel1: TPanel;
    edtMultiThreadCount: TEdit;
    btnMultiThread: TButton;
    CheckBox1: TCheckBox;
    Label1: TLabel;
    sbInternalInfo: TSpeedButton;
    N3: TMenuItem;
    N5: TMenuItem;
    N7: TMenuItem;
    N8: TMenuItem;
    N9: TMenuItem;
    N10: TMenuItem;
    N11: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure miFileOpenClick(Sender: TObject);
    procedure miFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure sbRunClick(Sender: TObject);
    procedure sbTraceIntoClick(Sender: TObject);
    procedure sbPauseClick(Sender: TObject);
    procedure sbResetClick(Sender: TObject);
    procedure miFileExitClick(Sender: TObject);
    procedure miViewClick(Sender: TObject);
    procedure miOutputClick(Sender: TObject);
    procedure miOptionsClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure miFileNewClick(Sender: TObject);
    procedure miFileSaveAsClick(Sender: TObject);
    procedure Debug1Click(Sender: TObject);
    procedure btnMultiThreadClick(Sender: TObject);
    procedure sbInternalInfoClick(Sender: TObject);
  private
    procedure WMGetMinMaxInfo(var M : TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  private
    IMain : TIThreadObject;
    procedure Done(Sender: TObject);
    procedure DebugProc(Sender: TObject);
    procedure UnAccesible;
  public
    InterObj : TInterObj;
    S : string;
    procedure AddToInter(Programa : string);
    procedure AfterCreate;
    procedure ButtonsEnabled(En : boolean);
    procedure ShowCallStackItem(index : integer);
  end;

var
  Main: TMain;

implementation

uses fEditor, fConsole, fOptions, fWatches, fCallStack,
  fBreakpoints, fAbout, iMTracer, RAIExpr, RAUtils;

{$R *.DFM}
var
  Capt : string;
const
  sNotImlemented = ' ';

type
  TIMain = class(TIThreadObject)
  protected
    function SyncSetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

procedure TMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  sbReset.Click;
  RegAuto.WriteInteger('Console', '.Visible', integer(Console.Visible));
  RegAuto.WriteInteger('Watches', '.Visible', integer(Watches.Visible));
  RegAuto.WriteInteger('CallStack', '.Visible', integer(CallStack.Visible));
  RegAuto.WriteInteger('Breakpoints', '.Visible', integer(Breakpoints.Visible));
  RegAuto.WriteString('', 'SearchPath', fOptions.SearchPath);
  Editor.RegAuto.Save;
  Watches.RegAuto.Save;
end;

procedure TMain.FormShow(Sender: TObject);
begin
  fOptions.SearchPath := RegAuto.ReadString('', 'SearchPath', fOptions.SearchPath);
  Console.Visible := boolean(RegAuto.ReadInteger('Console', '.Visible', 0));
  Watches.Visible := boolean(RegAuto.ReadInteger('Watches', '.Visible', 0));
  CallStack.Visible := boolean(RegAuto.ReadInteger('CallStack', '.Visible', 0));
  Breakpoints.Visible := boolean(RegAuto.ReadInteger('Breakpoints', '.Visible', 0));
  InterObj.SearchPath := fOptions.SearchPath;
  if RegAuto.ReadString(Name, '.Top', '') = '' then
  begin
   {first run}
    Editor.OpenUnit(ExePath+'MySecondUnit.itr', true);
    Editor.OpenUnit(ExePath+'Windows.itr', true);
    Editor.OpenUnit(ExePath+'MyFirstUnit.itr', true);
    miOutputClick(miOutput);
    fOptions.SearchPath := ExePath;
    InterObj.SearchPath := ExePath;
  end;
end;

procedure TMain.miFileOpenClick(Sender: TObject);
begin
  Editor.FileOpen;
end;

procedure TMain.miFileClick(Sender: TObject);
begin
  if (Sender = miFileSave) or (Sender = sbSave) then Editor.FileSave
  else if (Sender = miFileSaveAll) or (Sender = sbSaveAll) then Editor.FileSaveAll
  else if Sender = miFileClose then Editor.FileClose
end;

procedure TMain.WMGetMinMaxInfo(var M : TWMGetMinMaxInfo);
begin
  inherited;
  if Visible then begin
    M.MinMaxInfo^.ptMaxTrackSize.Y := Height;
    M.MinMaxInfo^.ptMinTrackSize.X := 137;
  end;  
end;

procedure TMain.FormCreate(Sender: TObject);
begin
  InterObj := TInterObj.Create;
  InterObj.IDebugObject.OnDebug := DebugProc;
  InterObj.OnDone := Done;
  IMain := TIMain.Create;
  InterObj.SyncSysObjs.Add(IMain);
  Capt := Caption;
end;

procedure TMain.AfterCreate;
begin
  Editor.OnGetUnitSource := Main.InterObj.GetUnitSource;
  Editor.RegAuto.Load;
  Watches.RegAuto.Load;
end;

procedure TMain.FormDestroy(Sender: TObject);
begin
  IUnknown(InterObj)._Release;
  IUnknown(IMain)._Release;
end;

{*************************** Debugger *********************************}
{*************************** Debugger *********************************}
{*************************** Debugger *********************************}
procedure TMain.Done(Sender: TObject);
var
  FA : TInterObj;
begin
  FA := (Sender as TInterObj);
  Console.Status := 'Ready';
  Caption := Capt;
  Console.TimerStop;
{  if Editor.ActiveTab <> nil then
    Editor.ActiveTab.EMemo.ShowExecutionPoint(-1, -1);}
  if FA.ErrCode = eOk then begin
    Console.MessAdd(FA.Res.Str);
  end
  else with FA do begin
    //  
    if not HiddenError(ErrCode) and (ErrUnitName <> '') then
      if Editor.OpenUnit(ErrUnitName, false) and (Editor.ActiveTab <> nil)
      then begin
        Editor.ActiveTab.EMemo.ShowErrorPoint(ErrPos);
        Editor.SetFocus;
      end
      else ShowMessage('Can''t find source for unit '''+ErrUnitName+'''');
    Console.MessAdd(FA.ErrMessage);
  end;
  CallStack.DebugDone;
  Watches.DebugDone;
  if (FA.ErrCode = eOk) and Checkbox1.Checked
  then begin
    Console.TimerStart;
    FA.RunInThread;
  end;
  IUnknown(FA)._Release;
  InterObj.IDebugObject.Reset;
end;

procedure TMain.AddToInter(Programa : string);
begin
  if Console.Status = 'Ready' then begin
    Editor.DeleteAllErrorPoint;
    InterObj.Check := false;
    Console.TimerStart;
    if (Programa = '') and (Editor.ActiveTab <> nil) then
      InterObj.Formula := 'main'
    else
      InterObj.Formula := Programa;
    if Editor.ActiveTab <> nil then
      InterObj.UnitName := Editor.ActiveTab.UnitName
    else InterObj.UnitName := '';
    InterObj.ReloadAll := true;
    IUnknown(InterObj)._AddRef;
    InterObj.RunInThread;
  end;
end;

procedure TMain.sbRunClick(Sender: TObject);
begin
  miFileSaveAll.Click;
  UnAccesible;
  case InterObj.IDebugObject.Status of
    dsStopped : InterObj.IDebugObject.Run;
    dsReady   : begin
      InterObj.IDebugObject.Reset;
      AddToInter('');
    end;  
  end;
  Console.Status := 'Running';
  Caption := Capt+' - Running';
end;

procedure TMain.sbTraceIntoClick(Sender: TObject);
begin
  miFileSaveAll.Click;
  UnAccesible;
  InterObj.IDebugObject.Pause;
  if InterObj.IDebugObject.Status = dsReady then AddToInter('')
  else
    if Sender = sbTraceInto then InterObj.IDebugObject.TraceInto
    else if Sender = sbStepOver then InterObj.IDebugObject.StepOver;
  Console.Status := 'Running';
  Caption := Capt+' - Running';
end;

procedure TMain.sbResetClick(Sender: TObject);
begin
  miFileSaveAll.Click;
  UnAccesible;
  InterObj.Terminate;
  if InterObj.IDebugObject.Status = dsStopped then
    InterObj.IDebugObject.Run;
  Console.Status := 'Ready';
  Caption := Capt+' - Ready';
end;

procedure TMain.sbPauseClick(Sender: TObject);
begin
  InterObj.IDebugObject.Pause;
end;

procedure TMain.DebugProc(Sender: TObject);
begin
  Console.Status := 'Stopped';
  Caption := Capt+' - Stopped';
  UnAccesible;
  ShowCallStackItem(0);
 {UpdateWatches}
  Watches.DebugProc;
 {UpdateCallStack}
  CallStack.DebugProc;
end;

procedure TMain.ShowCallStackItem(index : integer);
  procedure Show(PosBeg, PosEnd : integer);
  begin
    if Editor.ActiveTab <> nil then
      if index = 0 then
        Editor.ActiveTab.EMemo.ShowExecutionPoint(PosBeg, PosEnd)
      else
        Editor.ActiveTab.EMemo.ShowCallStackPoint(PosBeg, PosEnd)
  end;
begin
  if Editor.ActiveTab <> nil then
    Editor.ActiveTab.EMemo.ShowCallStackPoint(-1, -1);
  with InterObj.IDebugObject.CallStack[index] do
  if PosBeg > -1 then begin
    if UnitName <> '' then
      if Editor.OpenUnit(UnitName, false) and (Editor.ActiveTab <> nil) then
        Show(PosBeg, PosEnd)
      else ShowMessage('Can''t find source for unit ''' +UnitName+'''');
  end else if Editor.ActiveTab <> nil then
    Show(-1, -1);
end;

procedure TMain.UnAccesible;
begin
  if Editor.ActiveTab <> nil then begin
    Editor.ActiveTab.EMemo.ShowExecutionPoint(-1, -1);
    Editor.ActiveTab.EMemo.ShowCallStackPoint(-1, -1);
  end;
 {UpdateWatches}
  Watches.DebugDone;
 {UpdateCallStack}
  CallStack.DebugDone;
end;




procedure TMain.miFileExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMain.miViewClick(Sender: TObject);
begin
  miOutput.Checked    := Console.Visible;
  miWatches.Checked   := Watches.Visible;
  miCallStack.Checked := CallStack.Visible;
  miBreakpoints.Checked := Breakpoints.Visible;
end;

procedure TMain.miOutputClick(Sender: TObject);
var
  Form : TForm;
begin
  if Sender = miOutput then Form := Console
  else if Sender = miWatches then Form := Watches
  else if Sender = miCallStack then Form := CallStack
  else if Sender = miBreakpoints then Form := Breakpoints
  else Form := nil;
  if Form <> nil then Form.Visible := not Form.Visible;
end;

procedure TMain.miOptionsClick(Sender: TObject);
begin
  fOptions.ShowModal;
  InterObj.SearchPath := fOptions.SearchPath;
end;


procedure TMain.ButtonsEnabled(En : boolean);
begin
  Main.sbRun.Enabled       := En;
  Main.sbReset.Enabled     := En;
  Main.sbStepOver.Enabled  := En;
  Main.sbTraceInto.Enabled := En;
  Main.sbPause.Enabled     := En;
  Main.miFileSaveAll.Enabled   := En;
  Main.miFileSave.Enabled      := En;
end;

procedure TMain.About1Click(Sender: TObject);
begin
  AboutShowModal;
end;

procedure TMain.miFileNewClick(Sender: TObject);
begin
  Editor.FileNew;
end;

procedure TMain.miFileSaveAsClick(Sender: TObject);
begin
  Editor.FileSaveAs;
end;
(*
{************************* TIBitmap ************************* }
type
  TIBitmap = class(TBitmap, IIObject)
  {*** interface}
  private
    FRefCount: Integer;
    function QueryInterface(const IID: TGUID; out Obj): Integer; stdcall;
  public
    function _AddRef: integer; stdcall;
    function _Release: integer; stdcall;
    function SetGetValue(var SGI : TSetGetInfo) : boolean;
    function TObj : TObject;
    function IObjectInfo : TIObjectInfo;
  {### interface}
  end;

function TIBitmap.QueryInterface(const IID: TGUID; out Obj): Integer;
begin
  Result := S_FALSE;
end;

function TIBitmap._AddRef: Longint;
begin
 {$IFDEF DEBUG}
  Inc(GlobalRefCount);
 {$ENDIF}
  Inc(FRefCount);
  Result := FRefCount;
end;

function TIBitmap._Release: Longint;
begin
 {$IFDEF DEBUG}
  dec(GlobalRefCount);
 {$ENDIF}
  Dec(FRefCount);
  Result := FRefCount;
  if FRefCount = 0 then Free;
end;

function TIBitmap.SetGetValue(var SGI : TSetGetInfo) : boolean;
begin
  Result := false;
end;

function TIBitmap.TObj : TObject;
begin
  Result := self;
end;

function TIBitmap.IObjectInfo : TIObjectInfo;
begin
  Result.ClassName := ClassName;
end;
{######################### TIBitmap ######################### }
*)
function TIMain.SyncSetGetVar(var SGI : TSetGetInfo) : boolean;
  function NameIs(S : string) : boolean;
  begin
    Result := Cmp(SGI.Name, iString(S));
  end;
var
  S : string;
  i : integer;
begin
  Result := true;
 if SGI.Get then with SGI do begin
  if NameIs('Writeln') or NameIs('Write')
  then begin
      Res.Empty;
      S := '';
      for i := 0 to Args.Count-1 do
        S := S+ Args[i].AsString;
      if NameIs('Writeln') then Console.MessAdd(S)
      else Console.memMess.Lines[Console.memMess.Lines.Count-1] :=
        Console.memMess.Lines[Console.memMess.Lines.Count-1]+S;
  end {else
  if ProcIs('open', 1, [ivString]) then
    Editor.OpenUnit(Args[0].vString, true)
  else if ProcIs('cls', 0, [ivEmpty]) then
    Console.memMess.Clear
  else if ProcIs('Bitmap', 0, [ivEmpty]) then
    Res.vObject := TIBitmap.Create
 } else Result := false;
 end else Result := false;
end;

procedure TMain.Debug1Click(Sender: TObject);
begin
{$IFDEF DEBUG} {CheckDebug;} {$ENDIF}
end;

procedure TMain.btnMultiThreadClick(Sender: TObject);
var
  O : TInterObj;
  IM : TIMain;
  i : integer;
begin
  miFileSaveAll.Click;
  Console.TimerStart;
  if Editor.ActiveTab <> nil then
  for i := 1 to StrToInt(edtMultiThreadCount.Text) do begin
    O := TInterObj.Create;
    O.SearchPath := fOptions.SearchPath;
    O.Formula := Editor.ActiveTab.EMemo.Lines.Text;
    O.UnitName := '';
    O.OnDone := Done;
    IM := TIMain.Create;
    O.SyncSysObjs.Add(IM);
    O.ReloadAll := true;
    O.RunInThread;
  end;
end;

procedure TMain.sbInternalInfoClick(Sender: TObject);
{
var
  FA : TInterObj;
  iU, iP : integer;
}
begin
  ShowMessage(sNotImlemented);
{
  FA := InterObj;
  if Editor.ActiveTab = nil then exit;
  Tracer.Writeln('__________________________');
  Tracer.Writeln(' InternalInfo..');
  FA.UnitName := Editor.ActiveTab.UnitName;
  FA.Formula := '1';
  FA.Update;
  Tracer.Writeln(' ');
  Tracer.Writeln(' = '+ IntToStr(FA.Prog.Units.Count));
  for iU := 0 to FA.Prog.Units.Count-1 do begin
    Tracer.Writeln(' '+ IntToStr(iU+1)+') '+FA.Prog.Units[iU].Name);
    Tracer.Writeln('   = '+ IntToStr(FA.Prog.Units[iU].Procs.Count));
    for iP := 0 to FA.Prog.Units[iU].Procs.Count-1 do begin
      Tracer.Writeln('  '+ IntToStr(iP+1)+') '
        + Proc2Str(FA.Prog.Units[iU].Procs[iP]));
    end;
  end;
}
end;

end.
