unit fRAInterTest;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, RAEditor, StdCtrls, RAIProgram,
  RAIProc, RARegAuto;

type
  TForm1 = class(TForm)
    RAEditor1: TRAEditor;
    Panel1: TPanel;
    memMess: TMemo;
    RAIProgram1: TRAIProgram;
    Label1: TLabel;
    Button1: TButton;
    Edit1: TEdit;
    Label2: TLabel;
    ComboBox1: TComboBox;
    bRun: TButton;
    ErrCode: TLabel;
    RegAuto2: TRegAuto;
    RegAuto1: TRegAuto;
    procedure bRunClick(Sender: TObject);
    procedure RAEditor1GetLineAttr(Sender: TObject; var Line: String;
      index: Integer; var Attrs: TLineAttrs);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RAIProgram1SyncSetGetVar(Sender: TObject;
      var SGI: TSetGetInfo; var Done: Boolean);
    procedure Button1Click(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure RegAuto1AfterLoad(Sender: TObject);
    procedure RegAuto1AfterSave(Sender: TObject);
    procedure RAIProgram1SetGetVar(Sender: TObject;
      var SGI: TSetGetInfo; var Done: Boolean);
  private
    { Private declarations }
    Parser : TIParser;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses RAConst, RAUtils, RAICalc, RAIClas, RAIFile, RAIIntf;

{$R *.DFM}

procedure TForm1.RAEditor1GetLineAttr(Sender: TObject; var Line: String;
  index: Integer; var Attrs: TLineAttrs);

  procedure SetColor(FC, BC : TColor; Style : TFontStyles);
  var
    i : integer;
  begin
    for i := Parser.PosBeg[0]+1 to Parser.PosEnd[0] do begin
      Attrs[i].FC := FC;
      Attrs[i].BC := BC;
      Attrs[i].Style := Style;
    end;
  end;

var
  Token : string;
  i : integer;
const
  Symbols = [',', ':', ';', '.', '[', ']', '(', ')', '=', '+',
             '-', '/', '<', '>', '%', '*', '~', '''', '\', '^'];
const
  RAInterKeyWords : string =
    ' begin end result args if then else endif goto while '+
    'do endwhile var true false or and not mod unit endunit '+
    'function end uses external const endconst class null ';

  function IsRAInterKeyWord(St : string) : boolean;
  begin
    Result := Pos(' '+ANSILowerCase(St)+' ', RAInterKeyWords) <> 0;
  end;

var
  FC : TColor;
  FS : TFontStyles;
begin
  Parser.pcProgram := PChar(Line);
  Parser.pcPos := Parser.pcProgram;
  FC := clOlive; { }
  FS := [fsItalic];
  Attrs[1].FC := FC;
  Attrs[1].Style := FS;
  Attrs[1].BC := (Sender as TRAEditor).Color;
  for i := 1 to 100 do
    Move(Attrs[1], Attrs[i], sizeof(Attrs[1]));
  try
    Token := Parser.Token;
    while Token <> '' do begin
      if IsRAInterKeyWord(Token) then SetColor(clBlack, (Sender as TRAEditor).Color, [fsBold])
      else if (Length(Token) = 1) and (Token[1] in Symbols) then
        SetColor(clBlue, (Sender as TRAEditor).Color, [])
      else if Lang.IsIntConstant(Token) or Lang.IsRealConstant(Token) then
        SetColor(clNavy, (Sender as TRAEditor).Color, [])
      else if Lang.IsStringConstant(Token) then
        SetColor(clPurple, (Sender as TRAEditor).Color, [])
      else
        SetColor(clBlack, (Sender as TRAEditor).Color, []);
      Token := Parser.Token;
    end;
  except

  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Parser := TIParser.Create;
  RAIProgram1.RAIObject.SysObjs.Add2(TIFileUtil.Create);
  RegAuto2.ReadSections(ComboBox1.Items);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Parser.Free;
end;

procedure TForm1.RAIProgram1SyncSetGetVar(Sender: TObject;
  var SGI: TSetGetInfo; var Done: Boolean);
var
  S : string;
  i : integer;
begin
  if Cmp('Writeln', SGI.Name) or Cmp('Write', SGI.Name) then
  begin
    SGI.Res.Empty;
    S := '';
    for i := 0 to SGI.Args.Count-1 do
      S := S+ SGI.Args[i].AsString;
    if Cmp('Writeln', SGI.Name) then
      memMess.Lines.Add(S)
    else
      memMess.Lines[memMess.Lines.Count-1] :=
        memMess.Lines[memMess.Lines.Count-1] + S;
    Done := true;
  end;
end;

procedure TForm1.RAIProgram1SetGetVar(Sender: TObject;
  var SGI: TSetGetInfo; var Done: Boolean);
begin
 { any neccessary functions may be here }
  if RAIProgram1.RAIObject.ProcIs('MyFunction', 1, [ivInteger]) then
  begin
    Done := true;
    SGI.Res.vReal := SGI.Args[0].vInteger / 12345;
  end;
end;

procedure TForm1.bRunClick(Sender: TObject);
begin
  RAIProgram1.ProgramSource := RAEditor1.Lines.Text;
  RAIProgram1.Run;
  memMess.Lines.Add(RAIProgram1.SResult);
  ErrCode.Caption := 'ErrCode: ' + IntToStr(RAIProgram1.ErrCode);
  if RAIProgram1.ErrCode > 0 then
  begin
    RAEditor1.SelLength := 0;
    RAEditor1.SelStart := RAIProgram1.ErrPos;
    RAEditor1.SetFocus;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  Ss : TStrings;
  S : string;
begin
  DecimalSeparator := '.';
  Ss := TStringList.Create;
  S := Edit1.Text;
  ParseString(S, Ss);
  with TCalc.Create do
  try
    ParseString(S, Ss);
    SetExp(Ss, 0, Ss.Count-1);
    Solve;
    memMess.Lines.Add(Res.AsString);
  finally
    Free;
    Ss.Free;
  end;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  RegAuto2.ReadSection(ComboBox1.Text, RAEditor1.Lines);
end;

procedure TForm1.RegAuto1AfterLoad(Sender: TObject);
begin
  ComboBox1.ItemIndex := RegAuto1.ReadInteger(Name, 'PrId', 0);
  ComboBox1Change(nil);
end;

procedure TForm1.RegAuto1AfterSave(Sender: TObject);
begin
  RegAuto1.WriteInteger(Name, 'PrId', ComboBox1.ItemIndex);
end;

end.
