{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       class       : TIParser
       description : text parsing

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit IParser;

interface

uses SysUtils, Classes;

const
  ieBadRemark = 1;

type

  TIParserStyle = (psPascal, psCpp);

  TIParser = class
  protected
    FpcProgram : PChar;
    FpcPos : PChar; //  
    FHistory : TStringList;
    FHistorySize : integer;
    FHistoryPtr  : integer;
    FStyle : TIParserStyle;

    function HistoryInd(index : integer) : integer;
    function GetHistory(index : integer) : string;
    function GetPosBeg(index : integer) : integer;
    function GetPosEnd(index : integer) : integer;
    procedure SetHistorySize(Size : integer);
    function GetPos : integer;
  public
    constructor Create;
    destructor Destroy; override;
    {     }
    function Token : string;
    {          }
    // function TokenL : string; -     #;-(
    {     }
    procedure RollBack(index : integer);
    property History[index : integer] : string read GetHistory;
    property PosBeg[index : integer] : integer read GetPosBeg;
    property PosEnd[index : integer] : integer read GetPosEnd;
    property HistorySize : integer read FHistorySize write SetHistorySize;
    property Pos : integer read GetPos;
    property pcPos : PChar read FpcPos write FpcPos;
    property pcProgram : PChar read FpcProgram write FpcProgram;
    property Style : TIParserStyle read FStyle write FStyle;
  end;

  EIParserError = class(Exception)
  public
    ErrCode : integer;
    Pos : integer;
    constructor Create(AErrCode : integer; APos : integer);
  end;

implementation

uses RAConst, RAUtils;

constructor EIParserError.Create(AErrCode : integer; APos : integer);
begin
  ErrCode := AErrCode;
  Pos     := APos;
end;

procedure IParserError(AErrCode : integer; APos : integer);
begin
  raise EIParserError.Create(AErrCode, APos);
end;

{*************************** TIParser ****************************}
constructor TIParser.Create;
begin
  inherited Create;
  FHistory := TStringList.Create;
  HistorySize := 10;
end;

destructor TIParser.Destroy;
begin
  FHistory.Free;
  inherited Destroy;
end;

function TIParser.Token : string;
var
  Buf : array [byte] of char;
  P, F : PChar;
const  
{$IFDEF RA_D}
  StSkip = [' ', #10, #13];
{$ENDIF RA_D}
{$IFDEF RA_B}
  StSkip = ' '#10#13;
{$ENDIF RA_B}

  procedure Skip;
  begin
    case P[0] of
      '{' :
        if FStyle = psPascal then begin
          F := StrScan(P+1, '}');
          if F = nil then IParserError(ieBadRemark, P - FpcProgram);
          P := F+1;
        end;
      '}' : if FStyle = psPascal then IParserError(ieBadRemark, P - FpcProgram);
      '(' :
        if (FStyle = psPascal) and (P[1] = '*') then begin
          F := P+2;
          while true do begin
            F := StrScan(F, '*');
            if F = nil then IParserError(ieBadRemark, P - FpcProgram);
            if F[1] = ')' then begin inc(P); break; end;
            inc(F);
          end;
          P := F+1;
        end;
      '*' :
        if FStyle = psPascal then begin
          if (P[1] = ')') then IParserError(ieBadRemark, P - FpcProgram)
        end else {if FStyle = psCpp}
          if (P[1] = '/') then IParserError(ieBadRemark, P - FpcProgram);
      '/' :
        if (P[1] = '/') then begin
          F := StrScan(P+1, #13);
          if F = nil then F := StrEnd(P+1);
          P := F;
        end else
        if (FStyle = psCpp) and (P[1] = '*') then begin
          F := P+2;
          while true do begin
            F := StrScan(F, '*');
            if F = nil then IParserError(ieBadRemark, P - FpcProgram);
            if F[1] = '/' then begin inc(P); break; end;
            inc(F);
          end;
          P := F+1;
        end;
    end;
    while CharInSet(P[0], StSkip) do inc(P);
  end;

var
  F1 : PChar;
  i : integer;
begin
 { New Token -     }
  F := FpcPos;
  P := FpcPos;
 { Firstly skip spaces and remarks }
  repeat
    F1 := P;
    Skip;
  until F1 = P;
  F := P;
  if CharInSet(P[0], StIdFirstSymbols) then
   { token }
  begin
    while CharInSet(P[0], StIdSymbols) do inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
  end else
  if CharInSet(P[0], StConstSymbols10) then
   { number }
  begin
    while CharInSet(P[0], StConstSymbols10) or (P[0] = '.') do inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
  end else
  if ((P[0] = '$') and CharInSet(P[1], StConstSymbols)) then
   { hex number }
  begin
    inc(P);
    while CharInSet(P[0], StConstSymbols) do inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
  end else
  if P[0] = '''' then
   { string constant -   }
  begin
    inc(P);
    while P[0] <> #0 do begin
      if (P[0] = '''') then
        if (P[1] = '''') then inc(P) else break;
      inc(P);
    end;
    inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
    i := 2;
    while i < Length(Result) -1 do begin
      if Result[i] = '''' then
        Delete(Result, i, 1);
      inc(i);
    end;
  end else
  if (FStyle = psCpp) and (P[0] = '"') then
   { C++ string constant -   }
  begin
    inc(P);
    while P[0] <> #0 do begin
      if (P[0] = '"') then
        if (P[1] = '"') then inc(P) else break;
      inc(P);
    end;
    inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
    i := 2;
    while i < Length(Result) -1 do begin
      if Result[i] = '"' then
        Delete(Result, i, 1);
      inc(i);
    end;
  end else
  if P[0] = #0 then
    Result := ''
  else begin
    Result := P[0];
    inc(P);
  end;
  
  FpcPos := P;
  FHistory[FHistoryPtr] := Result;
  FHistory.Objects[FHistoryPtr] := TObject(Pos-1);
  inc(FHistoryPtr);
  if FHistoryPtr > FHistorySize-1 then FHistoryPtr := 0;
end;

function TIParser.HistoryInd(index : integer) : integer;
begin
  Result := FHistoryPtr-1-index;
  if Result < 0 then Result := Result + FHistorySize;
end;

function TIParser.GetHistory(index : integer) : string;
begin
  Result := FHistory[HistoryInd(index)];
end;

function TIParser.GetPosEnd(index : integer) : integer;
begin
  Result := integer(FHistory.Objects[HistoryInd(index)])+1;
end;

function TIParser.GetPosBeg(index : integer) : integer;
var
  i : integer;
begin
  i := HistoryInd(index);
  Result := integer(FHistory.Objects[i])-Length(FHistory[i])+1;
end;

procedure TIParser.SetHistorySize(Size : integer);
{$IFDEF DEBUG}
var i : integer;
{$ENDIF}
begin
  while Size > FHistorySize do begin
    FHistory.Add('');
    inc(FHistorySize);
  end;
  while Size < FHistorySize do begin
    FHistory.Delete(0);
    dec(FHistorySize);
  end;
 {$IFDEF DEBUG}
  for i := 0 to FHistorySize-1 do FHistory[i] := '';
 {$ENDIF}
  FHistoryPtr := 0;
end;

function TIParser.GetPos : integer;
begin
  Result := pcPos - FpcProgram;
end;

procedure TIParser.RollBack(index : integer);
begin
  FpcPos := PosEnd[index]+FpcProgram;
  dec(FHistoryPtr, index);
  if FHistoryPtr < 0 then
    FHistoryPtr := FHistorySize+FHistoryPtr;
end;

{########################### TIParser ###########################}

procedure ParseString(const S : string; Ss : TStrings);
var
  Parser : TIParser;
  Token : string;
begin
  Ss.Clear;
  Parser := TIParser.Create;
  try
    Parser.pcProgram := PChar(S);
    Parser.pcPos := Parser.pcProgram;
    Token := Parser.Token;
    while Token <> '' do
    begin
      Ss.Add(Token);
      Token := Parser.Token;
    end;
  finally
    Parser.Free;
  end;
end;


end.


