{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       components  : TRegAutoGrid, TRAFilterCheckBox
       description : Small components
       required    : TRegAuto

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

//{$DEFINE TCheckBox95} // - uncomment this line, if you have TCheckBox95 component

unit RACt;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, RARegAuto, Grids, ComCtrls
{$IFDEF TCheckBox95}
  , EnhButtn {TCheckBox95}
{$ENDIF TCheckBox95}
  ;

type

  TRegAutoGrid = class(TComponent)
  private
    FRegAuto : TRegAuto;
    procedure SetRegAuto(Value : TRegAuto);
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure RegAutoNotify(Sender : TObject; Operation : TRegAutoOperation);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property RegAuto : TRegAuto read FRegAuto write SetRegAuto;
  end;


  TRAStatusBar = class(TStatusBar)
  public
    constructor Create(AOwner : TComponent); override;
  end;


 { TRAFilterCheckBox -    
      ListFilter   }

 {$IFDEF TCheckBox95}
  TRACheckBox = class(TCheckBox95)
 {$ELSE}
  TRACheckBox = class(TCheckBox)
 {$ENDIF TCheckBox95}
  private
    FAllowGrayed : boolean;
    FCanvas : TCanvas;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMControlListChange(var Message: TCMControlListChange); message CM_CONTROLLISTCHANGE;
  protected
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property AllowGrayed : boolean read FAllowGrayed write FAllowGrayed;
  end;

  TRAComboBox = class(TComboBox)
  private
    FDropWidth : integer;
    procedure SetDropWidth(ADropWidth : integer);
  protected
    procedure CreateWnd; override;
  public
  published
    property DropWidth : integer read FDropWidth write SetDropWidth;
  end;

  {     Grid}
  procedure RegAutoSaveGrid(Grid : TCustomGrid; RegAuto : TRegAuto;
    Section : string);
  procedure RegAutoRestoreGrid(Grid : TCustomGrid; RegAuto : TRegAuto;
    Section : string);

  {     ListView}
  procedure RegAutoSaveListView(ListView : TCustomListView;
    RegAuto : TRegAuto; Section : string);
  procedure RegAutoRestoreListView(ListView : TCustomListView;
    RegAuto : TRegAuto; Section : string);

implementation

uses RADsgn;

procedure RegAutoSaveGrid(Grid : TCustomGrid; RegAuto : TRegAuto;
  Section : string);
var
  i : integer;
begin
  for i := 0 to TDrawGrid(Grid).ColCount-1 do
    RegAuto.WriteInteger(Section, 'ColumnWidth'+IntToStr(i),
      TDrawGrid(Grid).ColWidths[i]);
end;

procedure RegAutoRestoreGrid(Grid : TCustomGrid; RegAuto : TRegAuto; Section : string);
var
  i : integer;
begin
  for i := 0 to TDrawGrid(Grid).ColCount-1 do
    TDrawGrid(Grid).ColWidths[i] :=
      RegAuto.ReadInteger(Section, 'ColumnWidth'+IntToStr(i),
        TDrawGrid(Grid).ColWidths[i]);
end;

procedure RegAutoSaveListView(ListView : TCustomListView; RegAuto : TRegAuto;
  Section : string);
var
  i : integer;
begin
  for i := 0 to TListView(ListView).Columns.Count-1 do
    RegAuto.WriteInteger(Section, 'ColumnWidth'+IntToStr(i),
      TListView(ListView).Columns[i].Width);
end;

procedure RegAutoRestoreListView(ListView : TCustomListView; RegAuto : TRegAuto;
  Section : string);
var
  i : integer;
begin
  for i := 0 to TListView(ListView).Columns.Count-1 do
    TListView(ListView).Columns[i].Width :=
      RegAuto.ReadInteger(Section, 'ColumnWidth'+IntToStr(i),
        TListView(ListView).Columns[i].Width);
end;

{********************* TRegAutoGrid *********************}
constructor TRegAutoGrid.Create(AOwner: TComponent);
var
  i : integer;
begin
  inherited Create(AOwner);
  for i := 0 to Owner.ComponentCount-1 do
    if Owner.Components[i] is TRegAuto then begin
      RegAuto := Owner.Components[i] as TRegAuto;
      break;
    end;
end;

procedure TRegAutoGrid.Loaded;
begin
  inherited Loaded;
end;

procedure TRegAutoGrid.SetRegAuto(Value : TRegAuto);
begin
  if Value <> FRegAuto then
  begin
    if FRegAuto <> nil then FRegAuto.RemoveNotify(RegAutoNotify);
    FRegAuto := Value;
    if FRegAuto <> nil then FRegAuto.AddNotify(RegAutoNotify);
  end;
end;

procedure TRegAutoGrid.RegAutoNotify(Sender : TObject; Operation : TRegAutoOperation);
var
  i, j : integer;
begin
  inherited Loaded;
  for j := 0 to Owner.ComponentCount-1 do
    if Owner.Components[j] is TCustomGrid then
      with TDrawGrid(Owner.Components[j] as TCustomGrid) do
        for i := 0 to ColCount-1 do
          case Operation of
            roBeforeLoad : ColWidths[i] := RegAuto.ReadInteger(Owner.Name+'\'+Name, 'ColumnWidth'+IntToStr(i), ColWidths[i]);
            roBeforeSave : RegAuto.WriteInteger(Owner.Name+'\'+Name, 'ColumnWidth'+IntToStr(i), ColWidths[i]);
          end;
end;

procedure TRegAutoGrid.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (AComponent = FRegAuto) and (Operation = opRemove) then
    RegAuto := nil;
end;


{********************* TRAStatusBar *********************}
constructor TRAStatusBar.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls];
end;

{********************* TRAFilterCheckBox *********************}
constructor TRACheckBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csAcceptsControls];
  if csDesigning in ComponentState then
  begin
    FCanvas := TControlCanvas.Create;
    TControlCanvas(FCanvas).Control := Self;
  end;
end;

destructor TRACheckBox.Destroy;
begin
  if csDesigning in ComponentState then
    FCanvas.Free;
  inherited Destroy;
end;

procedure TRACheckBox.CMControlListChange(var Message: TCMControlListChange);
begin
  inherited;
  if  Message.Inserting then
    Message.Control.Enabled := State = cbChecked;
end;

procedure TRACheckBox.Click;
var
  i : integer;
begin
  inherited Click;
  for i := 0 to ControlCount - 1 do
    TControl(Controls[i]).Enabled := State = cbChecked;
end;

procedure TRACheckBox.MouseDown(Button : TMouseButton; Shift : TShiftState; X, Y : Integer);
begin
  if (Button = mbRight) and FAllowGrayed then
    if State = cbGrayed then
      State := cbUnchecked else
      State := cbGrayed
end;

procedure TRACheckBox.WMPaint(var Message: TWMPaint);
begin
  inherited;
  if csDesigning in ComponentState then
  begin
    FCanvas.Handle := Message.DC;
    DrawDesignFrame(FCanvas, ClientRect);
    FCanvas.Handle := 0;
  end;
end;

procedure TRAComboBox.CreateWnd;
var
  Tmp : integer;
begin
  inherited CreateWnd;
  if DropWidth = 0 then
    DropWidth := Width
  else
  begin
    Tmp := DropWidth;
    DropWidth := 0;
    DropWidth := Tmp;
  end;
end;

procedure TRAComboBox.SetDropWidth(ADropWidth : integer);
begin
  if FDropWidth <> ADropWidth then begin
    FDropWidth := ADropWidth;
    Perform(CB_SETDROPPEDWIDTH, FDropWidth, 0);
  end;
end;

end.
