{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       component   : TRAEditor
       description : 'Delphi IDE'-like Editor

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAEditor;

{$DEFINE DEBUG}
{$IFNDEF RAEDITOR_NOEDITOR}
 {$DEFINE RAEDITOR_EDITOR}    {if not RAEDITOR_EDITOR then mode = Viewer}
{$ENDIF}
{$DEFINE RAEDITOR_DEFLAYOT}   {set default keyboard layot}
{$IFNDEF RAEDITOR_NOUNDO}
  {$DEFINE RAEDITOR_UNDO}       {enable undo}
{$ENDIF}
{$IFNDEF RAEDITOR_NOCOMPLETION}
  {$DEFINE RAEDITOR_COMPLETION} {enable code completion}
{$ENDIF}
{ $DEFINE RAEDITOR_NOOPTIMIZE}{disable fast redraw on resizing}

{$IFNDEF RAEDITOR_EDITOR}
  {$UNDEF RAEDITOR_DEFLAYOT}
  {$UNDEF RAEDITOR_UNDO}
  {$UNDEF RAEDITOR_COMPLETION}
{$ENDIF RAEDITOR_EDITOR}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ClipBrd, RAScrollBar;

const
  Max_X             = 1024; {max symbols per row}
  Max_X_Scroll      = 256;  {max symbols per row for scrollbar max -     }
  FLeftMargin       = 1;
  Inclusive         = 0;    {Inclusive mode off =        }
  GutterRightMargin = 2;

type

  TCellRect = record
    Width,
    Height : integer;
  end;

  TLineAttr = record
    FC, BC : TColor;
    Style  : TFontStyles;
  end;

  TRACustomEditor = class;

  TLineAttrs = array[0..Max_X] of TLineAttr;
  TOnGetLineAttr = procedure(Sender : TObject; var Line : string; index : integer; var Attrs : TLineAttrs) of object;
  TOnChangeStatus = TNotifyEvent;

  TEditorStrings = class(TStringList)
  private
    FRAEditor : TRACustomEditor;
    FUpdateLock : integer;
    procedure SetTextStr(const Value: string); override;
    procedure StringsChanged(Sender : TObject);
    procedure SetInternal(index : integer; value : string);
  public
    constructor Create;
    property Internal[index : integer] : string write SetInternal;
  end;

  TModifiedAction = (maInsert, maDelete);

  TBookMark = record
    X, Y : integer;
    Valid : boolean;
  end;
  TBookMarkNum = 0..9;
  TBookMarks = array[TBookMarkNum] of TBookMark;

  TEditorClient = class
  private
    FRAEditor : TRACustomEditor;
    Top : integer;
    function Left : integer;
    function Height : integer;
    function Width : integer;
    function ClientWidth : integer;
    function ClientHeight : integer;
    function ClientRect : TRect;
    function BoundsRect : TRect;
    function GetCanvas : TCanvas;
    property Canvas : TCanvas read GetCanvas;
  end;

  TGutter = class
  private
    FRAEditor : TRACustomEditor;
  public
    procedure Paint;
    procedure Invalidate;
  end;
  TOnPaintGutter = procedure(Sender : TObject; Canvas : TCanvas) of object;

  TEditCommand = word;

  TEditKey = class
  public
    Key : word;
    Shift : TShiftState;
    Command : TEditCommand;
    constructor Create(const ACommand : TEditCommand; const AKey : word; const AShift : TShiftState);
  end;

  TKeyboard = class
  private
    List : TList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(const ACommand : TEditCommand; const AKey : word; const AShift : TShiftState);
    procedure Clear;
    function Command(const AKey : word; const AShift : TShiftState) : TEditCommand;
   {$IFDEF RAEDITOR_DEFLAYOT}
    procedure SetDefLayot;
   {$ENDIF RAEDITOR_DEFLAYOT}
  end;

  ERAEditorError = class(Exception);

 {$IFDEF RAEDITOR_UNDO}
  TUndoBuffer = class;

  TUndo = class
  private
    FRAEditor : TRACustomEditor;
    function UndoBuffer : TUndoBuffer;
  public
    constructor Create(ARAEditor : TRACustomEditor);
    procedure Undo; dynamic; abstract;
    procedure Redo; dynamic; abstract;
  end;

  TUndoBuffer = class(TList)
  private
    FPtr : integer;
    InUndo : boolean;
    GroupUndo : boolean;
    function LastUndo : TUndo;
    function NewGroup(AUndo : TUndo) : boolean;
  public
    procedure Add(AUndo : TUndo);
    procedure Undo;
    procedure Redo;
   {$IFDEF RA_D35H}
    procedure Clear; override;
   {$ELSE}
    procedure Clear;
   {$ENDIF RA_D35H}
    procedure Delete;
  end;
 {$ENDIF RAEDITOR_UNDO}

 {$IFDEF RAEDITOR_COMPLETION}
  TCompletion = class;
  TOnCompletion = procedure(Sender : TObject; var Cancel : boolean) of object;
 {$ENDIF RAEDITOR_COMPLETION}


 {*** TRACustomEditor }

  TRACustomEditor = class(TCustomControl)
  private
   { internal objects }
    FLines : TEditorStrings;
    scbHorz : TRAControlScrollBar95;
    scbVert : TRAControlScrollBar95;
    EditorClient : TEditorClient;
    FGutter : TGutter;
    FKeyboard : TKeyboard;
    FBookMarks : TBookMarks;
   {$IFDEF RAEDITOR_UNDO}
    FUndoBuffer : TUndoBuffer;
   {$ENDIF RAEDITOR_UNDO}
   {$IFDEF RAEDITOR_COMPLETION}
    FCompletion : TCompletion;
   {$ENDIF RAEDITOR_COMPLETION}

   { internal - Columns and rows attributes }
    FCols, FRows : integer;
    FLeftCol, FTopRow : integer;
   // FLeftColMax, FTopRowMax : integer;
    FLastVisibleCol, FLastVisibleRow : integer;
    FCaretX, FCaretY : integer;
    FVisibleColCount : integer;
    FVisibleRowCount : integer;

   { internal - other flags and attributes }
    FUpdateLock : integer;
    RClientRect : TRect;
    FNeedRepaint : boolean;
    FCellRect  : TCellRect;
   {$IFDEF RAEDITOR_EDITOR}
    IgnoreKeyPress : boolean;
   {$ENDIF RAEDITOR_EDITOR}

   { internal - selection attributes }
    FSelected : boolean;
    FSelBlock : boolean; {   }
    FSelBegX, FSelBegY, FSelEndX, FSelEndY : integer;
    FUpdateSelBegY, FUpdateSelEndY : integer;
    FSelStartX, FSelStartY : integer;
    FSelStart, FSelLength : integer; {   TMemo}
    FclSelectBC, FclSelectFC : TColor;

   { mouse support }
    timerScroll : TTimer;
    MouseMoveY, MouseMoveXX, MouseMoveYY : integer;
    FDbl : boolean;

   { visual attributes - properties }
    FBorderStyle: TBorderStyle;
    FGutterColor : TColor;
    FGutterWidth : integer;
    FVisibleRightMargin : boolean;
    FRightMargin : integer;
    FRightMarginColor : TColor;
    FScrollBars : TScrollStyle;
    FDoubleClickLine : boolean;

   { non-visual attributes - properties }
    FInsertMode : boolean;
    FReadOnly   : boolean;
    FModified : boolean;


   { Events }
    FOnGetLineAttr : TOnGetLineAttr;
    FOnKeyDown: TKeyEvent;
    FOnKeyPress: TKeyPressEvent;
    FOnKeyUp: TKeyEvent;
    FOnChange  : TNotifyEvent;
    FOnSelectionChange : TNotifyEvent;
    FOnChangeStatus : TOnChangeStatus;
    FOnScroll : TNotifyEvent;
    FOnResize  : TNotifyEvent;
    FOnMouseDown: TMouseEvent;
    FOnMouseMove: TMouseMoveEvent;
    FOnMouseUp: TMouseEvent;
    FOnDblClick : TNotifyEvent;
    FOnPaintGutter : TOnPaintGutter;
   {$IFDEF RAEDITOR_COMPLETION}
    FOnCompletionIdentifer : TOnCompletion;
    FOnCompletionTemplate : TOnCompletion;
    FOnCompletionDrawItem : TDrawItemEvent;
    FOnCompletionMeasureItem : TMeasureItemEvent;
   {$ENDIF RAEDITOR_COMPLETION}

   { internal message processing }
   {$IFNDEF RA_D4H}
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
   {$ENDIF RA_D4H}
    procedure WMEraseBkgnd(var Message: TWmEraseBkgnd); message WM_ERASEBKGND;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMSetFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SETCURSOR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;

    procedure UpdateEditorSize;
   {$IFDEF RAEDITOR_COMPLETION}
    procedure DoCompletionIdentifer(var Cancel : boolean);
    procedure DoCompletionTemplate(var Cancel : boolean);
   {$ENDIF RAEDITOR_COMPLETION}
    procedure ScrollTimer(Sender : TObject);

   { property methods }
    procedure SetGutterWidth(AWidth : integer);
    procedure SetGutterColor(AColor : TColor);
    function GetLines : TStrings;
    procedure SetBorderStyle(Value: TBorderStyle);
    procedure SetLines(ALines : TStrings);
    function GetSelStart : integer;
    procedure SetSelStart(const ASelStart : integer);
    procedure SetSelLength(const ASelLength : integer);
    procedure SetMode(index : integer; Value : boolean);
    procedure SetCaretPosition(const index, Pos : integer);
    procedure SetCols(ACols : integer);
    procedure SetRows(ARows : integer);
    procedure SetScrollBars(Value : TScrollStyle);
  protected
    LineAttrs : TLineAttrs;
   {$IFDEF RA_D4H}
    procedure Resize; override;
   {$ELSE}
    procedure Resize; dynamic;
   {$ENDIF RA_D4H}
    procedure CreateWnd; override;
    procedure CreateParams(var Params : TCreateParams); override;
    procedure Loaded; override;
    procedure Paint; override;
    procedure BeginUpdate;
    procedure EndUpdate;
    procedure ScrollBarScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure Scroll(const Vert : boolean; const ScrollPos : integer);
    procedure PaintCol(const Col : integer);
    procedure PaintRow(const Row : integer);
    procedure PaintLine(const Line : integer; ColBeg, ColEnd : integer);
    procedure PaintCaret(const bShow : boolean);
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
   {$IFDEF RAEDITOR_EDITOR}
    procedure KeyPress(var Key: Char); override;
   {$ENDIF RAEDITOR_EDITOR}
    procedure SetSel(const SelX, SelY : integer);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DblClick; override;
    procedure DrawRightMargin;
    procedure PaintSelection;
    procedure SetUnSelected;
    procedure Mouse2Cell(const X, Y : integer; var CX, CY : integer);
    procedure Mouse2Caret(const X, Y : integer; var CX, CY : integer);
    procedure CaretCoord(const X, Y : integer; var CX, CY : integer);
    function PosFromMouse(const X, Y : integer) : integer;

   {$IFDEF RAEDITOR_UNDO}
    procedure CantUndo;
   {$ENDIF RAEDITOR_UNDO}
    procedure SetCaretInternal(X, Y : integer);

   {$IFDEF RAEDITOR_EDITOR}
    procedure ChangeBookMark(const BookMark : TBookMarkNum; const Valid : boolean);
   {$ENDIF RAEDITOR_EDITOR}

   { triggers for descendants }
    procedure Changed; dynamic;
    procedure StatusChanged; dynamic;
    procedure SelectionChanged; dynamic;
    procedure ChangeAttr(Line, ColBeg, ColEnd : integer); dynamic;
    procedure GutterPaint(Canvas : TCanvas); dynamic;
    procedure BookmarkCnanged(BookMark : integer); dynamic;
   {$IFDEF RAEDITOR_COMPLETION}
    procedure CompletionIdentifer(var Cancel : boolean); dynamic;
    procedure CompletionTemplate(var Cancel : boolean); dynamic;
   {$ENDIF RAEDITOR_COMPLETION}
   { don't use method TextModified: see comment at method body }
    procedure TextModified(Pos : integer; Action : TModifiedAction; Text : string); dynamic;
    property Gutter : TGutter read FGutter;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetLeftTop(ALeftCol, ATopRow : integer);
    procedure ClipBoardCopy;
    procedure ClipBoardPaste;
    procedure ClipBoardCut;
    procedure DeleteSelected;
    function CalcCellRect(const X, Y : integer) : TRect;
    procedure SetCaret(X, Y : integer);
    procedure CaretFromPos(const Pos : integer; var X, Y : integer);
    function PosFromCaret(const X, Y : integer) : integer;
    function GetSelText : string;
    function GetWordOnCaret : string;
   {$IFDEF RAEDITOR_EDITOR}
    procedure InsertText(const Text : string);
    procedure ReplaceWord(const NewString : string);
    procedure ReplaceWord2(const NewString : string);
    procedure Command(ACommand : TEditCommand); virtual;
   {$ENDIF}
    property LeftCol : integer read FLeftCol;
    property TopRow  : integer read FTopRow;
    property VisibleColCount : integer read FVisibleColCount;
    property VisibleRowCount : integer read FVisibleRowCount;
    property LastVisibleCol : integer read FLastVisibleCol;
    property LastVisibleRow : integer read FLastVisibleRow;
    property Cols : integer read FCols write SetCols;
    property Rows : integer read FRows write SetRows;
    property CaretX : integer index 0 read FCaretX write SetCaretPosition;
    property CaretY : integer index 1 read FCaretY write SetCaretPosition;
    property Modified : boolean read FModified write FModified;
    property SelStart : integer read GetSelStart write SetSelStart;
    property SelLength : integer read FSelLength write SetSelLength;
    property SelText : string read GetSelText;
    property BookMarks : TBookMarks read FBookMarks;
    property Keyboard : TKeyboard read FKeyboard;
    property CellRect : TCellRect read FCellRect;
  public { published in descendants }
    property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
    property Lines : TStrings read GetLines write SetLines;
    property ScrollBars : TScrollStyle read FScrollBars write SetScrollBars default ssBoth;
    property Cursor default crIBeam;
    property Color default clWindow;

    property GutterWidth : integer read FGutterWidth write SetGutterWidth;
    property GutterColor : TColor read FGutterColor write SetGutterColor default clBtnFace;
    property RightMarginVisible : boolean read FVisibleRightMargin write FVisibleRightMargin;
    property RightMargin : integer read FRightMargin write FRightMargin;
    property RightMarginColor : TColor read FRightMarginColor write FRightMarginColor default clBtnFace;
    property InsertMode : boolean index 0 read FInsertMode write SetMode default true;
    property ReadOnly : boolean  index 1 read FReadOnly write SetMode default false;
    property DoubleClickLine : boolean read FDoubleClickLine write FDoubleClickLine default false;
   {$IFDEF RAEDITOR_COMPLETION}
    property Completion : TCompletion read FCompletion write FCompletion;
   {$ENDIF RAEDITOR_COMPLETION}

    property OnGetLineAttr : TOnGetLineAttr read FOnGetLineAttr write FOnGetLineAttr;
    property OnChangeStatus : TOnChangeStatus read FOnChangeStatus write FOnChangeStatus;
    property OnScroll : TNotifyEvent read FOnScroll write FOnScroll;
    property OnResize  : TNotifyEvent read FOnResize write FOnResize;
    property OnKeyDown : TKeyEvent read FOnKeyDown write FOnKeyDown;
    property OnKeyPress: TKeyPressEvent read FOnKeyPress write FOnKeyPress;
    property OnKeyUp   : TKeyEvent read FOnKeyUp write FOnKeyUp;
    property OnChange  : TNotifyEvent read FOnChange write FOnChange;
    property OnSelectionChange : TNotifyEvent read FOnSelectionChange write FOnSelectionChange;
    property OnMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    property OnMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;
    property OnDblClick : TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnPaintGutter : TOnPaintGutter read FOnPaintGutter write FOnPaintGutter;
   {$IFDEF RAEDITOR_COMPLETION}
    property OnCompletionIdentifer : TOnCompletion read FOnCompletionIdentifer write FOnCompletionIdentifer;
    property OnCompletionTemplate : TOnCompletion read FOnCompletionTemplate write FOnCompletionTemplate;
    property OnCompletionDrawItem: TDrawItemEvent read FOnCompletionDrawItem write FOnCompletionDrawItem;
    property OnCompletionMeasureItem: TMeasureItemEvent read FOnCompletionMeasureItem write FOnCompletionMeasureItem;
   {$ENDIF RAEDITOR_COMPLETION}
   {$IFDEF RA_D4H}
    property DockManager;
   {$ENDIF RA_D4H}
  end;

  TRAEditor = class(TRACustomEditor)
  published
    property BorderStyle;
    property Lines;
    property ScrollBars;
    property GutterWidth;
    property GutterColor;
    property RightMarginVisible;
    property RightMargin;
    property RightMarginColor;
    property InsertMode;
    property ReadOnly;
    property DoubleClickLine;
   {$IFDEF RAEDITOR_COMPLETION}
    property Completion;
   {$ENDIF RAEDITOR_COMPLETION}
    property OnGetLineAttr;
    property OnChangeStatus;
    property OnScroll;
    property OnResize;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnChange;
    property OnSelectionChange;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnPaintGutter;
   {$IFDEF RAEDITOR_COMPLETION}
    property OnCompletionIdentifer;
    property OnCompletionTemplate;
    property OnCompletionDrawItem;
    property OnCompletionMeasureItem;
   {$ENDIF RAEDITOR_COMPLETION}

   { TCustomControl }
    property Align;
    property Enabled;
    property Color;
    property Ctl3D;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabStop;
    property Visible;
   {$IFDEF RA_D4H}
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property Constraints;
    property UseDockManager default True;
    property DockSite;
    property DragKind;
    property ParentBiDiMode;
    property OnCanResize;
    property OnConstrainedResize;
    property OnDockDrop;
    property OnDockOver;
    property OnEndDock;
    property OnGetSiteInfo;
    property OnStartDock;
    property OnUnDock;
   {$ENDIF RA_D4H}
  end;

 {$IFDEF RAEDITOR_COMPLETION}
  TCompletionList = (cmIdentifers, cmTemplates);

  TCompletion = class(TPersistent)
  private
    FRAEditor : TRACustomEditor;
    FPopupList : TListBox;
    FIdentifers : TStrings;
    FTemplates : TStrings;
    FItems : TStringList;
    FItemIndex : integer;
    FMode : TCompletionList;
    FDefMode : TCompletionList;
    FItemHeight : integer;
    FTimer : TTimer;
    FEnabled : boolean;
    FVisible : boolean;
    FDropDownCount : integer;
    FDropDownWidth : integer;
    FListBoxStyle : TListBoxStyle;
    FCaretChar : char;
    FCRLF : string;
    FSeparator : string;
    function DoKeyDown(Key : Word; Shift : TShiftState) : boolean;
    procedure DoKeyPress(Key : Char);
    procedure OnTimer(Sender : TObject);
    procedure FindSelItem(var Eq : boolean);
    procedure ReplaceWord(const NewString : string);

    procedure SetStrings(index : integer; AValue : TStrings);
    function GetItemIndex : integer;
    procedure SetItemIndex(AValue : integer);
    function GetInterval : cardinal;
    procedure SetInterval(AValue : cardinal);
    procedure MakeItems;
    function GetItems : TStrings;
  public
    constructor Create2(ARAEditor : TRACustomEditor);
    destructor Destroy; override;
    procedure DropDown(const AMode : TCompletionList; const ShowAlways : boolean);
    procedure DoCompletion(const AMode : TCompletionList);
    procedure CloseUp(const Apply : boolean);
    procedure SelectItem;
    property ItemIndex : integer read GetItemIndex write SetItemIndex;
    property Visible : boolean read FVisible write FVisible;
    property Mode : TCompletionList read FMode write FMode;
    property Items : TStringList read FItems;
  published
    property DropDownCount : integer read FDropDownCount write FDropDownCount default 6;
    property DropDownWidth : integer read FDropDownWidth write FDropDownWidth default 300;
    property Enabled : boolean read FEnabled write FEnabled default false;
    property Identifers : TStrings index 0 read FIdentifers write SetStrings;
    property Templates : TStrings index 1 read FTemplates write SetStrings;
    property ItemHeight : integer read FItemHeight write FItemHeight;
    property Interval : cardinal read GetInterval write SetInterval;
    property ListBoxStyle : TListBoxStyle read FListBoxStyle write FListBoxStyle;
    property CaretChar : char read FCaretChar write FCaretChar;
    property CRLF : string read FCRLF write FCRLF;
    property Separator : string read FSeparator write FSeparator;
  end;
{$ENDIF RAEDITOR_COMPLETION}

const
 {Edit commands}
  {Cursor}
  ecLeft         = 1;
  ecUp           = ecLeft +1;
  ecRight        = ecLeft +2;
  ecDown         = ecLeft +3;
  {Cursor with select}
  ecSelLeft      = 9;
  ecSelUp        = ecSelLeft +1;
  ecSelRight     = ecSelLeft +2;
  ecSelDown      = ecSelLeft +3;
  {Cursor  }
  ecPrevWord     = ecSelDown +1;
  ecNextWord     = ecPrevWord +2;
  ecSelPrevWord  = ecNextWord +1;
  ecSelNextWord  = ecSelPrevWord +1;

  ecWindowTop    = ecSelNextWord +1;
  ecWindowBottom = ecWindowTop +1;
  ecPrevPage     = ecWindowTop +2;
  ecNextPage     = ecWindowTop +3;
  ecSelPrevPage  = ecWindowTop +4;
  ecSelNextPage  = ecWindowTop +5;

  ecBeginLine    = ecSelNextPage +1;
  ecEndLine      = ecBeginLine +1;
  ecBeginDoc     = ecBeginLine +2;
  ecEndDoc       = ecBeginLine +3;
  ecSelBeginLine = ecBeginLine +4;
  ecSelEndLine   = ecBeginLine +5;
  ecSelBeginDoc  = ecBeginLine +6;
  ecSelEndDoc    = ecBeginLine +7;

  ecScrollLineUp   = ecSelEndDoc +1;
  ecScrollLineDown = ecScrollLineUp +1;

  ecInsertPara       = 101;
  ecBackspace        = ecInsertPara +1;
  ecDelete           = ecInsertPara +2;
  ecChangeInsertMode = ecInsertPara +3;

  ecDeleteSelected   = ecInsertPara +10;
  ecClipboardCopy    = ecInsertPara +11;
  ecClipboardCut     = ecClipboardCopy +1;
  ecClipBoardPaste   = ecClipboardCopy +2;

  ecUndo             = ecClipBoardPaste +1;
  ecRedo             = ecUndo +1;

  ecSetBookmark0     = ecRedo +1;
  ecSetBookmark1     = ecSetBookmark0 +1;
  ecSetBookmark2     = ecSetBookmark0 +2;
  ecSetBookmark3     = ecSetBookmark0 +3;
  ecSetBookmark4     = ecSetBookmark0 +4;
  ecSetBookmark5     = ecSetBookmark0 +5;
  ecSetBookmark6     = ecSetBookmark0 +6;
  ecSetBookmark7     = ecSetBookmark0 +7;
  ecSetBookmark8     = ecSetBookmark0 +8;
  ecSetBookmark9     = ecSetBookmark0 +9;

  ecGotoBookmark0    = ecSetBookmark9 +1;
  ecGotoBookmark1    = ecGotoBookmark0 +1;
  ecGotoBookmark2    = ecGotoBookmark0 +2;
  ecGotoBookmark3    = ecGotoBookmark0 +3;
  ecGotoBookmark4    = ecGotoBookmark0 +4;
  ecGotoBookmark5    = ecGotoBookmark0 +5;
  ecGotoBookmark6    = ecGotoBookmark0 +6;
  ecGotoBookmark7    = ecGotoBookmark0 +7;
  ecGotoBookmark8    = ecGotoBookmark0 +8;
  ecGotoBookmark9    = ecGotoBookmark0 +9;

 {$IFDEF RAEDITOR_COMPLETION}
  ecCompletionIdentifers = ecGotoBookmark9 +1;
  ecCompletionTemplates  = ecCompletionIdentifers +1;
 {$ENDIF RAEDITOR_COMPLETION}

{$IFDEF RAEDITOR_EDITOR}

{$ENDIF RAEDITOR_EDITOR}

implementation

uses RAUtils, Consts, RAConst, iMTracer;

{$IFDEF RAEDITOR_UNDO}

type

  TCaretUndo = class(TUndo)
    FCaretX, FCaretY : integer;
    constructor Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer);
    procedure Undo; override;
    procedure Redo; override;
  end;

  TInsertUndo = class(TCaretUndo)
    FText : string;
    constructor Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer; AText : string);
    procedure Undo; override;
    procedure Redo; override;
  end;

  TDeleteUndo = class(TInsertUndo)
    procedure Undo; override;
    procedure Redo; override;
  end;

  TBackspaceUndo = class(TDeleteUndo)
    procedure Undo; override;
    procedure Redo; override;
  end;

  TReplaceUndo = class(TCaretUndo)
    FBeg, FEnd : integer;
    FText, FNewText : string;
    constructor Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer; ABeg, AEnd : integer; AText, ANewText : string);
    procedure Undo; override;
    procedure Redo; override;
  end;

  TDeleteSelectedUndo = class(TDeleteUndo)
    FSelBlock : boolean; {   }
    FSelBegX, FSelBegY, FSelEndX, FSelEndY : integer;
    constructor Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer; AText : string; ASelBlock : boolean; ASelBegX, ASelBegY, ASelEndX, ASelEndY : integer);
    procedure Undo; override;
    procedure Redo; override;
  end;

 { // TSelectUndo not implemented
  TSelectUndo = class(TCaretUndo)
    FSelBegX, FSelBegY, FSelEndX, FSelEndY : integer;
  end;
 }

{$ENDIF RAEDITOR_UNDO}


{********************* Debug ***********************}
procedure Debug(const S : string);
begin
  Tracer.Writeln(S);
end;
{
procedure BeginTick;
begin
//  Tracer.TimerStart(1);
end;

procedure EndTick;
begin
//  Tracer.TimerStop(1);
end;
}
{#################### Debug ######################}

procedure Err;
begin
  MessageBeep(0);
end;

constructor TEditorStrings.Create;
begin
  inherited Create;
  OnChange := StringsChanged;
end;

procedure TEditorStrings.SetTextStr(const Value: string);
begin
  inherited SetTextStr(Value);
  FRAEditor.SetLines(nil);
end;

procedure TEditorStrings.StringsChanged(Sender : TObject);
begin
  if FUpdateLock = 0 then FRAEditor.SetLines(nil);
end;

procedure TEditorStrings.SetInternal(index : integer; value : string);
begin
  inc(FUpdateLock);
  try
    inherited Strings[index] := Value;
  finally
    dec(FUpdateLock);
  end;
end;

function TEditorClient.GetCanvas : TCanvas;
begin
  Result := FRAEditor.Canvas;
end;

function TEditorClient.Left : integer;
begin
  Result := FRAEditor.GutterWidth+2;
end;

function TEditorClient.Height : integer;
begin
  if FRAEditor.FScrollBars in [ssHorizontal, ssBoth] then
    Result := Max(FRAEditor.Height-GetSystemMetrics(SM_CYHSCROLL), 0) else
    Result := Max(FRAEditor.Height, 0);
end;

function TEditorClient.Width : integer;
begin
  if FRAEditor.FScrollBars in [ssVertical, ssBoth] then
    Result := Max(FRAEditor.Width-Left-GetSystemMetrics(SM_CXVSCROLL), 0) else
    Result := Max(FRAEditor.Width-Left, 0) ;
end;

function TEditorClient.ClientWidth : integer;
begin
  Result := Width;
end;

function TEditorClient.ClientHeight : integer;
begin
  Result := Height;
end;

function TEditorClient.ClientRect : TRect;
begin
  Result := Bounds(Left, Top, Width, Height);
end;

function TEditorClient.BoundsRect : TRect;
begin
  Result := Bounds(0, 0, Width, Height);
end;

procedure TGutter.Invalidate;
{var
  R : TRect;}
begin
//  Owner.Invalidate;
//  R := Bounds(0, 0, FRAEditor.GutterWidth, FRAEditor.Height);
//  InvalidateRect(FRAEditor.Handle, @R, false);
  Paint;
end;

procedure TGutter.Paint;
begin
  with FRAEditor, Canvas do
  begin
    Brush.Style := bsSolid;
    Brush.Color := FGutterColor;
    FillRect(Bounds(0, EditorClient.Top, GutterWidth, EditorClient.Height));
    Pen.Width := 1;
    Pen.Color := clWhite;
    MoveTo(GutterWidth - 2, EditorClient.Top);
    LineTo(GutterWidth - 2, EditorClient.Top + EditorClient.Height);
    Pen.Width := 2;
    MoveTo(GutterWidth + 1, EditorClient.Top);
    LineTo(GutterWidth + 1, EditorClient.Top + EditorClient.Height);
    Pen.Width := 1;
    Pen.Color := clGray;
    MoveTo(GutterWidth - 1, EditorClient.Top);
    LineTo(GutterWidth - 1, EditorClient.Top + EditorClient.Height);
  end;
  with FRAEditor do GutterPaint(Canvas);
end;



{*********************** TRACustomEditor ***********************}

constructor TRACustomEditor.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := [csCaptureMouse, csClickEvents{, csOpaque}, csDoubleClicks, csReplicatable];
  FInsertMode := true;
  FLines := TEditorStrings.Create;
  FLines.FRAEditor := self;
  FKeyboard := TKeyboard.Create;
  FRows := 1;
  FCols := 1;
 {$IFDEF RAEDITOR_UNDO}
  FUndoBuffer := TUndoBuffer.Create;
  FUndoBuffer.GroupUndo := true;
 {$ENDIF RAEDITOR_UNDO}

  FRightMarginColor := clSilver;
  FBorderStyle := bsSingle;
  Ctl3d := true;
  Height := 40; Width := 150;
  ParentColor := false;
  Color := clWindow;
  Cursor := crIBeam;
  TabStop := true;

  FScrollBars := ssBoth;
  scbHorz := TRAControlScrollBar95.Create;
  scbVert := TRAControlScrollBar95.Create;
  scbVert.Kind := sbVertical;
  scbHorz.OnScroll := ScrollBarScroll;
  scbVert.OnScroll := ScrollBarScroll;

  FGutterColor := clBtnFace;

  EditorClient := TEditorClient.Create;
  EditorClient.FRAEditor := Self;
  FGutter := TGutter.Create;
  FGutter.FRAEditor := Self;

  FLeftCol := 0; FTopRow  := 0;
  FclSelectBC := clNavy;
  FclSelectFC := clWhite;
  FSelected := false;
  FCaretX := 0;
  FCaretY := 0;

  timerScroll := TTimer.Create(Self);
  timerScroll.Enabled := false;
  timerScroll.Interval := 100;
  timerScroll.OnTimer := ScrollTimer;

 {$IFDEF RAEDITOR_EDITOR}

 {$IFDEF RAEDITOR_DEFLAYOT}
  FKeyboard.SetDefLayot;
 {$ENDIF RAEDITOR_DEFLAYOT}

 {$IFDEF RAEDITOR_COMPLETION}
  FCompletion := TCompletion.Create2(Self);
 {$ENDIF RAEDITOR_COMPLETION}

 {$ENDIF RAEDITOR_EDITOR}
end;

destructor TRACustomEditor.Destroy;
begin
  FLines.Free;
  scbHorz.Free;
  scbVert.Free;
  EditorClient.Free;
  FKeyboard.Free;
{$IFDEF RAEDITOR_EDITOR}
 {$IFDEF RAEDITOR_UNDO}
  FUndoBuffer.Free;
 {$ENDIF RAEDITOR_UNDO}
 {$IFDEF RAEDITOR_COMPLETION}
  FCompletion.Free;
 {$ENDIF RAEDITOR_COMPLETION}
{$ENDIF RAEDITOR_EDITOR}
  inherited Destroy;
end;

procedure TRACustomEditor.Loaded;
begin
  inherited Loaded;
  UpdateEditorSize;
{  Rows := FLines.Count;
  Cols := Max_X; }
end;

{**************   ***************}
procedure TRACustomEditor.CreateParams(var Params: TCreateParams);
const
  BorderStyles: array[TBorderStyle] of cardinal = (0, WS_BORDER);
const
  ScrollStyles: array[TScrollStyle] of cardinal = (0, WS_HSCROLL, WS_VSCROLL, WS_HSCROLL or WS_VSCROLL);
begin
  inherited CreateParams(Params);
  with Params do
    begin
      Style := Style or BorderStyles[FBorderStyle] or ScrollStyles[FScrollBars];
      if NewStyleControls and Ctl3D and (FBorderStyle = bsSingle) then
      begin
        Style := Style and not WS_BORDER;
        ExStyle := ExStyle or WS_EX_CLIENTEDGE;
      end;
      WindowClass.style := WindowClass.style and not (CS_HREDRAW or CS_VREDRAW);
    end;
end;

{$IFNDEF RA_D4H}
procedure TRACustomEditor.WMSize(var Message: TWMSize);
begin
  inherited;
  if not (csLoading in ComponentState) then Resize;
end;
{$ENDIF RA_D4H}

procedure TRACustomEditor.Resize;
begin
  UpdateEditorSize;
end;

procedure TRACustomEditor.CreateWnd;
begin
  inherited CreateWnd;
  if FScrollBars in [ssHorizontal, ssBoth] then
    scbHorz.Handle := Handle;
  if FScrollBars in [ssVertical, ssBoth] then
    scbVert.Handle := Handle;
  FNeedRepaint := true;
end;

procedure TRACustomEditor.SetBorderStyle(Value: TBorderStyle);
begin
  if FBorderStyle <> Value then
  begin
    FBorderStyle := Value;
    RecreateWnd;
  end;
end;

procedure TRACustomEditor.CMFontChanged(var Message: TMessage);
begin
  inherited;
  UpdateEditorSize;
end;

procedure TRACustomEditor.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 {$IFDEF RAEDITOR_NOOPTIMIZE}
  inherited;
  Message.Result := 1;
 {$ELSE}
  Message.Result := 0;
 {$ENDIF}
end;

procedure TRACustomEditor.PaintSelection;
var
  iR : integer;
begin
  for iR := FUpdateSelBegY to FUpdateSelEndY do
    PaintLine(iR, -1, -1);
end;

procedure TRACustomEditor.SetUnSelected;
begin
  if FSelected then
  begin
    FSelected := false;
    PaintSelection;
  end;
end;

procedure TRACustomEditor.WMSetCursor(var Message: TWMSetCursor);
var
  P: TPoint;
begin
  GetCursorPos(P);
  P := ScreenToClient(P);
  if (P.X < GutterWidth) and (Cursor = crIBeam) then
  begin
    Message.Result := 1;
    Windows.SetCursor(Screen.Cursors[crArrow])
  end else
    inherited;
end;
{##############   ###############}

{**************  ***************}
function IsRectEmpty(R : TRect) : boolean;
begin
  Result := (R.Top = R.Bottom) and (R.Left = R.Right);
end;

function TRACustomEditor.CalcCellRect(const X, Y : integer) : TRect;
begin
  Result := Bounds(
    EditorClient.Left+X*FCellRect.Width+1,
    EditorClient.Top+Y*FCellRect.Height,
    FCellRect.Width,
    FCellRect.Height)
end;

procedure TRACustomEditor.Paint;
var
  iR : integer;
  RIR, RIB, CR, ECR : TRect;
  BX, EX, BY, EY : integer;
const
  DebugRas : integer = 0;
begin
  if FUpdateLock > 0 then exit;
//  BeginTick;
 {$IFDEF RAEDITOR_NOOPTIMIZE}
  FNeedRepaint := true;
 {$ENDIF}
 { -    }
  PaintCaret(false);
  ECR := EditorClient.Canvas.ClipRect;
  CR := EditorClient.BoundsRect;
  if not(FNeedRepaint or EqualRect(ECR, CR)) then
  begin
   { }
    RIR := Rect(RClientRect.Right, 0, CR.Right, {CR}RClientRect.Bottom);
    RIB := Rect(0, RClientRect.Bottom, CR.Right, CR.Bottom);
    if RIR.Left >= RIR.Right then SetRectEmpty(RIR);{  -   }
    if RIB.Top >= RIB.Bottom then SetRectEmpty(RIB);{  -   }
    if IsRectEmpty(RClientRect) or FNeedRepaint then
      for iR := FTopRow to FTopRow + FVisibleRowCount + 1 do
        PaintLine(iR, -1, -1)
    else
    if not (IsRectEmpty(RIR) and IsRectEmpty(RIB)) then
    begin
      {  -     Cell}
      {*      -   (whitenbs)}
       EditorClient.Canvas.Brush.Color := Color;
       EditorClient.Canvas.FillRect(RIR);
      {#}
      if not IsRectEmpty(RIB) then
      begin
       {  }
        RIB.Top := RIB.Top div FCellRect.Height;
        RIB.Bottom := RIB.Bottom div FCellRect.Height ;
        for iR := FTopRow + RIB.Top to FTopRow + RIB.Bottom do
          PaintLine(iR, -1, -1);
      end;
      if not IsRectEmpty(RIR) then
      begin
       {  }
        RIR.Left := RIR.Left div FCellRect.Width;
        RIR.Right := RIR.Right div FCellRect.Width + 1;
        for iR := FTopRow to FTopRow + FVisibleRowCount + 1 do
          PaintLine(iR, RIR.Left - 1, RIR.Right);
      end
    end;
  end else
  begin
   { Paint}
    BX := ECR.Left div FCellRect.Width - 3{?};
    EX := ECR.Right div FCellRect.Width + 1;
    BY := ECR.Top div FCellRect.Height;
    EY := ECR.Bottom div FCellRect.Height;
    for iR := BY to EY do
      PaintLine(FTopRow + iR, FLeftCol + BX,  FLeftCol + EX)
  end;
  SetRectEmpty(RClientRect);
//  inc(DebugRas);
//  EndTick;
  PaintCaret(true);
  FGutter.Paint;
  FNeedRepaint := false;
end;

procedure TRACustomEditor.BeginUpdate;
begin
  inc(FUpdateLock);
end;

procedure TRACustomEditor.EndUpdate;
begin
  dec(FUpdateLock);
  if FUpdateLock = 0 then
  begin
    FNeedRepaint := true;
    Invalidate;
  end;
end;

procedure TRACustomEditor.UpdateEditorSize;
const
  BiggestSymbol = 'W';
begin
  if (csLoading in ComponentState) then exit;
  EditorClient.Canvas.Font := Font;
  FCellRect.Width  := EditorClient.Canvas.TextWidth(BiggestSymbol);
  FCellRect.Height := EditorClient.Canvas.TextHeight(BiggestSymbol)+1;

  FVisibleColCount := Trunc(EditorClient.ClientWidth / FCellRect.Width);
  FVisibleRowCount := Trunc(EditorClient.ClientHeight / FCellRect.Height);
  FLastVisibleCol := FLeftCol+FVisibleColCount-1;
  FLastVisibleRow := FTopRow+FVisibleRowCount-1;
  Rows := FLines.Count;
  Cols := Max_X_Scroll;
  scbHorz.Page := FVisibleColCount;
  scbVert.Page := FVisibleRowCount;
  scbHorz.LargeChange := Max(FVisibleColCount, 1);
  scbVert.LargeChange := Max(FVisibleRowCount, 1);
  scbVert.Max := Max(1, FRows-1+FVisibleRowCount{-1});
  FGutter.Invalidate;
end;

procedure TRACustomEditor.PaintCol(const Col : integer);
var
  iR : integer;
begin
  for iR := FTopRow to FLastVisibleRow+1 do
    PaintLine(iR, Col, Col)
end;

procedure TRACustomEditor.PaintRow(const Row : integer);
begin
  PaintLine(Row + FTopRow, -1, -1)
end;

procedure TRACustomEditor.PaintLine(const Line : integer; ColBeg, ColEnd : integer);

  procedure ChangeSelectedAttr;

    procedure DoChange(const iBeg, iEnd : integer);
    var
      i : integer;
    begin
      for i := iBeg to iEnd do
        begin
          LineAttrs[i+1].FC := FclSelectFC;
          LineAttrs[i+1].BC := FclSelectBC;
        end;
    end;

  begin
    if (Line = FSelBegY) and (Line = FSelEndY) then
      DoChange(FSelBegX, FSelEndX-1+Inclusive)
    else
    begin
      if Line = FSelBegY then DoChange(FSelBegX, FSelBegX + FVisibleColCount);
      if (Line > FSelBegY) and (Line < FSelEndY) then DoChange(ColBeg, ColEnd);
      if Line = FSelEndY then DoChange(ColBeg, FSelEndX-1+Inclusive);
    end;
  end;

  function ExpandTabs(const S : string): string;
  var
    i : integer;
  begin
    Result := '';
    for i := 1 to Length(S) do
      if S[i] = #9 then
        Result := Result + '  '
      else
        Result := Result + S[i];
  end;

var
  Ch : string;
  R : TRect;
  i, iC, jC, SL, MX : integer;
  S : string;
  LA : TLineAttr;
begin
  if (Line < FTopRow) or (Line > FTopRow+FVisibleRowCount) then exit;
  Debug('PaintLine '+IntToStr(Line));
  if ColEnd < 0 then ColEnd := FLeftCol+FVisibleColCount else inc(ColEnd, FLeftCol);
  ColEnd := Min(ColEnd, Max_X - 1);
  if ColBeg < 0 then ColBeg := FLeftCol else inc(ColBeg, FLeftCol);
  i := ColBeg;
  if (Line > -1) and (Line < FLines.Count) {and (Length(FLines[Line]) > 0)} then
    with EditorClient do
    begin
      LineAttrs[ColBeg].FC := self.Font.Color;
      LineAttrs[ColBeg].Style := self.Font.Style;
      LineAttrs[ColBeg].BC := self.Color;
      for iC := ColBeg to ColEnd+1 do
        Move(LineAttrs[ColBeg], LineAttrs[iC], sizeof(LineAttrs[1]));
      S := ExpandTabs(FLines[Line]);
      if Assigned(FOnGetLineAttr) then FOnGetLineAttr(Self, S, Line, LineAttrs);
      if FSelected then ChangeSelectedAttr;{   }
      ChangeAttr(Line, ColBeg, ColEnd);

     { }
      EditorClient.Canvas.Brush.Color := LineAttrs[1].BC;
      EditorClient.Canvas.FillRect(Bounds(EditorClient.Left, (Line-FTopRow)*FCellRect.Height, 1, FCellRect.Height));

     {  -   (  )}
      SL := Length(S);
      if SL > ColEnd then MX := ColEnd else MX := SL;

      i := ColBeg;
      while i < MX do
        with Canvas do
        begin
          iC := i+1;
          LA := LineAttrs[iC];
          jC := iC+1;
          if iC <= SL then Ch := S[iC] else Ch := ' ';
          while (jC <= MX+1) and
                CompareMem(@LA, @LineAttrs[jC], sizeof(LineAttrs[1])) do
          begin
            if jC <= SL then Ch := Ch+S[jC] else Ch := Ch+' ';
            inc(jC);
          end;
          Brush.Color := LA.BC;
          Font.Color  := LA.FC;
          Font.Style  := LA.Style;
          R := CalcCellRect(i-FLeftCol, Line-FTopRow);
          TextOut(R.Left, R.Top, Ch);

         { }
          FillRect(Bounds(R.Left, R.Bottom-1, FCellRect.Width*Length(Ch), 2));

          i := jC-1;
        end;
    end
  else
  begin
    EditorClient.Canvas.Brush.Color := Color;
    EditorClient.Canvas.FillRect(Bounds(EditorClient.Left, (Line-FTopRow)*FCellRect.Height, 1, FCellRect.Height));
  end;
 {  }
  R := Bounds(CalcCellRect(i-FLeftCol, Line-FTopRow).Left ,
    (Line-FTopRow)*FCellRect.Height,
    (FLeftCol+FVisibleColCount-i+2)*FCellRect.Width,
     FCellRect.Height);
  {       }
  if FSelected and not FSelBlock and (Line >= FSelBegY) and (Line < FSelEndY) then
    EditorClient.Canvas.Brush.Color := FclSelectBC
  else
    EditorClient.Canvas.Brush.Color := Color;
  EditorClient.Canvas.FillRect(R);
  DrawRightMargin;
end;

procedure TRACustomEditor.ChangeAttr(Line, ColBeg, ColEnd : integer);
begin
end;

procedure TRACustomEditor.DrawRightMargin;
var
  F : integer;
begin
  if FVisibleRightMargin and (FRightMargin > FLeftCol)
     and (FRightMargin < FLastVisibleCol+3) then
    with EditorClient.Canvas do
    begin
     { RightMargin Line}
      Pen.Color := FRightMarginColor;
      F := CalcCellRect(FRightMargin-FLeftCol, 0).Left;
      MoveTo(F, EditorClient.Top);
      LineTo(F,EditorClient.Top+EditorClient.Height);
    end;
end;

{************************** Scroll **************************}
procedure TRACustomEditor.WMHScroll(var Message: TWMHScroll);
begin
  scbHorz.DoScroll(Message);
end;

procedure TRACustomEditor.WMVScroll(var Message: TWMVScroll);
begin
  scbVert.DoScroll(Message);
end;

procedure TRACustomEditor.ScrollBarScroll(Sender: TObject; ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  case ScrollCode of
    scLineUp..scPageDown, {scPosition,} scTrack{, scEndScroll} :
      begin
        if Sender = scbVert then Scroll(true, ScrollPos)
        else if Sender = scbHorz then Scroll(false, ScrollPos);
      end;
  end;
//  Tracer.Writeln(IntToStr((Sender as TRAControlScrollBar95).Position));
end;

procedure TRACustomEditor.Scroll(const Vert : boolean; const ScrollPos : integer);
var
  R, RClip : TRect;
  OldFTopRow, iR : integer;
begin
//  BeginTick;
  PaintCaret(false);
  if Vert then
  begin{Vertical Scroll}
   {}
    OldFTopRow := FTopRow;
    FTopRow := ScrollPos;
    if Abs((OldFTopRow-ScrollPos)*FCellRect.Height) < EditorClient.Height then
    begin
      R := EditorClient.ClientRect;
      R.Bottom := R.Top+CellRect.Height*(FVisibleRowCount+1); {??}
      RClip := R;
      ScrollDC(
        EditorClient.Canvas.Handle,              // handle of device context
        0,                                   // horizontal scroll units
        (OldFTopRow-ScrollPos)*FCellRect.Height, // vertical scroll units
        R,             // address of structure for scrolling rectangle
        RClip,         // address of structure for clipping rectangle
        0,             // handle of scrolling region
        nil            // address of structure for update rectangle
      );
      if OldFTopRow < ScrollPos then {Up}
        for iR := FTopRow-OldFTopRow+1 downto 0 do
          PaintRow(FVisibleRowCount-iR)
      else {Down}
        for iR := OldFTopRow-FTopRow-1 downto 0 do
          PaintRow(iR);
      FGutter.Invalidate;
    end else
      Invalidate{Paint};
  end else
  {Horizontal Scroll}
  begin
    {}
    if FLeftCol-1 = ScrollPos then
    begin {Left}
      FLeftCol := ScrollPos;
      //  EditorPaintColRow(false, FLeftCol + FVisibleColCount);
      Invalidate;{Paint};//!
    end else
    if FLeftCol+1 = ScrollPos then
    begin {Right}
      FLeftCol := ScrollPos;
      //  EditorPaintColRow(false, FLeftCol);
      Invalidate{Paint};//!
    end else
    if FLeftCol <> ScrollPos then
    begin
      FLeftCol := ScrollPos;
      Invalidate{Paint};
    end;
  end;
  FLastVisibleRow := FTopRow+FVisibleRowCount-1;
  FLastVisibleCol := FLeftCol+FVisibleColCount-1;
  DrawRightMargin;
  PaintCaret(true);
  if Assigned(FOnScroll) then FOnScroll(Self);
//  EndTick;
end;
{####################### Scroll #########################}
{###############################  ##########################3#}

{************** Caret ***************}
procedure TRACustomEditor.PaintCaret(const bShow : boolean);
var
  R : TRect;
begin
  if not bShow then
    HideCaret(Handle)
  else
  if Focused then
  begin
    R := CalcCellRect(FCaretX - FLeftCol, FCaretY - FTopRow);
    SetCaretPos(R.Left-1, R.Top+1);
    ShowCaret(Handle)
  end
end;

procedure TRACustomEditor.SetCaretInternal(X, Y : integer);
var
  R : TRect;
begin
  if (X = FCaretX) and (Y = FCaretY) then exit;
  // 
  Y := Min(Y, FLines.Count-1);
  Y := Max(Y, 0);
  if Y < FTopRow then SetLeftTop(FLeftCol, Y)
  else if Y > Max(FLastVisibleRow, 0) then
    SetLeftTop(FLeftCol, Y-FVisibleRowCount+1);
  if X < 0 then X := 0;
  if X < FLeftCol then SetLeftTop(X, FTopRow)
  else if X > FLastVisibleCol then
    SetLeftTop(X-FVisibleColCount+1, FTopRow);

  R := CalcCellRect(X - FLeftCol, Y - FTopRow);
  SetCaretPos(R.Left-1, R.Top+1);
  if Assigned(FOnChangeStatus) and ((FCaretX <> X) or (FCaretY <> Y)) then
  begin
    FCaretX := X; FCaretY := Y;
    FOnChangeStatus(Self);
  end;
  FCaretX := X; FCaretY := Y;
end;

procedure TRACustomEditor.SetCaret(X, Y : integer);
begin
  if (X = FCaretX) and (Y = FCaretY) then exit;
 {$IFDEF RAEDITOR_UNDO}
  TCaretUndo.Create(Self, FCaretX, FCaretY);
 {$ENDIF RAEDITOR_UNDO}
  SetCaretInternal(X, Y);
end;

procedure TRACustomEditor.SetCaretPosition(const index, Pos : integer);
begin
  if index = 0 then
    SetCaret(Pos, FCaretY)
  else
    SetCaret(FCaretX, Pos)
end;

procedure TRACustomEditor.WMSetFocus(var Message: TWMSetFocus);
begin
  CreateCaret(Handle, 0, 2, CellRect.Height-2);
  PaintCaret(true);
end;

procedure TRACustomEditor.WMKillFocus(var Message: TWMSetFocus);
begin
  inherited;
 {$IFDEF RAEDITOR_COMPLETION}
  if FCompletion.FVisible then FCompletion.CloseUp(false);
 {$ENDIF RAEDITOR_COMPLETION}
  DestroyCaret;
end;

{############### Caret ###############}

{**************  ***************}
procedure TRACustomEditor.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

procedure TRACustomEditor.KeyDown(var Key: Word; Shift: TShiftState);
{$IFDEF RAEDITOR_EDITOR}
var
  Com : word;
{$ENDIF RAEDITOR_EDITOR}
begin
 {$IFDEF RAEDITOR_COMPLETION}
  if FCompletion.FVisible then
    if FCompletion.DoKeyDown(Key, Shift) then exit else
  else
    FCompletion.FTimer.Enabled := false;
 {$ENDIF RAEDITOR_COMPLETION}
  if Assigned(FOnKeyDown) then FOnKeyDown(Self, Key, Shift);
 {$IFDEF RAEDITOR_EDITOR}
  Com := FKeyboard.Command(Key, Shift);
  if Com > 0 then Command(Com);
  IgnoreKeyPress := (Com > 0);
 {$IFDEF RAEDITOR_COMPLETION}
  if (Com = ecBackSpace) then
    FCompletion.DoKeyPress(#8);
 {$ENDIF RAEDITOR_COMPLETION}
 {$ENDIF RAEDITOR_EDITOR}
end;

procedure TRACustomEditor.KeyUp(var Key: Word; Shift: TShiftState);
begin
  PaintCaret(false);
  if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
  PaintCaret(true);
end;

{$IFDEF RAEDITOR_EDITOR}
procedure TRACustomEditor.KeyPress(var Key: Char);
var
  S : string;
begin
  if IgnoreKeyPress then begin IgnoreKeyPress := false; exit end;
  if FReadOnly then exit;
  PaintCaret(false);
  if Assigned(FOnKeyPress) then FOnKeyPress(Self, Key);
  while FCaretY > FLines.Count-1 do FLines.Add('');
  while FCaretX > Length(FLines[FCaretY]) do FLines.Internal[FCaretY] := FLines[FCaretY]+' ';
  case Key of
    #32..#255 :
      begin
       {$IFDEF RAEDITOR_COMPLETION}
        if not HasChar(Key, RAEditorCompletionChars) then
          FCompletion.DoKeyPress(Key);
       {$ENDIF RAEDITOR_COMPLETION}
        begin
          DeleteSelected;
          S := FLines[FCaretY];
          if FInsertMode then
          begin
           {$IFDEF RAEDITOR_UNDO}
            TInsertUndo.Create(Self, FCaretX, FCaretY, Key);
           {$ENDIF RAEDITOR_UNDO}
            Insert(Key, S, FCaretX+1);
          end else
          begin
           {$IFDEF RAEDITOR_UNDO}
            CantUndo;
           {$ENDIF RAEDITOR_UNDO}
            S[FCaretX+1] := Key;
          end;
          FLines.Internal[FCaretY] := S;
          SetCaretInternal(FCaretX+1, FCaretY);
          TextModified(SelStart, maInsert, Key);
          PaintLine(FCaretY, -1, -1);
          Changed;
        end;
      {$IFDEF RAEDITOR_COMPLETION}
      if HasChar(Key, RAEditorCompletionChars) then
        FCompletion.DoKeyPress(Key);
      {$ENDIF RAEDITOR_COMPLETION}
    end;
  end;
  PaintCaret(true);
end;
{$ENDIF RAEDITOR_EDITOR}

{$IFDEF RAEDITOR_EDITOR}
type
  EComplete = class(EAbort);

procedure TRACustomEditor.Command(ACommand : TEditCommand);
var
  X, Y : integer;
 {$IFDEF RAEDITOR_UNDO}
  CaretUndo : boolean;
 {$ENDIF RAEDITOR_UNDO}

type
  TPr = procedure of object;

  procedure DoAndCorrectXY(Pr : TPr);
  begin
    Pr;
    X := FCaretX; Y := FCaretY;
   {$IFDEF RAEDITOR_COMPLETION}
    CaretUndo := false;
   {$ENDIF RAEDITOR_COMPLETION}
  end;

  function Com(const Args : array of TEditCommand): boolean;
  var
    i : integer;
  begin
    Result := true;
    for i := 0 to High(Args) do
      if Args[i] = ACommand then exit;
    Result := false;
  end;

  procedure Complete;
  begin
   {may be other exception used in future versions }
    raise EComplete.Create('');
  end;

var
  F : integer;
  S : string;
  B : boolean;
begin
  X := FCaretX; Y := FCaretY;
 {$IFDEF RAEDITOR_UNDO}
  CaretUndo := true;
 {$ENDIF RAEDITOR_UNDO}
  PaintCaret(false);
  inc(FLines.FUpdateLock);
  try
  try
    case ACommand of

     { }

      ecLeft, ecRight, ecSelLeft, ecSelRight :
        begin
          if Com([ecSelLeft, ecSelRight]) and not FSelected then SetSel(X, Y);
          if Com([ecLeft, ecSelLeft]) then dec(X) else inc(X);
          if Com([ecSelLeft, ecSelRight]) then SetSel(X, Y) else SetUnSelected;
        end;
      ecUp, ecDown, ecSelUp, ecSelDown :
        if Com([ecUp, ecSelUp]) or (Y < FRows-1) then
        begin
          if Com([ecSelUp, ecSelDown]) and not FSelected then SetSel(X, Y);
          if Com([ecUp, ecSelUp]) then dec(Y) else inc(Y);
          if Com([ecSelUp, ecSelDown]) then SetSel(X, Y) else SetUnSelected;
        end;
      ecPrevWord :
        begin
          S := FLines[Y];
          B := false;
          for F := X-1 downto 0 do
            if B then
              if (S[F+1] in Separators) then
                begin
                  X := F+1;
                  break;
                end else
            else
              if not (S[F+1] in Separators) then
                B := true;
          if X = FCaretX then
            X := 0;
          SetUnselected;
        end;
      ecNextWord :
        begin
          S := FLines[Y];
          B := false;
          for F := X to Length(S)-1 do
            if B then
              if not (S[F+1] in Separators) then
              begin
                X := F;
                break;
              end else
            else
              if (S[F+1] in Separators) then
                B := true;
          if X = FCaretX then
            X := Length(S);
          SetUnselected;
        end;
      ecScrollLineUp, ecScrollLineDown :
        begin
          if not ((ACommand = ecScrollLineDown) and
             (Y >= FLines.Count-1) and (Y = FTopRow)) then
          begin
            if ACommand = ecScrollLineUp then F := -1 else F := 1;
            scbVert.Position := scbVert.Position+F;
            Scroll(true, scbVert.Position);
          end;
          if Y < FTopRow then Y := FTopRow
          else if Y > FLastVisibleRow then Y := FLastVisibleRow;
        end;
      ecBeginLine, ecSelBeginLine, ecBeginDoc, ecSelBeginDoc,
      ecEndLine, ecSelEndLine, ecEndDoc, ecSelEndDoc :
        begin
          if Com([ecSelBeginLine, ecSelBeginDoc, ecSelEndLine, ecSelEndDoc]) and
             not FSelected then
            SetSel(FCaretX, Y);
          if Com([ecBeginLine, ecSelBeginLine]) then
            X := 0 else
          if Com([ecBeginDoc, ecSelBeginDoc]) then
          begin
            X := 0;
            Y := 0;
            SetLeftTop(0, 0);
          end else
          if Com([ecEndLine, ecSelEndLine]) then
            X := Length(FLines[Y])
          else if Com([ecEndDoc, ecSelEndDoc]) then
          begin
            Y := FLines.Count-1;
            X := Length(FLines[Y]);
            SetLeftTop(X-FVisibleColCount, Y - FVisibleRowCount div 2);
          end;
          if Com([ecSelBeginLine, ecSelBeginDoc, ecSelEndLine, ecSelEndDoc]) then
            SetSel(X, Y)
          else
            SetUnSelected;
        end;
      ecPrevPage :
        begin
          scbVert.Position := scbVert.Position - scbVert.LargeChange;
          Scroll(true, scbVert.Position);
          Y := Y - FVisibleRowCount;
          SetUnSelected;
        end;
      ecNextPage :
        begin
          scbVert.Position := scbVert.Position + scbVert.LargeChange;
          Scroll(true, scbVert.Position);
          Y := Y + FVisibleRowCount;
          SetUnSelected;
        end;
      ecSelPrevPage :
        begin
          BeginUpdate;
          SetSel(X, Y);
          scbVert.Position := scbVert.Position - scbVert.LargeChange;
          Scroll(true, scbVert.Position);
          Y := Y - FVisibleRowCount;
          SetSel(X, Y);
          EndUpdate;
        end;
      ecSelNextPage :
        begin
          BeginUpdate;
          SetSel(X, Y);
          scbVert.Position := scbVert.Position + scbVert.LargeChange;
          Scroll(true, scbVert.Position);
          Y := Y + FVisibleRowCount;
          if Y <= FLines.Count - 1 then
            SetSel(X, Y) else
            SetSel(X, FLines.Count - 1);
          EndUpdate;
        end;
      ecWindowTop : Y := FTopRow;
      ecWindowBottom : Y := FTopRow+FVisibleRowCount - 1;

     {}

      ecInsertPara :
        if not FReadOnly then
        begin
          DeleteSelected;
         {$IFDEF RAEDITOR_UNDO}
          TInsertUndo.Create(Self, FCaretX, FCaretY, #13#10);
          CaretUndo := false;
         {$ENDIF RAEDITOR_UNDO}
          if FLines.Count = 0 then FLines.Add('');
          F := SelStart-1;
          FLines.Insert(Y+1, Copy(FLines[Y], X+1, Length(FLines[Y])));
          FLines.Internal[Y] := Copy(FLines[Y], 1, X);
          X := 0;
          inc(Y);
          UpdateEditorSize;
          TextModified(F, maInsert, #13#10);
          Invalidate;
          Changed;
        end;
      ecBackspace :
        if not FReadOnly then
          if X > 0 then
          begin
           {  }
            DeleteSelected;
            if X <= Length(FLines[Y]) then
            begin
             {$IFDEF RAEDITOR_UNDO}
              TBackspaceUndo.Create(Self, FCaretX, FCaretY, FLines[Y][X]);
              CaretUndo := false;
             {$ENDIF RAEDITOR_UNDO}
              F := SelStart-1; S := FLines.Text[F];
              FLines.Internal[Y] := Copy(FLines[Y], 1, X-1) +
                Copy(FLines[Y], X+1, Length(FLines[Y]));
              TextModified(F, maDelete, S);
            end;
            dec(X);
            PaintLine(Y, -1, -1);
            Changed;
          end else if Y > 0 then
          begin
           {  }
            DeleteSelected;
            F := SelStart-2; S := FLines.Text[SelStart-2]+FLines.Text[SelStart-1];
            X := Length(FLines[Y-1]);
           {$IFDEF RAEDITOR_UNDO}
            TBackspaceUndo.Create(Self, FCaretX, FCaretY, #13);
            CaretUndo := false;
           {$ENDIF RAEDITOR_UNDO}
            FLines.Internal[Y-1] := FLines[Y-1] + FLines[Y];
            FLines.Delete(Y);
            dec(Y);
            UpdateEditorSize;
            TextModified(F, maDelete, S);
            Invalidate;
            Changed;
          end;
      ecDelete :
        if not FReadOnly then
        begin
          if FLines.Count = 0 then FLines.Add('');
          if FSelected then
            DoAndCorrectXY(DeleteSelected)
          else if X < Length(FLines[Y]) then
          begin
           {  }
           {$IFDEF RAEDITOR_UNDO}
            TDeleteUndo.Create(Self, FCaretX, FCaretY, FLines[Y][X+1]);
            CaretUndo := false;
           {$ENDIF RAEDITOR_UNDO}
            DeleteSelected;
            S := FLines.Text[SelStart];
            FLines.Internal[Y] := Copy(FLines[Y], 1, X) +
              Copy(FLines[Y], X+2, Length(FLines[Y]));
            TextModified(SelStart, maDelete, S);
            PaintLine(FCaretY, -1, -1);
            Changed;
          end else if (Y >= 0) and (Y <= FLines.Count-2) then
          begin
           {  }
            DeleteSelected;
            S := FLines.Text[SelStart+1]+FLines.Text[SelStart+2];
            FLines.Internal[Y] := FLines[Y] + FLines[Y+1];
            FLines.Delete(Y+1);
            UpdateEditorSize;
            TextModified(SelStart, maDelete, S);
            Invalidate;
            Changed;
          end;
        end;
      ecDeleteSelected :
        if not FReadOnly and FSelected then
          DoAndCorrectXY(DeleteSelected);
      ecClipBoardCut :
        if not FReadOnly then
          DoAndCorrectXY(ClipBoardCut);
      ecChangeInsertMode :
        begin
          FInsertMode := not FInsertMode;
          if Assigned(FOnChangeStatus) then FOnChangeStatus(Self);
        end;
      ecClipBoardCopy : ClipBoardCopy;
      ecClipBoardPaste : DoAndCorrectXY(ClipBoardPaste);
      ecSetBookmark0..ecSetBookmark9 :
        ChangeBookMark(ACommand-ecSetBookmark0, true);
      ecGotoBookmark0..ecGotoBookmark9 :
        begin
          ChangeBookMark(ACommand-ecGotoBookmark0, false);
          X := FCaretX;
          Y := FCaretY;
        end;
     {$IFDEF RAEDITOR_UNDO}
      ecUndo :
        if not FReadOnly then
        begin
          FUndoBuffer.Undo;
          PaintCaret(true);
          Complete;
        end;
      ecRedo :
        if not FReadOnly then
        begin
          FUndoBuffer.Redo;
          PaintCaret(true);
          Complete;
        end;
     {$ENDIF RAEDITOR_UNDO}
     {$IFDEF RAEDITOR_COMPLETION}
      ecCompletionIdentifers :
        if not FReadOnly then
        begin
          FCompletion.DoCompletion(cmIdentifers);
          PaintCaret(true);
          Complete;
        end;
      ecCompletionTemplates :
        if not FReadOnly then
        begin
          FCompletion.DoCompletion(cmTemplates);
          PaintCaret(true);
          Complete;
        end;
     {$ENDIF RAEDITOR_COMPLETION}
    end;
   {$IFDEF RAEDITOR_UNDO}
    if CaretUndo then
      SetCaret(X, Y)
    else
      SetCaretInternal(X, Y);
   {$ELSE}
    SetCaret(X, Y);
   {$ENDIF RAEDITOR_UNDO}
  except
    on E : EComplete do
    else raise;
  end;
  finally
    dec(FLines.FUpdateLock);
    PaintCaret(true);
  end;
end;
{$ENDIF}

{$IFDEF RAEDITOR_EDITOR}
procedure TRACustomEditor.ChangeBookMark(const BookMark : TBookMarkNum; const Valid : boolean);

  procedure SetXY(X, Y : integer);
  var
    X1, Y1 : integer;
  begin
    X1 := FLeftCol;
    Y1 := FTopRow;
    if (Y < FTopRow) or (Y > FLastVisibleRow) then
      Y1 := Y - (FVisibleRowCount div 2);
    if (X < FLeftCol) or (X > FVisibleColCount) then
      X1 := X - (FVisibleColCount div 2);
    SetLeftTop(X1, Y1);
    SetCaret(X, Y);
  end;

begin
  if Valid then
    if FBookMarks[Bookmark].Valid and (FBookMarks[Bookmark].Y = FCaretY) then
      FBookMarks[Bookmark].Valid := false
    else
    begin
      FBookMarks[Bookmark].X := FCaretX;
      FBookMarks[Bookmark].Y := FCaretY;
      FBookMarks[Bookmark].Valid := true;
    end
  else
  if FBookMarks[Bookmark].Valid then
    SetXY(FBookMarks[Bookmark].X, FBookMarks[Bookmark].Y);
  BookmarkCnanged(BookMark);
end;
{$ENDIF}

procedure TRACustomEditor.BookmarkCnanged(BookMark : integer);
begin
  FGutter.Invalidate;
end;


{###############  ###############}

procedure TRACustomEditor.SelectionChanged;
begin
 {abstract}
end;

procedure TRACustomEditor.SetSel(const SelX, SelY : integer);

  procedure UpdateSelected;
  var
    iR : integer;
  begin
    if FUpdateSelBegY < FSelBegY then
      for iR := FUpdateSelBegY to FSelBegY do PaintLine(iR, -1, -1)
    else
      for iR := FSelBegY to FUpdateSelBegY do PaintLine(iR, -1, -1);
    if FUpdateSelEndY < FSelEndY then
      for iR := FUpdateSelEndY to FSelEndY do PaintLine(iR, -1, -1)
    else
      for iR := FSelEndY to FUpdateSelEndY do PaintLine(iR, -1, -1);
    SelectionChanged;
    if Assigned(FOnSelectionChange) then FOnSelectionChange(Self);
  end;

begin
  if not FSelected then
  begin
    FSelStartX := SelX; FSelStartY := SelY;
    FSelEndX := SelX; FSelEndY := SelY;
    FSelBegX := SelX; FSelBegY := SelY;
    FSelected := true;
  end else
  begin
    FUpdateSelBegY := FSelBegY; FUpdateSelEndY := FSelEndY;
    if SelY <= FSelStartY then FSelBegY := SelY;
    if SelY >= FSelStartY then FSelEndY := SelY;
    if (SelY < FSelStartY) or ((SelY = FSelStartY) and (SelX <= FSelStartX)) then FSelBegX := SelX;
    if (SelY > FSelStartY) or ((SelY = FSelStartY) and (SelX >= FSelStartX)) then FSelEndX := SelX;
    if FSelBegY < 0 then FSelBegY := 0;
    FSelected := true;
  end;
  UpdateSelected;
  { TSelectUndo.Create(..) }
  if FUpdateSelBegY > FSelBegY then FUpdateSelBegY := FSelBegY;
  if FUpdateSelEndY < FSelEndY then FUpdateSelEndY := FSelEndY;
end;

{**************  ***************}
procedure TRACustomEditor.Mouse2Cell(const X, Y : integer; var CX, CY : integer);
begin
  CX := Round((X-EditorClient.Left) / FCellRect.Width);
  CY := (Y-EditorClient.Top) div FCellRect.Height;
end;

procedure TRACustomEditor.Mouse2Caret(const X, Y : integer; var CX, CY : integer);
begin
  Mouse2Cell(X, Y, CX, CY);
  if CX < 0 then CX := 0;
  if CY < 0 then CY := 0;
  CX := CX + FLeftCol;
  CY := CY + FTopRow;
  if CX > FLastVisibleCol then CX := FLastVisibleCol;
  if CY > FLines.Count - 1 then CY := FLines.Count - 1;
end;

procedure TRACustomEditor.CaretCoord(const X, Y : integer; var CX, CY : integer);
begin
  CX := X - FLeftCol;
  CY := Y - FTopRow;
  if CX < 0 then CX := 0;
  if CY < 0 then CY := 0;
  CX := FCellRect.Width * CX;
  CY := FCellRect.Height * CY;
end;

procedure TRACustomEditor.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  XX, YY : integer;
begin
  if FDbl then begin FDbl := false; exit end;
 {$IFDEF RAEDITOR_COMPLETION}
  if FCompletion.FVisible then FCompletion.CloseUp(false);
 {$ENDIF RAEDITOR_COMPLETION}
  Mouse2Caret(X, Y, XX, YY);
  if (XX = FCaretX) and (YY = FCaretY) then exit;
  PaintCaret(false);
  SetUnSelected;
  SetFocus;
  SetCaret(XX, YY);
  PaintCaret(true);
  if Assigned(FOnMouseDown) then FOnMouseDown(Self, Button, Shift, X, Y);
end;

procedure TRACustomEditor.DblClick;
var
  iBeg, iEnd : integer;
begin
  FDbl := true;
  if Assigned(FOnDblClick) then FOnDblClick(Self);
  if FDoubleClickLine then
  begin
    PaintCaret(false);
    SetSel(0, FCaretY);
    if FCaretY = FLines.Count -1 then begin
      SetSel(Length(FLines[FCaretY]), FCaretY);
      SetCaret(Length(FLines[FCaretY]), FCaretY);
    end else begin
      SetSel(0, FCaretY +1);
      SetCaret(0, FCaretY +1);
    end;
    PaintCaret(true);
  end else if GetWordOnPosEx(FLines[FCaretY], FCaretX, iBeg, iEnd) <> '' then
  begin
    PaintCaret(false);
    SetSel(iBeg - 1, FCaretY);
    SetSel(iEnd - 1, FCaretY);
    SetCaret(iEnd - 1, FCaretY);
    PaintCaret(true);
  end;
end;

procedure TRACustomEditor.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  timerScroll.Enabled := false;
  if Assigned(FOnMouseUp) then FOnMouseUp(Self, Button, Shift, X, Y);
end;

procedure TRACustomEditor.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  if Shift = [ssLeft] then begin
    PaintCaret(false);
    MouseMoveY := Y;
    Mouse2Caret(X, Y, MouseMoveXX, MouseMoveYY);
    if MouseMoveYY <= FLastVisibleRow then begin
      SetSel(MouseMoveXX, MouseMoveYY);
      SetCaret(MouseMoveXX, MouseMoveYY);
    end;
    timerScroll.Enabled := (Y < 0) or (Y > ClientHeight);
    PaintCaret(true);
  end;
  if Assigned(FOnMouseMove) then FOnMouseMove(Self, Shift, X, Y);
end;

procedure TRACustomEditor.ScrollTimer(Sender: TObject);
begin
  if (MouseMoveY < 0) or (MouseMoveY > ClientHeight) then
  begin
    if (MouseMoveY < -20) then dec(MouseMoveYY, FVisibleRowCount) else
    if (MouseMoveY < 0) then dec(MouseMoveYY) else
    if (MouseMoveY > ClientHeight + 20) then inc(MouseMoveYY, FVisibleRowCount) else
    if (MouseMoveY > ClientHeight) then inc(MouseMoveYY);
    PaintCaret(false);
    SetSel(MouseMoveXX, MouseMoveYY);
    SetCaret(MouseMoveXX, MouseMoveYY);
    PaintCaret(true);
  end;
end;

{##############  ###############}

{************** ClipBoard ***************}
function TRACustomEditor.GetSelText : string;
var
  S : string;
  i : integer;
begin
  Result := '';
  if not FSelected then exit;
  if not FSelBlock then
  begin
    if (FSelBegY < 0) or (FSelBegY > FLines.Count-1) then Err;
    if FSelBegY = FSelEndY then
      S := Copy(FLines[FSelEndY], FSelBegX+1, FSelEndX-FSelBegX+Inclusive)
    else
    begin
      S := Copy(FLines[FSelBegY], FSelBegX+1, Length(FLines[FSelBegY]));
      for i := FSelBegY+1 to FSelEndY-1 do
        S := S +#13#10+FLines[i];
      S := S+#13#10+Copy(FLines[FSelEndY], 1, FSelEndX+Inclusive);
    end;
    Result := S;
  end;                       
end;

procedure TRACustomEditor.ClipBoardCopy;
begin
  if not FSelBlock then
    ClipBoard.SetTextBuf(PChar(GetSelText));
end;

{$IFDEF RAEDITOR_EDITOR}
procedure TRACustomEditor.InsertText(const Text : string);
var
  S : string;
  P : integer;
  X, Y : integer;
begin
  PaintCaret(false);
  BeginUpdate;
  S := FLines.Text;
  P := PosFromCaret(FCaretX, FCaretY);
  {$IFDEF RAEDITOR_UNDO}
  TInsertUndo.Create(Self, FCaretX, FCaretY, Text);
  //FUndoBuffer.EndGroup;
  {$ENDIF RAEDITOR_UNDO}
  Insert(Text, S, P+1);
  TextModified(P, maInsert, Text);
  FLines.Text := S; {!!!   }
  CaretFromPos(P+Length(Text), X, Y);
  SetCaretInternal(X, Y);
  Changed;
  EndUpdate;
  PaintCaret(true);
end;

{      NewString}
{  NewString    #13, #10}
procedure TRACustomEditor.ReplaceWord(const NewString : string);
var
  iBeg, iEnd : integer;

  function GetWordOnPos2(S : string; P :integer) : string;
  begin
    Result := '';
    if P < 1 then exit;
    if (S[P] in Separators) and ((P < 1) or (S[P-1] in Separators)) then inc(P);
    iBeg := P;
    while iBeg >= 1 do
      if S[iBeg] in Separators then break else dec(iBeg);
    inc(iBeg);
    iEnd := P;
    while iEnd <= Length(S) do
      if S[iEnd] in Separators then break else inc(iEnd);
    if iEnd > iBeg then
      Result := Copy(S, iBeg, iEnd - iBeg) else
      Result := S[P];
  end;

var
  S, W : string;
  X : integer;
  F : integer;
begin
  PaintCaret(false);
  BeginUpdate;
  F := PosFromCaret(FCaretX, FCaretY);
  S := FLines[FCaretY];
  while FCaretX > Length(S) do S := S + ' ';
  W := Trim(GetWordOnPos2(S, FCaretX));
  if W = '' then begin iBeg := FCaretX + 1; iEnd := FCaretX end;
 {$IFDEF RAEDITOR_UNDO}
  CantUndo;
  //TReplaceUndo.Create(Self, FCaretX - Length(W), FCaretY, iBeg, iEnd, W, NewString);
 {$ENDIF RAEDITOR_UNDO}
//  LW := Length(W);
  if FSelected then
  begin
    if (FSelBegY <= FCaretY) or (FCaretY >= FSelEndY) then
      //  LW ..
  end;
  Delete(S, iBeg, iEnd - iBeg);
  Insert(NewString, S, iBeg);
  FLines.Internal[FCaretY] := S;
  X := iBeg + Length(NewString) - 1;
  TextModified(F, maInsert, NewString);
  PaintLine(FCaretY, -1, -1);
  SetCaretInternal(X, FCaretY);
  Changed;
  EndUpdate;
  PaintCaret(true);
end;

{      NewString}
procedure TRACustomEditor.ReplaceWord2(const NewString : string);
var
  S, S1, W : string;
  P, X, Y : integer;
  iBeg, iEnd : integer;
  NewCaret : integer;
begin
  PaintCaret(false);
  if FCaretX > Length(FLines[FCaretY]) then
    FLines.Internal[FCaretY] := FLines[FCaretY] +
     Spaces(FCaretX - Length(FLines[FCaretY]));
  S := FLines.Text;
  P := PosFromCaret(FCaretX, FCaretY);
  W := Trim(GetWordOnPosEx(S, P, iBeg, iEnd));
  if W = '' then begin iBeg := P+1; iEnd := P end;
  S1 := NewString;
  NewCaret := Length(NewString);
 {$IFDEF RAEDITOR_UNDO}
  TReplaceUndo.Create(Self, FCaretX, FCaretY, iBeg, iEnd, W, S1);
 {$ENDIF RAEDITOR_UNDO}
//  LW := Length(W);
  if FSelected then
  begin
    if (FSelBegY <= FCaretY) or (FCaretY >= FSelEndY) then
      //  LW ..
  end;
  Delete(S, iBeg, iEnd - iBeg);
  Insert(S1, S, iBeg);
  FLines.Text := S; {!!!   }
  CaretFromPos(iBeg + NewCaret - 1, X, Y);
  SetCaretInternal(X, Y);
  Changed;
  PaintCaret(true);
end;
{$ENDIF RAEDITOR_EDITOR}

procedure TRACustomEditor.ClipBoardPaste;
var
  S, ClipS : string;
  Len, P : integer;
  H : THandle;
  X, Y : integer;
begin
  if FReadOnly then exit;
  if (FCaretY > FLines.Count-1) and (FLines.Count > 0) then Err;
  BeginUpdate;
  H := ClipBoard.GetAsHandle(CF_TEXT);
  Len := GlobalSize(H);
  if Len = 0 then exit;
  PaintCaret(false);
 {$IFDEF RAEDITOR_UNDO}
  //FUndoBuffer.BeginGroup;
 {$ENDIF RAEDITOR_UNDO}
  DeleteSelected;
  SetLength(ClipS, Len);
  SetLength(ClipS, ClipBoard.GetTextBuf(PChar(ClipS), Len));
  if FLines.Count > 0 then
    while FCaretX > Length(FLines[FCaretY]) do
      FLines.Internal[FCaretY] := FLines[FCaretY] + ' ';
  S := FLines.Text;
  if FLines.Count > 0 then
    P := PosFromCaret(FCaretX, FCaretY) else
    P := 0;
 {$IFDEF RAEDITOR_UNDO}
  TInsertUndo.Create(Self, FCaretX, FCaretY, ClipS);
  //FUndoBuffer.EndGroup;
 {$ENDIF RAEDITOR_UNDO}
  Insert(ClipS, S, P + 1);
  TextModified(P, maInsert, ClipS);
  FLines.Text := S; {!!!   }
  CaretFromPos(P + Length(ClipS), X, Y);
  SetCaretInternal(X, Y);
  Changed;
  EndUpdate;
  PaintCaret(true);
end;

procedure TRACustomEditor.ClipBoardCut;
begin
  ClipBoardCopy;
  DeleteSelected;
end;

procedure TRACustomEditor.DeleteSelected;
var
  S : string;
  iBeg, iEnd, X, Y : integer;
begin
  if FSelected then
  begin
    PaintCaret(false);
    S := FLines.Text;
    iBeg := PosFromCaret(FSelBegX, FSelBegY);
    iEnd := PosFromCaret(FSelEndX, FSelEndY);
   {$IFDEF RAEDITOR_UNDO}
    TDeleteSelectedUndo.Create(Self, FCaretX, FCaretY, GetSelText, FSelBlock, FSelBegX, FSelBegY, FSelEndX, FSelEndY);
   {$ENDIF RAEDITOR_UNDO}
    Delete(S, iBeg+1, iEnd-iBeg);
    FSelected := false;
    TextModified(iBeg, maDelete, GetSelText);
    FLines.Text := S; {!!!   }
    CaretFromPos(iBeg, X, Y);
    SetCaretInternal(X, Y);
    Changed;
    PaintCaret(true);
  end;
end;
{############### ClipBoard ###############}

procedure TRACustomEditor.SetGutterWidth(AWidth : integer);
begin
  FGutterWidth := AWidth;
  UpdateEditorSize;
end;

procedure TRACustomEditor.SetGutterColor(AColor : TColor);
begin
  FGutterColor := AColor;
  FGutter.Invalidate;
end;

function TRACustomEditor.GetLines : TStrings;
begin
  Result := FLines;
end;

procedure TRACustomEditor.SetLines(ALines : TStrings);
begin
  if ALines <> nil then FLines.Assign(ALines);
  FSelected := false;
  UpdateEditorSize;
  if Showing then Invalidate;
end;

procedure TRACustomEditor.SetCols(ACols : integer);
begin
  if FCols <> ACols then
  begin
    FCols := Max(ACols, 1);
    scbHorz.Max := FCols-1;
  end;
end;

procedure TRACustomEditor.SetRows(ARows : integer);
begin
  if FRows <> ARows then
  begin
    FRows := Max(ARows, 1);
    scbVert.Max := Max(1, FRows-1+FVisibleRowCount-1);
  end;
end;

procedure TRACustomEditor.SetLeftTop(ALeftCol, ATopRow : integer);
begin
  if ALeftCol < 0 then ALeftCol := 0;
  if (FLeftCol <> ALeftCol) then
  begin
    scbHorz.Position := ALeftCol;
    Scroll(false, ALeftCol);
  end;
  if ATopRow < 0 then ATopRow := 0;
  if (FTopRow <> ATopRow) then
  begin
    scbVert.Position := ATopRow;
    Scroll(true, ATopRow);
  end;
end;

procedure TRACustomEditor.SetScrollBars(Value : TScrollStyle);
begin
  if FScrollBars <> Value then
  begin
    FScrollBars := Value;
    RecreateWnd;
  end;
end;

procedure TRACustomEditor.TextModified(Pos : integer; Action : TModifiedAction; Text : string);
begin
 { This method don't called at all cases, when text is modified,
   so don't use it. Later I may be complete it. }
end;

procedure TRACustomEditor.Changed;
begin
  FModified := true;
  if Assigned(FOnChange) then FOnChange(self);
  StatusChanged;
end;

procedure TRACustomEditor.StatusChanged;
begin
  if Assigned(FOnChangeStatus) then FOnChangeStatus(self);
end;

procedure TRACustomEditor.CaretFromPos(const Pos : integer; var X, Y : integer);
{   Pos -   -  }
begin
  RAUtils.GetXYByPos(FLines.Text, Pos, X, Y);
end;

function TRACustomEditor.PosFromCaret(const X, Y : integer) : integer;
{}
var
  i : integer;
begin
  if Y > FLines.Count-1 then
    Result := -1
  else
  begin
    Result := 0;
    for i := 0 to Y-1 do
      inc(Result, Length(FLines[i])+2{CR/LF});
    if X < Length(FLines[Y]) then
      inc(Result, X)
    else
      inc(Result, Length(FLines[Y]))
  end;
end;

function TRACustomEditor.PosFromMouse(const X, Y : integer) : integer;
var
  X1, Y1 : integer;
begin
  Mouse2Caret(X, Y, X1, Y1);
  if (X1 < 0) or (Y1 < 0) then
    Result := -1
  else
    Result := PosFromCaret(X1, Y1);
end;

function TRACustomEditor.GetSelStart : integer;
begin
  Result := PosFromCaret(FCaretX, FCaretY);
end;

procedure TRACustomEditor.SetSelStart(const ASelStart : integer);
begin
  FSelected := true;
  FSelStart := ASelStart;
  CaretFromPos(ASelStart, FSelBegX, FSelBegY);
  FCaretX := FSelBegX;
  FCaretY := FSelBegY;
  //PaintSelection;
//  EditorPaint;
end;

procedure TRACustomEditor.SetSelLength(const ASelLength : integer);
begin
  FSelected := true;
  CaretFromPos(FSelStart+ASelLength, FSelEndX, FSelEndY);
  //PaintSelection;
  Invalidate;
end;

procedure TRACustomEditor.GutterPaint(Canvas : TCanvas);
begin
  if Assigned(FOnPaintGutter) then FOnPaintGutter(Self, Canvas);
end;

procedure TRACustomEditor.SetMode(index : integer; Value : boolean);
var
  PB : ^boolean;
begin
  case index of
    0 : PB := @FInsertMode;
    else {1 :} PB := @FReadOnly;
  end;
  if PB^ <> Value then
  begin
    PB^ := Value;
    StatusChanged;
  end;
end;

function TRACustomEditor.GetWordOnCaret : string;
begin
  Result := GetWordOnPos(FLines[CaretY], CaretX);
end;


constructor TEditKey.Create(const ACommand : TEditCommand; const AKey : word; const AShift : TShiftState);
begin
  Key := AKey;
  Shift := AShift;
  Command := ACommand;
end;

constructor TKeyboard.Create;
begin
  List := TList.Create;
end;

destructor TKeyboard.Destroy;
begin
  Clear;
  List.Free;
end;

procedure TKeyboard.Add(const ACommand : TEditCommand; const AKey : word; const AShift : TShiftState);
begin
  List.Add(TEditKey.Create(ACommand, AKey, AShift));
end;

procedure TKeyboard.Clear;
var
  i : integer;
begin
  for i := 0 to List.Count - 1 do
    TObject(List[i]).Free;
  List.Clear;
end;

function TKeyboard.Command(const AKey : word; const AShift : TShiftState) : TEditCommand;
var
  i : integer;
begin
  Result := 0;
  for i := 0 to List.Count-1 do
    with TEditKey(List[i]) do
      if (Key = AKey) and (Shift = AShift) then
      begin
        Result := Command;
        exit;
      end;
end;

{$IFDEF RAEDITOR_EDITOR}
{$IFDEF RAEDITOR_DEFLAYOT}
procedure TKeyboard.SetDefLayot;
begin
  Clear;
  Add(ecLeft        , VK_LEFT   , []);
  Add(ecRight       , VK_RIGHT  , []);
  Add(ecUp          , VK_UP     , []);
  Add(ecDown        , VK_DOWN   , []);
  Add(ecSelLeft     , VK_LEFT   , [ssShift]);
  Add(ecSelRight    , VK_RIGHT  , [ssShift]);
  Add(ecSelUp       , VK_UP     , [ssShift]);
  Add(ecSelDown     , VK_DOWN   , [ssShift]);
  Add(ecBeginLine   , VK_HOME   , []);
  Add(ecSelBeginLine, VK_HOME   , [ssShift]);
  Add(ecBeginDoc    , VK_HOME   , [ssCtrl]);
  Add(ecSelBeginDoc , VK_HOME   , [ssCtrl, ssShift]);
  Add(ecEndLine     , VK_END    , []);
  Add(ecSelEndLine  , VK_END    , [ssShift]);
  Add(ecEndDoc      , VK_END    , [ssCtrl]);
  Add(ecSelEndDoc   , VK_END    , [ssCtrl, ssShift]);
  Add(ecPrevWord    , VK_LEFT   , [ssCtrl]);
  Add(ecNextWord    , VK_RIGHT  , [ssCtrl]);
  Add(ecSelPrevWord , VK_LEFT   , [ssCtrl, ssShift]);
  Add(ecSelNextWord , VK_RIGHT  , [ssCtrl, ssShift]);

  Add(ecWindowTop   , VK_PRIOR  , [ssCtrl]);
  Add(ecWindowBottom, VK_NEXT   , [ssCtrl]);
  Add(ecPrevPage    , VK_PRIOR  , []);
  Add(ecNextPage    , VK_NEXT   , []);
  Add(ecSelPrevPage , VK_PRIOR  , [ssShift]);
  Add(ecSelNextPage , VK_NEXT   , [ssShift]);
  Add(ecScrollLineUp  , VK_UP     , [ssCtrl]);
  Add(ecScrollLineDown, VK_DOWN   , [ssCtrl]);

  Add(ecChangeInsertMode, VK_INSERT, []);

  Add(ecInsertPara    , VK_RETURN , []);
  Add(ecBackspace     , VK_BACK   , []);
  Add(ecDelete        , VK_DELETE , []);
  Add(ecDeleteSelected, VK_DELETE , [ssCtrl]);
  Add(ecClipboardCopy , VK_INSERT , [ssCtrl]);
  Add(ecClipboardCut  , VK_DELETE , [ssShift]);
  Add(ecClipBoardPaste, VK_INSERT , [ssShift]);

  Add(ecClipboardCopy , ord('C') , [ssCtrl]);
  Add(ecClipboardCut  , ord('X') , [ssCtrl]);
  Add(ecClipBoardPaste, ord('V') , [ssCtrl]);


  Add(ecSetBookmark0  , ord('0') , [ssCtrl, ssShift]);
  Add(ecSetBookmark1  , ord('1') , [ssCtrl, ssShift]);
  Add(ecSetBookmark2  , ord('2') , [ssCtrl, ssShift]);
  Add(ecSetBookmark3  , ord('3') , [ssCtrl, ssShift]);
  Add(ecSetBookmark4  , ord('4') , [ssCtrl, ssShift]);
  Add(ecSetBookmark5  , ord('5') , [ssCtrl, ssShift]);
  Add(ecSetBookmark6  , ord('6') , [ssCtrl, ssShift]);
  Add(ecSetBookmark7  , ord('7') , [ssCtrl, ssShift]);
  Add(ecSetBookmark8  , ord('8') , [ssCtrl, ssShift]);
  Add(ecSetBookmark9  , ord('9') , [ssCtrl, ssShift]);

  Add(ecGotoBookmark0  , ord('0') , [ssCtrl]);
  Add(ecGotoBookmark1  , ord('1') , [ssCtrl]);
  Add(ecGotoBookmark2  , ord('2') , [ssCtrl]);
  Add(ecGotoBookmark3  , ord('3') , [ssCtrl]);
  Add(ecGotoBookmark4  , ord('4') , [ssCtrl]);
  Add(ecGotoBookmark5  , ord('5') , [ssCtrl]);
  Add(ecGotoBookmark6  , ord('6') , [ssCtrl]);
  Add(ecGotoBookmark7  , ord('7') , [ssCtrl]);
  Add(ecGotoBookmark8  , ord('8') , [ssCtrl]);
  Add(ecGotoBookmark9  , ord('9') , [ssCtrl]);

 {$IFDEF RAEDITOR_UNDO}
  Add(ecUndo           , ord('Z') , [ssCtrl]);
  Add(ecUndo           , VK_BACK  , [ssAlt]);
 {$ENDIF RAEDITOR_UNDO}

 {$IFDEF RAEDITOR_COMPLETION}
  Add(ecCompletionIdentifers , VK_SPACE   , [ssCtrl]);
  Add(ecCompletionTemplates  , ord('J')   , [ssCtrl]);
 {$ENDIF RAEDITOR_COMPLETION}

end;
{$ENDIF RAEDITOR_DEFLAYOT}


{$IFDEF RAEDITOR_UNDO}

procedure RedoNotImplemented;
begin
  raise ERAEditorError.Create('Redo not yet implemented');
end;

procedure TRACustomEditor.CantUndo;
begin
  FUndoBuffer.Clear;
end;

procedure TUndoBuffer.Add(AUndo : TUndo);
begin
  if InUndo then exit;
  while (Count > 0) and (FPtr < Count-1) do
  begin
    TUndo(Items[FPtr+1]).Free;
    inherited Delete(FPtr+1);
  end;
  inherited Add(AUndo);
  FPtr := Count-1;
end;

procedure TUndoBuffer.Undo;
var
  UndoClass : TClass;
begin
  InUndo := true;
  try
    if LastUndo <> nil then
    begin
      UndoClass := LastUndo.ClassType;
      while (LastUndo <> nil) and (UndoClass = LastUndo.ClassType) do
      begin
        LastUndo.Undo;
        dec(FPtr);
        if not GroupUndo then break;
      end;
    end;
  finally
    InUndo := false;
  end;
end;

procedure TUndoBuffer.Redo;
begin
end;

procedure TUndoBuffer.Clear;
begin
  while Count > 0 do
  begin
    TUndo(Items[0]).Free;
    inherited Delete(0);
  end;
end;

procedure TUndoBuffer.Delete;
begin
  if Count > 0 then
  begin
    TUndo(Items[Count-1]).Free;
    inherited Delete(Count-1);
  end;
end;

function TUndoBuffer.LastUndo : TUndo;
begin
  if (FPtr >= 0) and (Count > 0) then
    Result := TUndo(Items[FPtr]) else
    Result := nil;
end;

function TUndoBuffer.NewGroup(AUndo : TUndo) : boolean;
begin
  Result := (LastUndo = nil) or (LastUndo.ClassType <> AUndo.ClassType)
end;


{* TUndo}
constructor TUndo.Create(ARAEditor : TRACustomEditor);
begin
  FRAEditor := ARAEditor;
  UndoBuffer.Add(Self);
end;

function TUndo.UndoBuffer : TUndoBuffer;
begin
  if FRAEditor <> nil then
    Result := FRAEditor.FUndoBuffer else
    Result := nil;
end;
{* TUndo}

{* TCaretUndo}
constructor TCaretUndo.Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer);
begin
  inherited Create(ARAEditor);
  FCaretX := ACaretX;
  FCaretY := ACaretY;
end;

procedure TCaretUndo.Undo;
begin
  with UndoBuffer do
  begin
    dec(FPtr);
    while GroupUndo and (FPtr >= 0) and not NewGroup(Self) do
      dec(FPtr);
    inc(FPtr);
    with TCaretUndo(Items[FPtr]) do
      FRAEditor.SetCaretInternal(FCaretX, FCaretY);
  end;
end;

procedure TCaretUndo.Redo;
begin
  RedoNotImplemented;
end;
{# TCaretUndo}

{* TInsertUndo}
constructor TInsertUndo.Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer; AText : string);
begin
  inherited Create(ARAEditor, ACaretX, ACaretY);
  FText := AText;
end;

procedure TInsertUndo.Undo;
var
  S, Text : string;
  iBeg : integer;
begin
  Text := '';
  with UndoBuffer do
  begin
    while (FPtr >= 0) and not NewGroup(Self) do
    begin
      Text := TInsertUndo(LastUndo).FText + Text;
      dec(FPtr);
      if not GroupUndo then break;
    end;
    inc(FPtr);
  end;
  with TInsertUndo(UndoBuffer.Items[UndoBuffer.FPtr]) do
  begin
    S := FRAEditor.FLines.Text;
    iBeg := FRAEditor.PosFromCaret(FCaretX, FCaretY);
    Delete(S, iBeg+1, Length(Text));
    FRAEditor.FLines.Text := S; {!!!   }
    FRAEditor.Changed;
    FRAEditor.SetCaret(FCaretX, FCaretY);
  end;
end;

procedure TInsertUndo.Redo;
begin
  RedoNotImplemented;
end;
{# TInsertUndo}

{* TDeleteUndo}
procedure TDeleteUndo.Undo;
var
  S, Text : string;
  iBeg : integer;
begin
  Text := '';
  with UndoBuffer do
  begin
    while (FPtr >= 0) and not NewGroup(Self) do
    begin
      Text := TDeleteUndo(LastUndo).FText + Text;
      dec(FPtr);
      if not GroupUndo then break;
    end;
    inc(FPtr);
  end;
  with TDeleteUndo(UndoBuffer.Items[UndoBuffer.FPtr]) do
  begin
    S := FRAEditor.FLines.Text;
    iBeg := FRAEditor.PosFromCaret(FRAEditor.FCaretX, FRAEditor.FCaretY);
    Insert(Text, S, iBeg + 1);
    FRAEditor.FLines.Text := S; {!!!   }
    FRAEditor.Changed;
    FRAEditor.SetCaret(FCaretX, FCaretY);
  end;
end;

procedure TDeleteUndo.Redo;
begin
  RedoNotImplemented;
end;
{# TDeleteUndo}

{* TBackspaceUndo}
procedure TBackspaceUndo.Undo;
var
  S, Text : string;
  iBeg : integer;
begin
  Text := '';
  with UndoBuffer do
  begin
    while (FPtr >= 0) and not NewGroup(Self) do
    begin
      Text := Text + TDeleteUndo(LastUndo).FText;
      dec(FPtr);
      if not GroupUndo then break;
    end;
    inc(FPtr);
  end;
  with TDeleteUndo(UndoBuffer.Items[UndoBuffer.FPtr]) do
  begin
    S := FRAEditor.FLines.Text;
    iBeg := FRAEditor.PosFromCaret(FRAEditor.FCaretX, FRAEditor.FCaretY);
    Insert(Text, S, iBeg + 1);
    FRAEditor.FLines.Text := S; {!!!   }
    FRAEditor.Changed;
    FRAEditor.SetCaret(FCaretX, FCaretY);
  end;
end;

procedure TBackspaceUndo.Redo;
begin
  RedoNotImplemented;
end;
{# TBackspaceUndo}

{* TReplaceUndo}
constructor TReplaceUndo.Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer; ABeg, AEnd : integer; AText, ANewText : string);
begin
  inherited Create(ARAEditor, ACaretX, ACaretY);
  FBeg := ABeg;
  FEnd := AEnd;
  FText := AText;
  FNewText := ANewText;
end;

procedure TReplaceUndo.Undo;
var
  S : string;
begin
//  raise ERAEditorError.Create('Undo not yet implemented');
  S := FRAEditor.FLines.Text;
  Delete(S, FBeg, Length(FNewText));
  Insert(FText, S, FBeg);
  FRAEditor.FLines.Text := S; {!!!   }
  FRAEditor.Changed;
  FRAEditor.SetCaretInternal(FCaretX, FCaretY);
end;

procedure TReplaceUndo.Redo;
begin
  RedoNotImplemented;
end;
{# TReplaceUndo}

{* TDeleteSelectedUndo}
constructor TDeleteSelectedUndo.Create(ARAEditor : TRACustomEditor; ACaretX, ACaretY : integer; AText : string; ASelBlock : boolean; ASelBegX, ASelBegY, ASelEndX, ASelEndY : integer);
begin
  inherited Create(ARAEditor, ACaretX, ACaretY, AText);
  FSelBlock := ASelBlock;
  FSelBegX := ASelBegX;
  FSelBegY := ASelBegY;
  FSelEndX := ASelEndX;
  FSelEndY := ASelEndY;
end;

procedure TDeleteSelectedUndo.Undo;
var
  S, Text : string;
  iBeg : integer;
begin
  Text := '';
  with UndoBuffer do
  begin
    while (FPtr >= 0) and not NewGroup(Self) do
    begin
      Text := TDeleteUndo(LastUndo).FText + Text;
      dec(FPtr);
      if not GroupUndo then break;
    end;
    inc(FPtr);
  end;
  with TDeleteUndo(UndoBuffer.Items[UndoBuffer.FPtr]) do
  begin
    S := FRAEditor.FLines.Text;
    iBeg := FRAEditor.PosFromCaret(FSelBegX, FSelBegY);
    Insert(Text, S, iBeg+1);
    { FSelBegX, FSelBegY}
   //!!!! ..
    FRAEditor.BeginUpdate;
    FRAEditor.FLines.Text := S; {!!!   }
    FRAEditor.Changed;
    FRAEditor.FSelBlock := FSelBlock;
    FRAEditor.FSelBegX  := FSelBegX;
    FRAEditor.FSelBegY  := FSelBegY;
    FRAEditor.FSelEndX  := FSelEndX;
    FRAEditor.FSelEndY  := FSelEndY;
    FRAEditor.FSelected := Length(FText) > 0;
    FRAEditor.SetCaret(FCaretX, FCaretY);
    FRAEditor.EndUpdate;
  end;
end;

procedure TDeleteSelectedUndo.Redo;
begin
  RedoNotImplemented;
end;
{# TDeleteSelectedUndo}

{$ENDIF RAEDITOR_UNDO}

{#####################  Undo  #####################}


{*********************  Code Completion  *********************}
{$IFDEF RAEDITOR_COMPLETION}

procedure TRACustomEditor.CompletionIdentifer(var Cancel : boolean);
begin
 {abstract}
end;

procedure TRACustomEditor.CompletionTemplate(var Cancel : boolean);
begin
 {abstract}
end;

procedure TRACustomEditor.DoCompletionIdentifer(var Cancel : boolean);
begin
  CompletionIdentifer(Cancel);
  if Assigned(FOnCompletionIdentifer) then FOnCompletionIdentifer(Self, Cancel);
end;

procedure TRACustomEditor.DoCompletionTemplate(var Cancel : boolean);
begin
  CompletionTemplate(Cancel);
  if Assigned(FOnCompletionTemplate) then FOnCompletionTemplate(Self, Cancel);
end;

type
  TPopupList = class(TListBox)
  private
    FTimer : TTimer;
    YY : integer;
   // HintWindow : THintWindow;
    procedure CMHintShow(var Message: TMessage); message CM_HINTSHOW;
    procedure WMCancelMode(var Message: TMessage); message WM_CancelMode;
    procedure OnTimer(Sender : TObject);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure DestroyWnd; override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

constructor TCompletion.Create2(ARAEditor : TRACustomEditor);
begin
  inherited Create;
  FRAEditor := ARAEditor;
  FPopupList := TPopupList.Create(FRAEditor);
  FItemHeight := FPopupList.ItemHeight;
  FDropDownCount := 6;
  FDropDownWidth := 300;
  FTimer := TTimer.Create(nil);
  FTimer.Enabled := false;
  FTimer.Interval := 800;
  FTimer.OnTimer := OnTimer;
  FIdentifers := TStringList.Create;
  FTemplates := TStringList.Create;
  FItems := TStringList.Create;
  FDefMode := cmIdentifers;
  FCaretChar := '|';
  FCRLF := '/n';
  FSeparator := '=';
end;

destructor TCompletion.Destroy;
begin
  inherited Destroy;
  FPopupList.Free;
  FIdentifers.Free;
  FTemplates.Free;
  FItems.Free;
end;

function TCompletion.GetItems : TStrings;
begin
  case FMode of
    cmIdentifers : Result := FIdentifers;
    cmTemplates  : Result := FTemplates;
    else Result := nil;
  end;
end;

{      NewString}
procedure TCompletion.ReplaceWord(const NewString : string);
var
  S, S1, W : string;
  P, X, Y : integer;
  iBeg, iEnd : integer;
  NewCaret : integer;
begin
  with FRAEditor do
  begin
    PaintCaret(false);
    BeginUpdate;
    if FCaretX > Length(FLines[FCaretY]) then
      FLines.Internal[FCaretY] := FLines[FCaretY] +
       Spaces(FCaretX - Length(FLines[FCaretY]));
    S := FLines.Text;
    P := PosFromCaret(FCaretX, FCaretY);
    W := Trim(GetWordOnPosEx(S, P, iBeg, iEnd));
    if W = '' then begin iBeg := P+1; iEnd := P end;
    case FMode of
      cmIdentifers :
        begin
          S1 := NewString;
          NewCaret := Length(NewString);
        end;
      cmTemplates  :
        begin
          S1 := ReplaceSokr1(NewString, FCRLF, #13#10 + Spaces(FCaretX-Length(W)));
          S1 := ReplaceSokr1(S1, FCaretChar, '');
          NewCaret := Pos(FCaretChar, NewString) - 1;
          if NewCaret = -1 then NewCaret := Length(NewString);
          //..
        end
      else raise ERAEditorError.Create('Invalid RAEditor Completion Mode');
    end;
   {$IFDEF RAEDITOR_UNDO}
    TReplaceUndo.Create(FRAEditor, FCaretX, FCaretY, iBeg, iEnd, W, S1);
   {$ENDIF RAEDITOR_UNDO}
  //  LW := Length(W);
    if FSelected then
    begin
      if (FSelBegY <= FCaretY) or (FCaretY >= FSelEndY) then
        //  LW ..
    end;
    Delete(S, iBeg, iEnd - iBeg);
    Insert(S1, S, iBeg);
    FLines.Text := S; {!!!   }
    CaretFromPos(iBeg + NewCaret - 1, X, Y);
    SetCaretInternal(X, Y);
    Changed;
    EndUpdate;
    PaintCaret(true);
  end;
end;

procedure TCompletion.DoKeyPress(Key : Char);
begin
  if FVisible then
    if HasChar(Key, RAEditorCompletionChars) then SelectItem else CloseUp(true)
  else
    if FEnabled then FTimer.Enabled := true;
end;

function TCompletion.DoKeyDown(Key: Word; Shift: TShiftState) : boolean;
begin
  Result := true;
  case Key of
    VK_ESCAPE : CloseUp(false);
    VK_RETURN : CloseUp(true);
    VK_UP, VK_DOWN, VK_PRIOR, VK_NEXT :
      FPopupList.Perform(WM_KEYDOWN, Key, 0);
    else Result := false;
  end;
end;

procedure TCompletion.DoCompletion(const AMode : TCompletionList);
var
  Eq : boolean;
  Cancel : boolean;
begin
  if FRAEditor.FReadOnly then exit;
  FMode := AMode;
  case FMode of
    cmIdentifers : DropDown(AMode, true);
    cmTemplates  :
      begin
        Cancel := false;
        FRAEditor.DoCompletionIdentifer(Cancel);
        if Cancel or (FTemplates.Count = 0) then exit;
        MakeItems;
        FindSelItem(Eq);
        if Eq then
          ReplaceWord(SubStr(FItems[ItemIndex], 2, FSeparator))
        else
          DropDown(AMode, true);
      end;
  end;
end;

procedure TCompletion.DropDown(const AMode : TCompletionList; const ShowAlways : boolean);
var
  ItemCount : Integer;
  P : TPoint;
  Y : Integer;
  PopupWidth, PopupHeight : Integer;
  SysBorderWidth, SysBorderHeight : Integer;
  R : TRect;
  Cancel : boolean;
  Eq : boolean;
begin
  CloseUp(false);
  FMode := AMode;
  with FRAEditor do
  begin
    Cancel := false;
    case FMode of
      cmIdentifers : FRAEditor.DoCompletionIdentifer(Cancel);
      cmTemplates  : FRAEditor.DoCompletionTemplate(Cancel)
    end;
    MakeItems;
    FindSelItem(Eq);
   // Cancel := not Visible and (ItemIndex = -1);
    if Cancel or (FItems.Count = 0) or (((ItemIndex = -1) or Eq) and not ShowAlways) then exit;
    FPopupList.Items := FItems;
    FPopupList.ItemHeight := FItemHeight;
    FVisible := true;
    SetItemIndex(FItemIndex);
    if FListBoxStyle in [lbStandard] then
      FPopupList.Style := lbOwnerDrawFixed else
      FPopupList.Style := FListBoxStyle;
    FPopupList.OnMeasureItem := FRAEditor.FOnCompletionMeasureItem;
    FPopupList.OnDrawItem := FRAEditor.FOnCompletionDrawItem;

    ItemCount := FItems.Count;
    SysBorderWidth := GetSystemMetrics(SM_CXBORDER);
    SysBorderHeight := GetSystemMetrics(SM_CYBORDER);
    R := CalcCellRect(FCaretX - FLeftCol, FCaretY - FTopRow + 1);
    P := R.TopLeft;
    P.X := ClientOrigin.X + P.X;
    P.Y := ClientOrigin.Y + P.Y;
    Dec(P.X, 2 * SysBorderWidth);
    Dec(P.Y, SysBorderHeight);
    if ItemCount > FDropDownCount then ItemCount := FDropDownCount;
    PopupHeight := ItemHeight * ItemCount + 2;
    Y := P.Y;
    if (Y + PopupHeight) > Screen.Height then
    begin
      Y := P.Y - PopupHeight - FCellRect.Height + 1;
      if Y < 0 then Y := P.Y;
    end;
    PopupWidth := FDropDownWidth;
    if PopupWidth = 0 then PopupWidth := Width + 2 * SysBorderWidth;
  end;
  FPopupList.Left := P.X;
  FPopupList.Top := Y;
  FPopupList.Width := PopupWidth;
  FPopupList.Height := PopupHeight;
  SetWindowPos(FPopupList.Handle, HWND_TOP, P.X, Y, 0, 0,
    SWP_NOSIZE or SWP_NOACTIVATE or SWP_SHOWWINDOW);
  FPopupList.Visible := true;
end;

procedure TCompletion.MakeItems;
var
  i : integer;
  S : string;
begin
  FItems.Clear;
  case FMode of
    cmIdentifers :
      for i := 0 to FIdentifers.Count -1 do
        FItems.Add(FIdentifers[i]);
    cmTemplates  :
      begin
        with FRAEditor do S := GetWordOnPos(FLines[CaretY], CaretX);
        for i := 0 to FTemplates.Count -1 do
          if ANSIStrLIComp(PChar(FTemplates[i]), PChar(S), Length(S)) = 0 then
            FItems.Add(FTemplates[i]);
        if FItems.Count = 0 then FItems.Assign(FTemplates);
      end;
  end;
end;

procedure TCompletion.FindSelItem(var Eq : boolean);

  function FindFirst(Ss : TSTrings; S : string) : integer;
  var
    i : integer;
  begin
    for i := 0 to Ss.Count - 1 do
      if ANSIStrLIComp(PChar(Ss[i]), PChar(S), Length(S)) = 0 then
      begin
        Result := i;
        exit;
      end;
    Result := -1;
  end;

var
  S : string;
begin
  with FRAEditor do S := GetWordOnPos(FLines[CaretY], CaretX);
  if Trim(S) = '' then
    ItemIndex := -1 else
    ItemIndex := FindFirst(FItems, S);
  Eq := (ItemIndex > -1) and Cmp(Trim(SubStr(FItems[ItemIndex], 0, FSeparator)), S);
end;

procedure TCompletion.SelectItem;
var
  Cancel : boolean;
  Param : boolean;
begin
  FindSelItem(Param);
  Cancel := not Visible and (ItemIndex = -1);
  case FMode of
    cmIdentifers : FRAEditor.DoCompletionIdentifer(Cancel);
    cmTemplates  : FRAEditor.DoCompletionTemplate(Cancel);
  end;
  if Cancel or (GetItems.Count = 0) then CloseUp(false);
end;

procedure TCompletion.CloseUp(const Apply : boolean);
begin
  FItemIndex := ItemIndex;
  FPopupList.Visible := false;
//  (FPopupList as TPopupList). HintWindow.ReleaseHandle;
  FVisible := false;
  FTimer.Enabled := false;
  if Apply and (ItemIndex > -1) then
    case FMode of
      cmIdentifers : ReplaceWord(SubStr(FItems[ItemIndex], 0, FSeparator));
      cmTemplates  : ReplaceWord(SubStr(FItems[ItemIndex], 2, FSeparator));
    end;
end;

procedure TCompletion.OnTimer(Sender : TObject);
begin
  DropDown(FDefMode, false);
end;

procedure TCompletion.SetStrings(index : integer; AValue : TStrings);
begin
  case index of
    0 : FIdentifers.Assign(AValue);
    1 : FTemplates.Assign(AValue);
  end;
end;

function TCompletion.GetItemIndex : integer;
begin
  Result := FItemIndex;
  if FVisible then
    Result := FPopupList.ItemIndex;
end;

procedure TCompletion.SetItemIndex(AValue : integer);
begin
  FItemIndex := AValue;
  if FVisible then
    FPopupList.ItemIndex := FItemIndex;
end;

function TCompletion.GetInterval : cardinal;
begin
  Result := FTimer.Interval;
end;

procedure TCompletion.SetInterval(AValue : cardinal);
begin
  FTimer.Interval := AValue;
end;



{**************  TPopupList  *************}
constructor TPopupList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Left := -1000;
  Visible := false;
  TabStop := False;
  ParentFont := false;
  Parent := Owner as TRACustomEditor;
  Ctl3D := false;
  FTimer := TTimer.Create(nil);
  FTimer.Enabled := false;
  FTimer.Interval := 200;
  FTimer.OnTimer := OnTimer;
  Style := lbOwnerDrawFixed;
  ItemHeight := 13;
//  HintWindow := THintWindow.Create(Self);
end;

destructor TPopupList.Destroy;
begin
  FTimer.Free;
//  HintWindow.Free;
  inherited Destroy;
end;

procedure TPopupList.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := Style {or WS_POPUP} or WS_BORDER;
    ExStyle := ExStyle or WS_EX_TOOLWINDOW;
    WindowClass.Style := WindowClass.Style or CS_SAVEBITS;
  end;
end;

procedure TPopupList.CreateWnd;
begin
  inherited CreateWnd;
  if not (csDesigning in ComponentState) then
    Windows.SetParent(Handle, 0);
//  CallWindowProc(DefWndProc, Handle, WM_SETFOCUS, 0, 0); {??}
end;

procedure TPopupList.DestroyWnd;
begin
  inherited DestroyWnd;
//  HintWindow.ReleaseHandle;
end;

procedure TPopupList.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  F : integer;
begin
  YY := Y;
  F := ItemAtPos(Point(X, Y), true);
  if KeyPressed(VK_LBUTTON) then
  begin
    F := ItemAtPos(Point(X, Y), true);
    if F > -1 then ItemIndex := F;
    FTimer.Enabled := (Y < 0) or (Y > ClientHeight);
    if (Y < - ItemHeight)  or (Y > ClientHeight + ItemHeight) then
      FTimer.Interval := 50 else
      FTimer.Interval := 200;
  end;
  if (F > -1) and not FTimer.Enabled then
  begin
    //Application.CancelHint;
   // Hint := Items[F];
  //  HintWindow.ActivateHint(Bounds(ClientOrigin.X + X, ClientOrigin.Y + Y, 300, ItemHeight), Items[F]);
  end;
end;

procedure TPopupList.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  F : integer;
begin
  MouseCapture := true;
  F := ItemAtPos(Point(X, Y), true);
  if F > -1 then ItemIndex := F;
end;

procedure TPopupList.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseCapture := false;
  (Owner as TRACustomEditor).FCompletion.CloseUp(
    (Button = mbLeft) and PtInRect(ClientRect, Point(X, Y)));
end;

procedure TPopupList.OnTimer(Sender : TObject);
begin
  if (YY < 0) then
    Perform(WM_VSCROLL, SB_LINEUP, 0)
  else if (YY > ClientHeight) then
    Perform(WM_VSCROLL, SB_LINEDOWN, 0);
end;

procedure TPopupList.WMCancelMode(var Message: TMessage);
begin
  (Owner as TRACustomEditor).FCompletion.CloseUp(false);
end;

procedure TPopupList.CMHintShow(var Message: TMessage);
begin
  Message.Result := 1;
end;

procedure TPopupList.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Offset, W : Integer;
  S : string;
begin
  if Assigned(OnDrawItem) then OnDrawItem(Self, Index, Rect, State) else
  begin
    Canvas.FillRect(Rect);
    Offset := 3;
    with (Owner as TRACustomEditor).FCompletion do
      case FMode of
        cmIdentifers :
          Canvas.TextOut(Rect.Left + Offset, Rect.Top, SubStr(Items[Index], 1, Separator));
        cmTemplates :
          begin
            Canvas.TextOut(Rect.Left + Offset, Rect.Top, SubStr(Items[Index], 1, Separator));
            Canvas.Font.Style := [fsBold];
            S := SubStr(Items[Index], 0, Separator);
            W := Canvas.TextWidth(S);
            Canvas.TextOut(Rect.Right - 2*Offset - W, Rect.Top, S);
          end;
      end;
  end;  
end;

{$ENDIF RAEDITOR_COMPLETION}

{$ENDIF RAEDITOR_EDITOR}




end.
