{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAICalc;

{ expression evaluator. very slow :-[ }

interface

uses SysUtils, Classes,
  RAIIntf, RAIClas, RAConst;

type

  TLexem = class(TIObject)
  private
    IVar : TIVar;
    Pos : integer;
    OpCode : TOpCode;
    constructor Create;
  public
    destructor Destroy;override;
  end;

  TLexems = class(TStringList)
  private
    function GetIVar(index : integer) : TIVar;
    function GetLexem(index : integer) : TLexem;
    function Pos(index : integer) : integer;
    property IVars [index : integer] : TIVar read GetIVar;
    property Lexem [index : integer] : TLexem read GetLexem;
  public
    destructor Destroy; override;
    function AddObject(const S: string; AObject: TObject): Integer; override;
    procedure Delete(Index: Integer); override;
    procedure Clear; override;
  end;

  TPriorLevel = 1..9;

  TGetLocalVar = function(AName : TIdentifer) : IIVar of object;

  TCalc = class
  protected
    SetGetFunc : TSetGetFunc;
    GetLocalVar : TGetLocalVar;
    Lexems : TLexems;

    function SetGetValue(IObj : IIObject; Name : TIdentifer; Args : IIArgs; Res : IIValue; Get : boolean) : boolean;
    procedure Validate;
    procedure DoSolve;
    function Identifer(ai : integer) : boolean;
    procedure BinaryOperation(ai : integer; OpCode : TOpCode; Left, Right, Res : IIValue); // dynamic;
  public
    Res  : TIValue;
    Checking : boolean;
    SGIDebugFlags : TSGIDebugFlags; { }
    constructor Create;
    destructor Destroy;override;
    procedure Clear;
    function SetExp(Prog : TStrings; iBeg, iEnd : integer) : boolean;
    procedure Solve;
    property OnSetGetFunc : TSetGetFunc write SetGetFunc;
    property OnGetLocalVar : TGetLocalVar write GetLocalVar;
  end;{TCalc}

implementation

const PROBELS = 5;{x   Lexems}

constructor TLexem.Create;
begin
  inherited Create;
  IVar := TIVar.Create0;
end;

destructor TLexem.Destroy;
begin
  IVar._Release;
  inherited Destroy;
end;

destructor TLexems.Destroy;
begin
  Clear;
  inherited Destroy;
end;

function TLexems.AddObject(const S: string; AObject: TObject): Integer;
begin
  if AObject = nil then
    AObject := TLexem.Create;
  Result := inherited AddObject(S, AObject);
end;

function TLexems.GetIVar(index : integer) : TIVar;
begin
  if Objects[index] = nil then
    Objects[index] := TLexem.Create;
  Result := (Objects[index] as TLexem).IVar;
end;

function TLexems.GetLexem(index : integer) : TLexem;
begin
  if Objects[index] = nil then
    Objects[index] := TLexem.Create;
  Result := Objects[index] as TLexem;
end;

procedure TLexems.Clear;
var
  i : integer;
begin
  for i := 0 to Count-1 do
    if Objects[i] <> nil then
      (Objects[i] as TLexem)._Release;
  inherited Clear;
end;

procedure TLexems.Delete(Index: Integer);
begin
  if Objects[index] <> nil then
    (Objects[index] as TLexem)._Release;
  inherited Delete(Index);
end;

function TLexems.Pos(index : integer) : integer;
begin
  if (Objects[index] <> nil) and (Objects[index] is TLexem) then
    Result := (Objects[index] as TLexem).Pos
  else Result := 0;
end;
{############################ TLexems ################################}


{**************************** TCalc ****************************}
constructor TCalc.Create;
begin
  inherited Create;
  Lexems := TLexems.Create;
  Res := TIVar.Create0;
end;

destructor TCalc.Destroy;
begin
  Lexems.Free;
  Res._Release;
  inherited Destroy;
end;

procedure TCalc.Clear;
begin
  Lexems.Clear;
end;

function TCalc.SetExp(Prog : TStrings; iBeg, iEnd : integer) : boolean;
var
  Lex : string;
  i : integer;
begin
  Lexems.Clear;
  if iEnd = -1 then iEnd := Prog.Count-1;
  if (iBeg <= iEnd) and (iEnd <= Prog.Count-1) then begin
    i := iBeg;
    while i <= iEnd do begin
      Lex := Prog[i];
      if (i > iBeg) and (Lex[1] in ['>', '=']) and
         (( (Lexems[i-1] = '<') and (Lex = '>') ) or
          ( (Lexems[i-1] = '<') and (Lex = '=') ) or
          ( (Lexems[i-1] = '>') and (Lex = '=') ) or
          ( (Lexems[i-1] = '=') and (Lex = '=') ) or
          ( (Lexems[i-1] = ':') and (Lex = '=') ))
      then begin
        Lexems[i-1] := Lexems[i-1] + Lex;
        Lexems.Lexem[i-1].OpCode := Lang.Znak2OpCode(Lexems[i-1]);
      end else
        with Lexems.Lexem[Lexems.Add(Lex)] do begin
          IVar.Str := iString(Lex);
          Pos := i - iBeg + 2;  // (2)=--------|
          OpCode := Lang.Znak2OpCode(Lex);  // |
        end;                                // |
      inc(i);                               // |
    end;                                    // |
    Lexems.Insert(0, '(');  // <--|------------|
    Lexems.Insert(0, ' ');  // <--|
    Lexems.AddObject(')', nil);
    Lexems.AddObject(' ', nil);
    Lexems.AddObject(' ', nil);
    Lexems.AddObject(' ', nil);
    Lexems.AddObject(' ', nil);
    Result := true;
  end else
    Result := false;
end;

procedure TCalc.Validate;
var
   i : integer;
   SkobkaLeft, SkobkaRight : integer;
   SkobkaSqLeft, SkobkaSqRight : integer;
   Lexi, Lexi1 : string;
   IVi, IVi1 : TLexem;
begin
  SkobkaLeft   := 0; SkobkaRight   := 0;
  SkobkaSqLeft := 0; SkobkaSqRight := 0;
 {     }
  for i := 1 to Lexems.Count - PROBELS do begin
    Lexi  := Lexems[i];
    Lexi1 := Lexems[i+1];
    IVi  := Lexems.Lexem[i];
    IVi1 := Lexems.Lexem[i+1];
    if Lang.Statement(Lexi) then
      InterErrorO(ieSeparatorExpected, i)
    else  
    if not( Lang.IsIdentifer(Lexi)
            or Lang.IsOpCode(IVi.OpCode)
            or (Lexi = '^')
            or Lang.IsConstant(Lexi)
            or ( (Length(Lexi) = 1) and (Lexi[1] in RAISeparators) )
          )
    then if (i > 2) and (Lexi = ':=')
         then InterErrorOB(ieIdentiferExpected, i-2, false)
         else InterErrorOV(ieUnknownLexem, i, Lexems[i])
    else
    if ( Lang.IsOpCode(IVi.OpCode) and Lang.IsOpCode(IVi1.OpCode) )
       and not
       ( Lang.IsBinaryOpCode(IVi.OpCode) and
         Lang.IsUnaryOpCode(IVi1.OpCode) )
    then InterErrorO(ieTwoZnaks, i+1)
    else if (Lang.IsConstant(Lexi)  or Lang.IsCalcIdentifer(Lexi)) and
            (Lang.IsConstant(Lexi1) or Lang.IsCalcIdentifer(Lexi1))
    then InterErrorOB(ieSeparatorExpected, i, false)
   {   }
    else if Lexems[i] = '(' then inc(SkobkaLeft)
    else if Lexems[i] = ')' then inc(SkobkaRight)
    else if Lexems[i] = '[' then inc(SkobkaSqLeft)
    else if Lexems[i] = ']' then inc(SkobkaSqRight)
  end;
 {   }
  if SkobkaLeft < SkobkaRight then InterErrorS(ieLeftBrices)
  else if SkobkaLeft > SkobkaRight then InterErrorOB(ieRightBrices, Lexems.Count - PROBELS, false)
  else if SkobkaSqLeft < SkobkaSqRight then InterErrorS(ieLeftBricesSqr)
  else if SkobkaSqLeft > SkobkaSqRight then  InterErrorS(ieRightBrices)
  else if Lang.IsOpCode(Lexems.Lexem[Lexems.Count-PROBELS-1].OpCode)
  then InterErrorOB(ieIdOrConstExpected, Lexems.Count-PROBELS-1, false)
  else
 {   }
  for i := 1 to Lexems.Count - PROBELS -1 do begin
    Lexi := Lexems[i];
    if Lang.IsOpCode(Lexems.Lexem[i].OpCode) or Lang.IsConstant(Lexi)
       or ( (Length(Lexi) = 1) and not (Lexi[1] in RAIIdSymbols) )
    then continue
    else if Lang.IsIdentifer(Lexi) then continue
    else InterErrorOV(ieUnknownLexem, i, Lexi);
  end;{for}
end;

{ Prior   }
function Prior(Op : TOpCode) : TPriorLevel;
begin
  case Op of
    ocNot, ocPtr          : Result := 8;
    ocProcent             : Result := 7;
    ocMul, ocDiv          : Result := 6;
    ocAnd                 : Result := 5;
    ocPlus, ocMinus, ocOr : Result := 4;
    ocEqu, ocMore,
    ocSmall, ocNotEqu     : Result := 3;
    ocEquSmall, ocEquMore : Result := 2;
    ocLet                 : Result := 1;
    else Result := 9; {  }
  end;
end;

procedure TCalc.BinaryOperation(ai : integer; OpCode : TOpCode; Left, Right, Res : IIValue);
var
  rLeft, rRight : Double;
  iLeft, iRight : integer;
  bLeft, bRight : boolean;
begin
 {  }
  if (Left.Typ = ivString) and (Right.Typ = ivString) then
    case OpCode of
      ocPlus : Res.vString := iString(string(Left.vString) + string(Right.vString));
      ocEqu  : Res.vBoolean := StrComp(Left.vString, Right.vString)=0;
      ocEquEqu : Res.vBoolean := Cmp(Left.vString, Right.vString);
      else InterErrorO(ieBadOpCode, ai) {  };
    end
  else
 {   }
  if (Left.Typ = ivBoolean) and (Right.Typ = ivBoolean) then begin
    bLeft := Left.vBoolean;
    bRight := Right.vBoolean;
    case OpCode of
      ocOr      : Res.vBoolean := bLeft  or bRight;
      ocAnd     : Res.vBoolean := bLeft and bRight;
      ocEqu     : Res.vBoolean := bLeft  =  bRight;
      ocMore    : Res.vBoolean := bLeft  >  bRight;
      ocSmall   : Res.vBoolean := bLeft  <  bRight;
      ocNotEqu  : Res.vBoolean := bLeft  <> bRight;
      ocEquSmall: Res.vBoolean := bLeft  <= bRight;
      ocEquMore : Res.vBoolean := bLeft  >= bRight;
      else InterErrorO(ieBadOpCode, ai) {  };
    end;
  end else
 {   }
  if IIValueIsInteger(Left) and IIValueIsInteger(Right) then begin
    iLeft := Left.vInteger;
    iRight := Right.vInteger;
    case OpCode of
      ocPlus    : Res.vInteger := iLeft + iRight;
      ocMinus   : Res.vInteger := iLeft - iRight;
      ocMul     : Res.vInteger := iLeft * iRight;
      ocDiv     : if iRight = 0 then InterErrorS(ieDivisionByZero)
                  else Res.vReal := iLeft / iRight;
      ocIntDiv  : if iRight = 0 then InterErrorS(ieDivisionByZero)
                  else Res.vInteger := iLeft div iRight;
      ocMod     : Res.vInteger := iLeft mod iRight;
      ocOr      : Res.vInteger := iLeft or iRight;
      ocAnd     : Res.vInteger := iLeft and iRight;
      ocProcent : Res.vReal := iLeft * iRight / 100;
      ocEqu     : Res.vBoolean := iLeft =  iRight;
      ocMore    : Res.vBoolean := iLeft >  iRight;
      ocSmall   : Res.vBoolean := iLeft <  iRight;
      ocNotEqu  : Res.vBoolean := iLeft <> iRight;
      ocEquSmall: Res.vBoolean := iLeft <= iRight;
      ocEquMore : Res.vBoolean := iLeft >= iRight;
      else InterErrorO(ieBadOpCode, ai) {  };
    end;{case}
  end else
 {     }
  if IIValueIsReal(Left) and IIValueIsReal(Right) then begin
    rLeft := Left.vReal;
    rRight := Right.vReal;
    case OpCode of
      ocPlus    : Res.vReal := rLeft + rRight;
      ocMinus   : Res.vReal := rLeft - rRight;
      ocMul     : Res.vReal := rLeft * rRight;
      ocDiv     : if rRight <> 0 then
                  Res.vReal := rLeft / rRight
                  else InterErrorS(ieDivisionByZero);
      ocProcent : Res.vReal := rLeft * rRight / 100;
      ocEqu     : Res.vBoolean := rLeft = rRight;
      ocMore    : Res.vBoolean := rLeft > rRight;
      ocSmall   : Res.vBoolean := rLeft < rRight;
      ocNotEqu  : Res.vBoolean := rLeft <> rRight;
      ocEquSmall: Res.vBoolean := rLeft <= rRight;
      ocEquMore : Res.vBoolean := rLeft >= rRight;
      else InterErrorO(ieBadOpCode, ai) {  };
    end;{case}
  end
  else InterErrorOB(ieTypeMistmach, ai, false)
end;

{************************* ,  **************************}
function TCalc.Identifer(ai : integer) : boolean;
{      }
var
  Lex : string;
  IArgs : TIArgs;
  i : integer;
  R, Go, DelObj : boolean;
  ParamsPos : TList;
  Obj : IIObject;
  IV : IIVar;
  procedure AddParam(IVar : TIVar);
  begin
    IArgs.Add(IVar);
    IVar._Release;
    ParamsPos.Add(pointer(i));
  end;
begin
  Result := false;
  Lex := Lexems.IVars[ai].vIdentifer;
  Go := true;
  if (Lexems.IVars[ai].Typ = ivIdentifer)
     or Lang.Cmp(Lex, kwResult) or Lang.Cmp(Lex, kwArgs)
  then begin
    pointer(IV) := nil;
    if Assigned(GetLocalVar) then
      IV := GetLocalVar(Lex);
    if IV <> nil then begin
      IIValueAssign(Lexems.Lexem[ai].IVar, IV);
     {$IFDEF RAINTER_DEBUG} Lexems[ai] := Lexems.Lexem[ai].IVar.Str; {$ELSE} Lexems[ai] := ''; {$ENDIF}
      Result := true;
    end else if SGIDebugFlags <> dfHint then begin
      IArgs := TIArgs.Create;
      if (Lexems[ai+1] = '.') then IArgs.ResTyp := ivAny
                              else IArgs.ResTyp := ivEmpty; {. }
      {$IFDEF RAINTER_DEBUG}
       IArgs.Debug := IArgs.Debug + ' Params To '+Lex;
      {$ENDIF}
      ParamsPos := TList.Create;
      try
        i := ai+2;
        DelObj := false;
        if Lexems[ai-1] = '.' then begin
          if (ai-3 >= 0) and (Lexems.Lexem[ai-3].OpCode = ocLet)
          then IArgs.ResTyp := ivAny;
          if Lexems.IVars[ai-2].Typ = ivClass then begin
            Lexems.IVars[ai-2]._AddRef;
            AddParam(Lexems.IVars[ai-2]);
            DelObj := true;
          end else if Lexems.IVars[ai-2].Typ <> ivObject then
            InterErrorOV(ieSyntax, ai, Lex)
        end;
        if Go and (Lexems[ai+1] = '(') then begin
         {  }
          while (i < Lexems.Count) do begin
            if Lexems[i] = ')' then break;
            if Lexems[i] = ',' then
              if Lexems[i-1] = ',' then
                AddParam(TIVar.Create0)
              else
            else
              case Lexems.IVars[i].Typ of
                ivInteger..ivObject : begin
                  Lexems.IVars[i]._AddRef;
                  AddParam(Lexems.IVars[i]);
                end;
                else begin Go := false; break end;
              end;
            inc(i);
          end;
        end;
        if Go then begin
          if (Lexems[ai-1] = '.') and (Lexems.IVars[ai-2].Typ = ivObject)
          then begin Obj := Lexems.IVars[ai-2].vObject; DelObj := true end
          else Obj := nil;
          Lexems.IVars[ai].Name := Lex;
          try
            R := SetGetValue(Obj, TIdentifer(Lex), IArgs, Lexems.IVars[ai], true);
          except
            on E : EInterError do begin
              if E.Pos = -1 then
              case E.ErrCode of
                ieParams, ieParam, ieTooManyParams,
                ieParamTypMistmach : if E.Op >-1 then
                  if E.Op > Lexems.Count-1 then E.Op := integer(ParamsPos[Lexems.Count-1]) else E.Op := integer(ParamsPos[E.Op]);
                ieNotEnoughParams  : begin E.Op := i; E.PosIsBeg := false end;
              end;
              raise
            end;
            on E : Exception do raise EInterError.CreateOLV(ieExternalError, E.Message, 0, 0, true, '');
          end;
          if R then begin
            {$IFDEF RAINTER_DEBUG} Lexems[ai] := Lexems.IVars[ai].Str; {$ELSE} Lexems[ai] := ''; {$ENDIF}
           {  }
            if Lexems[ai+1] = '(' then begin
              i := ai+1;
              while (i < Lexems.Count-1) and (Lexems[i] <> ')') do
                Lexems.Delete(i);
              Lexems.Delete(i);
            end;
            if DelObj then begin
              Lexems.Delete(ai-1);
              Lexems.Delete(ai-2);
            end;
            Result := true;
          end
          else InterErrorOV(ieUnknownIdentifer, ai, Lex)
        end;
      finally
        IArgs._Release;
        ParamsPos.Free;
      end;
    end;
  end;
end;
{######################### ,  ##########################}

procedure TCalc.DoSolve;
var
  Ras : integer;{    }
  i : integer;
  LexemsCount : integer;
  LexFirst, LexLast : integer;
  LexZ, LexL, LexR : string[255];
  Solv : boolean;
  aL, aZ, aR : integer;
  ai  : integer absolute aL;
  IVL, IVZ, IVR : TLexem;
  IV : TIVar;
begin
  Ras := Lexems.Count + 10{ };
  repeat
    dec(Ras);
    if Ras <= 0 then InterErrorS(ieCikl);//!!!
    Solv := false;
    LexemsCount := Lexems.Count;
    LexFirst := 0;
    LexLast  := LexemsCount-1-PROBELS;
    for i := LexFirst+1 to LexLast do begin
      Solv := true;
      aL := i-1; aZ := i; aR := i+1;
      LexZ := Lexems[aZ]; IVZ := Lexems.Lexem[aZ];
      LexL := Lexems[aL]; IVL := Lexems.Lexem[aL];
      LexR := Lexems[aR]; IVR := Lexems.Lexem[aR];
     {  }
      if (IVR.OpCode = ocPtr) and (IVZ.IVar.Typ <> ivIdentifer) then
        if IVZ.IVar.Typ = ivPointer then begin
          if IVZ.IVar.vPointer <> nil then
            if HasIntf(IVZ.IVar.vPointer, IIVarObj) then begin
              IV := IIVarObj(GetIntf(IVZ.IVar.vPointer, IIVar)).IVar; IV._AddRef;
              Lexems.Lexem[aZ].IVar._Release;
              Lexems.Lexem[aZ].IVar := IV;
            end else IIValueAssign(IVZ.IVar, IVZ.IVar.vPointer)
          else IVZ.IVar.Empty;
          {$IFDEF RAINTER_DEBUG} Lexems[aZ] := IVZ.IVar.Str; {$ELSE} Lexems[aZ] := ''; {$ENDIF}
          Lexems.Delete(aR);
          break;
        end else InterErrorO(iePtrNotLocal, ai)
      else
     {   }
      if (IVL.OpCode = ocAt) and Assigned(GetLocalVar) then
        if Lang.IsValidIdentifer(LexZ) then begin
          IVZ.IVar.vPointer := GetLocalVar(LexZ);
          IVZ.IVar.vPointer._Release;
          if IVZ.IVar.vPointer = nil then
            InterErrorO(ieAtNotLocal, ai);
          {$IFDEF RAINTER_DEBUG} Lexems[aZ] := IVZ.IVar.Str; {$ELSE} Lexems[aZ] := ''; {$ENDIF}
          Lexems.Delete(aL);
          break;
        end else InterErrorO(ieAtNotLocal, ai)
      else
     {*******************  }
      if (IVZ.OpCode = ocLet) and  (IVR.IVar.Typ <> ivIdentifer) and
        // (IVR.IVar.IsConstant or (IVR.IVar.Typ = ivPointer)) and
         ( (Prior(ocLet) >= Prior(Lexems.Lexem[aR+1].OpCode)) or
           (Lexems[aR+1] = ')') or (Lexems[aR+1] = ',') )
      then
        if Lang.IsCalcIdentifer(LexL) then
          if (Lexems[aL-1] = '.') and (Lexems.IVars[aL-2].Typ = ivObject) then
            {   }
            if SetGetValue(Lexems.IVars[aL-2].vObject, LexL, nil, IVR.IVar, false) then begin
              Lexems.Delete(aZ);
              Lexems.Delete(aL);
              Lexems.Delete(aL-1);
              Lexems.Delete(aL-2);
              break;
            end else InterErrorOV(ieUnknownIdentifer, aL, LexL)
          else begin
            {  }
            if SetGetValue(nil, LexL, nil, IVR.IVar, false) then begin
              Lexems.Delete(aZ);
              Lexems.Delete(aL);
              break;
            end else InterErrorOV(ieUnknownIdentifer, aL, LexL)
          end
      //else InterErrorO(ieLetNotIdentifer, ai)
        else begin
          IIValueAssign(IVL.IVar, IVR.IVar);
          {$IFDEF RAINTER_DEBUG} Lexems[aL] := IVL.IVar.Str; {$ELSE} Lexems[aL] := ''; {$ENDIF}
          Lexems.Delete(aZ);
          Lexems.Delete(aL);
          break;
        end
     {###################  }
      else
     {'(','-','',')' = '(','-',')'}
      if ( IVZ.OpCode in [ocMinus, ocPlus] ) and
         ( IVR.IVar.Typ in [ivReal, ivInteger]) and
         ( (LexL = '(') or (LexL = ',') {or (IVL.OpCode = ocUnknown)} ) then
      begin
          if LexZ = '-' then begin
            if IVR.IVar.VTyp = ivReal then
              IVZ.IVar.vReal := -IVR.IVar.vReal
            else
              IVZ.IVar.vInteger := -IVR.IVar.vInteger;
           {$IFDEF RAINTER_DEBUG}
            Lexems[aZ] := IVZ.IVar.Str;
           {$ENDIF}
            Lexems[aZ] := IVZ.IVar.Str;
            Lexems.Delete(aR);
          end else
            Lexems.Delete(aZ);
          break;
      end else
      {Not}
      if (IVZ.OpCode = ocNot) and (IVR.IVar.Typ=ivBoolean) then
      begin
        IVZ.IVar.vBoolean := not IVR.IVar.vBoolean;
        {$IFDEF RAINTER_DEBUG}
         Lexems[aZ] := IVZ.IVar.Str;
        {$ENDIF}
        Lexems.Delete(aR);
        break;
      end else
     {    ,   }
      if Lang.IsBinaryOpCode(IVZ.OpCode) and
          IIValueIsConstant(IVL.IVar) and IIValueIsConstant(IVR.IVar) and
         {    black'a}
         ( (Prior(IVZ.OpCode) > Prior(Lexems.Lexem[aL-1].OpCode)) or
           (Lexems[aL-1] = '(') or (Lexems[aL-1] = ',') )
            and
         ( (Prior(IVZ.OpCode) >= Prior(Lexems.Lexem[aR+1].OpCode)) or
           (Lexems[aR+1] = ')') or (Lexems[aR+1] = ','))      then
      begin
        BinaryOperation(aZ, IVZ.OpCode, IVL.IVar, IVR.IVar, IVZ.IVar);
        {$IFDEF RAINTER_DEBUG} Lexems[aZ] := IVZ.IVar.Str; {$ELSE} Lexems[aZ] := ''; {$ENDIF}
        Lexems.Delete(aR);
        Lexems.Delete(aL);
        break;
      end else
      {, }
      if (IVR.OpCode <> ocLet) and (IVZ.OpCode = ocUnknown) and
         Identifer(i) then break
      else
     {remove brices around constant}
      if (LexL = '(') and (Lexems[aR] = ')') and
         not Lang.IsValidIdentifer(Lexems[aL-1])
      then begin
        Lexems.Delete(aR);
        Lexems.Delete(aL);
        break;
      end else Solv := false;
    end;{for}
  until not Solv;
end;{DoSolve}

procedure TCalc.Solve;
var
  EOp : integer;
begin
 { if not (Assigned(SetGetFunc) and Assigned(GetLocalVar)) then
    InterErrorS(ieCalcEventNotAssigned); }
  try
    if Checking then exit; {   ,    }
    try
     // Validate;
      DoSolve;
    except
      on E : EInterError do begin
       // Validate;
        raise;
      end;
    end;
    if Lexems.Count = PROBELS+1 then
      IIValueAssign(Res, Lexems.IVars[1])
    else begin
     // Validate;
      InterErrorS(ieResultNotSolved);
    end;
  except
    on E : EInterError do begin
      if E.Pos = -1 then begin
        EOp := E.Op;
        if E.Op < Lexems.Count then begin
          E.Op := Lexems.Pos(EOp);
          if not E.PosIsBeg then inc(E.Op, Length(Lexems[EOp]));
        end;
        if E.Op <> 0 then E.Op := E.Op-2;
      end;
      raise;
    end;
  end;
end;

function TCalc.SetGetValue(IObj : IIObject; Name : TIdentifer; Args : IIArgs; Res : IIValue; Get : boolean) : boolean;
var
  SGI : TSetGetInfo;
begin
  if Assigned(SetGetFunc) then begin
    pointer(SGI.IObjOut) := pointer(IObj);
    SGI.Name := iString(Name);
    pointer(SGI.Args) := pointer(Args);
    pointer(SGI.Res) := pointer(Res);
    SGI.Get := Get;
    SGI.DebugFlags := SGIDebugFlags;
    Result := SetGetFunc(SGI);
  end else
    Result := false;
end;

end.



