{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAIClas;

{ implementation of RAInter interfaces }

interface

uses Windows, SysUtils, Classes,
  RAIIntf, RAConst;

const
  K = '''';
  RAIConstSymbols   = ['0'..'9', 'a'..'f', 'A'..'F'];
  RAIConstSymbols10 = ['0'..'9'];
  RAISeparators     = ['(', ')', '[', ']', ',', '.', ';'];

type
  TZnak      = string {[3]}; {LongString improve performance in this context}
  TConst     = string [16];
  TIdentifer = string;
  TLangKW    = string;

const
  kwBEGIN     =  'begin'     ;
  kwEND       =  'end'       ;
  kwRESULT    =  'result'    ;
  kwARGS      =  'args'      ;
  kwIF        =  'if'        ;
  kwTHEN      =  'then'      ;
  kwELSE      =  'else'      ;
  kwENDIF     =  'endif'     ;
  kwGOTO      =  'goto'      ;
  kwWHILE     =  'while'     ;
  kwDO        =  'do'        ;
  kwENDWHILE  =  'endwhile'  ;
  kwVAR       =  'var'       ;
  kwTRUE      =  'true'      ;
  kwFALSE     =  'false'     ;
  kwOR        =  'or'        ;
  kwAND       =  'and'       ;
  kwNOT       =  'not'       ;
  kwDIV       =  'div'       ;
  kwMOD       =  'mod'       ;
  kwUNIT      =  'unit'      ;
  kwENDUNIT   =  'endunit'   ;
  kwFUNC      =  'function'  ;
  kwENDFUNC   =  'end'       ;
  kwUSES      =  'uses'      ;
  kwEXTERNAL  =  'external'  ;
  kwCONST     =  'const'     ;
  kwENDCONST  =  'endconst'  ;
  kwCLASS     =  'class'     ;
  kwENDCLASS  =  'end'       ;

  kwReservedLang = ' '+kwBEGIN+' '+kwEND+' '+kwRESULT+' '+kwARGS+' '+kwIF+' '+kwTHEN+' '+kwELSE+' '+
    kwENDIF+' '+kwGOTO+' '+kwWHILE+' '+kwDO+' '+kwENDWHILE+' '+kwVAR+' '+kwTRUE+' '+kwFALSE+' '+
    kwOR+' '+kwAND+' '+kwNOT+' '+kwMOD+' '+kwUNIT+' '+kwENDUNIT+' '+kwFUNC+' '+kwENDFUNC+' '+
    kwUSES+' '+kwEXTERNAL+' '+kwCONST+' '+kwENDCONST+' '+kwCLASS+' '+kwENDCLASS+' ';

  kwReserved = ' '+kwBEGIN+' '+kwEND+' '{+kwRESULT+' '+kwARGS+' '}+kwIF+' '+kwTHEN+' '+kwELSE+' '+
    kwENDIF+' '+kwGOTO+' '+kwWHILE+' '+kwDO+' '+kwENDWHILE+' '+kwVAR+' '+kwTRUE+' '+kwFALSE+' '+
    kwOR+' '+kwAND+' '+kwNOT+' '+kwMOD+' '+kwUNIT+' '+kwENDUNIT+' '+kwFUNC+' '+kwENDFUNC+' '+
    kwUSES+' '+kwEXTERNAL+' '+kwCONST+' '+kwENDCONST+' '+kwCLASS+' '+kwENDCLASS+' ';

type

  TOpCode = (ocUnknown, ocPlus, ocMinus, ocMul, ocDiv, ocProcent, ocPtr, ocAt,
             ocEqu, ocNotEqu, ocEquEqu, ocMore, ocSmall, ocEquMore, ocEquSmall,
             ocOr, ocAnd, ocNot, ocIntDiv, ocMod, ocLet);

  TSetGetFunc = function(var SGI : TSetGetInfo) : boolean of object;

  TIObject = class(TObject, IUnknown, IIObject)
  {*** interface}
  private
    FRefCount: Integer;
  protected
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
  public
    function _AddRef: integer; virtual; stdcall;
    function _Release: integer; virtual; stdcall;
    function SetGetValue(var SGI : TSetGetInfo) : boolean;
    procedure GetIObjectData(var IObjectData : TIObjectData);
  {### interface}
  private
    PSGI : PSetGetInfo;
  protected
    function SetGetVar(var SGI : TSetGetInfo) : boolean; dynamic;
  public
   {$IFDEF RAINTER_DEBUG}
    Debug : string;
   {$ENDIF}
    constructor Create;
   {$IFDEF RAINTER_DEBUG}
    destructor Destroy; override;
   {$ENDIF}
    function NameIs(Name : TIdentifer) : boolean;
    function ProcIs(Name : TIdentifer; Count : integer; Params : array of TIValueTyp) : boolean;
    function ClassIs(Name : TIdentifer) : boolean;
  end;

  TIValueRec = record
    {ivIdentifer, ivClass, ivString, ivFormula, }
    vString  : string;
    vObject  : IIObject;
    vPointer : IIValue ;
    case Typ : TIValueTyp of
      ivInteger : (vInteger : integer );
      ivReal    : (vReal    : double  );
      ivBoolean : (vBoolean : boolean );
  end;

  TIValue = class(TIObject, IUnknown, IIObject, IIValue)
  private
    Vr : TIValueRec;
    sResult : string;
   {***  property}
    procedure SetStr(S : iString);
    function GetStr : iString;
    function GetAsString : iString;
    function GetTyp : TIValueTyp;
    procedure SetTyp(ATyp : TIValueTyp);
    procedure SetvString   (vString    : iString);
    procedure SetvClass    (vClass     : iString);
    procedure SetvFormula  (vFormula   : iString);
    procedure SetvIdentifer(vIdentifer : iString);
    procedure SetvInteger(vInteger : integer );
    procedure SetvReal   (vReal    : double  );
    procedure SetvBoolean(vBoolean : boolean );
    procedure SetvObject (vObject  : IIObject);
    procedure SetvPointer(vPointer : IIValue );
    function IIValue.GetvIdentifer = GetvString;
    function IIValue.GetvFormula   = GetvString;
    function IIValue.GetvClass     = GetvString;
    function GetvString    : iString;
    function GetvInteger : integer ;
    function GetvReal    : double  ;
    function GetvBoolean : boolean ;
    function GetvObject  : IIObject;
    function GetvPointer : IIValue ;
   {###  property}
  public
    procedure SetVal(Typ : TIValueTyp; const AValue);
    constructor Create(Typ : TIValueTyp; const AValue);
    constructor Create0;
    destructor Destroy; override;
    procedure Empty;

    property Typ : TIValueTyp read GetTyp write SetTyp;
    property Str : iString read GetStr write SetStr;
    property AsString : iString read GetAsString write SetStr;

    property vString    : iString  read GetvString  write SetvString   ;
    property vIdentifer : iString  read GetvString  write SetvIdentifer;
    property vFormula   : iString  read GetvString  write SetvFormula  ;
    property vClass     : iString  read GetvString  write SetvClass    ;
    property vInteger   : integer  read GetvInteger write SetvInteger;
    property vReal      : double   read GetvReal    write SetvReal   ;
    property vBoolean   : boolean  read GetvBoolean write SetvBoolean;
    property vObject    : IIObject read GetvObject  write SetvObject ;
    property vPointer   : IIValue  read GetvPointer write SetvPointer;

  end;

  TIVar = class;
  IIVarObj = interface
    ['{8C6BA270-08FC-11d1-BA5A-F44E9C5EF10B}']
    function IVar : TIVar;
  end;
  TIVar = class(TIValue, IIVar, IIVarObj)
  public
    Name  : TIdentifer; { }
    VTyp : TIValueTyp;
   // ByRef : boolean;    { .   TIArgs}
    constructor CreateN(AName : TIdentifer; AValue : TIValue);
    function GetName : iString;
    function IVar : TIVar;
    function IIVar.GetvIdentifer = GetvString;
    function IIVar.GetvFormula   = GetvString;
    function IIVar.GetvClass     = GetvString;
    procedure SetName(AName : iString);
  end;

  TIList = class(TIObject)
  {TIVar list}
  private
    FList : TList;
    function GetItem(index : integer) : IIVar;
  protected
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function Add(IVar : IIVar) : integer;
    procedure Delete(index : integer);
    function Count : integer;
   {$IFDEF RAINTER_DEBUG}
    function _AddRef: integer; override;
    function _Release: integer; override;
   {$ENDIF}
    function GetVarByName(AName : TIdentifer) : IIVar;
    property Items [index : integer] : IIVar read GetItem; default;
  end;

  TIArgs  = class(TIList, IIArgs)
  public
    ResTyp : TIValueTyp;
    function GetResTyp : TIValueTyp;
  end;

  TOList = class(TList)
  public
   {$IFDEF RA_D4H}
    procedure Clear; override;
   {$ELSE}
    procedure Clear;
   {$ENDIF RA_D4H}
    destructor Destroy; override;
  end;

  Lang = class
  public
    class function ReservedLangWord(const Word : string) : boolean;
    class function ReservedWord(const Word : string) : boolean;
    class function Statement(const Word : string) : boolean;

    class function IsBinaryOperationZnak(const St : TZnak) : boolean;
    class function IsUnaryOperationZnak(const St : TZnak) : boolean;
    class function IsOperationZnak(const St : TZnak) : boolean;

    class function Znak2OpCode(const Op : TZnak) : TOpCode;
    class function IsBinaryOpCode(const OpCode : TOpCode) : boolean;
    class function IsUnaryOpCode(const OpCode : TOpCode) : boolean;
    class function IsOpCode(const OpCode : TOpCode) : boolean;

    class function IsValidIdentifer(const ID : string): boolean;
    class function IsCalcIdentifer(const ID : string): boolean;
    class function Cmp(const Str : string; const LangKW : TLangKW) : boolean;
    class function IsIdentifer(const ID : string): boolean;
    class function IsIntConstant(const St : TConst) : boolean;
    class function IsRealConstant(const St : TConst) : boolean;
    class function IsStringConstant(const St : string) : boolean;
    class function IsConstant(const St : string) : boolean;
    class function GetString(const S : string) : string;
  end;

  function Cmp(const S1, S2 : string) : boolean;
  function HiddenError(const ErrCode : integer) : boolean;
  function sTyp2IValueTyp(const sTyp : string) : TIValueTyp;

  { interface helpers -    }
  procedure IIValueAssign(Dest, Source : IIValue);
  function IIValueIsConstant(AIValue : IIValue) : boolean;
  function IIValueIsReal(AIValue : IIValue) : boolean;
  function IIValueIsInteger(AIValue : IIValue) : boolean;
  procedure IIValueSetStr(AIValue : IIValue; S : iString);
  procedure IIValueGetStr(AIValue : IIValue; var S : string);
  function HasIntf(Unknown : IUnknown; const GUID : TGUID) : boolean;
  function GetIntf(Unknown : IUnknown; const GUID : TGUID) : IUnknown;
  function IIArgsCheckParams(AIArgs : IIArgs; Count : integer; Params : array of TIValueTyp) : boolean;

  function IObjectAddRef(Obj : TObject; var FRefCount : Longint) : Longint;
  function IObjectRelease(Obj : TObject; var FRefCount : Longint) : Longint;
  
const
  cpAny = High(integer);

 {Error raising routines} 
  procedure InterErrorO(const ResErr : TInterError; const AOp : integer);
  procedure InterErrorOL(const ResErr : TInterError; const AOp, ALine : integer);
  procedure InterErrorOLB(const ResErr : TInterError; const AOp, ALine : integer; const Beg : boolean);
  procedure InterErrorOB(const ResErr : TInterError; const AOp : integer; const Beg : boolean);
  procedure InterErrorOLV(const ResErr : TInterError; const AOp, ALine : integer; const VarName : string);
  procedure InterErrorOV(const ResErr : TInterError; const AOp : integer; const VarName : string);
  procedure InterErrorV(const ResErr : TInterError; const VarName : string);
  procedure InterErrorS(const ResErr : TInterError);
  procedure InterErrorP(const ResErr : TInterError; const APos : integer);
  procedure InterErrorPV(const ResErr : TInterError; const APos : integer; const VarName : string);

type

  EInterError = class(Exception)
  private
    FErrCode  : integer;
    FMess     : string;
    procedure SetErrCode(const AErrCode : integer);
  public
    Line, Op : integer;
    PosIsBeg : boolean;
    VarName : string;
    Pos     : integer;
    ProcName : string;
    UnitName : string;
    constructor CreateOLV(const ResErr : TInterError; const Mes : string; const AOp, ALine : integer; const APosIsBeg : boolean; const AVarName : string);
    constructor CreateP(const ResErr : TInterError; const Mes : string; const AVarName : string; const APos : integer);
    constructor CreateC(const ResErr : TInterError; const Mes : string);
    property ErrCode  : integer read FErrCode write SetErrCode;
  end;


{$IFDEF RAINTER_DEBUG}
 var
  GlobalRefCount : integer = 0;
  GlobalObjCount : integer = 0;
  GlobalObjs : TStrings = nil;
  GlobalRefs : TStrings = nil;
  AppHandle : longint = 0;
  function GetGlobalObjs(index : integer) : TIObject;
  procedure CheckDebug;
{$ENDIF}

implementation

{$R RAIError.res}

{$IFDEF RAINTER_DEBUG}
var
  DebugLock: TRTLCriticalSection;

procedure LockThread;
begin
  EnterCriticalSection(DebugLock);
end;

procedure UnLockThread;
begin
  LeaveCriticalSection(DebugLock);
end;

function GetGlobalObjs(index : integer) : TIObject;
begin
  LockThread;
  if (GlobalObjs <> nil) and (index < GlobalObjs.Count)
  then Result := TIObject(GlobalObjs.Objects[index])
  else Result := nil;
  UnLockThread;
end;
{$ENDIF}

function Cmp(const S1, S2 : string) : boolean;
begin
  Result := CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, PChar(S1),
    Length(S1), PChar(S2), Length(S2)) - 2 = 0;
end;

function HiddenError(const ErrCode : integer) : boolean;
begin
  Result := ErrCode in [ieOk, ieUserBreak];
end;

function HasIntf(Unknown : IUnknown; const GUID : TGUID) : boolean;
var
  O : IUnknown;
begin
  Result := Unknown.QueryInterface(GUID, O) = S_OK;
end;

function GetIntf(Unknown : IUnknown; const GUID : TGUID) : IUnknown;
var
  O : IUnknown;
begin
  if Unknown.QueryInterface(GUID, O) = S_OK then
    Result := O else
    Result := nil
end;

procedure IIValueAssign(Dest, Source : IIValue);
begin
  Dest.Empty; {release any objects}
  case Source.Typ of
    ivString    : Dest.vString    :=  Source.vString   ;
    ivIdentifer : Dest.vIdentifer :=  Source.vIdentifer;
    ivFormula   : Dest.vFormula   :=  Source.vFormula  ;
    ivClass     : Dest.vClass     :=  Source.vClass    ;
    ivInteger   : Dest.vInteger   :=  Source.vInteger  ;
    ivReal      : Dest.vReal      :=  Source.vReal     ;
    ivBoolean   : Dest.vBoolean   :=  Source.vBoolean  ;
    ivObject    : Dest.vObject    :=  Source.vObject   ;
    ivPointer   : Dest.vPointer   :=  Source.vPointer  ;
  end;
end;

function IIValueIsConstant(AIValue : IIValue) : boolean;
begin
  Result := AIValue.Typ in [ivInteger, ivReal, ivBoolean, ivString];
end;

function IIValueIsReal(AIValue : IIValue) : boolean;
begin
  Result := (AIValue.Typ = ivReal) or IIValueIsInteger(AIValue);
end;

function IIValueIsInteger(AIValue : IIValue) : boolean;
begin
  Result := AIValue.Typ = ivInteger;
end;

function IIArgsCheckParams(AIArgs : IIArgs; Count : integer; Params : array of TIValueTyp) : boolean;
//function TIList.CheckParams(lCount : integer; Params : array of TIValueTyp) : boolean;
{       (Args)
  (Params).   GetVar.
  Count -   
           Count > 0,      ;
           Count < 0,       ;
           Count = cpAny,  -  .
      ivEmpty,     -  }
var
  i : integer;
begin
  if Count <> cpAny then begin
    if Count >= 0 then
      if AIArgs.Count < Count then
        InterErrorO(ieNotEnoughParams, 0)
      else if AIArgs.Count > Count then
        InterErrorO(ieTooManyParams, Count)
      else
    else {Count < 0}
      if AIArgs.Count < -Count then
        InterErrorO(ieNotEnoughParams, 0)
  end;
  for i := Low(Params) to High(Params) do
    if (i < AIArgs.Count) and (Params[i] <> ivEmpty) and
       (Params[i] <> AIArgs[i].Typ) and
       not ((Params[i] = ivReal) and IIValueIsInteger(AIArgs[i]))
    then InterErrorO(ieParamTypMistmach, i);
  Result := true;
end;

procedure IIValueSetStr(AIValue : IIValue; S : iString);
begin
  with AIValue, Lang do
    if S = nil then Empty
    else
      case S[0] of
        '''' :
          if IsStringConstant(S) then
            vString := iString(Lang.GetString(S));
        '0'..'9', '$' :
          if IsIntConstant(S) then
            try
              vInteger := StrToInt(S)
            except
              InterErrorS(ieIntegerOverflow);
            end
          else
          if IsRealConstant(S) then vReal := StrToFloat(S);
        'A'..#255 :
          if Cmp(S, kwTRUE) then vBoolean := true
          else if Cmp(S, kwFALSE) then vBoolean := false
          else if IsValidIdentifer(S) then vIdentifer := S
          else vFormula := S;
        else vFormula := S;
      end;
end;


procedure IIValueGetStr(AIValue : IIValue; var S : string);
begin
  with AIValue, Lang do
    case Typ of
      ivString  : S := K+ vString +K;
      ivIdentifer, ivFormula, ivClass : S := vString;
      ivInteger : S := IntToStr(vInteger);
      ivReal    : S := FloatToStr(vReal);
      ivBoolean : if vBoolean then S := kwTRUE else S := kwFALSE;
      ivObject  : if vObject <> nil then S := 'object' else S := 'nil';
      else S := '';
    end;
end;

{***************************** Lang *****************************}
class function Lang.Cmp(const Str : string; const LangKW : TLangKW) : boolean;
begin
  Result := RAIClas.Cmp(Str, LangKW);
end;

class function Lang.Znak2OpCode(const Op : TZnak) : TOpCode;
var
  No : boolean;
begin
  Result := ocUnknown;
  No := true;
  if Length(Op) = 1 then begin
    No := false;
    case Op[1] of
      '+' : Result := ocPlus   ;
      '-' : Result := ocMinus  ;
      '*' : Result := ocMul    ;
      '/' : Result := ocDiv    ;
      '%' : Result := ocProcent;
      '=' : Result := ocEqu    ;
      '>' : Result := ocMore   ;
      '<' : Result := ocSmall  ;
      '^' : Result := ocPtr    ;
      '@' : Result := ocAt     ;
      '0'..'9' : exit;
      else No := true;
    end;{case}
  end;{then}
  if No then begin
    if Op[1] in RAIConstSymbols10 then exit
    else if Op = '<>' then Result := ocNotEqu
    else if Op = '==' then Result := ocEquEqu
    else if Op = '<=' then Result := ocEquSmall
    else if Op = '>=' then Result := ocEquMore
    else if Op = ':=' then Result := ocLet
    else if Lang.Cmp(Op, kwOR) then Result := ocOr
    else if Lang.Cmp(Op, kwAND) then Result := ocAnd
    else if Lang.Cmp(Op, kwNOT) then Result := ocNot
    else if Lang.Cmp(Op, kwDIV) then Result := ocIntDiv
    else if Lang.Cmp(Op, kwMOD) then Result := ocMod
  end;
end;

class function Lang.IsIdentifer(const ID : string): boolean;
var
  i, L : integer;
begin
  Result := false;
  L := Length(ID);
  if L = 0 then exit;
  if not (ID[1] in RAIIdFirstSymbols) then exit;
  for i:=1 to L do begin
    if not (ID[i] in RAIIdSymbols) then exit;
  end;
  Result := true;
end;

class function  Lang.IsCalcIdentifer(const ID : string): boolean;
begin
  Result := IsIdentifer(ID) and
           ( Cmp(ID, kwResult) or Cmp(ID, kwArgs) or
             not (ReservedLangWord(ID)))
end;                 

class function  Lang.IsValidIdentifer(const ID : string): boolean;
begin
  Result := IsIdentifer(ID) and not (ReservedLangWord(ID));
end;

class function Lang.ReservedLangWord(const Word : string) : boolean;
begin
  Result := Pos(' '+LowerCase(Word)+' ', kwReservedLang) > 0;
end;

class function Lang.ReservedWord(const Word : string) : boolean;
begin
  Result := Pos(' '+LowerCase(Word)+' ', kwReserved) > 0;
end;

class function Lang.Statement(const Word : string) : boolean;
begin
  Result := Cmp(Word, kwBEGIN) or Cmp(Word, kwEND) or
    Cmp(Word, kwIF) or Cmp(Word, kwTHEN) or
    Cmp(Word, kwELSE) or Cmp(Word, kwENDIF) or
    Cmp(Word, kwFUNC) or Cmp(Word, kwENDFUNC) or
    Cmp(Word, kwWHILE) or Cmp(Word, kwDO) or
    Cmp(Word, kwENDWHILE)
end;

class function Lang.IsRealConstant(const St : TConst) : boolean;
var
  i, j : integer;
  Point : boolean;
begin
  Result := false;
  if (St = '.') or (St = '') then exit;
  if St[1] = '-' then
    if Length(St) = 1 then exit else j := 2
  else j := 1;
  Point := false;
  for i := j to Length(St) do
    if St[i] = '.' then
       if Point then exit else Point := true
    else if (St[i] < '0') or (St[i] > '9') then exit;
  Result := true;
end;

class function Lang.IsIntConstant(const St : TConst) : boolean;
var
  i, j : integer;
  Sym : set of char;
begin
  Result := false;
  if (Length(St) = 0) or ((Length(St) = 1) and (St[1]='$')) then exit;
  Sym := RAIConstSymbols10;
  if (St[1] = '-') or (St[1] = '$') then begin
    if Length(St) = 1 then exit else j := 2;
    if St[1] = '$' then Sym := RAIConstSymbols;
  end else j := 1;
  for i := j to Length(St) do
    if not (St[i] in Sym) then exit;
  Result := true;
end;

class function Lang.IsOperationZnak(const St : TZnak) : boolean;
begin
  if IsBinaryOperationZnak(St) or
     IsUnaryOperationZnak(St)
  then Result := true
  else Result := false;
end;

class function Lang.IsBinaryOperationZnak(const St : TZnak) : boolean;
begin
  if ( (Length(St) = 1) and
       (St[1] in ['+', '-', '*', '/', '%', '=', '<', '>']) )
     or (St = '>=') or (St = '<=') or (St = '<>') or (St = ':=') or (St = '==')
  then Result := true
  else begin
    if Cmp(St, kwAND) or Cmp(St, kwOR)
    then Result := true
    else Result := false;
  end;
end;

class function Lang.IsUnaryOperationZnak(const St : TZnak) : boolean;
begin
   if Cmp(St, kwNOT) or (St = '@') or (St = '-')
   then Result := true
   else Result := false;
end;

class function Lang.IsBinaryOpCode(const OpCode : TOpCode) : boolean;
begin
  Result := OpCode in [ocPlus .. ocMod];
end;

class function Lang.IsUnaryOpCode(const OpCode : TOpCode) : boolean;
begin
  Result := OpCode in [ocNot, ocMinus, ocAt];
end;

class function Lang.IsOpCode(const OpCode : TOpCode) : boolean;
begin
  Result := OpCode in [ocPlus .. ocLet];
end;

class function Lang.IsStringConstant(const St : string) : boolean;
var
  LS : integer;
begin
  LS := Length(St);
  Result := not ( (LS < 2) or (St[1] <> '''') or (St[LS] <> '''') );
end;

class function Lang.IsConstant(const St : string) : boolean;
begin
  Result := IsRealConstant(St) or IsIntConstant(St) or IsStringConstant(St);
end;

class function Lang.GetString(const S : string) : string;
{   }
var
  LS : integer;
begin
  Result := S;
  LS := Length(S);
  if (LS < 2) or (S[1] <> '''') or (S[LS] <> '''')
  then exit; //error
  Result := Copy(S, 2, LS-2);
end;
{############################# Lang #############################}


function IObjectAddRef(Obj : TObject; var FRefCount : Longint) : Longint;
begin
 {$IFDEF DEBUG}
  LockThread;
  Inc(GlobalRefCount);
  GlobalRefs.Add(Obj.ClassName);
  UnLockThread;
 {$ENDIF}
  Inc(FRefCount);
  Result := FRefCount;
end;

function IObjectRelease(Obj : TObject; var FRefCount : Longint) : Longint;
begin
 {$IFDEF DEBUG}
  LockThread;
  dec(GlobalRefCount);
  GlobalRefs.Delete(GlobalRefs.IndexOf(Obj.ClassName));
  UnLockThread;
 {$ENDIF}
  Dec(FRefCount);
  Result := FRefCount;
  if FRefCount = 0 then Obj.Free;
end;

{*************************** TIObject ***************************}
 {************* IIObject interface *************}
function TIObject.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

function TIObject._AddRef: Longint;
begin
  Result := IObjectAddRef(Self, FRefCount);
end;

function TIObject._Release: Longint;
begin
  Result := IObjectRelease(Self, FRefCount);
end;

procedure TIObject.GetIObjectData(var IObjectData : TIObjectData);
begin
  IObjectData.ClassName := ClassName;
end;
 {############# IIObject interface #############}

constructor TIObject.Create;
begin
  inherited Create;
 {$IFDEF RAINTER_DEBUG}
  LockThread;
  Inc(GlobalObjCount);
  if GlobalObjs = nil then begin
    GlobalObjs := TStringList.Create;
    GlobalRefs := TStringList.Create;
  end;
  Debug := classname+' : '+IntToStr(GlobalObjCount);
  GlobalObjs.AddObject(Debug, self);
  UnLockThread;
 {$ENDIF}
  _AddRef;
end;

{$IFDEF RAINTER_DEBUG}
destructor TIObject.Destroy;
begin
  LockThread;
  Dec(GlobalObjCount);
  GlobalObjs.Delete(GlobalObjs.IndexOfObject(self));
  if GlobalObjs.Count = 0 then begin
    GlobalObjs.Free; GlobalObjs := nil;
    GlobalRefs.Free; GlobalRefs := nil;
  end;
  UnLockThread;
  inherited Destroy;
end;
{$ENDIF}

function TIObject.SetGetValue(var SGI : TSetGetInfo) : boolean;
begin
  PSGI := @SGI;
  Result := SetGetVar(SGI);
end;

function TIObject.SetGetVar(var SGI : TSetGetInfo) : boolean;
begin
  Result := false;
end;

function TIObject.NameIs(Name : TIdentifer) : boolean;
begin
  Result := Cmp(PSGI^.Name, Name);
end;

function TIObject.ProcIs(Name : TIdentifer; Count : integer; Params : array of TIValueTyp) : boolean;
begin
  try
    Result := NameIs(Name) and IIArgsCheckParams(PSGI^.Args, Count, Params);
  except
    on E : EInterError do begin E.VarName := Name; raise; end
    else raise;
  end;
end;

function TIObject.ClassIs(Name : TIdentifer) : boolean;
begin
  Result := (PSGI^.Args.Count > 0) and
            (PSGI^.Args[0].Typ = ivClass) and
            Cmp(PSGI^.Args[0].vString, Name);
end;
{########################### TIObject ###########################}

{*************************** TIValue ***************************}
constructor TIValue.Create(Typ : TIValueTyp; const AValue);
begin
  inherited Create;
  SetVal(Typ, AValue);
end;

constructor TIValue.Create0;
begin
  inherited Create;
  Typ := ivEmpty;
end;

destructor TIValue.Destroy;
begin
  Empty;
  inherited Destroy;
end;

procedure TIValue.SetVal(Typ : TIValueTyp; const AValue);
begin
  Self.Typ := Typ;
  case Typ of
    ivString  : Vr.vString  := string(AValue);
    ivReal    : Vr.vReal    := double(AValue);
    ivInteger : Vr.vInteger := integer(AValue);
    ivBoolean : Vr.vBoolean := boolean(AValue);
    ivObject  : begin
      Vr.vObject := IIObject(AValue);
      Vr.vObject._AddRef;
    end;
    ivPointer : begin
      Vr.vPointer := TIValue(AValue);
      Vr.vPointer._AddRef;
    end;
  end;
end;

procedure TIValue.SetStr(S : iString);
begin
  IIValueSetStr(IIValue(Self), S);
end;

function TIValue.GetStr : iString;
begin
  IIValueGetStr(IIValue(Self), sResult);
  Result := PChar(sResult);
end;

function TIValue.GetAsString : iString;
begin
  if Typ = ivString then Result := vString
  else Result := GetStr;
end;

function TIValue.GetTyp : TIValueTyp;
begin
  Result := Vr.Typ;
end;

procedure TIValue.SetTyp(ATyp : TIValueTyp);
begin
  if Vr.Typ = ivObject then Vr.vObject := nil
  else if Vr.Typ = ivPointer then Vr.vPointer := nil;
  Vr.Typ := ATyp;
end;

procedure TIValue.SetvString(vString : iString);
begin
  Typ := ivString;
  Vr.vString := vString;
end;

procedure TIValue.SetvIdentifer(vIdentifer : iString);
begin
  Typ := ivIdentifer;
  Vr.vString := vIdentifer;
end;

procedure TIValue.SetvFormula(vFormula : iString);
begin
  Typ := ivFormula;
  Vr.vString := vFormula;
end;

procedure TIValue.SetvClass(vClass : iString);
begin
  Typ := ivClass;
  Vr.vString := vClass;
end;

procedure TIValue.SetvInteger(vInteger : integer);
begin
  Typ := ivInteger;
  Vr.vInteger := vInteger;
end;

procedure TIValue.SetvReal(vReal : double);
begin
  Typ := ivReal;
  Vr.vReal := vReal;
end;

procedure TIValue.SetvBoolean(vBoolean : boolean);
begin
  Typ := ivBoolean;
  Vr.vBoolean := vBoolean;
end;

procedure TIValue.SetvObject(vObject : IIObject);
begin
  Empty;
  Typ := ivObject;
  Vr.vObject := vObject;
end;

procedure TIValue.SetvPointer(vPointer : IIValue );
begin
  Empty;
  Typ := ivPointer;
  Vr.vPointer := vPointer;
end;

function TIValue.GetvString : iString;
begin
  case Typ of
    ivInteger : sResult := IntToStr(Vr.vInteger);
    ivString, ivIdentifer, ivClass, ivFormula : sResult := Vr.vString;
    ivReal : sResult := FloatToStr(Vr.vReal);
    else sResult := '';
  end;
  Result := iString(sResult);
end;

function TIValue.GetvInteger : integer ;
begin
  case Typ of
    ivInteger : Result := Vr.vInteger;
    ivString, ivFormula : Result := StrToInt(Vr.vString);
    ivReal    : Result := Round(Vr.vReal);
    else Result := 0;
  end;
end;

function TIValue.GetvReal    : double  ;
begin
  if Typ = ivInteger then Result := Vr.vInteger
  else Result := Vr.vReal;
end;

function TIValue.GetvBoolean : boolean ;
begin
  Result := Vr.vBoolean;
end;

function TIValue.GetvObject  : IIObject;
begin
  if Typ = ivObject then Result := Vr.vObject
                    else Result := nil;
end;

function TIValue.GetvPointer  : IIValue;
begin
  if Typ = ivPointer then Result := Vr.vPointer
                     else Result := nil;
end;

procedure TIValue.Empty;
begin
  Typ := ivEmpty;
end;
{########################### TIValue ###########################}

{*************************** TIVar ***************************}
constructor TIVar.CreateN(AName : TIdentifer; AValue : TIValue);
begin
  inherited Create0;
  Name := AName;
  IIValueAssign(Self, AValue);
end;

function TIVar.IVar : TIVar;
begin
  Result := self;
end;

function TIVar.GetName : iString;
begin
  sResult := Name;
  Result := iString(sResult);
end;

procedure TIVar.SetName(AName : iString);
begin
  Name := AName;
end;

{########################### TIVar ###########################}

{*************************** TIList ***************************}
constructor TIList.Create;
begin
  inherited Create;
  FList := TList.Create;
end;

destructor TIList.Destroy;
begin
  Clear;
  FList.Free;
  inherited Destroy;
end;

procedure TIList.Clear;
var
  i : integer;
begin
  for i := 0 to FList.Count-1 do
    IIVar(FList[i])._Release;
  FList.Clear;  
end;

{$IFDEF RAINTER_DEBUG}
function TIList._AddRef: integer;
begin
  Result := inherited _AddRef;
end;

function TIList._Release: integer;
begin
  Result := inherited _Release;
end;
{$ENDIF}

function TIList.Count : integer;
begin
  Result := FList.Count;
end;

function TIList.Add(IVar : IIVar) : integer;
begin
  Result := FList.Add(pointer(IVar));
  IVar._AddRef;
end;

procedure TIList.Delete(index : integer);
begin
  IIVar(FList[index])._Release;
  FList.Delete(index);
end;

function TIList.GetItem(index : integer) : IIVar;
begin
  if (index >= 0) and (index < FList.Count) then
    Result := IIVar(FList[index])
  else Result := nil;
end;

function TIList.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  index : integer;
begin
  with SGI do begin
  Res.Empty;
  Result := inherited SetGetVar(SGI);
  if not Result and Get then begin
    Result := true;
    if ProcIs('Add', 1, [ivEmpty]) then
      Res.vInteger := Add(Args[0])
    else if ProcIs('Items', 1, [ivInteger]) then
      if (Args[0].vInteger >= 0) and (Args[0].vInteger < FList.Count) then
        IIValueAssign(Res, GetItem(Args[0].vInteger))
      else InterErrorO(ieParam, 0)
    else if ProcIs('Delete', 1, [ivInteger]) then begin
      index := Args[0].vInteger;
      if not ((index >= 0) and (index < FList.Count)) then
        InterErrorO(ieParam, 0);
      Delete(index);
      Res.Empty;
    end else if ProcIs('Count', 0, [ivEmpty]) then
      Res.vInteger := Count
    else Result := false;
  end;
  end;
end;

function TIList.GetVarByName(AName : TIdentifer) : IIVar;
var
  i : integer;
begin
  Result := nil;
  for i := 0 to FList.Count-1 do
    if Cmp(Items[i].Name, AName) then begin
      Result := Items[i];
      break;
    end;
end;

destructor TOList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

procedure TOList.Clear;
var i : integer;
begin
  for i:=0 to Count-1 do
    TObject(Items[i]).Free;
  inherited Clear;
end;
{########################### TIList ###########################}

{*************************** TIArgs ***************************}
function TIArgs.GetResTyp : TIValueTyp;
begin
  Result := ResTyp;
end;
{########################### TIArgs ###########################}



function sTyp2IValueTyp(const sTyp : string) : TIValueTyp;
begin
  if Cmp(sTyp, 'integer') then Result := ivInteger
  else if Cmp(sTyp, 'boolean') then Result := ivBoolean
  else if Cmp(sTyp, 'string') then Result := ivString
  else Result := ivEmpty;
end;

{***************************** EInterError *****************************}
procedure InterErrorS(const ResErr : TInterError);
begin
  raise EInterError.CreateOLV(ResErr, '', 0, 0, true, '');
end;

procedure InterErrorO(const ResErr : TInterError; const AOp : integer);
begin
  raise EInterError.CreateOLV(ResErr, '', AOp, 0, true, '');
end;

procedure InterErrorOL(const ResErr : TInterError; const AOp, ALine : integer);
begin
  raise EInterError.CreateOLV(ResErr, '', AOp, ALine, true, '');
end;

procedure InterErrorOLB(const ResErr : TInterError; const AOp, ALine : integer; const Beg : boolean);
begin
  raise EInterError.CreateOLV(ResErr, '', AOp, ALine, Beg, '');
end;

procedure InterErrorOB(const ResErr : TInterError; const AOp : integer; const Beg : boolean);
begin
  raise EInterError.CreateOLV(ResErr, '', AOp, 0, Beg, '');
end;

procedure InterErrorOV(const ResErr : TInterError; const AOp : integer; const VarName : string);
begin
  raise EInterError.CreateOLV(ResErr, '', AOp, 0, true, VarName);
end;

procedure InterErrorV(const ResErr : TInterError; const VarName : string);
begin
  raise EInterError.CreateOLV(ResErr, '', 0, 0, true, VarName);
end;

procedure InterErrorOLV(const ResErr : TInterError; const AOp, ALine : integer; const VarName : string);
begin
  raise EInterError.CreateOLV(ResErr, '', AOp, ALine, true, VarName);
end;

procedure InterErrorP(const ResErr : TInterError; const APos : integer);
begin
  raise EInterError.CreateP(ResErr, '', '', APos);
end;

procedure InterErrorPV(const ResErr : TInterError; const APos : integer; const VarName : string);
begin
  raise EInterError.CreateP(ResErr, '', VarName, APos);
end;

{__________________________________________________}

constructor EInterError.CreateOLV(const ResErr : TInterError; const Mes : string; const AOp, ALine : integer; const APosIsBeg : boolean; const AVarName : string);
begin
  inherited Create(Mes);
  FMess := Mes;
  PosIsBeg := APosIsBeg;
  Op := AOp;
  Line := ALine;
  VarName := AVarName;
  Pos := -1;
  ErrCode := ResErr;
end;

constructor EInterError.CreateP(const ResErr : TInterError; const Mes : string; const AVarName : string; const APos : integer);
begin
  inherited Create(Mes);
  FMess := Mes;
  Pos := APos;
  VarName := AVarName;
  ErrCode := ResErr;
end;

constructor EInterError.CreateC(const ResErr : TInterError; const Mes : string);
begin
  inherited Create(Mes);
  FMess := Mes;
  ErrCode := ResErr;
end;

procedure EInterError.SetErrCode(const AErrCode : integer);
begin
  FErrCode := AErrCode;
  Message := Format(LoadStr(FErrCode), [VarName]) +'  '+ FMess;
end;
{############################## EInterError ###############################}



{*************************** RAINTER_DEBUG ****************************}
{$IFDEF RAINTER_DEBUG}
procedure CheckDebug;
var
  i : integer;
  SS : TStringList;
  FN : TFileName;
  Obj : TIObject;
  S : PChar;
begin
  if GlobalObjCount > 0 then begin
    SS := TStringList.Create;
    FN := ExtractFilePath(ParamStr(0))+'interError.log';
    if FileExists(FN) then SS.LoadFromFile(FN);
    SS.Add('------------------'+DateToStr(Date)+' '+TimeToStr(Time)+'--------------------');
    SS.Add('  : '+IntToStr(GlobalObjCount)+' >'+IntToStr(GlobalRefCount));
    SS.Add(': '+IntToStr(GlobalObjs.Count));
    for i := 0 to GlobalObjs.Count-1 do begin
      Obj := TIObject(GlobalObjs.Objects[i]);
      SS.Add(Obj.Debug +' : '+IntToStr(Obj._AddRef-1));
      Obj._Release;
    end;
    SS.Add('');
    SS.Add(': '+IntToStr(GlobalRefs.Count));
    for i := 0 to GlobalRefs.Count-1 do SS.Add(GlobalRefs[i]);
    SS.SaveToFile(FN);
    SS.Free;
    S := PChar('   : '+
      IntToStr(GlobalObjCount)+'  '
      +IntToStr(GlobalRefCount)+' .'+#13
      +' '''+FN+'''');
    MessageBox(AppHandle, S, '  ', MB_OK+MB_ICONERROR);
  end;
end;
{$ENDIF}

{$IFDEF RAINTER_DEBUG}
initialization
  InitializeCriticalSection(DebugLock);
  GetGlobalObjs(0); { linker   GetGlobalObjs}
finalization
  CheckDebug;
  DeleteCriticalSection(DebugLock);
{$ENDIF}
end.
