{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAIImpl;

{
     TIInterObject,
      ,
      Thread',
     
   . (3  1)
}

interface

uses Classes, SysUtils,
  RAIIntf, RAIClas, RAIExpr;

type

{ TIThread }
  TSetGetFunc = function(var SGI : TSetGetInfo) : boolean of object;

  TIThread = class(TThread)
  protected
    Go : TThreadMethod;
    SetGetMethod : TSetGetFunc;
    PSGI : PSetGetInfo;
    Exist : boolean;
    procedure Execute; override;
    procedure SyncSetGetMethod;
  public
    constructor Create(procGo : TThreadMethod);
    constructor CreateSuspended(procGo : TThreadMethod; Susp : boolean);
    function SyncSetGetVar(var SGI : TSetGetInfo; Method: TSetGetFunc) : boolean;
  end;

  TUnknownList = class(TList)
  public
    destructor Destroy; override;
   {$IFDEF RA_D4H}
    procedure Clear; override;
   {$ELSE}
    procedure Clear;
   {$ENDIF RA_D4H}
    procedure Add(Unknown : IUnknown);
    procedure Add2(Unknown : IUnknown);
  end;

  IIThreadObject = interface
    ['{8C6BA2A0-08FC-11d1-BA5A-F44E9C5EF10B}']
    procedure SetThread(AThread : TIThread);
    property trdProg : TIThread write SetThread;
  end;

  TIThreadObject = class(TIObject, IIObject, IIThreadObject)
  private
    trdProg  : TIThread; { <> nil,      Thread}
    function SetGetValue(var SGI : TSetGetInfo) : boolean;
    procedure SetThread(AThread : TIThread);
  protected
    function SyncSetGetVar(var SGI : TSetGetInfo) : boolean; dynamic;
  public
    constructor Create;
  end;

  TITranslator = class(TIThreadObject, IITranslator)
  protected
    function Translate(var TranslateData : TTranslateData) : boolean; dynamic;  abstract;
    procedure Clear; dynamic; abstract;
  end;

  TIDebugObject = class;

  {  }
  TIInterObject = class(TITranslator, IIInterObject)
  private
    IInter : IIInter;
    FSysObjs        : TUnknownList {of IIObject};  //        
    FSyncSysObjs    : TUnknownList {of IIObject};  //        
    FExtTranslators : TUnknownList {of IITranslator}; //   
   // FUnitObjs : TINList; //           uses
    FOnDone  : TNotifyEvent;
    FTerminated : boolean;
    FIDebugObject : TIDebugObject;
    function {IIInterObject.}GetRes : IIValue;
    procedure ThreadOnDone(Sender : TObject);
  protected
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
    function SyncSetGetVar(var SGI : TSetGetInfo) : boolean; override;
    function Translate(var TranslateData : TTranslateData) : boolean; override;
    procedure Clear; override;
    function {IIInterObject.}IsTerminated : boolean;
    procedure ProgramDone; dynamic;
  public
    {RunParams - in}
    UnitName    : string; {   ( = '',   Formula        (,  UnitName = '',  Formula = 'MyUnit.MyFunc'.   UnitName = 'MyUnit',    Formula = 'MyFunc'))}
    Formula     : string    ;
    ReloadAll   : boolean   ; {      Unit'}
    RemoveUnit  : boolean   ;
    CompileOnly : boolean   ;
    AppHandle   : longint   ;
   {  - out}
    ErrProcName : string;{   EInterError}
    ErrUnitName : string;{   EInterError}
    ErrMessage  : string;  { }
    ErrPos      : integer; {   pcProgram}
    ErrCode     : integer;
    ErrVarName  : string;  {.   }
    ErrIObj     : IIObject;
    Res : IIValue;
    sResult : string;

    constructor Create;
    destructor Destroy; override;
    procedure AddRef;
    function Release : longint;
    procedure Terminate;

    procedure {IIInterObject.}GetRunParams(var RunParams : TRunParams);
    procedure {IIInterObject.}Done(var Error : TProgError);

    {   }
    function {IIInterObject.}GetUnitSource(UnitName : iString; var UnitSource, UnitFullName : iString) : boolean; dynamic;
    procedure Run;
    procedure RunInThread;

    property SysObjs        : TUnknownList read FSysObjs;
    property SyncSysObjs    : TUnknownList read FSyncSysObjs;
    property ExtTranslators : TUnknownList read FExtTranslators;
   // property UnitObjs : TINList read FUnitObjs; 
    property OnDone  : TNotifyEvent read FOnDone write FOnDone;
    property IDebugObject : TIDebugObject read FIDebugObject;
  end;

  {  }
  TDebugStatus = (dsReady, dsRuning, dsStopped);
  TBreakpoints = class;

  TIDebugObject = class(TInterfacedObject, IIDebugObject)
  private
    FIDebugger : IIDebugger;
    FIInterObject : TIInterObject;
    FOnDebug : TNotifyEvent;
    FDebugMode   : boolean; { }
    DebugFuncEnabled : boolean; {   }
    DebugTrace  : boolean; {    }
    DebugStep   : boolean;
    FBreakpoints : TBreakpoints;
    CS : TCallStackItem;
   {***  StepOver}
    Funcs : integer;
   {###}
    FStatus : TDebugStatus;
    procedure SyncDebug;
    procedure CheckFDebug;
    function CheckBreakpoints : boolean;
    procedure SetDebugMode(ADebugMode : boolean);
    function GetIDebugger : IIDebugger;
    function GetCallStack : IICallStack;

    property DebugMode : boolean read FDebugMode write SetDebugMode;
  protected
    function {IIDebugObject.}GetDebugMode : boolean;
    procedure {IIDebugObject.}Debug;
    procedure Suspend;
    procedure Resume;
  public
    constructor Create(AIInterObject : TIInterObject);
    destructor Destroy; override;
    function EvaluateModify(Watch : string; var Value : string; SGIDebugFlags : TSGIDebugFlags) : boolean;
    procedure TraceInto;
    procedure StepOver;
    procedure Run;
    procedure Reset;
    procedure Pause;
    property OnDebug : TNotifyEvent read FOnDebug write FOnDebug;
    property IDebugger : IIDebugger read GetIDebugger;
    property Status : TDebugStatus read FStatus;
    property CallStack : IICallStack read GetCallStack;
    property Breakpoints : TBreakpoints read FBreakpoints;
  end;

{********************** TBreakpoints ***********************}
  TBreakpoint = class
  public
    UnitName : string;
    PosBeg, PosEnd : integer; {    }
    Condition : string;
    Pass      : integer;
    PassCount : integer;
  end;

  TBreakpoints = class(TList)
  private
    function GetItem(ind : integer) : TBreakpoint;
  public
    destructor Destroy; override;
    function IndexOf(AUnitName : string; APosBeg, APosEnd : integer;
      ACondition : string; APass : integer) : integer;
    procedure Delete(index : integer);
    property Items[index : integer] : TBreakpoint read GetItem; default;
  end;
{###################### TBreakpoints #######################}


implementation

{*************************** TIThread ***************************}
constructor TIThread.Create(procGo : TThreadMethod);
begin
  CreateSuspended(procGo, False);
end;

constructor TIThread.CreateSuspended(procGo : TThreadMethod; Susp : boolean);
begin
  inherited Create(Susp);
  Go := procGo;
  FreeOnTerminate := True;
end;

procedure TIThread.Execute;
begin
  Go;
end;

function TIThread.SyncSetGetVar(var SGI : TSetGetInfo; Method: TSetGetFunc) : boolean;
begin
  SetGetMethod := Method;
  PSGI := @SGI;
  Synchronize(SyncSetGetMethod);
  Result := Exist;
end;

procedure TIThread.SyncSetGetMethod;
begin
  Exist := SetGetMethod(PSGI^);
end;
{########################### TIThread ###########################}

destructor TUnknownList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

procedure TUnknownList.Clear;
var
  i : integer;
begin
  for i := 0 to Count-1 do
    IUnknown(Items[i])._Release;
  inherited Clear;
end;

procedure TUnknownList.Add(Unknown : IUnknown);
begin
  Unknown._AddRef;
  inherited Add(pointer(Unknown));
end;

procedure TUnknownList.Add2(Unknown : IUnknown);
begin
  inherited Add(pointer(Unknown));
end;

{*************************** TIThreadObject ***************************}
constructor TIThreadObject.Create;
begin
  _AddRef;
end;

function TIThreadObject.SetGetValue(var SGI : TSetGetInfo) : boolean;
var
  IThreadObject : IIThreadObject;
begin
  Result := inherited SetGetValue(SGI);
  if not Result then
    if trdProg <> nil then
      Result := trdProg.SyncSetGetVar(SGI, SyncSetGetVar)
    else
      Result := SyncSetGetVar(SGI);
  if Result and (SGI.Res.Typ = ivObject) then begin
    IThreadObject := IIThreadObject(GetIntf(SGI.Res.vObject, IIThreadObject));
    if IThreadObject <> nil then
      IThreadObject.trdProg := trdProg;
  end;
end;

function TIThreadObject.SyncSetGetVar(var SGI : TSetGetInfo) : boolean;
begin
  Result := false;
end;

procedure TIThreadObject.SetThread(AThread : TIThread);
begin
  trdProg := AThread;
end;
{########################### TITranslator ###########################}


{*************************** TIInterObject ***************************}
constructor TIInterObject.Create;
begin
  inherited Create;
  if CreateInterObject(IID_IIValue, Res) <> 0 then {Error};
  FIDebugObject := TIDebugObject.Create(Self);
  FSysObjs        := TUnknownList.Create;
  FSyncSysObjs    := TUnknownList.Create;
  FExtTranslators := TUnknownList.Create;
//  FUnitObjs := TINList.Create;
end;

destructor TIInterObject.Destroy;
begin
  Res := nil;
  FIDebugObject._Release;
  FSysObjs.Free;
  FSyncSysObjs.Free;
  FExtTranslators.Free;
//  FUnitObjs._Release;
  inherited Destroy;
end;

function TIInterObject.QueryInterface(const IID: TGUID; out Obj): HResult;
const
  E_NOINTERFACE = $80004002;
begin
  if GetInterface(IID, Obj) or FIDebugObject.GetInterface(IID, Obj) then
    Result := 0 else
    Result := E_NOINTERFACE;
end;

(*
function TIInterObject.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  i, iU : integer;
  Obj : TINObject;
  found : boolean;
begin
  Result := inherited SetGetVar(SGI);
  with SGI do begin
  if not Result then
    {   }
    for i:=0 to FSysObjs.Count-1 do begin
      Obj := TINObject(FSysObjs[i]);
      if (Length(Name) > 0) and Cmp(Obj.Name, Name) and Get then
        Res.vObject := Obj
      else begin
        Result := Obj.SetGetValue(SGI);
        if Result then break;
      end;
    end;
  if not Result then
    {   }
    if (Prg <> nil) and (Prg.CurUnit <> nil) then
     {         }
      for i:=0 to FUnitObjs.Count-1 do begin
        Obj := TINObject(FUnitObjs[i]);
        if Cmp(Obj.Name, Name) then found := true else begin
          found := false;
          for iU := 0 to Prg.CurUnit.SysUsesUnits.Count-1 do
            if Cmp(Prg.CurUnit.SysUsesUnits[iU], Obj.Name) then begin
              found := true;
              break
            end;
        end;
        if found then
          if (Length(Name) > 0) and Cmp(Obj.Name, Name) and Get then begin
            Res.vObject := Obj;
            Result := true;
            break;
          end else begin
            Result := Obj.SetGetValue(SGI);
            if Result then break;
          end;
      end;
  end;
end;

{    for i:=0 to FSyncSysObjs.Count-1 do begin
      Obj := TINObject(FSyncSysObjs[i]);
      if (Length(SGI.Name) > 0) and Cmp(Obj.Name, SGI.Name) and SGI.Get then
        Res.vObject := Obj
      else begin
        Result := Obj.SetGetValue(SGI);
        if Result then break;
      end;
    end;
}    
*)

function TIInterObject.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  i : integer;
begin
  Result := false;
  if Cmp('ApplicationHandle', SGI.Name) then begin
    SGI.Res.vInteger := AppHandle;
    Result := true;
  end else
  {   }
  for i:=0 to FSysObjs.Count-1 do begin
     Result := IIObject(GetIntf(IUnknown(FSysObjs[i]), IIObject)).SetGetValue(SGI);
      if Result then break;
  end;
end;

function TIInterObject.SyncSetGetVar(var SGI : TSetGetInfo) : boolean;
var
  i : integer;
begin
  Result := false;
  {   }
  for i:=0 to FSyncSysObjs.Count-1 do begin
     Result := IIObject(GetIntf(IUnknown(FSyncSysObjs[i]), IIObject)).SetGetValue(SGI);
      if Result then break;
  end;
end;

function TIInterObject.Translate(var TranslateData : TTranslateData) : boolean;
var
  i : integer;
begin
  Result := false;
  for i:=0 to FExtTranslators.Count-1 do begin
    Result := IITranslator(FExtTranslators[i]).Translate(TranslateData);
    if Result then break;
  end;
end;

procedure TIInterObject.Clear;
{
var
  i : integer;
}
begin
{  for i:=0 to FExtTranslators.Count-1 do
    TITObject(FExtTranslators[i]).Clear;}
end;

function TIInterObject.GetUnitSource(UnitName : iString; var UnitSource, UnitFullName : iString) : boolean;
begin
  Result := false;
end;

procedure TIInterObject.ThreadOnDone(Sender : TObject);
{  FOnDone  Synhronize}
begin
  trdProg := nil;
  ProgramDone;
  if Assigned(FOnDone) then FOnDone(Self);
end;

procedure TIInterObject.RunInThread;
var
  trdP : TIThread;
begin
  if trdProg <> nil then raise Exception.Create('RunInThread Error');
  trdP := TIThread.Create(Run);
  trdProg := trdP;
  trdP.OnTerminate := ThreadOnDone;
end;

procedure TIInterObject.Run;
begin
  FTerminated := false;
  if CreateInterObject(IID_IIInter, IInter) <> 0 then exit;{error}
  try
    FIDebugObject.FStatus := dsRuning;
    FIDebugObject.CheckFDebug;
    IInter.Run(Self);
  finally
    IInter := nil; //IInter._Release;
    ProgramDone;
    if (trdProg = nil) and Assigned(FOnDone) then FOnDone(Self);
    FIDebugObject.FStatus := dsReady;
  end;
end;

function TIInterObject.IsTerminated : boolean;
begin
  Result := FTerminated;
end;

procedure TIInterObject.Terminate;
begin
  if (trdProg <> nil) then trdProg.Terminate;
  FTerminated := true;
end;

procedure TIInterObject.GetRunParams(var RunParams : TRunParams);
begin
  RunParams.UnitName    := iString(UnitName);
  RunParams.Formula     := iString(Formula);
  RunParams.ReloadAll   := ReloadAll  ;
  RunParams.RemoveUnit  := RemoveUnit ;
  RunParams.CompileOnly := CompileOnly;
  RunParams.AppHandle   := AppHandle;
end;

procedure TIInterObject.Done(var Error : TProgError);
begin
  ErrProcName := Error.ErrProcName;
  ErrUnitName := Error.ErrUnitName;
  ErrMessage  := Error.ErrMessage ;
  ErrPos      := Error.ErrPos     ;
  ErrCode     := Error.ErrCode    ;
  ErrVarName  := Error.ErrVarName ;
  ErrIObj     := Error.ErrIObj    ;
end;

procedure TIInterObject.ProgramDone;
begin
end;

function TIInterObject.GetRes : IIValue;
begin
  Result := Res;
end;

procedure TIInterObject.AddRef;
begin
  IUnknown(Self)._AddRef;
end;

function TIInterObject.Release : longint;
begin
  Result := IUnknown(Self)._Release;
end;

{************************* TIDebugObject **************************}
const
  DebugValueVarUndefined = ' ';
  DebugValueUnAccesible  = '[  ]';

constructor TIDebugObject.Create(AIInterObject : TIInterObject);
begin
  inherited Create;
  FIInterObject := AIInterObject;
  FBreakpoints := TBreakpoints.Create;
  _AddRef;
end;

destructor TIDebugObject.Destroy;
begin
  FBreakpoints.Free;
  inherited Destroy;
end;

procedure TIDebugObject.Debug;
begin
  if FIInterObject.trdProg <> nil then
    FIInterObject.trdProg.Synchronize(SyncDebug)
  else
    SyncDebug;
  FIDebugger := nil;
end;

function TIDebugObject.GetIDebugger;
begin
  if (FIDebugger = nil) and (FIInterObject.IInter <> nil) then
    FIDebugger := IIDebugger(GetIntf(FIInterObject.IInter, IID_IIDebugger));
  Result := FIDebugger;
end;

function TIDebugObject.GetDebugMode : boolean;
begin
  Result := FDebugMode;
end;

procedure TIDebugObject.SetDebugMode(ADebugMode : boolean);
begin
  if ADebugMode <> FDebugMode then begin
    FDebugMode := ADebugMode;
    if IDebugger <> nil then
      IDebugger.DebugMode := FDebugMode;
  end;
  if not FDebugMode then begin
    DebugTrace := false;
    DebugStep  := false;
  end;
end;

procedure TIDebugObject.CheckFDebug;
begin
  {  breakpoint,    }
  DebugMode := (FBreakpoints.Count > 0) or DebugTrace or DebugStep;
end;

procedure TIDebugObject.Reset;
begin
  DebugMode := false;
  Funcs := 1;
end;

{  Synchronize}
procedure TIDebugObject.SyncDebug;
var
  Stop : boolean;
begin
  CS := IDebugger.CallStack[0];
  if CS.FuncName[0] = '' {Delphi - } then exit; {    }
  Stop := false;
  {  breakpoint}
  if FBreakpoints.Count > 0 then begin
    DebugFuncEnabled := true;
    Stop := CheckBreakpoints;
    DebugFuncEnabled := false;
  end;
  { TraceInto  }
  if DebugTrace then Stop := true;
  { DebugStep}
  if not Stop and DebugStep then begin
    if IDebugger.CallStack.Count <= Funcs then Stop := true;
  end;
  if Stop then begin
    Funcs := IDebugger.CallStack.Count;
    DebugFuncEnabled := true;
    if Assigned(FOnDebug) then begin
      Suspend;
      FOnDebug(Self);
    end;
  end;
end;

procedure TIDebugObject.TraceInto;
begin
  if not DebugFuncEnabled then exit;
  DebugFuncEnabled := false;
  DebugTrace := true;
  DebugStep  := false;
  Resume;
end;

procedure TIDebugObject.StepOver;
begin
  if not DebugFuncEnabled then exit;
  DebugFuncEnabled := false;
  DebugTrace := false;
  DebugStep  := true;
  Resume;
end;

procedure TIDebugObject.Run;
begin
{ -   ...!!!}
  if not DebugFuncEnabled then exit;
  DebugFuncEnabled := false;
  DebugTrace := false;
  DebugStep  := false;
  Funcs := 1;
  CheckFDebug;
  Resume;
end;

procedure TIDebugObject.Pause;
begin
//  if DebugFuncEnabled then exit;{  }
  DebugTrace := true;
  DebugStep  := false;
  DebugMode  := true;
  DebugFuncEnabled := true;
end;

function TIDebugObject.EvaluateModify(Watch : string; var Value : string; SGIDebugFlags : TSGIDebugFlags) : boolean;
var
  P : iString;
begin
  if DebugFuncEnabled then begin
    Result := IDebugger.EvaluateModify(iString(Watch), P, SGIDebugFlags);
    Value := P;
  end else begin
    Result := false;
    Value := DebugValueUnAccesible;
  end;  
end;

procedure TIDebugObject.Suspend;
begin
  if FIInterObject.trdProg <> nil then FIInterObject.trdProg.Suspend;
  FStatus := dsStopped;
end;

procedure TIDebugObject.Resume;
begin
  if FIInterObject.trdProg <> nil then FIInterObject.trdProg.Resume;
  FStatus := dsRuning;
end;

function TIDebugObject.GetCallStack : IICallStack;
begin
  Result := IDebugger.CallStack;
end;

function TIDebugObject.CheckBreakpoints : boolean;
var
  i : integer;
  Value : string;
const
  sTrue = '';
begin
  Result := false;
  for i := 0 to FBreakpoints.Count-1 do with FBreakpoints[i] do
    if Cmp(PChar(UnitName), CS.UnitName) and
      (((PosBeg <= CS.PosBeg) and (CS.PosBeg <= PosEnd)) or
       ((PosBeg <= CS.PosEnd) and (CS.PosEnd <= PosEnd)))
     then begin
       if Condition = '' then Value := sTrue
       else if not EvaluateModify(Condition, Value, dfAll) then exit; {Error in expression}
       if Value = sTrue then
         if Pass = 0 then begin Result := true; exit end
         else begin
           inc(PassCount);
           if PassCount > Pass then begin Result := true; exit end
         end;
     end;
end;

{********************** TBreakpoints ***********************}
destructor TBreakpoints.Destroy;
begin
  while Count > 0 do Delete(0);
  inherited Destroy;
end;

procedure TBreakpoints.Delete(index : integer);
begin
  Items[index].Free;
  inherited Delete(index);
end;

function TBreakpoints.GetItem(ind : integer) : TBreakpoint;
begin
  Result := TBreakpoint(inherited Items[ind]);
end;

function TBreakpoints.IndexOf(AUnitName : string; APosBeg, APosEnd : integer;
      ACondition : string; APass : integer) : integer;
begin
  Result := -1;
end;

{###################### TBreakpoints #######################}
{######################### TIDebugObject ##########################}




end.
