{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAIProc;

interface

uses SysUtils, Classes,
  RAIIntf, RAICalc, RAIClas, RAConst;

type

  TIParser = class
  protected
    FpcProgram : PChar;
    FpcPos : PChar; //  
    FHistory : TStringList;
    FHistorySize : integer;
    FHistoryPtr  : integer;

    function HistoryInd(index : integer) : integer;
    function GetHistory(index : integer) : string;
    function GetPosBeg(index : integer) : integer;
    function GetPosEnd(index : integer) : integer;
    procedure SetHistorySize(Size : integer);
    function GetPos : integer;
  public
    constructor Create;
    destructor Destroy; override;
    {     }
    function Token : string;
    {          }
    // function TokenL : string; -     #;-(
    {     }
    procedure RollBack(index : integer);
    property History[index : integer] : string read GetHistory;
    property PosBeg[index : integer] : integer read GetPosBeg;
    property PosEnd[index : integer] : integer read GetPosEnd;
    property HistorySize : integer read FHistorySize write SetHistorySize;
    property Pos : integer read GetPos;
    property pcPos : PChar read FpcPos write FpcPos;
    property pcProgram : PChar read FpcProgram write FpcProgram;
  end;

  { }
  procedure GetBlockByPos(pcProgram : iString; Pos : integer; var PosBeg, PosEnd : integer);

type

  TStatement = class
  public
    pcBeg, pcEnd : PChar;
  end;

 { .   }
  TExecStatements = class(TList)
  public
    procedure DeleteLast;
    function Last : TStatement;
  end;
{________________________ IF __________________________}
  TIF  = class(TStatement)
  public
   // pcIf    : PChar; = pcBeg
    pcThen  : PChar;
    pcElse  : PChar;
  //  pcEndIf : PChar; = pcEnd
  end;

  TIFS = TOList;
{_______________________ WHILE ___________________________}
  TWHILE  = class(TStatement)
  public
   // pcWhile    : PChar; = pcBeg
    pcDo    : PChar;
    iWhile : integer; // 
   // pcEndWhile : PChar; = pcEnd
  end;

  TWHILES = TOList;
{__________________________________________________}

{________________________ GOTO __________________________}
  TGOTO = class
  private
    Name  : TIdentifer;{ }
    pcPos : PChar;{ }
  end;

  TGOTOS = class(TOList)
  public
    procedure AddGOTO(Name : TIdentifer; pcPos : PChar);
    function GetPos(Name : TIdentifer; var pcPos : PChar): boolean;
  end;
{__________________________________________________}

  TProc    = class;
  TProcCom = class;

  TDebugProc = procedure of object;

  TUnitSetGetValue = function(PUnit : TObject; SGI : TSetGetInfo) : boolean of object;

  TProcCom = class
  public
   {****   GetResult}
    IInterObject : IIInterObject;
    Debug     : boolean;
    DebugProc : TDebugProc;
   {####   GetResult}
    Checking : boolean;
    SetGetValue : TUnitSetGetValue;
    Name : TIdentifer;
    PUnit : TObject;
    UnitName : TIdentifer; // = '',   Formula,
    UnitFunc : TIdentifer; //  -   
    pcProgram : PChar;   { }
    OffsBeg, OffsEnd : integer; {   }
    IObj : IIObject;
    Args : IIArgs;
  end;

{***************************** TProc *******************************}
  TProc = class
  protected
    FIObj : IIObject;
    FArgs : IIArgs;
    FName : TIdentifer;  {}
    Expres : TCalc;
    IFS    : TIFS;
    WHILES : TWHILES;
    Execs : TExecStatements;
    GOTOS     : TGOTOS;
    VARS      : TIList;
    Checking  : boolean;  { ,    }
    CheckVars : TStrings;
    ObjSetGetValue : TUnitSetGetValue;
    Parser : TIParser;
    ProcCom : TProcCom;
    pcProgram : PChar;   { }
    OffsBeg, OffsEnd : integer; {   }
    procedure Runing;
    procedure Clear;
    function SetGetValue(var SGI : TSetGetInfo) : boolean;
  public
    Working   : boolean;
    Res       : TIValue; {  }
    PUnit : TObject;
    PosBeg, PosEnd : integer; { }
    constructor Create(lProcCom : TProcCom);
    destructor Destroy;  override;
    procedure Solve;
    procedure Check;
    function DebugEvaluate(Expression : string; var Value : string; var SGIDebugFlags : TSGIDebugFlags) : boolean;
    { Value   ' := ',   DebugModify }

    property Name : TIdentifer read FName;   {}
    property IObj : IIObject read FIObj;
    property Args : IIArgs read FArgs;
    property Formula : PChar read pcProgram;
  end;

  procedure ParseString(const S : string; Ss : TStrings);

implementation


procedure TExecStatements.DeleteLast;
begin
  Delete(Count-1);
end;

function TExecStatements.Last : TStatement;
begin
  if Count = 0 then Result := nil else
    Result := TStatement(Items[Count-1]);
end;

{------------------------ GOTO --------------------------}
{ GOTO}
procedure TGOTOS.AddGOTO(Name : TIdentifer; pcPos : PChar);
var
  GT : TGOTO;
begin
  if not GetPos(Name, pcPos) then begin
    GT := TGOTO.Create;
    GT.Name := Name;
    GT.pcPos := pcPos;
    Add(GT);
  end;
end;

function TGOTOS.GetPos(Name : TIdentifer; var pcPos : PChar) : boolean;
var i : integer;
begin
  Result := false;
  for i:=0 to Count-1 do
    if TGOTO(Items[i]).Name = Name then begin
      pcPos := TGOTO(Items[i]).pcPos;
      Result := true;{}
      exit;
    end;
end;

{************************** TProc ******************************}
constructor TProc.Create(lProcCom : TProcCom);
var
  IVar : TIVar;
  i : integer;
begin
  inherited Create;
  ProcCom   := lProcCom;
  FIObj     := ProcCom.IObj;
  if FIObj <> nil then FIObj._AddRef;
  Parser := TIParser.Create;
  pcProgram := ProcCom.pcProgram;
  OffsBeg   := ProcCom.OffsBeg;
  OffsEnd   := ProcCom.OffsEnd;
  IFS       := TIFS.Create;
  WHILES    := TWHILES.Create;
  Execs     := TExecStatements.Create;
  GOTOS     := TGOTOS.Create;
  VARS      := TIList.Create;
  PUnit     := ProcCom.PUnit;
  FArgs := ProcCom.Args;
  if FArgs = nil then FArgs := TIArgs.Create
  else FArgs._AddRef;
(*
 {$IFDEF DEBUG}
  FArgs.Debug := 'Args';
 {$ENDIF}
*)
  ObjSetGetValue := ProcCom.SetGetValue;

  Expres  := TCalc.Create;
  Expres.OnSetGetFunc := SetGetValue;
  Expres.OnGetLocalVar := VARS.GetVarByName;

  FName     := lProcCom.Name;
  Working   := false;
  Checking  := false;
  CheckVars := TStringList.Create;
  Res := TIValue.Create0;

  IVar := TIVar.Create0;
  IVar.Name := kwRESULT;
  VARS.Add(IVar);
  IVar._Release;
  IVar := TIVar.Create0;
  IVar.vObject := FArgs;
  IVar.Name := kwARGS;
  VARS.Add(IVar);
  IVar._Release;
  for i := 0 to FArgs.Count-1 do
    VARS.Add(FArgs[i]);
end;

destructor TProc.Destroy;
begin
  Expres.Destroy;
  IFS.Free;
  WHILES.Free;
  Execs.Free;
  GOTOS.Free;
  VARS._Release;
  CheckVars.Free;
  Res._Release;
  if FArgs <> nil then FArgs._Release;
  FArgs := nil;
  if FIObj <> nil then FIObj._Release;
  FIObj := nil;
  Parser.Free;
  inherited Destroy;
end;

procedure TProc.Clear;
begin
  Expres.Clear;
  IFS.Clear;
  Execs.Clear;
  WHILES.Clear;
  GOTOS.Clear;
  VARS.Clear;
  CheckVars.Clear;
  Working := false;
  FArgs._Release; FArgs := nil;
end;

procedure TProc.Runing;
var
  Token, Token1, Token2 : string;
  prEnd : boolean;
  State : (tsBeforeBegin, tsAfterBegin);
  SS : TStringList;
  procedure DebugStep(PosBeg, PosEnd : integer);
  begin
   {   }
    Self.PosBeg := PosBeg;
    Self.PosEnd := PosEnd-1;
    ProcCom.DebugProc; {  }
  end;

type
  TStopFunc = function(S : string) : boolean;

  procedure sExpression(Stop : TStopFunc; DebugPosBeg : integer);
  var
    P : integer;
  begin
    SS.Clear;
    P := Parser.PosBeg[0];
    while not (Stop(Token) or (Token = '')) do begin
      SS.Add(Token);
      Token := Parser.Token;
    end;
   {  }
    if ProcCom.Debug then DebugStep(DebugPosBeg, Parser.PosEnd[1]);
    Expres.SetExp(SS, 0, -1);
    try
      Expres.Solve;{    ,   Exception}
    except
      on E : EInterError do begin
        if E.Pos = -1 {E.ProcName = ''} then begin
          Parser.pcPos := pcProgram+P;
          while E.Op >= 0 do begin
            Parser.Token;
            dec(E.Op);
          end;
          E.Pos := Parser.PosBeg[0];
        end;
        raise;
      end;
    end;
    SS.Clear;
  end;

  procedure sIF;
    function FindIfStatements : TIF;
    begin
      Result := TIF.Create;
      Result.pcBeg := Parser.pcPos;
      try
       { THEN}
        while true do begin
          Token := Parser.Token;
          if Lang.Cmp(Token, kwTHEN) then begin
            Result.pcThen := Parser.pcPos;
            break;
          end else
          if Lang.Statement(Token) or (Token = '') then
            InterErrorP(ieThenNotFound, Result.pcBeg-Parser.pcProgram)
        end;{loop}
       { ELSE}
        while true do begin
          Token := Parser.Token;
          if Lang.Cmp(Token, kwELSE) then begin
            Result.pcElse := Parser.pcPos;
            break;
          end else
          if Lang.Cmp(Token, kwENDIF) then begin
            Result.pcEnd := Parser.pcPos;
            break;
          end else
          if Lang.Cmp(Token, kwIF) then
            FindIfStatements
          else
          if Lang.Statement(Token) or (Token = '') then
            InterErrorP(ieEndIfNotFound, Result.pcBeg-Parser.pcProgram)
          else
        end;{loop}
        if Result.pcEnd = nil then
         { ENDIF}
          while true do begin
            Token := Parser.Token;
            if Lang.Cmp(Token, kwENDIF) then begin
              Result.pcEnd := Parser.pcPos;
              break;
            end else
            if Lang.Cmp(Token, kwIF) then
              FindIfStatements
            else
            if Lang.Statement(Token) or (Token = '') then
              InterErrorP(ieEndIfNotFound, Result.pcBeg-Parser.pcProgram)
          end;{loop}
        IFS.Add(Result);
      except
        Result.Free;
        Result := nil;
      end;
    end;
  var
    IFF : TIF;
    P : integer;
    function Stop(S : string) : boolean;
    begin
      Result := Lang.Cmp(S, kwTHEN) 
    end;
  begin
    P := Parser.PosBeg[0];
    IFF := FindIfStatements;
    if IFF = nil then InterErrorS(ieUnknown);
    //  
    Parser.pcPos := IFF.pcBeg;
    Token := Parser.Token;
    sExpression(@Stop, P);
    if Expres.Res.vBoolean then
      Parser.pcPos := IFF.pcThen
    else
      if IFF.pcElse <> nil then Parser.pcPos := IFF.pcElse
                           else Parser.pcPos := IFF.pcEnd;
    if Parser.pcPos <> IFF.pcEnd then
      Execs.Add(IFF);                     
  end;

  procedure sDoWhile;
    function Stop(S : string) : boolean;
    begin
      Result := Lang.Cmp(S, kwDO)
    end;
  var
   kWHILE : TWHILE;
  begin
    kWHILE := Execs.Last as TWHILE;
    //  
    Parser.pcPos := kWHILE.pcBeg;
    Token := Parser.Token;
    sExpression(@Stop, kWHILE.iWhile);
    if Expres.Res.vBoolean then
      Parser.pcPos := kWHILE.pcDo
    else begin
      Parser.pcPos := kWHILE.pcEnd;
      Execs.DeleteLast;
    end;  
  end;

  procedure sWhile;
    function FindWhileStatements : TWHILE;
    begin
      Result := TWHILE.Create;
      Result.iWhile := Parser.PosBeg[0];
      Result.pcBeg := Parser.pcPos;
      try
       { DO}
        while true do begin
          Token := Parser.Token;
          if Lang.Cmp(Token, kwDO) then begin
            Result.pcDo := Parser.pcPos;
            break;
          end else
          if Lang.Statement(Token) or (Token = '') then
            InterErrorP(ieDoNotFound, Result.pcBeg-Parser.pcProgram)
        end;{loop}
       { ENDWHILE}
        while true do begin
          Token := Parser.Token;
          if Lang.Cmp(Token, kwENDWHILE) then begin
            Result.pcEnd := Parser.pcPos;
            break;
          end else
          if Lang.Cmp(Token, kwWHILE) then
            FindWhileStatements
          else
          if Lang.Statement(Token) or (Token = '') then
            InterErrorP(ieEndWhileNotFound, Result.pcBeg-Parser.pcProgram)
          else
        end;{loop}
        WHILES.Add(Result);
      except
        Result.Free;
        Result := nil;
      end;
    end;
  var
   kWHILE : TWHILE;
  begin
    kWHILE := FindWhileStatements;
    if kWHILE = nil then InterErrorS(ieUnknown);
    Execs.Add(kWHILE);
    sDoWhile;
  end;

  procedure sGoto(Name : TIdentifer);
  var
    bnFindLabel : boolean;
    P : integer;
  begin
    P := Parser.Pos;
    bnFindLabel := GOTOS.GetPos(Name, Parser.FpcPos);
    if not bnFindLabel then begin
     {    , ..    }
      Parser.pcPos := pcProgram+OffsBeg;
      bnFindLabel := false;
      while not bnFindLabel do begin
        Token := Parser.Token;
        if Lang.Cmp(Token, kwEND) or (Token = '') then begin
          prEnd := true;
          InterErrorPV(ieLabelNotFound, P, Name);
        end else
        if Lang.IsValidIdentifer(Token) then begin
          Token1 := Parser.Token;
          Token2 := Parser.Token;
          if (Token1 = ':') and (Token2 <> '=') then begin
            {     }
            Parser.RollBack(1);
            GOTOS.AddGOTO(Token, Parser.pcPos);
            if Cmp(Token, Name) then begin
             {   - ,  }
              bnFindLabel := true;
              break;
            end;
          end;
        end;
      end;{while}
    end;{if}
    if not bnFindLabel then
      InterErrorPV(ieLabelNotFound, P, Name);
    while (Execs.Last <> nil) and
          (Parser.pcPos < Execs.Last.pcBeg)
    do Execs.DeleteLast;
    while (Execs.Last <> nil) and
          (Parser.pcPos > Execs.Last.pcEnd)
    do Execs.DeleteLast;
  end;

  procedure sVar;
  {  }
    function Stop : boolean;
    begin
      Result := Lang.Cmp(Token, kwBEGIN);
    end;
  var
    NeedSeparator : boolean;
    CheckVars : TStrings;
    sTyp : TIdentifer;
    i : integer;
    IVar : TIVar;
  begin
    CheckVars := TStringList.Create;
    NeedSeparator := false;
    try
      repeat
        Token := Parser.Token;
        if Stop then break
        else
        if NeedSeparator then
          if (Token = ',') or (Token = ';') then NeedSeparator := false{next}
          else if Token = ':' then begin
           {  }
            sTyp := Parser.Token;
            if Lang.IsValidIdentifer(sTyp) then
              for i := 0 to CheckVars.Count-1 do begin
                IVar := TIVar.Create0;
                IVar.Name := CheckVars[i];
                IVar.VTyp := sTyp2IValueTyp(sTyp);
                IVar.Typ := IVar.VTyp;
                VARS.Add(IVar);
                IVar._Release;
              end
            else InterErrorPV(ieVarDeclName, Parser.Pos, sTyp);
            CheckVars.Clear;
          end
          else InterErrorP(ieSeparatorExpected, Parser.Pos)
        else if Lang.ReservedLangWord(Token) then InterErrorPV(ieVarDeclName, Parser.Pos, Token)
        else if Lang.IsIdentifer(Token) then begin
         {***     }
          if (CheckVars.IndexOf(Token)<> -1) or (VARS.GetVarByName(Token) <> nil)
          then InterErrorPV(ieIdentiferRedeclared, Parser.PosBeg[0], Token);
          CheckVars.Add(Token);
          NeedSeparator := true;
         {###     }
        end
        else if Token = ':' then
          InterErrorP(ieIdentiferExpected, Parser.PosBeg[0])
        else InterErrorP(ieSyntax, Parser.PosBeg[0]);
      until false;{loop}
    finally
      CheckVars.Free;
    end;
    Parser.RollBack(1);
  end;

  procedure sFunc;
    function Stop(S : string) : boolean;
    begin
      Result := S = ';'
    end;
  begin
    sExpression(@Stop, Parser.PosBeg[0]);
  end;

  procedure sBeforeBegin;
  begin
    { }
    if Lang.Cmp(Token, kwVAR) then
      sVar
    else if Lang.Cmp(Token, kwBEGIN) then begin
     {  }
      if ProcCom.Debug then DebugStep(Parser.PosBeg[0], Parser.PosEnd[0]);
      State := tsAfterBegin;
    end else InterErrorP(ieBegin, Parser.Pos)
  end;{BeforeBegin}

  procedure sAfterBegin;
  {var
    Offs : integer;}
  begin
    if Token = ';' then {continue}
    else if Lang.IsCalcIdentifer(Token) then begin
      Token1 := Parser.Token;
      Token2 := Parser.Token;
      if (Token1 = ':') and (Token2 <> '=') then begin
       {}
        Parser.RollBack(1);
        GOTOS.AddGOTO(Token, Parser.pcPos);
      end else begin
       {}
        Parser.RollBack(2);
        sFunc;
      end;
    end else if Lang.Cmp(Token, kwIF) then
      sIf
    else if Lang.Cmp(Token, kwELSE) then begin
      Parser.pcPos := Execs.Last.pcEnd;
      Execs.DeleteLast
    end else if Lang.Cmp(Token, kwENDIF) then
      Execs.DeleteLast
    else if Lang.Cmp(Token, kwWHILE) then
      sWhile
    else if Lang.Cmp(Token, kwENDWHILE) then
      if (Execs.Last = nil) or not (Execs.Last is TWHILE) then
        InterErrorP(ieSyntax, Parser.PosEnd[1])
      else begin
       {  }
        if ProcCom.Debug then DebugStep(Parser.PosBeg[0], Parser.PosEnd[0]);
        sDoWhile;
      end
    else if Lang.Cmp(Token, kwGOTO) then
     {GoTo}
      if Lang.IsIdentifer(Parser.Token) then begin
        if Parser.Token <> ';' then
          InterErrorP(ieSeparatorExpected, Parser.PosEnd[1]);
        {  }
        if ProcCom.Debug then DebugStep(Parser.PosBeg[2], Parser.PosEnd[1]);
        sGoto(Parser.History[1]);
      end else InterErrorP(ieLabel, Parser.PosBeg[0])
    else begin
   {!!!      
              !!!}
    {  Offs := Parser.Pos;
      if not (IObj.Translate(self, Offs) and (Offs <> Parser.Pos)) then }
        InterErrorP(ieSyntax, Parser.Pos);
    end;
  end;{AfterBegin}

{____________  _______________ Runing _____________ ____________}
begin
  prEnd := false;
  Parser.pcProgram := pcProgram;
  Parser.pcPos := Parser.pcProgram+OffsBeg;
  State := tsBeforeBegin;
  SS := TStringList.Create;
  try
  {    -  }
  if StrScan(pcProgram+OffsBeg, ';') = nil then begin
    Token := Parser.Token;
    if Token = '' then InterErrorS(ieFormulaEmpty);
//    if Token = '' then begin Res.vInteger := 0; exit end;{!!!}
    sFunc;
    IIValueAssign(VARS.GetVarByName(kwRESULT), Expres.Res);
  end else
  repeat
    if ProcCom.IInterObject.IsTerminated then InterErrorS(ieUserBreak);
    Token := Parser.Token;
    if Token = '' then InterErrorP(ieEndUnit, Parser.Pos)
    else
    if Lang.Cmp(Token, kwENDFUNC) then prEnd := true
    else if Lang.Cmp(Token, kwENDUNIT) then
      InterErrorP(ieEndFunc, Parser.Pos)
    else
      case State of
        tsBeforeBegin : sBeforeBegin;
        tsAfterBegin  : sAfterBegin;
      end;
  until prEnd;
 {  }
  if ProcCom.Debug then DebugStep(Parser.PosBeg[0], Parser.PosEnd[0]);
  finally
    SS.Free;
  end;
end;

procedure TProc.Solve;
begin
    if Working then InterErrorS(ieUnknown);
    Working := true;
    DecimalSeparator := '.';
  try
    Runing;
    IIValueAssign(Res, VARS.GetVarByName(kwRESULT));
   {Line := 0; Line := 1 div Line;  {    }
  except
    on E : EInterError do begin
      {Result := 'EInterError';}
      if E.Pos = -1 {E.ProcName = ''} then begin
        E.ProcName := Name;
        E.Pos := 0;//?
      end;
      Clear;
      raise;
    end;
    on E : Exception do begin
      Clear;
      raise;
    end;
  end;
  Clear;
end;

procedure TProc.Check;
begin
  Checking := true;
  Expres.Checking := true;
  try
    Solve;
  finally
    Expres.Checking := false;
    Checking := false;
  end;
end;

function TProc.DebugEvaluate(Expression : string; var Value : string; var SGIDebugFlags : TSGIDebugFlags) : boolean;
var
  Parser : TIParser;
  SS : TStrings;
  OldSGIDebugFlags : TSGIDebugFlags;
begin
  Result := false;
  Value := '  Evaluate';
  Parser := TIParser.Create;
  SS := TStringList.Create;
  OldSGIDebugFlags := Expres.SGIDebugFlags;
  try
  Parser.pcPos := PChar(Expression);
  try
    while Parser.Token <> '' do SS.Add(Parser.History[0]);
    if SS.Count > 0 then begin
      Expres.SetExp(SS, 0, -1);
      Expres.SGIDebugFlags := SGIDebugFlags;
      Expres.Solve;
      Value := Expres.Res.Str;
      if IIValueIsConstant(Expres.Res) then Result := true;
    end else Value := '';
  except
    on E : EInterError do Value := E.Message;//'  ';
  end;
  finally
    Expres.SGIDebugFlags := OldSGIDebugFlags;
    Parser.Free;
    SS.Free;
  end;
end;

function TProc.SetGetValue(var SGI : TSetGetInfo) : boolean;
var
  IVar : IIVar;
begin
  if SGI.IObjOut = nil then begin
    pointer(IVar) := nil;
    IVar := VARS.GetVarByName(SGI.Name);
    Result := IVar <> nil;
    if Result then
      if SGI.Get then IIValueAssign(SGI.Res, IVar) else IIValueAssign(IVar, SGI.Res);
    if Result then exit else begin SGI.IObjOut := FIObj; SGI.IObjOut._Release{Delphi - }; end;
  end;
  if Assigned(ObjSetGetValue) then
    Result := ObjSetGetValue(PUnit, SGI)
  else Result := false;
end;
{######################## TProc ##########################}

{         
  -1  -1,      }
procedure GetBlockByPos(pcProgram : iString; Pos : integer; var PosBeg, PosEnd : integer);
var
  Lexems : TStringList;
  LexemsPos : TList;
  lOp : integer;
  OpBeg, OpEnd : integer;

  function FillLexems : boolean;
  var
    Parser : TIParser;
    Token : string;
    P : PChar;
  begin
    Result := false;
    Parser := TIParser.Create;
    Lexems.Clear; LexemsPos.Clear;
    try
      Parser.pcProgram := pcProgram;
      Parser.pcPos := pcProgram;
      P := nil; lOp := -1;
      repeat
        Token := Parser.Token;
        if Token = ';' then P := Parser.pcPos;
      until (Parser.Pos > Pos) or (Token = '');
      if (P <> nil) and (Token <> '') then begin
        Parser.pcPos := P;
        Token := Parser.Token;
        while (Token <> '') and (Token <> ';') do begin
          Lexems.Add(Token);
          LexemsPos.Add(pointer(Parser.PosBeg[0]));
          if (Pos >= Parser.PosBeg[0]) and
             (Pos <= Parser.PosBeg[0]+Length(Token))
          then lOp := Lexems.Count-1;  
          Token := Parser.Token;
        end;
        if lOp > -1 then Result := true;
      end;
    finally
      Parser.Free;
    end;
  end;

  procedure GetOperatorBegEnd;
  var
    i : integer;
    Lex : string;
    function CmpL(LangKW : TLangKW) : boolean;
    begin
      Result := Lang.Cmp(Lex, LangKW);
    end;
  begin
    OpBeg := -1; OpEnd := -1;
    Lex := Lexems[lOp];
   {  }
    if CmpL(kwTHEN) or
       CmpL(kwELSE) or
       CmpL(kwBEGIN) or
       CmpL(kwUSES) or
       CmpL(kwFUNC) or
       CmpL(kwDO) or
       CmpL(kwVAR)
    then exit;

    OpBeg := 0;
    if lOp >= 0 then begin
      for i := lOp downto 0 do begin
      { ,     }
      {  if , := , goto , begin , then, endif , end
       var , uses -    }
        Lex := Lexems[i];
        if CmpL(kwIF) or
           CmpL(kwWHILE)
        then begin OpBeg := i; break end
        else if CmpL(kwTHEN) or
                CmpL(kwELSE) or
                CmpL(kwDO)
        then begin OpBeg := i+1; break end
        else if CmpL(kwBEGIN) then begin OpBeg := i+1; break end
        else if CmpL(kwENDIF) or
                CmpL(kwEND) or
                CmpL(kwENDWHILE)
        then begin OpBeg := i; break end
        else if CmpL(kwGOTO)
        then begin OpBeg := i; break end
        { }
        else if CmpL(kwUSES) or
                CmpL(kwFUNC) or
                CmpL(kwVAR)
        then exit
        {#}
        else if (i = 0) and Lang.IsIdentifer(Lex)
        then begin OpBeg := i; break end
      end;
    end else OpBeg := 0;
    if lOp <= Lexems.Count-1 then begin
      OpEnd := 0;
      for i := lOp to Lexems.Count-1 do begin
        Lex := Lexems[i];
      { ,    }
      { then, :
        else , endif,  end - }
        if CmpL(kwTHEN) or
           CmpL(kwELSE) or
           CmpL(kwDO) or
           CmpL(kwENDWHILE)
        then begin OpEnd := i-1; break end
      end;
      if OpEnd = 0 then OpEnd := Lexems.Count-1;
    end;
    if OpEnd < OpBeg then OpEnd := OpBeg;//!
  end;

begin
  PosBeg := -1; PosEnd := -1;
  Lexems := TStringList.Create;
  LexemsPos := TList.Create;
  try
    if FillLexems then begin
      GetOperatorBegEnd;
      if (OpBeg >= 0) and (OpEnd >= 0) then begin
        PosBeg := integer(LexemsPos[OpBeg]);
        PosEnd := integer(LexemsPos[OpEnd]) + Length(Lexems[OpEnd]);
      end;
    end;  
  finally
    Lexems.Free;
    LexemsPos.Free;
  end;
end;

{*************************** TIParser ****************************}
constructor TIParser.Create;
begin
  inherited Create;
  FHistory := TStringList.Create;
  HistorySize := 10;
end;

destructor TIParser.Destroy;
begin
  FHistory.Free;
  inherited Destroy;
end;

function TIParser.Token : string;
var
  Buf : array [byte] of char;
  P, F : PChar;

  procedure Skip;
  begin
    case P[0] of
      '{' :
        begin
          F := StrScan(P+1, '}');
          if F = nil then InterErrorS(ieBadRemark);
          P := F+1;
        end;
      '(' :
        if (P[1] = '*') then begin
          F := P+2;
          while true do begin
            F := StrScan(F, '*');
            if F = nil then InterErrorS(ieBadRemark);
            if F[1] = ')' then begin inc(P); break; end;
            inc(F);
          end;
          P := F+1;
        end;
      '}' : InterErrorS(ieBadRemark);
      '*' : if (P[1] = ')') then InterErrorS(ieBadRemark);
      '/' :
        if (P[1] = '/') then begin
          F := StrScan(P+1, #13);
          if F = nil then F := StrEnd(P+1);
          P := F;
        end;
    end;
    while (P[0] in [' ', #10, #13]) do inc(P);
  end;

var
  F1 : PChar;
  i : integer;
begin
 { New Token -     }
  F := FpcPos;
  P := FpcPos;
 { Firstly skip spaces and remarks }
  repeat
    F1 := P;
    Skip;
  until F1 = P;
  F := P;
  if (P[0] in RAIIdFirstSymbols) then
   { token }
  begin
    while (P[0] in RAIIdSymbols) do inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
  end else
  if (P[0] in RAIConstSymbols10) then
   { number }
  begin
    while (P[0] in RAIConstSymbols10) or (P[0] = '.') do inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
  end else
  if ((P[0] = '$') and (P[1] in RAIConstSymbols)) then
   { hex number }
  begin
    inc(P);
    while (P[0] in RAIConstSymbols) do inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
  end else
  if P[0] = '''' then
   { string constant -   }
  begin
    inc(P);
    while P[0] <> #0 do begin
      if (P[0] = '''') then
        if (P[1] = '''') then inc(P) else break;
      inc(P);
    end;
    inc(P);
    StrLCopy(Buf, F, P-F);
    Result := StrPas(Buf);
    i := 2;
    while i < Length(Result) -1 do begin
      if Result[i] = '''' then
        Delete(Result, i, 1);
      inc(i);
    end;
  end else
  if P[0] = #0 then
    Result := ''
  else begin
    Result := P[0];
    inc(P);
  end;
  
  FpcPos := P;
  FHistory[FHistoryPtr] := Result;
  FHistory.Objects[FHistoryPtr] := TObject(Pos-1);
  inc(FHistoryPtr);
  if FHistoryPtr > FHistorySize-1 then FHistoryPtr := 0;
end;

function TIParser.HistoryInd(index : integer) : integer;
begin
  Result := FHistoryPtr-1-index;
  if Result < 0 then Result := Result + FHistorySize;
end;

function TIParser.GetHistory(index : integer) : string;
begin
  Result := FHistory[HistoryInd(index)];
end;

function TIParser.GetPosEnd(index : integer) : integer;
begin
  Result := integer(FHistory.Objects[HistoryInd(index)])+1;
end;

function TIParser.GetPosBeg(index : integer) : integer;
var
  i : integer;
begin
  i := HistoryInd(index);
  Result := integer(FHistory.Objects[i])-Length(FHistory[i])+1;
end;

procedure TIParser.SetHistorySize(Size : integer);
{$IFDEF DEBUG}
var i : integer;
{$ENDIF}
begin
  while Size > FHistorySize do begin
    FHistory.Add('');
    inc(FHistorySize);
  end;
  while Size < FHistorySize do begin
    FHistory.Delete(0);
    dec(FHistorySize);
  end;
 {$IFDEF DEBUG}
  for i := 0 to FHistorySize-1 do FHistory[i] := '';
 {$ENDIF}
  FHistoryPtr := 0;
end;

function TIParser.GetPos : integer;
begin
  Result := pcPos - FpcProgram;
end;

procedure TIParser.RollBack(index : integer);
begin
  FpcPos := PosEnd[index]+FpcProgram;
  dec(FHistoryPtr, index);
  if FHistoryPtr < 0 then
    FHistoryPtr := FHistorySize+FHistoryPtr;
end;

{########################### TIParser ###########################}

procedure ParseString(const S : string; Ss : TStrings);
var
  Parser : TIParser;
  Token : string;
begin
  Ss.Clear;
  Parser := TIParser.Create;
  try
    Parser.pcProgram := PChar(S);
    Parser.pcPos := Parser.pcProgram;
    Token := Parser.Token;
    while Token <> '' do
    begin
      Ss.Add(Token);
      Token := Parser.Token;
    end;
  finally
    Parser.Free;
  end;  
end;


end.

