{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAIProg;

{ program evaluator }

interface

uses Windows, Classes, SysUtils,
  RAIIntf, RAIClas, RAICalc, RAIProc;

type


{*********************** TInter **********************}

  TNObject = class
  public
    Name : TIdentifer;
  end;

  TINObject = class(TIObject)
  public
    Name : TIdentifer;
  end;

  TINList = class(TIObject)
  {   ( )   
    TNObject  TINObject}
  private
    List : TList;
    FSorted : boolean;
    function GetObject(index : integer) : TObject;
    function GetObjByName(lName : TIdentifer) : TObject;
    function Find(const S: string; var Index: Integer): Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function Count : integer;
    function Add(Obj : TObject) : integer;
    procedure Sort;
    property Objects[index : integer] : TObject read GetObject; default;
    property Objs[index : string] : TObject read GetObjByName;
  end;

  TPUnit  = class;
  TPUnits = class;
  TCallStack = TList;

  TInter = class(TInterfacedObject, IIInter, IIDebugger, IICallStack)
  private
    DebugStopped : boolean;
    RunProcs    : TCallStack;
    FuncLevel : integer;
    ProcCom   : TProcCom;
    Units    : TPUnits;
    CurUnit   : TPUnit;
    Lexems    : TStringList;
    FError     : TProgError;
    RunParams : TRunParams;
    IInterObject : IIInterObject;
    IDebugObject : IIDebugObject;
    ITranslator  : IITranslator;
    IObjectInfo  : IIObjectInfo;
    IObjectData  : TIObjectData;
    sResult : string; {  }
    function SetGetVar(PUnit : TObject; SGI : TSetGetInfo) : boolean;
    procedure ProcRun;
    procedure DebugProc; {  }
    procedure SetDebugMode(ADebugMode : boolean);
    function CurProc : TProc;
    function EvaluateModify(Watch : iString; var Value : iString; var SGIDebugFlags : TSGIDebugFlags) : boolean;
    function IICallStack.Count = GetCallStackCount;
    function IICallStack.GetItem = GetCallStackItem;
    function GetCallStack : IICallStack;
    function GetCallStackCount : integer;
    function GetCallStackItem(index : integer) : TCallStackItem;
  public

    constructor Create;
    destructor Destroy; override;
    procedure Clear; { }
    procedure Run(AIInterObject : IIInterObject);
    property Error : TProgError read FError;
  end;

{####################### TInter ######################}

{**************************** Classes *****************************}

  {    }
  TPProp = class(TNObject)
  public
   // Name : TIdentifer;
    VTyp : TIValueTyp;
    sTyp : TIdentifer;
    IValue : TIValue; {=nil,  Class   }
    GetFunc,
    SetFunc  : PChar; {  = nil,  Name -  }
    constructor Create;
  end;

  { }
  TPClass = class(TINObject)
  private
    FProps : TINList;
   // Name : TIdentifer; { }
   // Pred : TIdentifer; {  }
  protected
    {    ,  .. }
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  public
    constructor Create;
    destructor Destroy; override;
    property Props : TINList read FProps;
  end;

  {  }
  TPClasses = class(TINList)
  protected
    {     }
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

  {  }
  TClassObject = class(TIList)
  private
    ClassInfo : TPClass;
  protected
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

 {**************** Units ****************}
  TFParam = class(TNObject)
  public
    //Name : TIdentifer;
    sTyp : TIdentifer;
  end;

  TFParams = class(TINList)
  private
    function GetFParam(index : integer) : TFParam;
  public
    property FParams [index : integer] : TFParam read GetFParam; default;
  end;

  TCustomProc = class(TNObject)
  public
    // Name : TIdentifer;
    Params : TFParams; //
    sTyp : TIdentifer; // 
    ResTyp : TIValueTyp;
    constructor Create;
    destructor Destroy; override;
    procedure CheckParams(Args : IIArgs);
  end;

  TUnitProc = class(TCustomProc)
  public
    pcProgram : PChar;   { }
    OffsBeg, OffsEnd : integer; {   }
  end;

  TDllProc = class(TCustomProc)
  public
    Lib  : TIdentifer;
    LibName : TIdentifer;
     {or}
    LibIndex : integer;
  end;

  TPProcs = class(TINList)
  private
    Owner : TPUnit;
  protected
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
    function GetProc(index : integer) : TCustomProc;
  public
    property Procs [index : integer] : TCustomProc read GetProc; default;
  end;

  TIConsts = class(TINList)
  protected
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

  TPUnit = class
  {   ,     .
          ,   .
     .
    ..        
      ,       }
   //      ...
  private
    pcProgram : PChar;
    Parser : TIParser;
    Classes : TPClasses;
    FName : TIdentifer;  { }
    Formula  : string;    { }
    FUsesUnits : TStrings; {   .
      TInter      GetValue, SetValue }
    SysUsesUnits : TStrings;
    Units : TPUnits;
    FProcs : TPProcs;
    Consts : TIConsts;
    IInterObject : IIInterObject; {.     }
    ITranslator  : IITranslator;
    FSysObjs : TList {of TIObject}; //      
    Expres  : TCalc;
    {**** }
    ProcCom : TProcCom;
    {#### }
    function ConstGetLocalVar(lName : TIdentifer) : IIVar;
    procedure Translating;
    function Save : boolean;
    procedure LoadUsesUnits;
  public
    constructor Create(AUnits : TPUnits; AIInterObject : IIInterObject; AUnitName : string; AFormula : iString);
    destructor Destroy; override;
    procedure Translate;
    function Load : boolean;
    function SetGetVar(var SGI : TSetGetInfo) : boolean;
    function Declared(Id : TIdentifer) : boolean;
    property Name : TIdentifer read FName;   {}
    property Procs : TPProcs read FProcs;
   {**   }
    property pcFormula : PChar read pcProgram;
    property SysObjs : TList read FSysObjs; {     }
    property UsesUnits : TStrings read FUsesUnits; {   .}
   {##}
  end;

  TPUnits = class(TList)
  private
    Owner : TInter;
    function GetUnit(index : integer) : TPUnit;
  public
    constructor Create(AOwner : TInter);
   {$IFDEF RA_D4H}
    procedure Clear; override;
   {$ELSE}
    procedure Clear;
   {$ENDIF RA_D4H}
    function GetUnitByName(lName : TIdentifer) : TPUnit;
    function SetGetVar(PUnit : TPUnit; SGI : TSetGetInfo; CheckUnitName : boolean) : boolean;
    function LoadUnit(lUnit : TIdentifer) : boolean;
    property Units [index : integer] : TPUnit read GetUnit; default;
  end;

  function CallDll(DllName, FuncName : string; Args : IIArgs; ResTyp : TIValueTyp; var iRes : longint) : boolean;
  function CallDllIns(Ins : HINST; FuncName : string; Args : IIArgs; ResTyp : TIValueTyp; var iRes : longint) : boolean; forward;

implementation

{******** TNList}
function TINListGetName(Obj : TObject) : TIdentifer;
begin
  if Obj is TINObject then
    Result := (Obj as TINObject).Name
  else if Obj is TNObject then
    Result := (Obj as TNObject).Name
  else if Obj is TIVar then
    Result := (Obj as TIVar).Name;
end;

constructor TINList.Create;
begin
  inherited Create;
  List := TList.Create;
end;

destructor TINList.Destroy;
begin
  Clear;
  List.Free;
  inherited Destroy;
end;

procedure TINList.Clear;
var
  i : integer;
begin
  for i := 0 to List.Count-1 do
    if Objects[i] is TIObject then
      (Objects[i] as TIObject)._Release
    else Objects[i].Free;
  List.Clear;
end;

function TINList.Count : integer;
begin
  Result := List.Count;
end;

function TINList.GetObject(index : integer) : TObject;
begin
  Result := TObject(List[index]);
end;

function TINList.GetObjByName(lName : TIdentifer) : TObject;
var
  i, F : integer;
begin
  if FSorted then
    if Find(lName, F) then Result := Objects[F] else Result := nil
  else begin
    Result := nil;
    for i := 0 to Count-1 do
      if Cmp(TINListGetName(Objects[i]), lName) then begin
        Result := Objects[i];
        break;
      end;
  end;
end;

function TINList.Add(Obj : TObject) : integer;
begin
  Result := List.Add(Obj);
  FSorted := false;
end;

procedure TINList.Sort;
  function ListSortCompare(Item1, Item2: Pointer): Integer;
  begin
    Result := ANSICompareText(TINListGetName(TObject(Item1)),
                              TINListGetName(TObject(Item2)));
  end;
begin
  List.Sort(@ListSortCompare);
  FSorted := true;
end;

function TINList.Find(const S: string; var Index: Integer): Boolean;
{from Classes.TStringList.Find}
var
  L, H, I, C: Integer;
begin
  Result := False;
  L := 0;
  H := List.Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    C := AnsiCompareText(TINListGetName(Objects[I]), S);
    if C < 0 then L := I + 1 else
    begin
      H := I - 1;
      if C = 0 then
      begin
        Result := True;
       // if Duplicates <> dupAccept then L := I;
      end;
    end;
  end;
  Index := L;
end;
{######## TNList}

{**************************** Classes *****************************}
constructor TPProp.Create;
begin
  VTyp := ivVariant;
end;

constructor TPClass.Create;
begin
  inherited Create;
  FProps := TINList.Create;
end;

destructor TPClass.Destroy;
begin
  FProps._Release;
  inherited Destroy;
end;

function TPClass.SetGetVar(var SGI : TSetGetInfo) : boolean;
{  }
var
  Obj : TClassObject;
  i : integer;
  IVar : TIVar;
begin
  Result := true;
  if Cmp(SGI.Name, 'Create') then begin
    Obj := TClassObject.Create;
    Obj.ClassInfo := self;
    for i := 0 to FProps.Count-1 do begin
      IVar := TIVar.Create0;
      IVar.Name := (FProps[i] as TPProp).Name;
      IVar.VTyp := (FProps[i] as TPProp).VTyp;
      IVar.Typ := IVar.VTyp;
      Obj.Add(IVar);
      IVar._Release;
    end;
    SGI.Res.vObject := Obj;
    Obj._Release;
  end else begin
    {   }
   InterErrorS(ieNotImplemented);
   // Result := false;
  end;
end;

function TPClasses.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  PClass : TPClass;
begin
  Result := false;
  PClass := Objs[SGI.Name] as TPClass;
  if PClass <> nil then begin
    Result := true;
    SGI.Res.vClass := iString(PClass.Name);
  end else
  if (SGI.Args.Count > 0) and (SGI.Args[0].Typ = ivClass)
  then begin
   { }
    PClass := Objs[SGI.Args[0].vClass] as TPClass;
    if PClass <> nil then begin
      SGI.Args.Delete(0);
      Result := PClass.SetGetVar(SGI)
    end;
  end
end;

function TClassObject.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  IVar : IIVar;
begin
  Result := false;
  if Cmp(SGI.Name, 'Free') then begin
    Result := true;
    SGI.Res.Empty;
    //_Release; //!!!!!!!!!!!
    exit;
  end;
  {if not Result then}
  {  }
  pointer(IVar) := nil;
  IVar := GetVarByName(SGI.Name);
  if IVar <> nil then begin
    if SGI.Get then
      IIValueAssign(SGI.Res, IVar)
    else
      IIValueAssign(IVar, SGI.Res);
    Result := true;
  end;
  {    }
  //..
  if not Result then
    Result := ClassInfo.SetGetVar(SGI); {  }
end;
{############################ TPClasses #############################}

function TFParams.GetFParam(index : integer) : TFParam;
begin
  Result := TFParam(Objects[index]);
end;

constructor TCustomProc.Create;
begin
  inherited Create;
  Params := TFParams.Create;
end;

destructor TCustomProc.Destroy;
begin
  if Params <> nil then Params._Release;
  inherited Destroy;
end;

procedure TCustomProc.CheckParams(Args : IIArgs);
var
  i : integer;
begin
  for i := 0 to Args.Count-1 do
    if i <= Params.Count-1 then
      Args[i].Name := iString(Params[i].Name)
    else
      Args[i].Name := iString('Arg'+IntToStr(i));//Debug !!!!!
end;

function TPProcs.GetProc(index : integer) : TCustomProc;
begin
  Result := TCustomProc(Objects[index]);
end;

function TPProcs.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  DllProc : TDllProc;
  UnitProc : TUnitProc;
  Pr : TCustomProc;
  iRes : integer;
begin
  Result := false;
  if not SGI.Get then exit;
  Pr := Objs[SGI.Name] as TCustomProc;
  if Pr = nil then exit;
  Result := true;
 {*   Args  Pr.Params}
  Pr.CheckParams(SGI.Args);
 {#}
  if Pr is TUnitProc then begin
    UnitProc := TUnitProc(Pr);
    SGI.Res.vFormula := UnitProc.pcProgram;
    Owner.ProcCom.OffsBeg  := UnitProc.OffsBeg; { }
    Owner.ProcCom.OffsEnd  := UnitProc.OffsEnd; { }
    Owner.ProcCom.UnitName := Owner.Name;       { }
    Owner.ProcCom.PUnit    := Owner;            { }
    //,   ,      ProcRun FormulaOffs
  end else begin
    DllProc := TDllProc(Pr);
   {   Dll}
    CallDll(DllProc.Lib, DllProc.LibName, SGI.Args, DllProc.ResTyp, iRes);
    SGI.Res.vInteger := iRes;
  end;
end;


type
  TIPUnit = class(TIObject)
  private
    PUnit : TPUnit;
    IObj  : IIObject;
  protected
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

{**********************     *****************}
function TIConsts.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  IVar : TIVar;
begin
  IVar := Objs[SGI.Name] as TIVar;
  if IVar <> nil then begin
    IIValueAssign(SGI.Res, IVar);
    Result := true;
  end else Result := false
end;

{######################     #################}


constructor TPUnit.Create(AUnits : TPUnits; AIInterObject : IIInterObject; AUnitName : string; AFormula : iString);
begin
  inherited Create;
  Parser := TIParser.Create;
  Units := AUnits;
  IInterObject  := AIInterObject;
  FName     := AUnitName;
  Formula   := AFormula;
  Classes   := TPClasses.Create;
  FUsesUnits := TStringList.Create;
  SysUsesUnits := TStringList.Create;
  FProcs    := TPProcs.Create;
  Consts    := TIConsts.Create;;
  FProcs.Owner := self;
  FSysObjs  := TList.Create;
  Expres  := TCalc.Create;
  Expres.OnGetLocalVar := ConstGetLocalVar;
end;

destructor TPUnit.Destroy;
var
  i : integer;
begin
  Classes._Release;
  FUsesUnits.Free;
  SysUsesUnits.Free;
  FProcs._Release;
  Consts._Release;
  IInterObject := nil;
  for i := 0 to FSysObjs.Count-1 do TIObject(FSysObjs[i])._Release;
  FSysObjs.Free;
  Parser.Free;
  Expres.Free;
  inherited Destroy;
end;

procedure TPUnit.Translating;
type
 {       CallBack}
  TCallBackInfo = record
    PClass : TPClass;
    CProc : TCustomProc;
  end;
  PCallBackInfo = ^TCallBackInfo;
  TFuncStr = function(CBI : PCallBackInfo; S : string) : boolean;
var
  Token : string;
  prEnd : boolean;
  Names, sTyps : TStrings;
  CBI : TCallBackInfo;

  function GetConstValue(Sep1, Sep2 : string) : string;
    function CmpSep(S : string) : boolean;
    begin
      Result := Cmp(S, Sep1) or Cmp(S, Sep2)
    end;
  var
    SS : TStringList;
    Token1 : string;
    P : integer;
  begin
    Token1 := Parser.Token;
    P := Parser.PosBeg[0];
    Token := Parser.Token;
    {     ,     Calc  }
    if CmpSep(Token) and Lang.IsConstant(Token1) then
      Result := Token1
    else begin
      SS := nil;
      try
        if (Token = '') or (Token1 = '') or CmpSep(Token1)
        then InterErrorP(ieConstantExpected, P);
        SS := TStringList.Create;
        SS.AddObject(Token1, TObject(P));
        while not CmpSep(Token) do begin
          if Lang.ReservedWord(Token) then InterErrorP(ieSyntax, P)
          else if Token = ';' then InterErrorPV(ieExpected, P, 'Name');
          SS.AddObject(Token, TObject(P));
          Token := Parser.Token;
          P := Parser.PosBeg[0];
          if Token = '' then InterErrorP(ieSyntax, Parser.Pos)
        end;
        Expres.SetExp(SS, 0, -1);
        try
          Expres.Solve;
        except
          on E : EInterError do begin
            if E.Pos = -1 then
              E.Pos := integer(SS.Objects[E.Op]);
            raise;
          end;
        end;
        Result := Expres.Res.Str;
      finally
        SS.Free;
      end;
    end;
  end;

  procedure VarsDeclare(Decl, Stop : TFuncStr);
  {  }
  var
    NeedSeparator : boolean;
    CheckVars : TStrings;
    sTyp : TIdentifer;
    j : integer;
  begin
    Names.Clear; sTyps.Clear;
    CheckVars := TStringList.Create;
    NeedSeparator := false;
    try
      repeat
        Token := Parser.Token;
        if Stop(@CBI, Token) then break
        else
        if NeedSeparator then
          if (Token = ',') or (Token = ';') then NeedSeparator := false{next}
          else if Token = ':' then begin
           {  }
            sTyp := Parser.Token;
            if Lang.IsValidIdentifer(sTyp) then begin
              Names.AddStrings(CheckVars);
              for j := 0 to CheckVars.Count-1 do sTyps.Add(sTyp);
            end else InterErrorPV(ieVarDeclName, Parser.Pos, sTyp);
            CheckVars.Clear;
          end
          else InterErrorP(ieSeparatorExpected, Parser.Pos)
        else if Lang.ReservedLangWord(Token) then InterErrorPV(ieVarDeclName, Parser.Pos, Token)
        else if Lang.IsIdentifer(Token) then begin
         {***     }
          if (CheckVars.IndexOf(Token)<> -1) or (Names.IndexOf(Token)<> -1)
             or Decl(@CBI, Token)
          then InterErrorPV(ieIdentiferRedeclared, Parser.PosBeg[0], Token);
          CheckVars.Add(Token);
          NeedSeparator := true;
         {###     }
        end
        else if Token = ':' then
          InterErrorP(ieIdentiferExpected, Parser.PosBeg[0])
        else InterErrorP(ieSyntax, Parser.PosBeg[0]);
      until false;{loop}
    finally
      CheckVars.Free;
    end;
  end;

  function FuncDeclare(Decl, Stop : TFuncStr) : TCustomProc;
  { }
  var
    Token1 : string;
    i : integer;
    UProc : TUnitProc;
    DllProc : TDllProc;
    P : PChar;
  begin
    {}
    UProc := TUnitProc.Create;
    CBI.CProc := UProc;
    try
      UProc.Name := Parser.Token;
      if not Lang.IsValidIdentifer(UProc.Name) then
        InterErrorP(ieIdentiferExpected, Parser.Pos);
      Token := Parser.Token;
      if Token = '(' then begin
       { }
       {*****   }
        VarsDeclare(Decl, Stop);
        for i := 0 to Names.Count-1 do begin
          UProc.Params.Add(TFParam.Create);
          UProc.Params[UProc.Params.Count-1].Name := Names[i];
          UProc.Params[UProc.Params.Count-1].sTyp := sTyps[i];
        end;
        if Parser.History[1] = ';' then
          InterErrorP(ieIdentiferExpected, Parser.PosBeg[0]);
       {#####   }
        Token := Parser.Token;
      end;
      if Token = ':' then begin
       { }
        UProc.sTyp := Parser.Token;
        if not Lang.IsValidIdentifer(UProc.sTyp) then
          InterErrorP(ieIdentiferExpected, Parser.Pos);
        UProc.ResTyp := sTyp2IValueTyp(UProc.sTyp);
        Token := Parser.Token;
      end;
      if Token <> ';' then
        InterErrorP(ieTZExpected, Parser.PosEnd[1]);
      P := Parser.pcPos;
      Token := Parser.Token;
      if Lang.Cmp(Token, kwExternal) then begin
        { }
        DllProc := TDllProc.Create;
        try
          DllProc.Params._Release;
          DllProc.Params := UProc.Params; UProc.Params := nil;
          DllProc.Name   := UProc.Name;
          DllProc.ResTyp := UProc.ResTyp;
          DllProc.Lib := Lang.GetString(GetConstValue('Name', 'Index'));
          if Cmp(Token, 'Name') then begin
           {    }
            Token1 := GetConstValue(';', ';');
            if not Lang.IsStringConstant(Token1) then
              InterErrorP(ieConstantExpected, Parser.Pos);
            DllProc.LibName := Lang.GetString(Token1)
          end else if Cmp(Token, 'Index') then begin
           {    }
            Token1 := GetConstValue(';', ';');
            if not Lang.IsIntConstant(Token1) then
              InterErrorP(ieConstantExpected, Parser.Pos);
            DllProc.LibIndex := StrToInt(Token1);
          end;
          Result := DllProc;
          DllProc := nil;
        finally
          DllProc.Free;
        end;
      end else begin
        Parser.pcPos := P;
        Result := UProc;
        UProc := nil;
      end;
    finally
      UProc.Free;
    end;
  end;

  procedure sConst;
  { }
  var
    Token, ConstValue : string;
    IVar : TIVar;
    P : PChar;
  begin
    repeat
      P := Parser.pcPos;
      Token := Parser.Token;
      if ProcCom.IInterObject.IsTerminated then InterErrorS(ieUserBreak);
     { if Lang.Cmp(Token, kwENDCONST) then break
      else}
      if Lang.IsValidIdentifer(Token) then
        if Parser.Token <> '=' then
          InterErrorPV(ieExpected, Parser.PosEnd[1], '=')
        else begin
          ConstValue := GetConstValue(';', ';');
          IVar := TIVar.Create0;
          IVar.Name := Token;
          IVar.Str := iString(ConstValue);
          Consts.Add(IVar);
        end
      else begin
        Parser.pcPos := P;
        break;
      end;
        //if Token <> ';' then InterErrorP(ieSyntax, Parser.Pos);
    until false;{loop}
  end;

  procedure sFunc;
  { }
    function InDeclared(PCBI : PCallBackInfo; S : TIdentifer) : boolean;
    begin
      Result := Cmp(PCBI^.CProc.Name, S);
    end;
    function Stop(PCBI : PCallBackInfo; S : string) : boolean;
    begin
      Result := S = ')';
    end;
  var
    CProc : TCustomProc;
    UProc : TUnitProc;
    P : PChar;
  begin
    CProc := FuncDeclare(@InDeclared, @Stop);
    if CProc = nil then InterErrorS(ieUnknown);
    try
    if CProc is TUnitProc then begin
      P := Parser.pcPos;
      Token := Parser.Token;
     { }
      UProc := CProc as TUnitProc;
        if not(Lang.Cmp(Token, kwBEGIN) or Lang.Cmp(Token, kwVAR))
        then InterErrorP(ieUnknownLexem, Parser.Pos);
        UProc.OffsBeg := P-pcProgram;
        UProc.pcProgram := pcProgram;
        while not Lang.Cmp(Token, kwEndFunc) do begin
          if (Token = '') or Lang.Cmp(Token, kwEndUnit) then InterErrorPV(ieEndFunc, Parser.Pos, CProc.Name)
          else if Lang.Cmp(Token, kwFUNC) then InterErrorP(ieFuncInFunc, Parser.Pos);
          Token := Parser.Token;
        end;
        UProc.OffsEnd := Parser.Pos;
        Token := Parser.Token;
    end;
    if Token <> ';' then InterErrorP(ieSyntax, Parser.Pos);
   { Proc  }
    except
      CProc.Free;
      raise;
    end;
    Procs.Add(CProc);
  end;

  procedure sUses;
  var
    NeedSeparator : boolean;
  begin
    NeedSeparator := false;
    repeat
      Token := Parser.Token;
      if Token = '' then InterErrorP(ieEndUnit, Parser.Pos);
      if NeedSeparator then
        if Token = ';' then exit
        else if Token <> ',' then InterErrorP(ieSeparatorExpected, Parser.PosEnd[1])
        else NeedSeparator := false
      else if Lang.IsValidIdentifer(Token) then begin
        if nil{IInterObject.UnitObjs.GetObjByName(Token)!!!!!!} <> nil then
          SysUsesUnits.Add(Token)
        else begin
          FUsesUnits.Add(Token);
          if not Units.LoadUnit(Token) then
            InterErrorPV(ieUnitNotFound, Parser.PosBeg[0], Token)
        end;
        NeedSeparator := true;
      end
      else InterErrorP(ieIdentiferExpected, Parser.PosEnd[1])
    until false; {loop}
  end;

  procedure sUnit;
  { }
  begin
    Token := Parser.Token;
    if Lang.IsValidIdentifer(Token) then FName := Token
    else InterErrorP(ieIdentiferExpected, Parser.PosEnd[0]);
    if Parser.Token <> ';' then InterErrorP(ieSeparatorExpected, Parser.PosEnd[1]);
  end;

  procedure sClass;
  { }
    function InClassDeclared(PCBI : PCallBackInfo; S : TIdentifer) : boolean;
    begin
      Result := PCBI^.PClass.FProps.Objs[S] <> nil;
    end;
    function Stop(PCBI : PCallBackInfo; S : string) : boolean;
    begin
      Result := Lang.Cmp(S, kwFUNC) or Lang.Cmp(S, kwENDCLASS);
    end;
  var
    Prop : TPProp;
    i : integer;
    P : PChar;
    PClass : TPClass;
  begin
    PClass := TPClass.Create;
    CBI.PClass := PClass;
    try
      PClass.Name := Parser.Token;
      if not Lang.IsValidIdentifer(PClass.Name) then
        InterErrorP(ieIdentiferExpected, Parser.PosEnd[1])
      else if Declared(PClass.Name) then
        InterErrorPV(ieIdentiferRedeclared, Parser.PosBeg[0], PClass.Name);
      P := Parser.pcPos;
      Token := Parser.Token;
      repeat
        if ProcCom.IInterObject.IsTerminated then InterErrorS(ieUserBreak);
        if Lang.Cmp(Token, kwENDCLASS) then break
        else
       {   }
        if Lang.IsValidIdentifer(Token) then begin
         {  }
          Parser.pcPos := P;
          VarsDeclare(@InClassDeclared, @Stop);
          for i := 0 to Names.Count-1 do begin
            Prop := TPProp.Create;
            Prop.Name := Names[i];
            Prop.sTyp := sTyps[i];
            PClass.FProps.Add(Prop);
          end;
        end else
        if Lang.Cmp(Token, kwFUNC) then begin
        //.. ...
          P := Parser.pcPos;
          Token := Parser.Token;
        end else InterErrorP(ieSyntax, Parser.Pos);
      until false;{loop}
      if Parser.Token <> ';' then InterErrorP(ieSeparatorExpected, Parser.Pos);
      Classes.Add(PClass);
    except
      PClass._Release;
      raise;
    end;
  end;

var
  Data : TTranslateData;
begin
  prEnd := false;
  Parser.pcProgram := pcProgram;
  Parser.pcPos := pcProgram;
  Names := TStringList.Create;
  sTyps := TStringList.Create;
  try
  repeat
    if ProcCom.IInterObject.IsTerminated then InterErrorS(ieUserBreak);
    Token := Parser.Token;
    if Token = '' then InterErrorP(ieEndUnit, Parser.Pos)
    else
    if Lang.Cmp(Token, kwENDUNIT) then prEnd := true
   { }
    else if Lang.Cmp(Token, kwFUNC) then
      sFunc
    else if Lang.Cmp(Token, kwCONST) then
      sConst
    else if Lang.Cmp(Token, kwCLASS) then
      sClass
   { }
    else if Lang.Cmp(Token, kwUNIT) then
      sUnit
    else if Lang.Cmp(Token, kwUSES) then
      sUses
    else
    if ITranslator <> nil then begin
    {      
              }
      Data.Offs := Parser.Pos;
      Data.UnitName := iString(Name);
      Data.pcProgram := pcProgram;
      if not (ITranslator.Translate(Data) and (Data.Offs <> Parser.Pos)) then
        InterErrorP(ieSyntax, Parser.Pos);
    end else
      InterErrorP(ieSyntax, Parser.Pos);
  until prEnd;
  finally
    Names.Free;
    sTyps.Free;
  end;
end;

procedure TPUnit.Translate;
begin
  if Units = nil then InterErrorS(ieUnknown);
  pcProgram := PChar(Formula);
  ProcCom := Units.Owner.ProcCom;
  try
    Translating;
    Consts.Sort;
    Procs.Sort;
  except
    on E : EInterError do begin
      if E.UnitName = '' then begin
        E.UnitName := Name;
      end;
      //Clear;
      raise;
    end;
    on E : Exception do begin
     // Clear;
      raise;
    end;
  end;
  Save;
end;

function TPUnit.Load : boolean;
var
  P, P1 : PChar;
begin
//  Result := false;
 {  }
 //..
  {   }
  if true then begin
    try
      Result := IInterObject.GetUnitSource(PChar(Name), P1, P);
      SetString(Formula, P1, StrLen(P1));
    except
      on E : Exception do begin
        if Units.Owner.IObjectInfo <> nil then
          E.Message := '   '''+Units.Owner.IObjectData.ClassName+''':'#13#10+E.Message
        else
          E.Message := '    ''IInterObject'':'#13#10+E.Message;
        raise EInterError.CreateOLV(ieInterObject, E.Message, 0, 0, true, '');
      end;
    end;
    if Result then Translate;
  end;
  if Result then LoadUsesUnits;
 //  ,  Translate;
end;

function TPUnit.Save : boolean;
begin
 //..
  Result := false;
end;

procedure TPUnit.LoadUsesUnits;
var i : integer;
begin
  for i := 0 to FUsesUnits.Count-1 do
    if not Units.LoadUnit(FUsesUnits[i]) then
      InterErrorPV(ieUnitNotFound, -1, FUsesUnits[i])
end;

function TPUnit.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  i : integer;
begin
 {     (    ) }
 {   }
  Result := Procs.SetGetVar(SGI);
  if not Result then Result := Consts.SetGetVar(SGI);
  if not Result then Result := Classes.SetGetVar(SGI);
  if not Result then begin
    {   }
    for i:=0 to FSysObjs.Count-1 do begin
      Result := TIObject(FSysObjs[i]).SetGetValue(SGI);
      if Result then break;
    end;
  end;
end;

function TPUnit.Declared(Id : TIdentifer) : boolean;
begin
  Result := (Procs.Objs[Id] <> nil) or
            (Classes.Objs[Id] <> nil)  ;
end;

function TPUnit.ConstGetLocalVar(lName : TIdentifer) : IIVar;
var
  i : integer;
  PUnit : TPUnit;
begin
  {   }
  Result := Consts.Objs[lName] as TIVar;
  if Result = nil then
  {   }
  for i := 0 to UsesUnits.Count-1 do begin
    PUnit := Units.GetUnitByName(UsesUnits[i]);
    if PUnit <> nil then begin
      Result := IIVar(PUnit.Consts.Objs[lName] as TIVar);
      if Result <> nil then break;
    end;
  end;
end;
{################### TPUnit ###################}

constructor TPUnits.Create(AOwner : TInter);
begin
  inherited Create;
  Owner := AOwner;
end;

function TPUnits.GetUnit(index : integer) : TPUnit;
begin
  Result := TPUnit(Items[index]);
end;

function TPUnits.GetUnitByName(lName : TIdentifer) : TPUnit;
var i : integer;
begin
  Result := nil;
  for i:=0 to Count-1 do begin
    if Cmp(Units[i].Name, lName) then begin
      Result := Units[i];{}
      break;
    end;
  end;
end;

function TPUnits.SetGetVar(PUnit : TPUnit; SGI : TSetGetInfo; CheckUnitName : boolean) : boolean;
var
  i : integer;
  PrUnit : TPUnit;
  procedure CreateIPUint;
  var
    IPUnit : TIPUnit;
  begin
    Result := true;
    IPUnit := TIPUnit.Create;
    IPUnit.PUnit := PrUnit;
    IPUnit.IObj := SGI.IObjOut;
    SGI.Res.vObject := IPUnit;
    IPUnit._Release;
  end;
begin
  Result := false;
  if CheckUnitName then begin
    if SGI.Get then begin
      { -}
      Result := LoadUnit(SGI.Name);
      if Result then begin
        PrUnit := GetUnitByName(SGI.Name);
        if PrUnit <> nil then CreateIPUint;
      end;
    end;
    exit;{!}
  end;
  if SGI.Get then begin
    PrUnit := GetUnitByName(SGI.Name);
    { -}
    if PrUnit <> nil then begin
      CreateIPUint;
      exit; {!}
    end;
  end;
 // if not Result then
    if PUnit <> nil then begin
     {         }
      Result := PUnit.SetGetVar(SGI);
      if not Result then
        for i := 0 to PUnit.FUsesUnits.Count-1 do begin
          PrUnit := GetUnitByName(PUnit.FUsesUnits[i]);
          if PrUnit <> nil then begin
            Result := PrUnit.SetGetVar(SGI);
            if Result then exit;
          end else raise Exception.Create('   '''+PUnit.FUsesUnits[i]+'''.');
        end
      else exit;
    end;
end;

function TPUnits.LoadUnit(lUnit : TIdentifer) : boolean;
var
  PUnit : TPUnit;
begin
  if GetUnitByName(lUnit) = nil then begin
    PUnit := TPUnit.Create(self, Owner.IInterObject, lUnit, '');
    Result := PUnit.Load;
    if Result then Add(PUnit) else PUnit.Free;
  end else Result := true;
end;

procedure TPUnits.Clear;
var
  i : integer;
begin
  for i := 0 to Count-1 do
    TObject(Items[i]).Free;
  inherited Clear;
end;

function TIPUnit.SetGetVar(var SGI : TSetGetInfo) : boolean;
begin
  pointer(SGI.IObjOut) := pointer(IObj);
  Result := PUnit.Units.Owner.SetGetVar(PUnit, SGI);
end;
{############################ TPUnit #############################}


{*********************** TInter **********************}
const MaxRec = 50; {   }

constructor TInter.Create;
begin
  inherited Create;
  _AddRef;
  ProcCom := TProcCom.Create;
  ProcCom.SetGetValue := SetGetVar;
  ProcCom.DebugProc := DebugProc;
  RunProcs := TCallStack.Create;
  Units   := TPUnits.Create(self);
  Lexems   := TStringList.Create;
end;

destructor TInter.Destroy;
begin
  Clear;
  RunProcs.Free;
  ProcCom.Free;
  Units.Free;
  Lexems.Free;
  inherited Destroy;
end;

procedure TInter.Clear;
begin
  Units.Clear;
  Lexems.Clear;
  if IInterObject <> nil then
    IInterObject.Clear;
  IInterObject := nil;
  IDebugObject := nil;
  ITranslator  := nil;
  IObjectInfo  := nil;
  ProcCom.IObj := nil;
end;

procedure TInter.ProcRun;
  function IsUnit : boolean;
  { -   }
  var
    Parser : TIParser;
    Token : string;
  begin
    if StrScan(ProcCom.pcProgram + ProcCom.OffsBeg, ';') = nil then begin
      Result := false;
      exit;
    end;
    Result := true;
    Parser := TIParser.Create;
    try
      Parser.pcPos := ProcCom.pcProgram + ProcCom.OffsBeg;
      {   kwBEGIN  kwVAR   ,  - }
      Token := Parser.Token;
      if Lang.Cmp(Token, kwBEGIN) or Lang.Cmp(Token, kwVAR) then
        Result := false;
    finally
      Parser.Free;
    end;
  end;
var
  RunProc : TProc;
  PProc : TUnitProc;
  F : integer;
  UnitName, UnitFunc : TIdentifer;
  procedure InitProcCom;
  begin
    PProc := CurUnit.Procs.Objs[UnitFunc] as TUnitProc;
    if PProc = nil then InterErrorV(ieFuncNotFound, UnitFunc)
    else begin
      ProcCom.pcProgram := PProc.pcProgram;
      ProcCom.OffsBeg   := PProc.OffsBeg;
      ProcCom.OffsEnd   := PProc.OffsEnd;
      ProcCom.Name      := UnitFunc;
      ProcCom.PUnit     := CurUnit;
    end;
  end;
var
  SGI : TSetGetInfo;
begin
  F := -1;
  try
    UnitName := ProcCom.UnitName;
    UnitFunc := ProcCom.UnitFunc;
    ProcCom.PUnit := nil;
    if UnitName = '' then begin
      { Formula}
      if IsUnit then begin
       {}
        CurUnit := TPUnit.Create(Units, IInterObject, '', ProcCom.pcProgram);
        F := Units.Add(CurUnit);
        CurUnit.Translate;
        if not RunParams.CompileOnly then begin
          if UnitFunc = '' then
             UnitFunc := 'main';//raise Exception.Create('UnitFunc = '''); {UnitFunc := 'main'};
          InitProcCom;
        end;
      end
    end else begin
     {   }
      CurUnit := Units.GetUnitByName(UnitName);
      if CurUnit = nil then begin
        CurUnit := TPUnit.Create(Units, IInterObject, UnitName, ProcCom.pcProgram);
        F := Units.Add(CurUnit);
        if not CurUnit.Load then
          InterErrorV(ieUnitNotFound, UnitName);
      end;
     { //    
      CurUnit := TPUnit.Create(Units, FIObj, UnitName, ProcCom.pcProgram);
      F := Units.Add(CurUnit);
      if not CurUnit.Load then
        InterErrorV(ieUnitNotFound, UnitName);
      }
      if ProcCom.pcProgram[0] = #0 then begin
        if UnitFunc = '' then raise Exception.Create('UnitFunc = '''''); {UnitFunc := 'main'};
        InitProcCom;
      end else begin
        ProcCom.Name  := UnitFunc;
        ProcCom.PUnit := CurUnit;
      end;
    end;
   {}
    RunProc := TProc.Create(ProcCom);
    ProcCom.Args := nil;
    RunProcs.Add(RunProc);
    try
     // if DebugStopped then InterErrorV(ieUnknown, RunProc.Name);
      inc(FuncLevel);
      if FuncLevel > MaxRec then InterErrorS(ieStackOverFlow);
      RunProc.Solve;
      IIValueAssign(IInterObject.Res, RunProc.Res);
      dec(FuncLevel);
      if RunProc.Name <> '' then begin
       {   }
        pointer(SGI.IObjOut) := pointer(RunProc.IObj);
        SGI.Name    := iString(RunProc.Name);
        pointer(SGI.Args) := nil;
        pointer(SGI.Res) := pointer(IInterObject.Res);{?}
        SGI.Get     := false;
        SetGetVar(nil, SGI);
      end;
    finally
      RunProc.Free;
      RunProcs.Delete(RunProcs.Count-1);
    end;
  finally
    if (F > -1) and RunParams.RemoveUnit then begin
      CurUnit.Free;
      Units.Delete(F);
    end;
  end;
end;

procedure TInter.Run(AIInterObject : IIInterObject);
begin
 try
  DecimalSeparator := '.';
  FError.ErrProcName := '';
  FError.ErrMessage  := '';
  FError.ErrCode     := ieOk;
  FuncLevel := 0;
//  ProgDebugInfo.CallStack := RunProcs;
  DebugStopped := false;
  try
    if AIInterObject = nil then InterErrorS(ieBadIInterObject);
    IInterObject := AIInterObject;
    IInterObject.GetRunParams(RunParams);
    if RunParams.ReloadAll then Units.Clear;
    ProcCom.IInterObject := IInterObject;
    ProcCom.IObj := IInterObject;
    ProcCom.pcProgram := PChar(RunParams.Formula);
    ProcCom.UnitName := RunParams.UnitName;
    ProcCom.UnitFunc := '';
    ProcCom.Name := '';
    ProcCom.Args := nil;
    ProcCom.OffsBeg := 0; ProcCom.OffsEnd := 0;
    {$IFDEF DEBUG} Clas.AppHandle := RunParams.AppHandle; {$ENDIF}
    IDebugObject := IIDebugObject(GetIntf(IInterObject, IIDebugObject));
    if IDebugObject <> nil then
      ProcCom.Debug := IDebugObject.DebugMode
    else
      ProcCom.Debug := false;
    ITranslator  := IITranslator(GetIntf(IInterObject, IITranslator));
    IObjectInfo  := IIObjectInfo(GetIntf(IInterObject, IIObjectInfo));
    if IObjectInfo <> nil then
      IObjectInfo.GetIObjectData(IObjectData);

    CurUnit := nil;
    //!!!!!! IInterObject.Prg := self;
    ProcRun;
  except
    on E : EInterError do begin
     { if E.ErrCode = ieInterObject then
        MessageBox(RunParams.AppHandle, PChar(E.Message), 'R&A ', MB_ICONERROR)
      else} begin
        FError.ErrCode := E.ErrCode;
        if (E.UnitName = '') and (CurUnit <> nil) then
          E.UnitName := CurUnit.Name;
        FError.ErrUnitName := E.UnitName;
        FError.ErrProcName := E.ProcName;
        FError.ErrPos      := E.Pos;
        FError.ErrVarName  := E.VarName;
        FError.ErrMessage := E.Message;
        //  ErrPos := 0;{  }
      end;
    end;
    on E : Exception do 
      raise EInterError.CreateC(ieInternal, E.Message);
      // MessageBox(RunParams.AppHandle, PChar(E.Message), 'R&A ', MB_ICONERROR);
  end;
 finally
   IDebugObject := nil;
   ITranslator  := nil;
   IObjectInfo  := nil;
   IInterObject.Done(FError);
 end;
end;

function TInter.CurProc : TProc;
begin
  Result := TProc(RunProcs[RunProcs.Count-1]);
end;

procedure TInter.DebugProc;
begin
 {   }
  if DebugStopped then exit;
  DebugStopped := true;
  if IDebugObject <> nil then
    try
      IDebugObject.Debug;
    except
      on E : Exception do begin
        E.Message := 'Error in Debugger ' + E.Message;
        raise;
      end;
    end;
  DebugStopped := false;
end;

function TInter.GetCallStack : IICallStack;
begin
  Result := IICallStack(Self);
end;

function TInter.GetCallStackCount : integer;
begin
  Result := RunProcs.Count;
end;

function TInter.GetCallStackItem(index : integer) : TCallStackItem;
var
  Prc : TProc;
begin
  if not DebugStopped then exit; {error}
  Prc := RunProcs[RunProcs.Count-1-index];
  with Result do begin
    if Prc.PUnit <> nil then
      UnitName := iString((Prc.PUnit as TPUnit).FName)
    else UnitName := '';
    FuncName := iString(Prc.Name);
    pcProgram := Prc.Formula;
    PosBeg := Prc.PosBeg;
    PosEnd := Prc.PosEnd;
    Args   := Prc.Args;
  end;
end;


function TInter.EvaluateModify(Watch : iString; var Value : iString; var SGIDebugFlags : TSGIDebugFlags) : boolean;
begin
  if not DebugStopped then
    Result := false
  else
    Result := CurProc.DebugEvaluate(Watch, sResult, SGIDebugFlags);
  Value := iString(sResult);
end;

procedure TInter.SetDebugMode(ADebugMode : boolean);
begin
  ProcCom.Debug := ADebugMode;
end;

function TInter.SetGetVar(PUnit : TObject; SGI : TSetGetInfo) : boolean;
var
  Obj : IIObject;
  UnitOk : boolean;
  function ObjSetGetValue : boolean;
  begin
    try
      Result := Obj.SetGetValue(SGI);
    except
      on E : EInterError do raise;
      on E : Exception do begin
        if IObjectInfo <> nil then
          E.Message := '   '''+IObjectData.ClassName+''':'#13#10+E.Message
        else
          E.Message := '    ''IInterObject'':'#13#10+E.Message;
        raise EInterError.CreateOLV(ieInterObject, E.Message, 0, 0, true, '');
      end;
    end;
  end;
begin
  if PUnit = nil then PUnit := CurUnit;
  if (SGI.IObjOut <> nil){ and (Name <> '')} then
  with SGI do begin
    Obj := IObjOut;
    ProcCom.UnitName := '';
    if (Obj = IIObject(IInterObject)) and (SGI.Args <> nil) then begin
      if PUnit <> nil then
        Result := Units.SetGetVar((PUnit as TPUnit), SGI, false)
      else Result := Units.SetGetVar(nil, SGI, false);
      UnitOk := true;
    end else begin Result := false; UnitOk := false end;
    if not Result then Result := ObjSetGetValue;
    if not Result and UnitOk then
      if PUnit <> nil then
        Result := Units.SetGetVar((PUnit as TPUnit), SGI, true)
      else Result := Units.SetGetVar(nil, SGI, true);
    if Get and Result then
     { }
      while Res.Typ = ivFormula do begin
        ProcCom.Name := Name;
        ProcCom.UnitFunc := Name;
        ProcCom.pcProgram := PChar(Res.vString);
        ProcCom.Args := Args;
        ProcCom.IObj := Obj;
        ProcRun;
        Name := '';
        IIValueAssign(Res, IInterObject.Res);
        ProcCom.Args := nil;
      end
  end else Result := false;
end;
{####################### TInter ######################}


{****************************   **************************}

{
function Proc2Str(Proc : TCustomProc) : string;
var
  i : integer;
  S : string;
begin
  Result := Proc.Name;
  S := '(';
  if Proc.Params.Count > 0 then begin
    for i := 0 to Proc.Params.Count-1 do begin
      Result := Result +S+Proc.Params[i].Name+': '+ Proc.Params[i].sTyp;
      S := '; ';
    end;
    Result := Result + ')';
  end;
  if Proc.sTyp <> '' then
    Result := Result + ': '+Proc.sTyp;
end;
}

function CallDll(DllName, FuncName : string; Args : IIArgs; ResTyp : TIValueTyp; var iRes : longint) : boolean;
var
  Ins : HINST;
begin
  Result := false;
  Ins := LoadLibrary(PChar(DllName));
  if Ins = 0 then exit;
  CallDllIns(Ins, FuncName, Args, ResTyp, iRes);
end;

function CallDllIns(Ins : HINST; FuncName : string; Args : IIArgs; ResTyp : TIValueTyp; var iRes : longint) : boolean;
var
  Func : procedure; far;
  iFunc : function : integer; far;
  fFunc : function : boolean; far;
  i : integer;
  Aint : integer;
  // Abyte : byte;
  // Aword : word;
  Apointer : pointer;
begin
  Result := false;
  Func := GetProcAddress(Ins, PChar(FuncName));
  iFunc := @Func; fFunc := @Func;
  if @Func <> nil then begin
    for i := Args.Count-1 downto 0 do begin
      case Args[i].Typ of
        ivInteger : begin Aint := Args[i].vInteger; asm push Aint end; end;
        // ivByte    : begin Abyte := Args[i].vByte; asm push Abyte end; end;
        // ivWord    : begin Aword := Args[i].vWord; asm push Aword end; end;
        ivString  : begin Apointer := PChar(Args[i].vString); asm push Apointer end; end;
      end;
    end;
    case ResTyp of
      // ivWord    : iRes := wFunc;
      ivInteger : iRes := iFunc;
      ivBoolean : iRes := integer(fFunc);
      else Func;
    end;
    Result := true;
  end;
end;





end.


