unit RAIProgram;

interface

uses
  Windows, Messages, SysUtils, Classes,
  RAIIntf, RAIImpl, RAISystem;

type

  TSetGetInfo = RAIIntf.TSetGetInfo;
  TSetGetVarEvent = procedure (Sender : TObject; var SGI : TSetGetInfo; var Done : boolean) of object;

  TRAIProgram = class(TComponent)
  private
    FProgramSource : string;
    FSResult : string;
    FErrCode : integer;
    FErrPos  : integer;
    FRAIObject : TIInterObject;
    FOnSetGetVar : TSetGetVarEvent;
    FOnSyncSetGetVar : TSetGetVarEvent;
    FOnProgramDone : TNotifyEvent;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function Run : integer;
    procedure RunInThread;
    property ErrPos : integer read FErrPos;
    property RAIObject : TIInterObject read FRAIObject;
  published
    property ProgramSource : string read FProgramSource write FProgramSource;
    property SResult : string read FSResult;
    property ErrCode : integer read FErrCode;
    property OnSetGetVar : TSetGetVarEvent read FOnSetGetVar write FOnSetGetVar;
    property OnSyncSetGetVar : TSetGetVarEvent read FOnSyncSetGetVar write FOnSyncSetGetVar;
    property OnProgramDone : TNotifyEvent read FOnProgramDone write FOnProgramDone;
  end;

implementation

type
  TIMyInterObject = class(TIInterObject)
  private
    FRAIProgram : TRAIProgram;
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
    function SyncSetGetVar(var SGI : TSetGetInfo) : boolean; override;
  protected
    procedure ProgramDone; override;
  end;

constructor TRAIProgram.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FRAIObject := TIMyInterObject.Create;
  (FRAIObject as TIMyInterObject).FRAIProgram := Self;
  FRAIObject.SysObjs.Add2(TISystem.Create);
end;

destructor TRAIProgram.Destroy;
begin
  IUnknown(FRAIObject)._Release;
  inherited Destroy;
end;

function TRAIProgram.Run : integer;
begin
  FRAIObject.Formula := FProgramSource;
  try
    FRAIObject.Run;
    FErrCode := FRAIObject.ErrCode;
    if FErrCode = 0 then
      FSResult := FRAIObject.Res.AsString else
      FSResult := FRAIObject.ErrMessage;
    FErrPos := FRAIObject.ErrPos;
  except
    on E : Exception do begin
      FSResult := E.Message;
      FErrCode := -1;
    end;
  end;
  Result := FErrCode;
end;

procedure TRAIProgram.RunInThread;
begin
  FRAIObject.RunInThread;
end;

function TIMyInterObject.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  Done : boolean;
begin
  Result := inherited SetGetVar(SGI);
  if Result then exit;
  if Assigned(FRAIProgram.FOnSetGetVar) then
  begin
    Done := false;
    FRAIProgram.FOnSetGetVar(FRAIProgram, SGI, Done);
    Result := Done;
  end else
    Result := false;
end;

function TIMyInterObject.SyncSetGetVar(var SGI : TSetGetInfo) : boolean;
var
  Done : boolean;
begin
  if Assigned(FRAIProgram.FOnSyncSetGetVar) then
  begin
    Done := false;
    FRAIProgram.FOnSyncSetGetVar(FRAIProgram, SGI, Done);
    Result := Done;
  end else
    Result := false;
end;

procedure TIMyInterObject.ProgramDone;
begin

  if Assigned(FRAIProgram.FOnProgramDone) then
    FRAIProgram.FOnProgramDone(FRAIProgram);
end;


end.
