{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : R&A interpreter

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

unit RAISystem;

interface

uses Windows, SysUtils, Classes, ActiveX, ComObj,
  RAIIntf, RAIClas, RAIProg;

{$DEFINE RAINTER_OLEAUTO}

type

  TIUnit = class(TINObject)
  protected
    Ids : TStringList;
  public  
    constructor Create;
    destructor Destroy; override;
  end;

  TISystem = class(TIUnit)
  private
    IAPI : TIObject;
  public
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
    constructor Create;
    destructor Destroy; override;
  end;

  TISysUtils = class(TINObject)
  public
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

{$IFDEF RAINTER_OLEAUTO}
  TIOleAuto = class(TINObject)
  public
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

{ Additional error code }
const
  ieOleAuto = 301;
{$ENDIF RAINTER_OLEAUTO}

implementation

{$IFDEF RAINTER_OLEAUTO}
uses OleConst;
{$ENDIF RAINTER_OLEAUTO}

constructor TIUnit.Create;
begin
  inherited Create;
  Name := ClassName;
  Ids := TStringList.Create;
end;

destructor TIUnit.Destroy;
begin
  Ids.Free;
  inherited Destroy;
end;


type
  TIDate = class(TIObject)
  public
    Date : TDateTime;
    constructor Create(lDate : TDateTime);
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;

  TIAPI = class(TIObject)
  private
    ResTyp : TIValueTyp;
    FDlls : TStringList;
  protected
    function AddDll(DllName : string) : boolean;
    function RemoveDll(DllName : string) : boolean;
  public
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
    constructor Create;
    destructor Destroy; override;
  end;

{*************************** TISystem *************************}
const
  fstring            = 0;
  finteger           = fstring    +1;
  flongint           = finteger   +1;
  fbyte              = flongint   +1;
  fword              = fbyte      +1;
  fboolean           = fword      +1;
  freal              = fboolean   +1;
  fTList             = freal      +1;
  fCreate            = fTList     +1;
  fIAPI              = fCreate    +1;
  fiCallDll          = fIAPI      +1;
  fCallDll           = fiCallDll  +1;
  fCopy              = fCallDll   +1;
  fLength            = fCopy      +1;
  fLowerCase         = fLength    +1;
  fUpperCase         = fLowerCase +1;
  fFloatToStr        = fUpperCase +1;
  fChr               = fFloatToStr +1;
  fDateTime          = fChr       +1;
  fAsc               = fDateTime  +1;
 {math}
  fArcTan            = fAsc       +1;
  fCos               = fArcTan    +1;
  fExp               = fCos       +1;
  fFrac              = fExp       +1;
  fInt               = fFrac      +1;
  fLn                = fInt       +1;
  fSin               = fLn        +1;
  fSqr               = fSin       +1;
  fSqrt              = fSqr       +1;
  fPi                = fSqrt      +1;
  fnull              = fPi        +1;

constructor TISystem.Create;
begin
  inherited Create;
  Name := 'System';
  IAPI := TIAPI.Create;
  Ids.AddObject('string'  , TObject(fstring  ));
  Ids.AddObject('integer' , TObject(finteger ));
  Ids.AddObject('longint' , TObject(flongint ));
  Ids.AddObject('byte'    , TObject(fbyte    ));
  Ids.AddObject('word'    , TObject(fword    ));
  Ids.AddObject('boolean' , TObject(fboolean ));
  Ids.AddObject('real'    , TObject(freal    ));
  Ids.AddObject('TList'   , TObject(fTList   ));
  Ids.AddObject('Create'  , TObject(fCreate  ));
  Ids.AddObject('API'     , TObject(fIAPI    ));
  Ids.AddObject('iCallDll', TObject(fiCallDll));
  Ids.AddObject('CallDll' , TObject(fCallDll ));
  Ids.AddObject('Copy'    , TObject(fCopy    ));
  Ids.AddObject('Length'  , TObject(fLength  ));
  Ids.AddObject('Chr'     , TObject(fChr     ));
  Ids.AddObject('UpperCase', TObject(fUpperCase));
  Ids.AddObject('LowerCase', TObject(fLowerCase));
  Ids.AddObject('FloatToStr' , TObject(fFloatToStr));
  Ids.AddObject('DateTime', TObject(fDateTime));
  Ids.AddObject('Asc'    , TObject(fAsc     ));
  Ids.AddObject('ArcTan' , TObject(fArcTan));
  Ids.AddObject('Cos'    , TObject(fCos   ));
  Ids.AddObject('Exp'    , TObject(fExp   ));
  Ids.AddObject('Frac'   , TObject(fFrac  ));
  Ids.AddObject('Int'    , TObject(fInt   ));
  Ids.AddObject('Ln'     , TObject(fLn    ));
  Ids.AddObject('Sin'    , TObject(fSin   ));
  Ids.AddObject('Sqr'    , TObject(fSqr   ));
  Ids.AddObject('Sqrt'   , TObject(fSqrt  ));
  Ids.AddObject('Pi'     , TObject(fPi    ));
  Ids.AddObject('null'   , TObject(fnull  ));
  Ids.Sort;
end;

destructor TISystem.Destroy;
begin
  IAPI._Release;
  inherited Destroy;
end;

function TISystem.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  DllName, FuncName : string;
  iRes : longint;
  Ind : integer;
  rArg : double;
begin
  if SGI.Get then Result := Ids.Find(SGI.Name, Ind)
  else Result := false;
  if Result then with SGI do begin
    Res.Empty;
    Ind := integer(Ids.Objects[Ind]);
    case Ind of
     { }
      fstring  : if IIArgsCheckParams(Args, 1, [ivEmpty]) then Res.vString  := Args[0].vString  ;
      finteger : if IIArgsCheckParams(Args, 1, [ivEmpty]) then Res.vInteger := Args[0].vInteger ;
      fboolean : if IIArgsCheckParams(Args, 1, [ivEmpty]) then Res.vBoolean := Args[0].vBoolean ;
      freal    : if IIArgsCheckParams(Args, 1, [ivEmpty]) then Res.vReal    := Args[0].vReal    ;
     {#}
      fnull    : { Res.Empty - allready empty };
      fTList   : if IIArgsCheckParams(Args, 0, [ivEmpty]) then Res.vClass   := Name             ;
      fCreate  : if ClassIs('TList') and IIArgsCheckParams(Args, 1, [ivClass]) then
        begin
          Res.vObject := TIList.Create;
          Res.vObject._Release;
        end else Result := false;
      fIAPI    : if IIArgsCheckParams(Args, 0, [ivEmpty]) then Res.vObject := IAPI;
      {************ CallDll}
      fiCallDll :
        if IIArgsCheckParams(Args, -2, [ivString, ivString]) then begin
          DllName := Args[0].vString;
          FuncName := Args[1].vString;
          Args.Delete(0); Args.Delete(0);
          if NameIs('iCallDll') then begin
            CallDll(DllName, FuncName, Args, ivInteger, iRes);
            Res.vInteger := iRes;
          end else Res.vBoolean := CallDll(DllName, FuncName, Args, ivEmpty, iRes);
        end;
       {############ CallDll}
      { }
      fCopy     : if IIArgsCheckParams(Args, 3, [ivString, ivInteger, ivInteger]) then
        Res.vString := iString(Copy(Args[0].vString, Args[1].vInteger, Args[2].vInteger));
      fLength   : if IIArgsCheckParams(Args, 1, [ivString]) then
        Res.vInteger := Length(Args[0].vString);
      fChr      : if IIArgsCheckParams(Args, 1, [ivInteger]) then
        Res.vString := iString(string(Chr(Args[0].vInteger)));
      fLowerCase : if IIArgsCheckParams(Args, 1, [ivString]) then
        Res.vString := iString(ANSILowerCase(Args[0].vString));
      fUpperCase : if IIArgsCheckParams(Args, 1, [ivString]) then
        Res.vString := iString(ANSIUpperCase(Args[0].vString));
      fFloatToStr : if IIArgsCheckParams(Args, 2, [ivReal, ivInteger]) then
        Res.vString := iString(FloatToStrF(Args[0].vReal, ffFixed	, 15, Args[1].vInteger));
     {    Real   Real}
      fArcTan, fCos, fExp, fFrac, fInt, fLn, fSin, fSqr, fSqrt :
        if IIArgsCheckParams(Args, 1, [ivReal]) then begin
          rArg := Args[0].vReal;
          case Ind of
            fArcTan   : rArg := ArcTan(rArg);
            fCos      : rArg := Cos   (rArg);
            fExp      : rArg := Exp   (rArg);
            fFrac     : rArg := Frac  (rArg);
            fInt      : rArg := Int   (rArg);
            fLn       : rArg := Ln    (rArg);
            fSin      : rArg := Sin   (rArg);
            fSqr      : rArg := Sqr   (rArg);
            fSqrt     : rArg := Sqrt  (rArg);
          end;
          Res.vReal := rArg;
        end;
     {#}
      fAsc      : if IIArgsCheckParams(Args, 1, [ivString]) then
        if Length(Args[0].vString) = 0 then Res.vInteger := 0
        else Res.vInteger := Ord(Args[0].vString[1]);
      fPi       : if IIArgsCheckParams(Args, 0, [ivEmpty]) then
        Res.vReal := Pi;
      fDateTime :
        if IIArgsCheckParams(Args,  0, [ivEmpty]) then
        begin
          Res.vObject := TIDate.Create(Date+Time);
          Res.vObject._Release;
        end;
    end;{case}
  end;
end;

{*************************** TIAPI *************************}
constructor TIAPI.Create;
begin
  inherited Create;
  ResTyp := ivEmpty;
  FDlls := TStringList.Create;
  AddDll('user32');
  AddDll('gdi32');
  AddDll('kernel32');
end;

destructor TIAPI.Destroy;
begin
  while FDlls.Count > 0 do RemoveDll(FDlls[0]);
  FDlls.Free;
  inherited Destroy;
end;

function TIAPI.AddDll(DllName : string) : boolean;
var
  Ins : HINST;
begin
  DllName := ANSIUpperCase(DllName);
  Ins := LoadLibrary(PChar(DllName));
  Result := Ins <> 0;
  if Result then
    FDlls.AddObject(DllName, TObject(Ins));
end;

function TIAPI.RemoveDll(DllName : string) : boolean;
var
  F : integer;
begin
  DllName := ANSIUpperCase(DllName);
  F := FDlls.IndexOf(DllName);
  if F <> -1 then begin
    FreeLibrary(HINST(FDlls.Objects[F]));
    FDlls.Delete(F);
    Result := true;
  end else Result := false;
end;

function TIAPI.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  iRes : longint;
  i : integer;
  j : boolean;
  FuncName : string;
begin
  with SGI do
  if Get then begin
    Result := true;
    if NameIs('i') and IIArgsCheckParams(Args, 0, [ivEmpty]) then
    begin
      Res.vObject := self;
      ResTyp := ivInteger;
    end
    else if ProcIs('AddDll', 1, [ivString]) then
      Res.vBoolean := AddDll(Args[0].vString)
    else if ProcIs('RemoveDll', 1, [ivString]) then
      Res.vBoolean := RemoveDll(Args[0].vString)
    else begin
      Result := false;
      FuncName := SGI.Name;
      for j := false to true do begin
        for i := 0 to FDlls.Count-1 do
          if CallDllIns(HINST(FDlls.Objects[i]), FuncName, Args, ResTyp, iRes) then begin
            case ResTyp of
              ivInteger : Res.vInteger := iRes;
              ivBoolean : Res.vBoolean := boolean(iRes);
              else Res.Empty;
            end;
            Result := true;
            break;
          end;{then}
        if not j then
          if not Result then FuncName := FuncName +'A' else break;
      end;{for j}
    end;
  end else Result := false;
end;
 {########################### TIAPI #########################}
{########################### TISystem #########################}

{*************************** TISysUtils *************************}
function TISysUtils.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  i : integer;
begin
  Result := false;
  with SGI do begin
  if Get then begin
    Result := true;
    Res.Empty;
    if NameIs('Sum') then begin
      Res.vFormula := '';
      for i := 0 to Args.Count-1 do
        if Args[i].Typ <> ivEmpty then
          Res.vFormula :=  iString(string(Res.vFormula) +'+'+ string(Args[i].Str));
      if Res.vFormula = '' then InterErrorOB(ieParams, 0, false);
    end
    else if ProcIs('ShowMessage', 1, [ivString]) then
      MessageBox(0, PChar(Args[0].vString), nil, MB_OK)
    else Result := false;
  end;
  end;
end;
{########################### TISysUtils #########################}

constructor TIDate.Create(lDate : TDateTime);
begin
  inherited Create;
  Date := lDate;
end;

function TIDate.SetGetVar(var SGI : TSetGetInfo) : boolean;
begin
  Result := true;
 {*Date}
  with SGI do begin
  if Get then
    if ProcIs('DateAsString', 0, [ivEmpty]) then
      Res.vString := iString(DateToStr(Date))
    else if ProcIs('TimeAsString', 0, [ivEmpty]) then
      Res.vString := iString(TimeToStr(Date))
    else if ProcIs('TickAsString', 0, [ivEmpty]) then
      Res.vString := iString(IntToStr(GetTickCount))
    else Result := false
  else
    if NameIs('DateAsString') then
      Date := StrToDate(Res.vString)
    else Result := false;
  end;
end;


{$IFDEF RAINTER_OLEAUTO}

{*************************** OleAuto ****************************}

type
  TIVariant = class;
  IIVariant = interface
    ['{8C6BA264-08FC-11d1-BA5A-F44E9C5EF10B}']
    function IVariant : TIVariant;
  end;

  TIVariant = class(TIObject, IIVariant)
  private
    VObj : Variant;
    ObjCreated     : boolean;
    OleInitialized : boolean;
    EMess : string;
    function IVariant : TIVariant;
    constructor CreateServer(OleServerName : string);
    constructor CreateVariant(lV : Variant);
    destructor Destroy; override;
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;


  
{*******************************************************}
{****************** From OleAuto unit ******************}
{*******************************************************}

const
{ Maximum number of dispatch arguments }

  MaxDispArgs = 64; {!!!}

{ Special variant type codes }

  varStrArg = $0048;

{ Parameter type masks }

  atVarMask  = $3F;
  atTypeMask = $7F;
  atByRef    = $80;


{ Call GetIDsOfNames method on the given IDispatch interface }
procedure GetIDsOfNames(Dispatch: IDispatch; Names: PChar;
  NameCount: Integer; DispIDs: PDispIDList);
var
  I, N: Integer;
  Ch: WideChar;
  P: PWideChar;
  NameRefs: array[0..MaxDispArgs - 1] of PWideChar;
  WideNames: array[0..1023] of WideChar;
begin
  I := 0;
  N := 0;
  repeat
    P := @WideNames[I];
    if N = 0 then NameRefs[0] := P else NameRefs[NameCount - N] := P;
    repeat
      Ch := WideChar(Names[I]);
      WideNames[I] := Ch;
      Inc(I);
    until Char(Ch) = #0;
    Inc(N);
  until N = NameCount;
  if Dispatch.GetIDsOfNames(GUID_NULL, @NameRefs, NameCount,
    LOCALE_SYSTEM_DEFAULT, DispIDs) <> 0 then
    raise EOleError.CreateFmt(SNoMethod, [Names]);
end;

{ Central call dispatcher }
procedure VarDispInvoke(Result: PVariant; const Instance: Variant;
  Names : PChar; CallDesc: PCallDesc; Params: Pointer); cdecl;
var
  Dispatch: Pointer;
  DispIDs: array[0..MaxDispArgs - 1] of Integer;
begin
  if TVarData(Instance).VType = varDispatch then
    Dispatch := TVarData(Instance).VDispatch
  else if TVarData(Instance).VType = (varDispatch or varByRef) then
    Dispatch := Pointer(TVarData(Instance).VPointer^)
  else raise EOleError.Create(SVarNotObject);
  GetIDsOfNames(IDispatch(Dispatch), Names, CallDesc^.NamedArgCount + 1, @DispIDs);
  if Result <> nil then VarClear(Result^);
  DispatchInvoke(IDispatch(Dispatch), CallDesc, @DispIDs, @Params, Result);
end;

{#######################################################}
{################## from OleAuto unit ##################}
{#######################################################}


constructor TIVariant.CreateServer(OleServerName : string);
begin
  inherited Create;
  VarClear(VObj);
  OleInitialized := OleInitialize(nil) = S_OK;
  if OleInitialized then
    try
      VObj := CreateOleObject(OleServerName);
      ObjCreated := true;
    except
      on E : Exception do begin
        EMess := E.Message;
        ObjCreated := false;
      end;
    end;
end;

constructor TIVariant.CreateVariant(lV : Variant);
begin
  inherited Create;
  OleInitialized := OleInitialize(nil) = S_OK;
  VObj := lV;
  ObjCreated := true;
end;

destructor TIVariant.Destroy;
begin
  VarClear(VObj);
  if OleInitialized then OleUnInitialize;
  inherited Destroy;
end;

function TIVariant.IVariant : TIVariant;
begin
  Result := self;
end;

function TIVariant.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  CallDesc : TCallDesc;
  Params : array[0..MaxDispArgs * 4 - 1] of byte;
  ParamsPtr : pointer absolute Params;
  Ptr : integer;

  procedure AddParam(Typ : byte; ParamSize : integer; var Param);
  begin
    CallDesc.ArgTypes[Ptr] := Typ;
    Move(Param, Params[Ptr], ParamSize);
    inc(Ptr, ParamSize);
  end;

var
  VRes  : Variant;
  PVRes : PVariant;
  Names : string;
  i : integer;
  Int : integer;
  Wrd : WordBool;
  Poin : pointer;
begin
  with SGI do begin
  if not (ObjCreated and Get) then begin Result := false; exit end;
  if TVarData(VObj).VType = varDispatch then begin
    Result := true;
   {   Ole Automation}
    with CallDesc do begin
      CallType := DISPATCH_METHOD or DISPATCH_PROPERTYGET;
      ArgCount := Args.Count;
      NamedArgCount := 0;
    end;
    Names := Name+#00;
    Ptr := 0;
    for i := 0 to Args.Count-1 do
      case Args[i].Typ of
        ivInteger : begin
          Int := Args[i].vInteger;
          AddParam(varInteger, sizeof(Int), Int);
        end;
        ivString  : begin
          Poin := PChar(Args[i].vString);
          AddParam(varStrArg, sizeof(Poin), Poin);
        end;
        ivBoolean : begin
          Wrd := WordBool(Args[i].vBoolean);
          AddParam(varBoolean, sizeof(wrd), wrd);
        end;
      end;
    {  (  )   ,
       }
    if Args.ResTyp = ivEmpty then PVRes := nil else PVRes := @VRes;
    try
      VarDispInvoke(PVRes, VObj, PChar(Names), @CallDesc, ParamsPtr);
    except
      on E : EOleError do
        raise EInterError.CreateOLV(ieOleAuto, E.Message, 0, -1, true, Name);
    end;
    case TVarData(VRes).VType of
      varBoolean  : Res.vBoolean := VRes;
      varInteger  : Res.vInteger := TVarData(VRes).VInteger;
      varDispatch : begin
        Res.vObject := TIVariant.CreateVariant(VRes);
        Res.vObject._Release;
      end;
      varOleStr   : Res.vString := TVarData(VRes).VString;
      else begin
        Res.Empty;
      end;
    end;
    VarClear(VRes);
  end else Result := false;
  end;
end;

function TIOleAuto.SetGetVar(var SGI : TSetGetInfo) : boolean;
var
  Obj : TIVariant;
begin
  with SGI do begin
  if not Get then begin Result := false; exit end;
  Result := true;
  if ProcIs('CreateOleObject', 1, [ivString]) then begin
    Obj := TIVariant.CreateServer(Args[0].vString);
    if not Obj.ObjCreated then
      raise EInterError.CreateOLV(ieOleAuto, Obj.EMess, 0, -1, true, Name);
    Res.vObject := Obj;
    Obj._Release;
  end else
  if ProcIs('VarClear', 1, [ivObject]) then
    if HasIntf(Args[0].vObject, IIVariant) then begin
      Res.Empty;
      VarClear(IIVariant(GetIntf(Args[0].vObject, IIVariant)).IVariant.VObj);
      IIVariant(GetIntf(Args[0].vObject, IIVariant)).IVariant.ObjCreated := false;
    end
    else InterErrorO(0, ieParamTypMistmach)
  else if ProcIs('VType', 1, [ivObject]) then
    Res.vInteger := TVarData(IIVariant(GetIntf(Args[0].vObject, IIVariant)).IVariant.VObj).VType
  else Result := false;
  end;
end;
{$ENDIF RAINTER_OLEAUTO}
{########################### OleAuto ############################}

{___________________________________________________________}

{
    else if ProcIs('Delay', 1, [ivInteger]) then
      Delay(Args[0].vInteger)
    else if ProcIs('ShowMessagePause', 2, [ivString, ivInteger]) then
      ShowMessagePause(Args[0].vString, Args[1].vInteger)
    else if ProcIs('', 1, [ivInteger]) then
      Res.vString := RAUtils.NumberByWord(Args[0].vInteger)
}
(*
{***    }
procedure Delay(msec : longint); forward;
procedure ShowMessagePause(S : string; P : longint); forward;
{###}
*)
(*
procedure Delay(msec : longint);
var
  Ticks : longint;
begin
  Ticks := GetTickCount;
  while GetTickCount - Ticks < msec do
    Application.ProcessMessages;
end;

procedure ShowMessagePause(S : string; P : longint);
var
  Form : TForm;
begin
  Form := TForm.Create(Application);
  with Form do begin
    BorderStyle  := bsDialog;
    BorderIcons  := [biSystemMenu, biHelp];
    with TLabel.Create(Form) do begin
      Parent := Form;
      Top := 20;
      Left := 20;
      Font.Name := 'Courier New';
      Caption := S;
    end;
    Position := poScreenCenter;
    Show;
    Update;
    Sleep(P);
    Free;
  end;
end;
*)
{###########################  ############################}

end.
