{***********************************************************
                R&A Library
       Copyright (C) 1996-98 R&A

       description : Register interpreter components

       programer   : black
       e-mail      : blacknbs@chat.ru
       www         : www.chat.ru\~blacknbs\ralib
************************************************************}

{$INCLUDE RA.INC}

{$IFDEF RA_D2}
 !!! R&A Interpreter not compatible with Delphi 2
{$ENDIF}

unit RAInterReg;

interface

procedure Register;

implementation

uses Windows, Classes, SysUtils, DB, DBTables, Forms,
  Controls, Dialogs, Menus, TypInfo, DsgnIntf, 
  RAIProgram, RAIDb, RAUtils, RAConst, RAIIntf, RAIImpl;

{$R RAInter.dcr}

{$IFDEF RAINTER_INTEGERPROPERTY}
type

  TRAIntegerProperty = class(TIntegerProperty)
  private
    Component : TPersistent;
    RAIP : TRAIProgram;
    procedure SetGetVar(Sender : TObject; var SGI : TSetGetInfo; var Done : boolean);
  public
    procedure SetValue(const Value : string); override;
   {$IFDEF RAINTER_INTEGERPROPERTY_DBLCLICK}
    procedure Edit; override;
   {$ENDIF RAINTER_INTEGERPROPERTY_DBLCLICK}
  end;

{$ENDIF RAINTER_INTEGERPROPERTY}

procedure Register;
begin
  RegisterComponents('R&&A Controls', [TRAIProgram]);
  RegisterClasses([TRAIField]);
 {$IFDEF RAINTER_INTEGERPROPERTY}
  RegisterPropertyEditor(TypeInfo(integer), TObject, '', TRAIntegerProperty);
  RegisterPropertyEditor(TypeInfo(cardinal), TObject, '', TRAIntegerProperty);
  RegisterPropertyEditor(TypeInfo(longint), TObject, '', TRAIntegerProperty);
  RegisterPropertyEditor(TypeInfo(smallint), TObject, '', TRAIntegerProperty);
  RegisterPropertyEditor(TypeInfo(shortint), TObject, '', TRAIntegerProperty);
  RegisterPropertyEditor(TypeInfo(word), TObject, '', TRAIntegerProperty);
  RegisterPropertyEditor(TypeInfo(byte), TObject, '', TRAIntegerProperty);
 {$ENDIF RAINTER_INTEGERPROPERTY}
end;


{************************** TRAIFieldCreator ***************************}

  { TRAIFieldCreator add menu item to dataset editor }

type

  TRAIFieldCreator = class(TComponent)
  private
    DataSource : TDataSource;
    FOnShow : TNotifyEvent;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Prepare(Sender : TObject);
    procedure MenuClick(Sender : TObject);
    destructor Destroy; override;
  end;

var
  RAIFieldCreator : TComponent;

procedure TRAIFieldCreator.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (Operation = opInsert) and
     AComponent.ClassNameIs('TDataSetEditor') then
  begin
    FOnShow := (AComponent as TForm).OnShow;
    (AComponent as TForm).OnShow := Prepare;
  end;
end;

procedure TRAIFieldCreator.Prepare(Sender : TObject);
var
  PopupMenu : TPopupMenu;
  MenuItem : TMenuItem;
begin
  if Assigned(FOnShow) then FOnShow(Sender);
  if (Sender as TForm).FindComponent('RAIFieldCreatorItem') <> nil then exit;
  PopupMenu := (Sender as TForm).FindComponent('LocalMenu') as TPopupMenu;
  DataSource := (Sender as TForm).FindComponent('DataSource') as TDataSource;
  if (PopupMenu = nil) or (DataSource = nil) then exit;
  MenuItem := TMenuItem.Create(Sender as TForm);
  with MenuItem do
  begin
    Caption := 'New RA&IField';
    Name := 'RAIFieldCreatorItem';
    ShortCut := Menus.ShortCut(ord('I'), [ssCtrl]);
    OnClick := MenuClick;
  end;
  PopupMenu.Items.Insert(2, MenuItem);
end;

procedure TRAIFieldCreator.MenuClick(Sender : TObject);
var
  N, FN : string;
  i : integer;
  A : boolean;
begin
  if DataSource.DataSet = nil then exit;
  with TRAIField.Create(DataSource.DataSet.Owner) do begin
    i := 1;
    repeat
      N := DataSource.DataSet.Name + 'RAIField'+IntToStr(i);
      FN := 'RAIField'+IntToStr(i);
      inc(i);
    until (DataSource.DataSet.Owner.FindComponent(N) = nil) and
          (DataSource.DataSet.FindField(FN) = nil);
    Name := N;
    FieldName := FN;
    A := DataSource.DataSet.Active;
    DataSource.DataSet.Close;
    DataSet := DataSource.DataSet;
    DataSource.DataSet.Active := A;
  end
end;

destructor TRAIFieldCreator.Destroy;
begin
  RAIFieldCreator := nil;
  inherited Destroy;
end;
{########################## TRAIFieldCreator ###########################}

{$IFDEF RAINTER_INTEGERPROPERTY}

{ TRAIntegerProperty }

type
  TIClass = class(TIThreadObject)
  public
    FObject : TObject;
    constructor Create(O : TObject);
    function SetGetVar(var SGI : TSetGetInfo) : boolean; override;
  end;


procedure RetProperty(Obj : TObject; var Done : boolean; SGI : TSetGetInfo);
  procedure CreateResObj(O : TObject);
  begin
    if O <> nil then
    begin
      Done := true;
      SGI.Res.vObject := TIClass.Create(O);
      SGI.Res.vObject._Release;
    end;  
  end;
var
  PropInf : PPropInfo;
  TypeInf : PTypeInfo;
  PropTyp : TTypeKind;
begin
  if SGI.Args.Count = 0 then
  begin
    TypeInf := Obj.ClassInfo;
    PropInf := TypInfo.GetPropInfo(TypeInf, SGI.Name);
    if PropInf <> nil then
    begin
      PropTyp := PropInf^.PropType^.Kind;
      if PropTyp <> tkClass then
      begin
        Done := true;
        SGI.Res.vInteger := GetOrdProp(Obj, PropInf);
      end else
        CreateResObj(TObject(GetOrdProp(Obj, PropInf)));
    end else
    if Obj is TForm then
      CreateResObj(TComponent(Obj).FindComponent(SGI.Name))
    else if (Obj is TComponent) and ((Obj as TComponent).Owner <> nil) then
      CreateResObj(TComponent(Obj).Owner.FindComponent(SGI.Name))
   { else if (Obj is TCollectionItem) and ((Obj as TComponent).Owner <> nil) then
      CreateResObj(TComponent(Obj).Owner.FindComponent(SGI.Name));}
  end;
end;

constructor TIClass.Create(O : TObject);
begin
  inherited Create;
  FObject := O;
end;

function TIClass.SetGetVar(var SGI : TSetGetInfo) : boolean;
begin
 {Find property}
  Result := inherited SetGetVar(SGI);
  if not Result and SGI.Get then
    RetProperty(FObject, Result, SGI);
end;

procedure TRAIntegerProperty.SetGetVar(Sender : TObject; var SGI : TSetGetInfo; var Done : boolean);
begin
  Done := false;
  RetProperty(Component, Done, SGI);
end;

procedure TRAIntegerProperty.SetValue(const Value: string);

  function Calc : longint;
  begin
    if RAIP = nil then
    begin
      RAIP := TRAIProgram.Create(nil);
      RAIP.OnSetGetVar := SetGetVar;
    end;
    RAIP.ProgramSource := Value;
    RAIP.Run;
    if RAIP.ErrCode <> 0 then
      raise Exception.Create(RAIP.SResult);
    Result := StrToInt(RAIP.SResult);
  end;

var
  Int : longint;
  PropInf : PPropInfo;
  i : integer;
begin
  try
  { if Value is a simple integer we do not need to run RAInter }
    Int := StrToInt(Value);
    SetOrdValue(Int);
  except
   { and now we really need do this }
    for i := 0 to PropCount - 1 do
    begin
      Component := GetComponent(i) as TPersistent;
      Int := Calc;
      PropInf := TypInfo.GetPropInfo(Component.ClassInfo, GetName);
      if PropInf <> nil then
        SetOrdProp(Component, PropInf, Int);
      Modified;
    end;
  end;
end;

{$IFDEF RAINTER_INTEGERPROPERTY_DBLCLICK}
procedure TRAIntegerProperty.Edit;
var
  Int, P : longint;
  PropInf : PPropInfo;
  i : integer;
  Component : TPersistent;
begin
  for i := 0 to PropCount - 1 do
  begin
    Component := GetComponent(i) as TPersistent;
    PropInf := TypInfo.GetPropInfo(Component.ClassInfo, GetName);
    if PropInf <> nil then
    begin
      Int := GetOrdProp(Component, PropInf);
     {$R-}
      if KeyPressed(VK_CONTROL) then P := 10 else P := 1;
      if KeyPressed(VK_SHIFT) then dec(Int, P) else inc(Int, P);
     {$R+}
      SetOrdProp(Component, PropInf, Int);
      Modified;
    end;
  end;
end;
{$ENDIF RAINTER_INTEGERPROPERTY_DBLCLICK}
{$ENDIF RAINTER_INTEGERPROPERTY}

initialization
  RAIFieldCreator := TRAIFieldCreator.Create(Application);
finalization
  RAIFieldCreator.Free;
end.
