{***************************************************************}
{                                                               }
{  Formula Compiler 3.0 for Delphi ( 16 & 32-bit )              }
{                                                               }
{  Copyright (c) 1996, 1997  Dimak & Vlad.S                     }
{                                                               }
{  e-mail:  dimak@uniyar.ac.ru  &  vlads@uniyar.ac.ru           }
{                                                               }
{***************************************************************}

unit FC;

{$IFDEF WIN32}

{$S-,H+,B-}

{$ELSE}

{$S-,F+,O-,B-}

{$ENDIF}

interface

uses SysUtils, Classes, Consts, FCLib, Dialogs;

{ Events }
type
  TVariableEvent = function(const VarName: string): Extended of object;
  TFunctionEvent = function(const FuncName: string; const X: array of Extended): Extended of object;

type
  { TCustomFormula - predecessor of TFormulaCompiler components  }
  { It provides all common properties and methods }
  TCustomFormula = class(TComponent)
  private
    FSource: String;
    FCode: PChar;            { compiled code of Source }
    FCodeSize: Integer;      { code size }
    FConst: PChar;           { array of used constants }
    FConstSize: Integer;     { size of FConst }
    FOptions: Integer;
    FLib: TFormulaLib;       { TFormulaLib - assigned user library }
    FUnknownNames: Boolean;
    FNameList: TStringList;
    FOnFunction: TFunctionEvent;
    FOnVariable: TVariableEvent;
    { Compile S into machine code }
    { Code - pointer to copiled code, CodeSize - size of code }
    { Cnst - pointer to array of constants, ConstSize - size of Cnst }
    { NameList - list of unknown names (aliases which are not defined in FLib) }
    function Compile(S: String;
                 var Code: PChar; var CodeSize: Integer;
                 var Cnst: PChar; var ConstSize: Integer;
                 NameList: TStringList): Integer;
    function GetArgs: Integer;
    procedure SetSource(NewValue: String);
    procedure SetUnknownNames(NewValue: Boolean);
    function  OnVarCall(Index: Integer): Extended;
    function  OnFuncCall(Index: Integer; ParamCount: Integer; Offs: Integer): Extended;
  protected
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    { sets new formula - returns error code  }
    function NewFunction(const S: String): Integer; virtual;
    { recompile Source }
    procedure Recompile;
    { Runtime only and read only . Number of arguments  }
    property Args: Integer read GetArgs;
    { formula }
    property Source: String read FSource write SetSource;
    { assigned user library }
    property Lib: TFormulaLib read FLib write FLib;
    property UnknownNames: Boolean read FUnknownNames write SetUnknownNames;
    property OnVariable: TVariableEvent read FOnVariable write FOnVariable;
    property OnFunction: TFunctionEvent read FOnFunction write FOnFunction;
  end;

  { TFormulaCompilerD - returns double type number as result of calculations }
  TFormulaCompilerD = class(TCustomFormula)
  public
    { evaluates result  }
    F: function(const X: array of Double): Double;
    constructor Create(AOwner: TComponent); override;
    function NewFunction(const S: String): Integer; override;
  published
    property Lib;
    property Source;
    property UnknownNames;
    property OnVariable;
    property OnFunction;
  end;

  { TFormulaCompilerE - returns extended type number as result of calculations }
  TFormulaCompilerE = class(TCustomFormula)
  public
    { evaluates result }
    F: function(const X: array of Extended): Extended;
    constructor Create(AOwner: TComponent); override;
    function NewFunction(const S: String): Integer; override;
  published
    property Lib;
    property Source;
    property UnknownNames;
    property OnVariable;
    property OnFunction;
  end;

implementation

uses WinTypes, WinProcs, FCUtils;

{ Compiler options }
const
  coValues   = $0001;
  coArray    = $0002;

  coDouble   = $0010;
  coExtended = $0020;

  coCall           = $0F00;
  coCallFromProc   = $0100;
  coCallFromMeth   = $0200;
  coCallDirect     = $0300;

{$IFNDEF WIN32}
{ Getting function address }
function GetFuncAddr(var Code): Pointer;
begin
  Result := Ptr(AllocDStoCSAlias(Seg(Code)), Ofs(Code));
end;
{$ENDIF}

{ TCustomFormula methods}
constructor TCustomFormula.Create;
const
  {$IFDEF WIN32}
  IniCode: PChar = #$D9#$E8#$9B#$C3;
  { FLD1 (D9E8); FWAIT (9B); RET (C3); }
  {$ELSE}
  IniCode: PChar = #$D9#$E8#$9B#$CB;
  { FLD1 (D9E8); FWAIT (9B); RET (CB); }
  {$ENDIF}
begin
  inherited Create(AOwner);
  FCodeSize := 4;
  GetMem(FCode, FCodeSize);
  Move(IniCode[0], FCode[0], FCodeSize);
  FConst := nil;
  FConstSize := 0;
  FSource := '1';
  FOptions := 0;
  FLib := nil;
  FUnknownNames := False;
  FNameList := TStringList.Create;
end;

function TCustomFormula.NewFunction(const S: String): Integer;
var
  tCode: PChar;
  tCodeSize: Integer;
  tConst: PChar;
  tConstSize: Integer;
  tN: TStringList;
begin
  tN := TStringList.Create;
  try
    { try to compile formula S }
    if FUnknownNames
      then Result := Compile(S, tCode, tCodeSize, tConst, tConstSize, tN)
      else Result := Compile(S, tCode, tCodeSize, tConst, tConstSize, nil);
    { if everything Ok  }
    if Result = fcOk
      then begin
        if FCode <> nil then FreeMem(FCode, FCodeSize);
        if FConst <> nil then FreeMem(FConst, FConstSize);
        FCode := tCode;
        FCodeSize := tCodeSize;
        FConst := tConst;
        FConstSize := tConstSize;
        FSource := S;
        FNameList.Assign(tN);
      end;
  finally
    tN.Free;
  end;
end;

{ recompile Source  }
procedure TCustomFormula.Recompile;
begin
  try
    { try to recompile source }
    FCCheck(NewFunction(FSource));
  except
    { on exception reset source to '1' }
    NewFunction('1');
    raise;
  end;
end;

{ get number of arguments in Source  }
function TCustomFormula.GetArgs: Integer;
begin
  Result := Arguments(FSource, FLib);
end;

procedure TCustomFormula.SetSource(NewValue: String);
begin
  if NewValue <> FSource
    then if csLoading in ComponentState
           then FSource := NewValue
           else FCCheck(NewFunction(NewValue));
end;

procedure TCustomFormula.SetUnknownNames(NewValue: Boolean);
begin
  if NewValue <> FUnknownNames
    then begin
      FUnknownNames := NewValue;
      Recompile;
    end;
end;

{ OnVarCall is called from the compiled code }
function TCustomFormula.OnVarCall(Index: Integer): Extended;
begin
  { call OnVariable event if assigned }
  if Assigned(FOnVariable)
    then Result := FOnVariable(FNameList[Index])
    else FCCheck(fcUnknownVar);
end;

{ OnFuncCall is called from the compiled code }
function TCustomFormula.OnFuncCall;
type
  TPseudoFunctionEvent = function(const Name: String;
    Data: Pointer; Size: Integer): Extended of object;
var
  P: Pointer;
  i: Integer;
begin
  { call OnFuntion event if assigned }
  if Assigned(FOnFunction)
    then begin
      { prepare to call event }
      asm
      {$IFDEF WIN32}
        mov eax, dword ptr Offs
        mov dword ptr P, eax
      {$ELSE}
        mov ax, ss
        mov word ptr P+2, ax
        mov ax, word ptr Offs
        mov word ptr P, ax
      {$ENDIF}
      end;
      Result := TPseudoFunctionEvent(FOnFunction)(FNameList[Index],
                      P, ParamCount - 1)
    end
    else FCCheck(fcUnknownFunc);
end;

destructor TCustomFormula.Destroy;
begin
  if FCode <> nil then FreeMem(FCode, FCodeSize);
  if FConst <> nil then FreeMem(FConst, FConstSize);
  FNameList.Free;
  inherited Destroy;
end;

procedure TCustomFormula.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if ( Operation = opRemove ) and ( AComponent = FLib )
    then FLib := nil;
end;

procedure TCustomFormula.Loaded;
begin
  inherited Loaded;
  try
    Recompile;
  except
  end;
end;

{ TFormulaCompilerD  methods }
constructor TFormulaCompilerD.Create;
begin
  inherited Create(AOwner);
  FOptions := coDouble or coArray or coCallDirect;
{$IFDEF WIN32}
  @F := FCode;
{$ELSE}
  @F := GetFuncAddr(FCode^);
{$ENDIF}
end;

function TFormulaCompilerD.NewFunction(const S: String): Integer;
begin
  Result := inherited NewFunction(S);
  if Result = fcOk
    then
    {$IFDEF WIN32}
      @F := FCode;
    {$ELSE}
      @F := GetFuncAddr(FCode^);
    {$ENDIF}
end;

{ TFormulaCompilerE methods }
constructor TFormulaCompilerE.Create;
begin
  inherited Create(AOwner);
  FOptions := coExtended or coArray or coCallDirect;
{$IFDEF WIN32}
  @F := FCode;
{$ELSE}
  @F := GetFuncAddr(FCode^);
{$ENDIF}
end;

function TFormulaCompilerE.NewFunction(const S: String): Integer;
begin
  Result := inherited NewFunction(S);
  if Result = fcOk
    then
    {$IFDEF WIN32}
      @F := FCode;
    {$ELSE}
      @F := GetFuncAddr(FCode^);
    {$ENDIF}
end;

{ Main method - compilation of expression }
function TCustomFormula.Compile;
var
  StackPtr: Integer;
  Args: TInternalExprArray;
  Constants: TArray;
  TempCode: TCodeArray;
  LinkerInfo: TIntArray;
const
  ArgOffset: array[1..3] of Byte = (0, 0, 0);
  Size: array[Boolean] of Byte = (SizeOf(Double), SizeOf(Extended));
{$IFDEF WIN32}
  c_dir_RETF = #$01#$C3;
  c_proc_RETF = #$01#$C3;
  cFLD_esp_c = #$03#$DB#$AC#$24;{+Integer}  { see PopStack     }
  cFSTP_esp_c = #$03#$DB#$BC#$24;{+Integer} { see PushStack    }
  cFLD_c = #$02#$DB#$2D;{+Integer}          { see LoadConst    }
  cMOV_ax_c = #$01#$B8;{+Integer}
  cMOV_dx_c = #$01#$BA;{+Integer}
  cMOV_bx_c = #$01#$BB;{+Integer}
  cMOV_cx_c = #$01#$B9;{+Integer}
  cMOV_ax_sp = #$02#$8B#$C4;
  cCALL_bx = #$02#$FF#$D3;{+Integer}
  cLOAD_ARG: array[Boolean, Boolean, Boolean, 1..3] of PChar = (
                   {Double} {!Array} {!First} {Call}
     { Double Params not First                                    }
     { FLD  [ebp+c]        FLD  [ebp+c]        FLD  [ebp+c]       }
    (((#$02#$DD#$45{+Byte},#$02#$DD#$45{+Byte},#$02#$DD#$45{+Byte}),
     { Double Params First                                        }
     { FLD  [ebp+c]        FLD  [ebp+c]        FLD  [ebp+c]       }
      (#$02#$DD#$45{+Byte},#$02#$DD#$45{+Byte},#$02#$DD#$45{+Byte})),
     { Double Array not First                                     }
     { FLD [edx+c]         FLD [edx+c]         FLD [eax+c]        }
     ((#$02#$DD#$42{+Byte},#$02#$DD#$42{+Byte},#$02#$DD#$40{+Byte}),
     { Double Array First                                         }
     { FLD [edx]    FLD [edx]    FLD [eax]                        }
      (#$02#$DD#$02,#$02#$DD#$02,#$02#$DD#$00))),
     { Extended Param not First                                   }
     { FLD [ebp+c]         FLD [ebp+c]         FLD [ebp+c]        }
    (((#$02#$DB#$6D{+Byte},#$02#$DB#$6D{+Byte},#$02#$DB#$6D{+Byte}),
     { Extended Param First                                       }
     { FLD [ebp+c]         FLD [ebp+c]         FLD [ebp+c]        }
      (#$02#$DB#$6D{+Byte},#$02#$DB#$6D{+Byte},#$02#$DB#$6D{+Byte})),
     { Extended Array not First                                   }
     { FLD [edx+c]         FLD [edx+c]         FLD [eax+c]        }
     ((#$02#$DB#$6A{+Byte},#$02#$DB#$6A{+Byte},#$02#$DB#$68{+Byte}),
     { Extended Array First                                       }
     { FLD [edx]    FLD [edx]    FLD [eax]                        }
      (#$02#$DB#$2A,#$02#$DB#$2A,#$02#$DB#$28)))
    );
{$ELSE}
  c_proc_RETF = #$01#$CB;
  c_dir_RETF = #$03#$CA#$06#$00; { retf 6 }
  c_proc_bp_prepare = #$03#$C4#$76#$0C;
    { les si, [bp+$0C] }
  c_dir_bp_prepare = #$06#$55#$8B#$EC#$C4#$76#$08;
    { push bp; mov bp, sp; les si, [bp+08] }
  c_dir_POP_bp = #$01#$5D;
  cFLD_bp_c = #$02#$DB#$AE;{+Integer}        { see PopStack     }
  cFSTP_bp_c = #$02#$DB#$BE;{+Integer}       { see PushStack    }
  cFLD_cs_c = #$03#$2E#$DB#$2E;{+Integer}    { see LoadConst    }
  cCall_c = #$01#$9A;{+Pointer}
  cPUSH_c = #$01#$68;{+Integer}
  cPUSH_es = #$01#$06;
  cPUSH_si = #$01#$56;
  cPOP_es = #$01#$07;
  cPOP_si = #$01#$5E;
  cMOV_ax_bp =#$02#$8B#$C5;
  cLOAD_ARG: array[Boolean, Boolean, Boolean] of PChar = (
                   {Double} {!Array} {!First} {Call}
     { Double Params not First                                    }
     { FLD [bp+c]                                                 }
    ((#$02#$DD#$46{+Byte},
     { Double Params First                                        }
     {FLD [bp+c]                                                  }
      #$02#$DD#$46{+Byte}),
     { Double Array not First                                     }
     { FLD es:[si+c]                                              }
     (#$03#$26#$DD#$44{+Byte},
     { Double Array First                                         }
     { FLD es:[si]                                                }
      #$03#$26#$DD#$04)),
     { Extended Param not First                                   }
     { FLD [bp+c]                                                 }
    ((#$02#$DB#$6D{+Byte},
     { Extended Param First                                       }
     { FLD [bp+c]                                                 }
      #$02#$DB#$6D{+Byte}),
     { Extended Array not First                                   }
     { FLD es:[si+c]                                              }
     (#$03#$26#$DB#$6C{+Byte},
     { Extended Array First                                       }
     { FLD es:[si]                                                }
      #$03#$26#$DB#$2C))
    );
{$ENDIF}
  { Misc }
  cFWAIT = #$01#$9B;
  cFLD_ST = #$01#$D9;{+C[i]}
  cFXCH_ST = #$01#$D9;{+C[8+i]}
  cFSTP_ST0 = #$02#$DD#$D8;
  cADD_sp_c = #$02#$81#$C4;{+Integer}
  cSUB_sp_c = #$02#$81#$EC;{+Integer}
  cSUB_ax_c = #$01#$2D;{+Integer}
  cADD_ax_c = #$01#$05;{+Integer}
  cPUSH_ax = #$01#$50;
  cPOP_ax = #$01#$58;
  cPUSH_bx = #$01#$53;
  cPOP_bx = #$01#$5B;
  { Binary operations }
  cFADDP = #$02#$DE#$C1;
  cFSUBP = #$02#$DE#$E9;
  cFMULP = #$02#$DE#$C9;
  cFDIVP = #$02#$DE#$F9;
  cFSUBRP = #$02#$DE#$E1;
  cFDIVRP = #$02#$DE#$F1;

{ Loads Const[i] into ST(0) }
procedure LoadConst(i: Extended);
var
  Index: Integer;
begin
  Index := Constants.IndexOf(i);
  if Index < 0 then Index := Constants.Add(i);
{$IFDEF WIN32}
  TempCode.AddCode(cFLD_c);
{$ELSE}
  TempCode.AddCode(cFLD_cs_c);
{$ENDIF}
  LinkerInfo.Add(TempCode.Count);
  TempCode.AddInteger(Index);
end;

procedure LoadUnknownVar(Index: Integer);
var
  i, fpuUsed, memUsed: Integer;
begin
  { Save stack }
  if StackPtr > 4 then fpuUsed := 4 else fpuUsed := StackPtr;
  if StackPtr > 4 then memUsed := StackPtr - 4 else memUsed := 0;
  for i := 1 to fpuUsed do
  begin
    {$IFDEF WIN32}
    TempCode.AddCode(cFSTP_esp_c);
    TempCode.AddInteger(-(memUsed + i)*SizeOf(Extended) - 8);
    {$ELSE}
    TempCode.AddCode(cFSTP_bp_c);
    TempCode.AddInteger(-(memUsed + i)*SizeOf(Extended) - 16);
    {$ENDIF}
  end;
  { Call event }
  TempCode.AddCode(cSUB_sp_c);
    TempCode.AddInteger((memUsed + fpuUsed + 1)*SizeOf(Extended) + 16);
  {$IFDEF WIN32}
  TempCode.AddCode(cPUSH_bx);
  TempCode.AddCode(cPUSH_ax);
  TempCode.AddCode(cMOV_ax_c); TempCode.AddInteger(Integer(Self));
  TempCode.AddCode(cMOV_dx_c); TempCode.AddInteger(Index);
  TempCode.AddCode(cMOV_bx_c);
    TempCode.AddInteger(Integer(Addr(TCustomFormula.OnVarCall)));
  TempCode.AddCode(cCALL_bx);
  TempCode.AddCode(cPOP_ax);
  TempCode.AddCode(cPOP_bx);
  {$ELSE}
  TempCode.AddCode(cPUSH_es);
  TempCode.AddCode(cPUSH_si);
  TempCode.AddCode(cPUSH_c); TempCode.AddInteger(Index);
  TempCode.AddCode(cPUSH_c); TempCode.AddInteger(LongRec(Self).Hi);
  TempCode.AddCode(cPUSH_c); TempCode.AddInteger(LongRec(Self).Lo);
  TempCode.AddCode(cCall_c);
    TempCode.AddInteger(Ofs(TCustomFormula.OnVarCall));
    TempCode.AddInteger(Seg(TCustomFormula.OnVarCall));
  TempCode.AddCode(cPOP_si);
  TempCode.AddCode(cPOP_es);
  {$ENDIF}
  TempCode.AddCode(cADD_sp_c);
    TempCode.AddInteger((memUsed + fpuUsed + 1)*SizeOf(Extended) + 16);
  { Restore stack }
  if fpuUsed > 0
    then begin
      { Save event result }
      {$IFDEF WIN32}
      TempCode.AddCode(cFSTP_esp_c);
      TempCode.AddInteger(-(memUsed + fpuUsed + 1)*SizeOf(Extended) - 8);
      {$ELSE}
      TempCode.AddCode(cFSTP_bp_c);
      TempCode.AddInteger(-(memUsed + fpuUsed + 1)*SizeOf(Extended) - 16);
      {$ENDIF}
      { Load in to FPU }
      for i := fpuUsed downto 1 do
      begin
      {$IFDEF WIN32}
        TempCode.AddCode(cFLD_esp_c);
        TempCode.AddInteger(-(memUsed + i)*SizeOf(Extended) - 8);
      {$ELSE}
        TempCode.AddCode(cFLD_bp_c);
        TempCode.AddInteger(-(memUsed + i)*SizeOf(Extended) - 16);
      {$ENDIF}
      end;
      { Load event result }
      {$IFDEF WIN32}
      TempCode.AddCode(cFLD_esp_c);
      TempCode.AddInteger(-(fpuUsed + memUsed + 1)*SizeOf(Extended) - 8);
      {$ELSE}
      TempCode.AddCode(cFLD_bp_c);
      TempCode.AddInteger(-(fpuUsed + memUsed + 1)*SizeOf(Extended) - 16);
      {$ENDIF}
    end;
end;

{ Loads i-th argument into ST(0) }
procedure LoadArg(i: Integer; StackFlag: Integer);
begin
  case Args.Elem[i] of
  'a': begin
         TempCode.AddCode(cLOAD_ARG[coExtended and FOptions <> 0,
                                    coArray and FOptions <> 0,
                                    Args.Id[i] = 0
         {$IFDEF WIN32}, FOptions div coCallFromProc {$ENDIF}]);
         if coArray and FOptions <> 0
           then begin
             if Args.Id[i] > 0
               then TempCode.Add(Args.Id[i]*Size[coExtended and FOptions <> 0])
           end
           else
             TempCode.Add(Args.Id[i]*Size[coExtended and FOptions <> 0] +
                          ArgOffset[FOptions div coCallFromProc]);
       end;
  'c': if Args.Id[i] > 0
         then TempCode.AddCode(BuiltInFuncs[Args.Id[i]].Code)
         else LoadConst(Args.Cnst[i]);
  'v': LoadUnknownVar(Args.Id[i]);
  's': begin
         if Args.Id[i] > 4
           then begin { Load from Memory part of Stack }
             {$IFDEF WIN32}
             TempCode.AddCode(cFLD_esp_c);
             TempCode.AddInteger(-(Args.Id[i]-4)*SizeOf(Extended) - 8);
             {$ELSE}
             TempCode.AddCode(cFLD_bp_c);
             TempCode.AddInteger(-(Args.Id[i]-4)*SizeOf(Extended) - 16);
             {$ENDIF}
           end
           else begin { Load from 8087 part of Stack }
             TempCode.AddCode(cFLD_ST);
             if StackPtr <= 4
               then TempCode.Add($C0 + StackFlag - 1 + StackPtr - Args.Id[i])
               else TempCode.Add($C0 + StackFlag - 1 + 4 - Args.Id[i]);
           end;
       end;
  end;
end;

{ Pushes ST(0) to Stack }
procedure PushStack;
begin
  Inc(StackPtr);
  if StackPtr > 4
    then begin
      {$IFDEF WIN32}
      TempCode.AddCode(cFSTP_esp_c);
      TempCode.AddInteger(-(StackPtr-4)*SizeOf(Extended) - 8);
      {$ELSE}
      TempCode.AddCode(cFSTP_bp_c);
      TempCode.AddInteger(-(StackPtr-4)*SizeOf(Extended) - 16);
      {$ENDIF}
    end;
end;

{ Deletes Stack elements }
procedure DelStack(Count: Integer);
var
  DelCount, i: Integer;
begin
  if StackPtr > 4
    then DelCount := 4 - (StackPtr - Count)
    else DelCount := Count;
  if DelCount > 0
    then begin
      TempCode.AddCode(cFXCH_ST);
      TempCode.Add($C8+DelCount);
      for i := 1 to DelCount do
        TempCode.AddCode(cFSTP_ST0);
    end;
  Dec(StackPtr, Count);
end;

{ _Compile }
function _Compile(S: String): Integer;
var
  InternalExpr, Postfix: TInternalExprArray;
  i, j, k, Top: Integer;
  ReverseOp: Boolean;
  Que: Byte;

{ Sets to coprocessor's stack }
procedure SetTo8087(Index: Integer; StackFlag: Integer);
begin
  with Postfix do
    case Elem[Index] of
    'a': LoadArg(Id[Index], StackFlag);
    'v': LoadUnknownVar(Id[Index]);
    'c': if Id[Index] < 0
           then LoadConst(Cnst[Index])
           else if Id[Index] < UserFuncBase
                  then TempCode.AddCode(BuiltInFuncs[Id[Index]].Code)
                  else Result := _Compile(FLib.Functions.Sources[Id[Index]-UserFuncBase]);
    's': begin
           if StackPtr > 4
             then begin
               {$IFDEF WIN32}
               TempCode.AddCode(cFLD_esp_c);
               TempCode.AddInteger(-(StackPtr-4)*SizeOf(Extended) - 8);
               {$ELSE}
               TempCode.AddCode(cFLD_bp_c);
               TempCode.AddInteger(-(StackPtr-4)*SizeOf(Extended) - 16);
               {$ENDIF}
             end;
           Dec(StackPtr);
         end;
    end;
end;

{ optimized compile of built-in functon }
procedure DoFuncComp;
var
  fpuUsed, memUsed, j: Integer;
begin
  with Postfix do
  if Elem[i - Id[i] - 1] = 'f'
    then begin
      { Built in function }
      if Id[i - Id[i] - 1] < UserFuncBase
      then begin
        if (i-1 <> Top) and (Top <> -1) then PushStack;
        { Unlimited arguments }
        if Cnst[i] > 0
        then begin
          { compile function with unlimited arguments  }
          if (i-1 <> Top) then SetTo8087(i-1, 1);
          for j := 2 to Id[i] do
          begin
            SetTo8087(i-j, 1);
            TempCode.AddCode(BuiltInFuncs[Id[i - Id[i] - 1]].Code);
          end;
        end
        else begin
          { generate code to put arguments to FPU stack, optimize exchange }
          { between memory and FPU stack depending on FPU register used by }
          { built in function }
          case Id[i] of
          1: if (i-1 <> Top) then SetTo8087(i-1, 1);
          2: begin
               if (i-1 <> Top) then SetTo8087(i-1, 1);
               SetTo8087(i-2, 2);
               if (Elem[i-2] = 's') and (Id[i-2] <= 4)
               then begin
                 TempCode.AddCode(cFXCH_ST);
                 TempCode.Add($C8+1);
               end;
             end;
          3: begin
               Que := 0;
               if (Elem[i-1] = 's')
               then Que := Que and $FE
               else Que := Que or 1;
               if (Elem[i-2] = 's') and (Id[i-2] <= 4)
                 then Que := Que and $FD
                 else Que := Que or 2;
               if (Elem[i-3] = 's') and (Id[i-3] <= 4)
                 then Que := Que and $FB
                 else Que := Que or 4;
               case Que of
               0: begin {sss}
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+2);
                        if (i-1 <> Top)
                          then Dec(StackPtr, 3)
                          else Dec(StackPtr, 2);
                      end;
               1: begin {ssc}
                        SetTo8087(i-1, 1);
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+2);
                        Dec(StackPtr, 2);
                      end;
               2: begin {scs}
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+1);
                        SetTo8087(i-2, 1);
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+1);
                        if (i-1 <> Top)
                          then Dec(StackPtr, 2)
                          else Dec(StackPtr, 1);
                      end;
               3: begin {scc}
                        SetTo8087(i-1, 1);
                        SetTo8087(i-2, 2);
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+1);
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+2);
                        Dec(StackPtr, 1);
                      end;
               4: begin {css}
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+1);
                        SetTo8087(i-3, 1);
                        if (i-1 <> Top)
                          then Dec(StackPtr, 2)
                          else Dec(StackPtr, 1);
                      end;
               5: begin {csc}
                        SetTo8087(i-1, 1);
                        TempCode.AddCode(cFXCH_ST);
                        TempCode.Add($C8+1);
                        SetTo8087(i-3, 2);
                        Dec(StackPtr, 1);
                      end;
               6: begin {ccs}
                        SetTo8087(i-2, 1);
                        SetTo8087(i-3, 2);
                        if (i-1 <> Top)
                          then Dec(StackPtr, 1);
                      end;
               7: begin {ccc}
                        SetTo8087(i-1, 1);
                        SetTo8087(i-2, 2);
                        SetTo8087(i-3, 3);
                      end;
               end;{case 0-7}
            end; { begin }
          end; {case 1-3}
          { add code of built in function }
          TempCode.AddCode(BuiltInFuncs[Id[i - Id[i] - 1]].Code);
        end;
      end
      { User function }
      else begin
        if Top <> -1 then PushStack;
        { Make references to arguments }
        for j := 1 to Id[i] do
          if Elem[i-j] = 'a'
            then Args.Insert(0, Args.Items[Id[i-j]+j-1])
            else Args.Insert(0, Items[i-j]);
        { Compile user function. Recursive call of _Compile }
        Result := _Compile(FLib.Functions.Sources[Id[i-Id[i]-1]-UserFuncBase]);
        { Remove args }
        for j := 1 to Id[i] do
          Args.Delete(0);
        { Remove used elements from our stack }
        k := 0;
        for j := 1 to Id[i] do
          if Elem[i-j] = 's' then Inc(k);
        DelStack(k);
      end;
    end
    else begin
      { if we encounter unknown name we should call onFunction  }
      if StackPtr > 4 then fpuUsed := 4 else fpuUsed := StackPtr;
      if StackPtr > 4 then memUsed := StackPtr - 4 else memUsed := 0;
      { Create parameters }
      if Top <> -1 then PushStack;
      for j := 1 to Id[i] do
      begin
        SetTo8087(i-j, 1);
        {$IFDEF WIN32}
        TempCode.AddCode(cFSTP_esp_c);
        TempCode.AddInteger(-(memUsed + fpuUsed + j)*SizeOf(Extended) - 8);
        {$ELSE}
        TempCode.AddCode(cFSTP_bp_c);
        TempCode.AddInteger(-(memUsed + fpuUsed + j)*SizeOf(Extended) - 16);
        {$ENDIF}
      end;
      { Save FPU stack to memory }
      for j := 1 to fpuUsed do
      begin
        {$IFDEF WIN32}
        TempCode.AddCode(cFSTP_esp_c);
        TempCode.AddInteger(-(memUsed + j)*SizeOf(Extended) - 8);
        {$ELSE}
        TempCode.AddCode(cFSTP_bp_c);
        TempCode.AddInteger(-(memUsed + j)*SizeOf(Extended) - 16);
        {$ENDIF}
      end;
      { Call event }
      TempCode.AddCode(cSUB_sp_c);
        TempCode.AddInteger((memUsed + fpuUsed + Id[i] + 1)*SizeOf(Extended) + 16);
      {$IFDEF WIN32}
      TempCode.AddCode(cPUSH_bx);
      TempCode.AddCode(cPUSH_ax);
      TempCode.AddCode(cMOV_ax_sp);
      TempCode.AddCode(cADD_ax_c); TempCode.AddInteger(26);
      TempCode.AddCode(cPUSH_ax);
      TempCode.AddCode(cMOV_ax_c); TempCode.AddInteger(Integer(Self));
      TempCode.AddCode(cMOV_dx_c); TempCode.AddInteger(Id[i - Id[i] - 1]);
      TempCode.AddCode(cMOV_cx_c); TempCode.AddInteger(Id[i]);
      TempCode.AddCode(cMOV_bx_c);
        TempCode.AddInteger(Integer(Addr(TCustomFormula.OnFuncCall)));
      TempCode.AddCode(cCALL_bx);
      TempCode.AddCode(cPOP_ax);
      TempCode.AddCode(cPOP_bx);
      {$ELSE}
      TempCode.AddCode(cPUSH_bx);
      TempCode.AddCode(cPUSH_es);
      TempCode.AddCode(cPUSH_si);
      TempCode.AddCode(cPUSH_c); TempCode.AddInteger(Id[i - Id[i] - 1]);
      TempCode.AddCode(cPUSH_c); TempCode.AddInteger(Id[i]);
      TempCode.AddCode(cMOV_ax_bp);
      TempCode.AddCode(cSUB_ax_c);
        TempCode.AddInteger((memUsed + fpuUsed + Id[i])*SizeOf(Extended) + 16);
      TempCode.AddCode(cPUSH_ax);
      TempCode.AddCode(cPUSH_c); TempCode.AddInteger(LongRec(Self).Hi);
      TempCode.AddCode(cPUSH_c); TempCode.AddInteger(LongRec(Self).Lo);
      TempCode.AddCode(cCall_c);
        TempCode.AddInteger(Ofs(TCustomFormula.OnFuncCall));
        TempCode.AddInteger(Seg(TCustomFormula.OnFuncCall));
      TempCode.AddCode(cPOP_si);
      TempCode.AddCode(cPOP_es);
      TempCode.AddCode(cPOP_bx);
      {$ENDIF}
      TempCode.AddCode(cADD_sp_c);
        TempCode.AddInteger((memUsed + fpuUsed + Id[i] + 1)*SizeOf(Extended) + 16);
      { Restore stack after that }
      if fpuUsed > 0
        then begin
          { Save event result }
          {$IFDEF WIN32}
          TempCode.AddCode(cFSTP_esp_c);
          TempCode.AddInteger(-(memUsed + fpuUsed + 1)*SizeOf(Extended) - 8);
          {$ELSE}
          TempCode.AddCode(cFSTP_bp_c);
          TempCode.AddInteger(-(memUsed + fpuUsed + 1)*SizeOf(Extended) - 16);
          {$ENDIF}
          { Load in prevously stored contents of FPU stack back }
          for j := fpuUsed downto 1 do
          begin
          {$IFDEF WIN32}
            TempCode.AddCode(cFLD_esp_c);
            TempCode.AddInteger(-(memUsed + j)*SizeOf(Extended) - 8);
          {$ELSE}
            TempCode.AddCode(cFLD_bp_c);
            TempCode.AddInteger(-(memUsed + j)*SizeOf(Extended) - 16);
          {$ENDIF}
          end;
          { Load event result }
          {$IFDEF WIN32}
          TempCode.AddCode(cFLD_esp_c);
          TempCode.AddInteger(-(fpuUsed + memUsed + 1)*SizeOf(Extended) - 8);
          {$ELSE}
          TempCode.AddCode(cFLD_bp_c);
          TempCode.AddInteger(-(fpuUsed + memUsed + 1)*SizeOf(Extended) - 16);
          {$ENDIF}
        end;
    end;
end;

begin
  { create InternalExpr array }
  InternalExpr := TInternalExprArray.Create;
  if InternalExpr <> nil
    then begin
      Postfix := TInternalExprArray.Create;
      if Postfix <> nil
        then begin
          { Parse S and put result into InternalExpr }
          Result := MakeInternalExpr(S, InternalExpr, NameList, FLib);
          { If parsing succefull }
          if Result = fcOk
            then begin
              { optimize expression }
              Optimize(InternalExpr);
              { makes RPN }
              MakePostfix(InternalExpr, Postfix);
              InternalExpr.Free;
              { Code generation }
              Top := -1;
              with Postfix do
              begin
                while Count >= 3 do
                begin
                  { Find the first executable group i.e (2 3 +) }
                  i := 2;
                  while not ( Elem[i] in (Operations + ['&', 'o']) ) do
                    Inc(i);
                  { if we should evaluate function }
                  if Elem[i] = '&'
                  then DoFuncComp  { compile function  }
                  else begin
                    { for binary operations : put arguments to FPU stack }
                    if (i-1 <> Top) and (i-2 <> Top) and (Top <> -1)
                      then PushStack;
                    if (i-2 = Top) or ((Elem[i-2] = 's') and (Id[i-2] <= 4))
                      then ReverseOp := True
                      else ReverseOp := False;
                    if (i-1 <> Top)
                      then if ReverseOp
                             then SetTo8087(i - 1, 2)
                             else SetTo8087(i - 1, 1);
                    if (i-2 <> Top)
                      then SetTo8087(i - 2, 2);
                    { Insert code for binary operations }
                    case Elem[i] of
                    '+': TempCode.AddCode(cFADDP);
                    '-': if ReverseOp
                           then TempCode.AddCode(cFSUBP)
                           else TempCode.AddCode(cFSUBRP);
                    '*': TempCode.AddCode(cFMULP);
                    '/': if ReverseOp
                           then TempCode.AddCode(cFDIVP)
                           else TempCode.AddCode(cFDIVRP);
                    '^': begin
                           if ReverseOp then
                           begin
                             TempCode.AddCode(cFXCH_ST);
                             TempCode.Add($C9);
                           end;
                           { Insert code of Pow function }
                           TempCode.AddCode(BuiltInFuncs[6].Code);
                         end;
                    { Insert code for logical operation }
                    'o': case Id[i] of
                           0: if ReverseOp { 0 = '>' }
                            {<} then TempCode.AddCode(BuiltInOperators[1].Code)
                            {>} else TempCode.AddCode(BuiltInOperators[0].Code);
                           1: if ReverseOp { 1 = '<' }
                            {>} then TempCode.AddCode(BuiltInOperators[0].Code)
                            {<} else TempCode.AddCode(BuiltInOperators[1].Code);
                           2: if ReverseOp { 2 = '>=' }
                           {<=} then TempCode.AddCode(BuiltInOperators[3].Code)
                           {>=} else TempCode.AddCode(BuiltInOperators[2].Code);
                           3: if ReverseOp { 3 = '<=' }
                           {>=} then TempCode.AddCode(BuiltInOperators[2].Code)
                           {<=} else TempCode.AddCode(BuiltInOperators[3].Code);
                                           { 4 = '=' }
                           4: TempCode.AddCode(BuiltInOperators[4].Code);
                                           { 5 = '<>' }
                           5: TempCode.AddCode(BuiltInOperators[5].Code);
                         end;
                    end;
                  end;
                  { Remove used elements }
                  if Elem[i] = '&'
                    then k := Id[i] + 1 else k := 2;
                  for j := 1 to k + 1 do
                    Delete(i-k);
                  { Push stack }
                  Top := i - k;
                  Insert(Top, MakeItem('s', StackPtr + 1, 0));
                end;
                { Final result }
                if Elem[0] <> 's' then SetTo8087(0, 0);
              end;
            end;
          Postfix.Free;
        end
        else Result := fcInternal;
    end
    else Result := fcInternal;
end;

var
  i, j, tI, tL, tA: Integer;
begin
  TempCode := TCodeArray.Create;
  Constants := TArray.Create(SizeOf(Extended));
  LinkerInfo := TIntArray.Create;
  { Current stack pointer }
  StackPtr := 0;
  Args := TInternalExprArray.Create;
  if (Args <> nil) and (TempCode <> nil) and (Constants <> nil) and
     (LinkerInfo <> nil)
  then begin
    tA := Arguments(S, FLib);
    for i := 0 to tA - 1 do
      Args.Add(MakeItem('a', i, 0));
    {$IFNDEF WIN32}
    if tA > 0
      then if coCallDirect and FOptions > 0
             then TempCode.AddCode(c_dir_bp_prepare)
             else TempCode.AddCode(c_proc_bp_prepare);
    {$ENDIF}
    Result := _Compile(S);
    if Result = fcOk
    then begin
      {$IFNDEF WIN32}
      if tA > 0
        then if coCallDirect and FOptions > 0
               then TempCode.AddCode(c_dir_POP_bp);
      {$ENDIF}
      if coCallDirect and FOptions > 0
        then TempCode.AddCode(c_dir_RETF)
        else TempCode.AddCode(c_proc_RETF);
      { Allocate memory for code and constants }
      CodeSize := TempCode.Count;
      ConstSize := Constants.Count*SizeOf(Extended);
      if ConstSize > 0
        then GetMem(Cnst, ConstSize)
        else Cnst := nil;
      GetMem(Code, CodeSize);
      if (Code <> nil) and ((ConstSize = 0) or (Cnst <> nil))
      then begin
        { Link }
        for i := 0 to LinkerInfo.Count - 1 do
        begin
          tL := LinkerInfo.Item[i];
          tI := TempCode.GetInteger(tL);
          for j := 1 to SizeOf(Integer) do
            TempCode.Delete(tL);
          TempCode.InsertInteger(tL, tI*SizeOf(Extended) +
          {$IFDEF WIN32}
            Integer(Cnst)
          {$ELSE}
            Ofs(Cnst^)
          {$ENDIF}
          );
        end;
        { Move Code }
        Move(TempCode.Data^, Code[0], TempCode.Count);
        { Move Constants }
        Move(Constants.Data^, Cnst[0], ConstSize);
      end
      else begin
        CodeSize := 0;
        Result := fcInternal
      end
    end
  end else Result := fcInternal;
  { Cleanup }
  if Args <> nil then Args.Free;
  if Constants <> nil then Constants.Free;
  if TempCode <> nil then TempCode.Free;
  if LinkerInfo <> nil then LinkerInfo.Free;
end;

end.
