{***************************************************************}
{                                                               }
{  FormulaLib Manager for Delphi ( 16 & 32-bit )                }
{                                                               }
{  Copyright (c) 1996, 1997  Dimak & Vlad.S                     }
{                                                               }
{  e-mail:  dimak@uniyar.ac.ru  &  vlads@uniyar.ac.ru           }
{                                                               }
{***************************************************************}

unit FCLibMan;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, TabNotBk, {$IFDEF WIN32} ComCtrls, {$ENDIF}
  FCLib;

{ TFormulaLibMan - dialog window which allows the user }
{ manage library contents }
type
  TFormulaLibMan = class(TForm)
    CancelBtn: TButton;
    EditBtn: TBitBtn;
    AddBtn: TBitBtn;
    DelBtn: TBitBtn;
    OkBtn: TButton;
    Notebook: TTabbedNotebook;
    { depending on NoteBook.PageIndex List shows aliales or functions }
    { from the Lib  }
    List: TListBox;
    procedure EditBtnClick(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure DelBtnClick(Sender: TObject);
    procedure NotebookChange(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
  private
    { user function and aliases library to edit }
    Lib: TFormulaLib;
    procedure UpdateList(Page: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    { edit library ALib }
    { Index = 0 - show user functions at startup }
    { Index = 1 - show user aliases  at startup }
    function Edit(ALib: TFormulaLib; Index: Integer): Boolean;
  end;

var
  FormulaLibMan: TFormulaLibMan;

implementation

{$R *.DFM}

uses FCLibEd;

constructor TFormulaLibMan.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Lib := TFormulaLib.Create(Self);
end;


function TFormulaLibMan.Edit(ALib: TFormulaLib; Index: Integer): Boolean;
begin
  Result := False;
  if Lib <> nil
    then begin
      { store ALib to Lib }
      Lib.Assign(ALib);
      Notebook.PageIndex := Index;
      { update list }
      UpdateList(Notebook.PageIndex);
      { update library after editing }
      if ShowModal = mrOk
        then begin
          ALib.Assign(Lib);
          Result := True;
        end;
    end;
end;

{ UpdateList refresh List: TListBox to show aliases of function  }
procedure TFormulaLibMan.UpdateList(Page: Integer);
var
  i: Integer;
begin
  List.Items.Clear;
  if Page = 0
    then
      for i := 0 to Lib.Functions.Count - 1 do
        List.Items.Add(Lib.Functions.Names[i] + ' = ' + Lib.Functions.Sources[i])
    else
      for i := 0 to Lib.Aliases.Count - 1 do
        List.Items.Add(Lib.Aliases.Names[i] + ' = ' + Lib.Aliases.Sources[i]);
  if List.Items.Count > 0 then List.ItemIndex := 0;
end;

{ Edit current entry }
procedure TFormulaLibMan.EditBtnClick(Sender: TObject);
var
  Temp: Integer;
begin
  { edit alias or function depending on NoteBook.PageIndex }
  with TFormulaEditor.Create(Self) do
    try
      if Notebook.PageIndex = 0
        then Edit(Lib.Functions, List.ItemIndex, '')
        else Edit(Lib.Aliases, List.ItemIndex, '');
    finally
      Free;
    end;
  { update visible list }
  Temp := List.ItemIndex;
  UpdateList(Notebook.PageIndex);
  List.ItemIndex := Temp;
end;

{ Add new entry }
procedure TFormulaLibMan.AddBtnClick(Sender: TObject);
var
  Temp: Integer;
begin
  { add alias or function depending on NoteBook.PageIndex }
  with TFormulaEditor.Create(Self) do
    try
      if Notebook.PageIndex = 0
        then Edit(Lib.Functions, -1, '')
        else Edit(Lib.Aliases, -1, '');
    finally
      Free;
    end;
  { update visible list }
  Temp := List.ItemIndex;
  UpdateList(Notebook.PageIndex);
  List.ItemIndex := Temp;
end;

{ delete current entry }
procedure TFormulaLibMan.DelBtnClick(Sender: TObject);
var
  Temp: Integer;
begin
  if ( List.ItemIndex >= 0 ) and
     ( MessageBox(0, 'Do You really want to delete this entry?', 'Delete',
           mb_YesNo or mb_IconQuestion) = id_Yes )
  then begin
  { actual delete }
    if Notebook.PageIndex = 0
      then Lib.Functions.Delete(List.ItemIndex)
      else Lib.Aliases.Delete(List.ItemIndex);
    Temp := List.ItemIndex;
    List.Items.Delete(List.ItemIndex);
    if Temp >= List.Items.Count
      then List.ItemIndex := List.Items.Count - 1
      else List.ItemIndex := Temp;
  end;
end;

procedure TFormulaLibMan.NotebookChange(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
  { update visible list when Notebook changes }
  UpdateList(NewTab);
end;

end.
