{
  Name    :   AccountTypes component.

  Purpose :   MTS component that Implements the IAccountTypes interface.

  Usage   :   This source code is supplied as example code with Delphi Informant
               magazine.
              The author makes no guarantee for its correctness or fitness
               for any purpose and cannot be held responsible for any loss
               or damage that may occur through its use or misuse.

  Author  :   Paul M. Fairhurst, July 1998.
}
unit AccountTypes;

interface

uses
  MtsObj, Mtx, ComObj, DelphiBankServer_TLB;

type
  TAccountTypes = class(TMtsAutoObject, IAccountTypes)
  protected
    function GetAccountType(AcctTypeID: Integer; var AccountName: WideString;
      var InterestRate, DefMinBalance: Single;
      var ResultStr: WideString): Integer; safecall;
    function AddAccountType(var AcctTypeID: Integer; const AccountName: WideString;
      InterestRate, DefMinBalance: Single;
      var ResultStr: WideString): Integer; safecall;
    { Protected declarations }
  end;

implementation

uses ComServ, SysUtils;

const
  cATSuccess : Integer = 0;
  cATFailure : Integer = -1;

function TAccountTypes.GetAccountType(AcctTypeID: Integer;
  var AccountName: WideString; var InterestRate, DefMinBalance: Single;
  var ResultStr: WideString): Integer;
const
  cMethodName : String = 'AccountTypes.GetAccountType';
begin
  Result := cATSuccess;

  try
    case AcctTypeID of
      1 : begin { Current Account Type. }
            AccountName   := 'Current Account';
            InterestRate  := 0.1;
            DefMinBalance := -100;
          end;
      2 : begin { Gold Account Type. }
            AccountName   := 'Gold Account';
            InterestRate  := 1.2;
            DefMinBalance := 1000;
          end;
      3 : begin { Savings Account Type. }
            AccountName   := 'Savings Account';
            InterestRate  := 4.9;
            DefMinBalance := 500;
          end;
      4 : begin { Savings Plus Account Type. }
            AccountName   := 'Savings Plus Account';
            InterestRate  := 5.9;
            DefMinBalance := 5000;
          end;
    else
      raise Exception.Create('Unknown account type.');
    end;

    ResultStr := cMethodName + ' - successful';
    SetComplete; { We're successfully done. }
  except
    on E: Exception do begin
      { Ooops. }
      Result    := cATFailure;
      ResultStr := cMethodName + ' exception - ' + E.Message;
      SetAbort; { Cannot continue. }
    end;
  end;
end;

function TAccountTypes.AddAccountType(var AcctTypeID: Integer;
  const AccountName: WideString; InterestRate, DefMinBalance: Single;
  var ResultStr: WideString): Integer;
const
  cMethodName : String = 'AccountTypes.AddAccountType';
begin
  { Not implemented yet. }
  Result    := cATFailure;
  ResultStr := cMethodName + ' is not implemented yet';
  SetAbort;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TAccountTypes, Class_AccountTypes,
    ciMultiInstance, tmApartment);
end.
