unit PopCal;

{
  Popup calendar.

  Written by Keith Wood.
  Version 1.0 - 9 June 1998.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Calendar, Buttons;

type
  TfrmCalendar = class(TForm)
    btnPrevYear: TBitBtn;
    btnPrevMonth: TBitBtn;
    btnNextMonth: TBitBtn;
    btnNextYear: TBitBtn;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    calCalendar: TCalendar;
    lblYearMonth: TLabel;
    procedure calCalendarChange(Sender: TObject);
    procedure btnPrevYearClick(Sender: TObject);
    procedure btnPrevMonthClick(Sender: TObject);
    procedure btnNextMonthClick(Sender: TObject);
    procedure btnNextYearClick(Sender: TObject);
    procedure calCalendarDblClick(Sender: TObject);
  private
    function GetDate: TDateTime;
    procedure SetDate(dDate: TDateTime);
  public
    property Date: TDateTime read GetDate write SetDate;
    class function Execute(dDate: TDateTime): TDateTime;
  end;

implementation

{$R *.DFM}

{ Only access to this class - pass initial date, returns selected date }
class function TfrmCalendar.Execute(dDate: TDateTime): TDateTime;
begin
  with TfrmCalendar.Create(Application) do
    try
      Result := dDate;
      Date := dDate;
      if ShowModal = mrOK then
        Result := Date;
    finally
      Free;
    end;
end;

{ Return the selected date }
function TfrmCalendar.GetDate: TDateTime;
begin
  with calCalendar do
    Result := EncodeDate(Year, Month, Day);
end;

{ Set the initial date }
procedure TfrmCalendar.SetDate(dDate: TDateTime);
var
  iYear, iMonth, iDay: Word;
begin
  if dDate = 0 then  { Not set - change to today }
    dDate := Date;
  DecodeDate(dDate, iYear, iMonth, iDay);
  with calCalendar do
  begin
    Year := iYear;
    Month := iMonth;
    Day := iDay;
  end;
  calCalendarChange(calCalendar);
end;

{ Update label on change of date }
procedure TfrmCalendar.calCalendarChange(Sender: TObject);
begin
  with calCalendar do
    lblYearMonth.Caption := FormatDateTime('mmmm yyyy', EncodeDate(Year, Month, Day));
end;

{ Move to previous year }
procedure TfrmCalendar.btnPrevYearClick(Sender: TObject);
begin
  calCalendar.PrevYear;
end;

{ Move to previous month }
procedure TfrmCalendar.btnPrevMonthClick(Sender: TObject);
begin
  calCalendar.PrevMonth;
end;

{ Move to next year }
procedure TfrmCalendar.btnNextMonthClick(Sender: TObject);
begin
  calCalendar.NextMonth;
end;

{ Move to next month }
procedure TfrmCalendar.btnNextYearClick(Sender: TObject);
begin
  calCalendar.NextYear;
end;

{ Double-click is the same as selecting }
procedure TfrmCalendar.calCalendarDblClick(Sender: TObject);
begin
  btnOK.Click;
end;

end.
