unit RepBase;

{
  Generic QuickReport to allow for common processing.

  Written by Keith Wood.
  Version 1.0 - 9 June 1998.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Quickrep, StdCtrls, ExtCtrls, DB, DBTables;

type
  TqrfBaseClass = class of TqrfBase;

  TqrfBase = class(TForm)
    qrpQuickReport: TQuickReport;
    qrbHeader: TQRBand;
    QRSysData1: TQRSysData;
    QRShape2: TQRShape;
    qrbDetail: TQRBand;
    qrbSummary: TQRBand;
    qrmSelection: TQRMemo;
    qrbFooter: TQRBand;
    QRShape1: TQRShape;
    qryReport: TQuery;
    dsrReport: TDataSource;
    QRSysData2: TQRSysData;
    QRSysData3: TQRSysData;
    procedure qrbSummaryBeforePrint(var PrintBand: Boolean);
  private
  protected
    procedure SetParams(slsParams: TStrings); virtual;
    procedure SetSelection(sSelection, sSortDesc: String);
  public
    class procedure ShowReport(sReport, sTitle: String;
      slsParams: TStrings; bPreview: Boolean);
  end;

implementation

{$R *.DFM}

uses
  RepParam;

{ Create the report, pass parameters to it and then show it }
class procedure TqrfBase.ShowReport(sReport, sTitle: String;
  slsParams: TStrings; bPreview: Boolean);
var
  clsReport: TqrfBaseClass;
begin
  { Find the class with this name.
    Each such class must be registered with Delphi, eg.
      initialization
        RegisterClass(TqrfReport1);
  }
  clsReport := TqrfBaseClass(FindClass(sReport));
  { Create a new report from this class }
  with clsReport.Create(Application) do
    try
      { Pass the selected parameters }
      SetParams(slsParams);
      { And preview or print the report }
      if bPreview then
        qrpQuickReport.Preview
      else
        qrpQuickReport.Print;
    finally
      Free;
    end;
end;

{ Set the title of the report }
procedure TqrfBase.SetParams(slsParams: TStrings);
begin
  qrpQuickReport.ReportTitle := slsParams.Values[sTitle];
end;

{ Set selection criteria and sort sequence for display }
procedure TqrfBase.SetSelection(sSelection, sSortDesc: String);
begin
  if sSelection <> '' then
    sSelection := 'Records selected where ' + sSelection;
  if sSortDesc <> '' then
  begin
    if sSelection = '' then
      sSelection := 'Sorted by ' + sSortDesc
    else
      sSelection := sSelection + ' and sorted by ' + sSortDesc;
  end;
  qrmSelection.text := sSelection;
end;

{ Don't show band if no selection criteria }
procedure TqrfBase.qrbSummaryBeforePrint(var PrintBand: Boolean);
begin
  PrintBand := (qrmSelection.Lines.Count > 0);
end;

end.
