unit ESDm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComServ, ComObj, VCLCom, StdVcl, BdeProv, DataBkr, DBClient, EbSrvr_TLB,
  Provider, Db, DBTables;

type
  TEbServer = class(TRemoteDataModule, IEbServer)
    Database1: TDatabase;
    CustomerTable: TTable;
    CustomerProv: TProvider;
    OrderTable: TTable;
    OrderProv: TProvider;
    procedure EbServerCreate(Sender: TObject);
    procedure EbServerDestroy(Sender: TObject);
    procedure OrderTableBeforePost(DataSet: TDataSet);
    procedure CustomerProvGetDataSetProperties(Sender: TObject;
      DataSet: TDataSet; out Properties: OleVariant);
  private
    { Private declarations }
    ClientConnection: IEbClient;
  public
    { Public declarations }
  protected
    function Get_CustomerProv: IProvider; safecall;
    function Get_OrderProv: IProvider; safecall;
    procedure FilterOn; safecall;
    procedure FilterOff; safecall;
    procedure ConnectClient(const Client: IEbClient); safecall;
    function IsDatabase: WordBool; safecall;
  end;

var
  EbServer: TEbServer;

implementation

uses ESMainF;

{$R *.DFM}

function TEbServer.Get_CustomerProv: IProvider;
begin
  Result := CustomerProv.Provider;
end;

function TEbServer.Get_OrderProv: IProvider;
begin
  Result := OrderProv.Provider;
end;

procedure TEbServer.EbServerCreate(Sender: TObject);
begin
  MainForm.IncrementCount;
end;

procedure TEbServer.EbServerDestroy(Sender: TObject);
begin
  MainForm.DecrementCount;
end;

procedure TEbServer.FilterOn;
begin
  CustomerTable.Filtered := True;
  if CustomerTable.Filtered then
    ClientConnection.ConfirmFilter('Filter Is On')
  else
    ClientConnection.ConfirmFilter('Filter Is Off');
end;

procedure TEbServer.FilterOff;
begin
  CustomerTable.Filtered := False;
  if CustomerTable.Filtered then
    ClientConnection.ConfirmFilter('Filter Is On')
  else
    ClientConnection.ConfirmFilter('Filter Is Off');
end;

procedure TEbServer.OrderTableBeforePost(DataSet: TDataSet);
begin
  with DataSet do
    if FieldByName('ShipDate').AsDateTime <
       FieldByName('SaleDate').AsDateTime then
      raise Exception.Create('Ship date must be greater than sale date.'); 
end;

procedure TEbServer.ConnectClient(const Client: IEbClient);
begin
  ClientConnection := Client;
end;

procedure TEbServer.CustomerProvGetDataSetProperties(Sender: TObject;
  DataSet: TDataSet; out Properties: OleVariant);
begin
  Properties := VarArrayCreate([0,1], varVariant);
  Properties[0] := VarArrayOf(['Filter', DataSet.Filter, True]);
  Properties[1] := VarArrayOf(['Filtered', DataSet.Filtered, False]);
end;

function TEbServer.IsDatabase: WordBool;
begin
  Result := False;
  try
    Database1.Connected := True;
    Result := True;
  except
  end; //try
end;

initialization
  TComponentFactory.Create(ComServer, TEbServer,
    Class_EbServer, ciMultiInstance, tmApartment);
end.
