unit EcMainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, Grids, DBGrids, ExtCtrls, DBCtrls, StdCtrls, Db,
  ComObj, EbSrvr_TLB, ActiveX, ComCtrls;

type
  TEcMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DBNavigator2: TDBNavigator;
    DBGrid2: TDBGrid;
    View1: TMenuItem;
    USOnly1: TMenuItem;
    AllCountries1: TMenuItem;
    ApplyChanges1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    ViewAllOrders1: TMenuItem;
    N2: TMenuItem;
    FilterLabel: TLabel;
    FilterStringLabel: TLabel;
    Label1: TLabel;
    SaveLocally1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure USOnly1Click(Sender: TObject);
    procedure AllCountries1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure ApplyChanges1Click(Sender: TObject);
    procedure ViewAllOrders1Click(Sender: TObject);
    procedure SaveLocally1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    procedure ConnectToServer;
    procedure DisableServerFeatures;
    function  LoadLocalData: Boolean;
  public
    { Public declarations }
  end;

  TCallBack = class(TAutoIntfObject, IEbClient)
    procedure ConfirmFilter(const Msg: WideString); safecall;
  end;

var
  EcMainForm:   TEcMainForm;
  IsConnected:  Boolean = False;

implementation

uses EcDm, EcAllOrd;

{$R *.DFM}

const
  CustomerFileName    = 'customer.ffd';
  OrderFileName       = 'order.ffd';
  AllOrderFileName    = 'allorder.ffd';

var
  ServerTypeLib:      ITypeLib;
  TypeLibResult:      HResult;
  CallBack:           IEbClient;

procedure TCallback.ConfirmFilter(const Msg: WideString);
begin
  EcMainForm.FilterLabel.Caption := Msg;
end;

procedure TEcMainForm.DisableServerFeatures;
{Disable features that are only available when connected
 to the application server.}
begin
  View1.Enabled := False;
  ApplyChanges1.Enabled := False;
  FilterStringLabel.Visible := False;
  FilterLabel.Visible := False;
end;

procedure TEcMainForm.ConnectToServer;
{If the -L command line parameter is not present try to connect
 to the application server. If the connection succeeds set
 the global variable IsConnected to True.}
begin
  {If the local command line parameter is present do not
   try to connect to the server.}
  if ParamCount > 0 then
    if UpperCase(ParamStr(1)) = '-L' then
    begin
      DisableServerFeatures;
      Exit;
    end; //if
  {Try to connect to the server. If the connection is established
   set IsConnected to True;}
  with MainDm do
  begin
    try
      EbConn.Connected := True;
      IsConnected := True;
    except
      DisableServerFeatures;
      Exit;
    end; //try
  end; //with
  {If the server is available see if the database is. If not
   shut down the server.}
  with MainDm do
  begin
    if not EbConn.AppServer.IsDatabase then
    begin
      IsConnected := False;
      EbConn.Connected := False;
    end; //if
  end; //with
end;

function TEcmainForm.LoadLocalData: Boolean;
{Try to load local data files. If they exist return True
 otherwise return False;}
begin
  Result := False;
  with MainDm do
  begin
    {If local files exist, load them.}
    if FileExists(CustomerFileName) then
    begin
      CustomerCds.LoadFromFile(CustomerFileName);
      Result := True;
    end; //if
    if FileExists(OrderFileName) then
      OrderCds.LoadFromFile(OrderFileName);
    if FileExists(AllOrderFileName) then
      AllOrdersCds.LoadFromFile(AllOrderFileName);
  end; //with
end;

procedure TEcMainForm.FormCreate(Sender: TObject);
var
  Srvr:        IEbServer;
begin
  {Try to connect to the server.}
  ConnectToServer;
  with MainDm do
  begin
    {If the server is not available and there is no local
     data to load notify the user and terminate.}
    if (LoadLocalData = False) and (IsConnected = False) then
    begin
      MessageDlg('There is no data available.',
                 mtInformation, [mbOK], 0);
      Application.Terminate;
      Exit;
    end; //if
    {Open the client datasets.}
    CustomerCds.Open;
    OrderCds.Open;
    {Display the filter expression.}
    FilterStringLabel.Caption := CustomerCds.GetOptionalParam('Filter');
  end; //with
  {If connected to the server set up the callback interface.}
  if IsConnected then
  begin
    TypeLibResult := LoadRegTypeLib(LIBID_EbSrvr, 1, 0, 0, ServerTypeLib);
    if TypeLibResult <> S_OK then
    begin
      MessageDlg('Error loading type library.', mtError, [mbOK], 0);
      Exit;
    end; //if
    //Create an instance of the TCallback object.
    CallBack := TCallback.Create(ServerTypeLib, IEbClient);
    //Get an interface reference to the server
    Srvr := IDispatch(MainDm.EbConn.AppServer) as IEbServer;
    //Pass the interface handle of the TCallback object to the server.
    Srvr.ConnectClient(CallBack);
  end; //if
end;

procedure TEcMainForm.USOnly1Click(Sender: TObject);
var
  IServer:     IEbServer;
begin
  with MainDm do
  begin
    IServer := IDispatch(EbConn.AppServer) as IEbServer;
    IServer.FilterOn;
    CustomerCds.Refresh;
    {Display the filter expression.}
    FilterStringLabel.Caption := CustomerCds.GetOptionalParam('Filter');
  end;
end;

procedure TEcMainForm.AllCountries1Click(Sender: TObject);
var
  IDispServer:       IEbServerDisp;
begin
  with MainDm do
  begin
    IDispServer := IDispatch(EbConn.AppServer) as IEbServerDisp;
    IDispServer.FilterOff;
    CustomerCds.Refresh;
    {Display the filter expression.}
    FilterStringLabel.Caption := CustomerCds.GetOptionalParam('Filter');
  end;
end;

procedure TEcMainForm.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TEcMainForm.ApplyChanges1Click(Sender: TObject);
begin
  with MainDm do
  begin
    with CustomerCds do
    begin
      {If there is an unposted record post it.}
      if State in [dsEdit, dsInsert] then Post;
      {If there are changes apply them. If the changes
       are applied successfully refresh.}
      if ChangeCount > 0 then
        if ApplyUpdates(ChangeCount) = 0 then Refresh;
    end; //with CustomerCds
    with OrderCds do
    begin
      {If there is an unposted record post it.}
      if State in [dsEdit, dsInsert] then Post;
      {If there are changes apply them. If the changes
       are applied successfully refresh.}
      if ChangeCount > 0 then
        if ApplyUpdates(ChangeCount) = 0 then Refresh;
    end; //with OrderCds
  end; //with MainDm
  {If local files exist, delete them now that the updates have
   been applied.}
  if FileExists(CustomerFileName) then
    DeleteFile(CustomerFileName);
  if FileExists(OrderFileName) then
    DeleteFile(OrderFileName);
  if FileExists(AllOrderFileName) then
    DeleteFile(AllOrderFileName);
end;

procedure TEcMainForm.ViewAllOrders1Click(Sender: TObject);
begin
  AllOrdersForm := TAllOrdersForm.Create(Self);
  AllOrdersForm.Show;
end;

procedure TEcMainForm.SaveLocally1Click(Sender: TObject);
begin
  with MainDm do
  begin
    CustomerCds.SaveToFile(CustomerFileName);
    OrderCds.SaveToFile(OrderFileName);
    AllOrdersCds.SaveToFile(AllOrderFileName);
  end; //with
end;

procedure TEcMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not IsConnected then
    SaveLocally1Click(Self);
end;

end.
